Package: BoolNet
Type: Package
Title: Construction, Simulation and Analysis of Boolean Networks
Version: 2.1.9
Date: 2023-10-02
Authors@R: c(person("Christoph", "Müssel", role = "aut"),
             person("Martin", "Hopfensitz", role = "aut"),
             person("Dao", "Zhou", role = "aut"),
             person(c("Hans", "A."), "Kestler", email = "hans.kestler@uni-ulm.de", role = c("aut", "cre")),
			 person("Armin", "Biere", role = "ctb", comment="contributed PicoSAT code"),
			 person(c("Troy", "D."), "Hanson", role = "ctb", comment="contributed uthash macros"))
Imports: igraph (>= 0.6), XML
Description: Functions to reconstruct, generate, and simulate synchronous, asynchronous, probabilistic, and temporal Boolean networks. Provides also functions to analyze and visualize attractors in Boolean networks <doi:10.1093/bioinformatics/btq124>.
License: Artistic-2.0
LazyLoad: yes
ByteCompile: TRUE
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2023-10-02 08:56:07 UTC; julian_schwab
Author: Christoph Müssel [aut],
  Martin Hopfensitz [aut],
  Dao Zhou [aut],
  Hans A. Kestler [aut, cre],
  Armin Biere [ctb] (contributed PicoSAT code),
  Troy D. Hanson [ctb] (contributed uthash macros)
Maintainer: Hans A. Kestler <hans.kestler@uni-ulm.de>
Repository: CRAN
Date/Publication: 2023-10-02 14:30:04 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 15:12:53 UTC; unix
Archs: BoolNet.so.dSYM

useDynLib(BoolNet, .registration = TRUE)

import("igraph", "XML")
importFrom("graphics", "abline", "arrows", "axis", "hist", "legend", "lines", "par", "plot", "rect", "text")
importFrom("grDevices", "adjustcolor")
importFrom("grDevices", "dev.interactive")
importFrom("stats", "dist", "kmeans", "quantile", "rnorm", "rpois", "runif")
importFrom("utils", "installed.packages", "tail")

S3method(print, AttractorInfo)
S3method(print, SymbolicSimulation)
S3method(print, BooleanNetwork)
S3method(print, SymbolicBooleanNetwork)
S3method(print, BooleanNetworkCollection)
S3method(print, ProbabilisticBooleanNetwork)
S3method(print, BooleanStateInfo)
S3method(print, TransitionTable)
S3method(print, MarkovSimulation)

export(
loadNetwork,
loadBioTapestry,
loadSBML,
saveNetwork,
getAttractors,
plotAttractors,
plotSequence,
plotStateGraph,
plotNetworkWiring,
plotPBNTransitions,
attractorsToLaTeX,
sequenceToLaTeX,
stateTransition,
getAttractorSequence,
getTransitionTable,
getStateSummary,
getBasinOfAttraction,
getPathToAttractor,
getTransitionProbabilities,
toPajek,
toSBML,
generateRandomNKNetwork,
generateCanalyzing,
generateNestedCanalyzing,
generateState,
generateTimeSeries,
perturbNetwork,
perturbTrajectories,
simplifyNetwork,
simulateSymbolicModel,
reconstructNetwork,
chooseNetwork,
fixGenes,
binarizeTimeSeries,
markovSimulation,
testNetworkProperties,
testAttractorRobustness,
testTransitionRobustness,
testIndegree,
truthTableToSymbolic,
symbolicToTruthTable)

perturbTrajectories <- function(network, 
                                measure = c("hamming", "sensitivity", "attractor"),
                                numSamples = 1000,
                                flipBits = 1,
                                updateType = c("synchronous","asynchronous","probabilistic"),
                                gene,
                                ...)
{
  symbolic <- inherits(network,"SymbolicBooleanNetwork")
  probabilistic <- inherits(network,"ProbabilisticBooleanNetwork") 
  stopifnot(inherits(network,"BooleanNetwork") || symbolic || probabilistic)
  
  measure <- match.arg(measure, c("hamming", "sensitivity", "attractor"))

  if (probabilistic && measure != "hamming")
    stop("Probabilistic networks are only supported for measure=\"hamming\"!")

  fixedIdx <- which(network$fixed != -1)
  nonFixedIdx <- which(network$fixed == -1)  

  if (symbolic)
  {
    maxDelay <- max(network$timeDelays)
    flipIndices <- unlist(mapply(function(x, i)
                          {
                           if (network$fixed[i] == -1)
                             seq_len(x) + (i - 1) * maxDelay
                           else
                            c()  
                          }, network$timeDelays, seq_along(network$genes)))
    startStates <- lapply(seq_len(numSamples), function(i)
    {
      m <- matrix(nrow=maxDelay, ncol=length(network$genes), 
             round(runif(n=maxDelay*length(network$genes))))
      m[,fixedIdx] <- network$fixed[fixedIdx]
      m
    })
  }
  else
  {
    flipIndices <- seq_along(network$genes)[nonFixedIdx]
    startStates <- lapply(seq_len(numSamples), function(i)
    {
      s <- round(runif(n=length(network$genes)))
      s[fixedIdx] <- network$fixed[fixedIdx]
      s
    })
  }
  
  if (length(flipIndices) == 0)
    stop("All genes in this network are fixed!")
  
  perturbedStates <- lapply(startStates, function(state)
  {
    flip <- sample(flipIndices, size=flipBits, replace=FALSE)
    state[flip] <- 1 - state[flip]
    return(state)
  })
  

  if (measure == "hamming")
  {
    dists <- mapply(function(state1, state2)
                     {
                       sum(stateTransition(network=network,
                                           state=state1,
                                           type=updateType,
                                           ...) !=
                           stateTransition(network=network,
                                           state=state2,
                                           type=updateType,
                                           ...))                                          
                     }, startStates, perturbedStates)/length(network$genes)
    return(list(stat=dists, value=mean(dists)))                    
  }
  else
  if (measure == "sensitivity")
  {
    if (missing(gene))
      stop("Parameter \"gene\" must be set for sensitivity analysis!")
    
    if (is.character(gene))
      geneIdx <- which(network$genes == gene)
    else
      geneIdx <- gene
    
    if (length(geneIdx) == 0)
      stop(paste("Unknown gene \"", gene, "\"!", sep=""))
    
    sensitivities <- unname(mapply(function(state1, state2)
                     {
                       stateTransition(network=network,
                                           state=state1,
                                           chosenGene = geneIdx,
                                           ...)[geneIdx] !=
                       stateTransition(network=network,
                                           state=state2,
                                           chosenGene = geneIdx,
                                           ...)[geneIdx]                                          
                     }, startStates, perturbedStates))
    return(list(stat=sensitivities, value=mean(sensitivities)))
  }
  else
  {
    if (symbolic)
    {
      att1 <-  simulateSymbolicModel(network, 
                                     startStates=startStates, 
                                     returnGraph=FALSE,
                                     canonical=TRUE,                                      
                                     ...)
      att2 <- simulateSymbolicModel(network, 
                                    startStates=perturbedStates, 
                                    returnGraph=FALSE, 
                                    canonical=TRUE,                                    
                                    ...)
      attractorAssignment1 <- att1$attractorAssignment
      attractorAssignment2 <- att2$attractorAssignment    
    }
    else
    {
      att1 <- getAttractors(network, 
                            startStates=startStates, 
                            returnTable=TRUE,
                            canonical=TRUE,                                         
                            ...)
      att2 <- getAttractors(network, 
                            startStates=perturbedStates, 
                            returnTable=TRUE,
                            canonical=TRUE,                            
                             ...)
                             
      tt1 <- getTransitionTable(att1)
      tt2 <- getTransitionTable(att2)
      attractorAssignment1 <- tt1$attractorAssignment
      attractorAssignment2 <- tt2$attractorAssignment
      names(attractorAssignment1) <- apply(tt1[,seq_along(network$genes)],1,paste,collapse="")
      names(attractorAssignment2) <- apply(tt2[,seq_along(network$genes)],1,paste,collapse="")
      
      attractorAssignment1 <- unname(attractorAssignment1[sapply(startStates,paste,collapse="")])
      attractorAssignment2 <- unname(attractorAssignment2[sapply(perturbedStates,paste,collapse="")])
      
    }
    
    attractorsEqual <- matrix(sapply(seq_along(att1$attractors), function(i1)
                       {
                         seq1 <- getAttractorSequence(att1, i1)
                         sapply(seq_along(att2$attractors), function(i2)
                         {
                            seq2 <- getAttractorSequence(att2, i2)
                            return(identical(seq1,seq2))
                         })
                       }), nrow=length(att1$attractors))
                           
    stat <- apply(cbind(attractorAssignment1, attractorAssignment2), 1, function(idx)
            {
              attractorsEqual[idx[1],idx[2]]
            })
    return(list(stat=stat, value=mean(stat)))
  }          
}

# Create LaTeX state tables of all attractors in <attractorInfo>.
# Genes are grouped according to <grouping>.
# An additional title can be supplied in <title>.
# If <plotFixed> is set, fixed variables are included in the plot.
# <onColor> and <offColor> specify the colors of ON/1 and OFF/0 states.
# <file> is the name of the output LaTeX document.
attractorsToLaTeX <- function (attractorInfo, subset, title = "", grouping = list(), plotFixed = TRUE, 
        onColor="[gray]{0.9}",offColor="[gray]{0.6}", 
        reverse=FALSE, file="attractors.tex")  
{
  stopifnot(inherits(attractorInfo,"AttractorInfo") || inherits(attractorInfo, "SymbolicSimulation"))
  
  if (inherits(attractorInfo,"AttractorInfo"))
  {
    numGenes <- length(attractorInfo$stateInfo$genes)
    geneNames <- attractorInfo$stateInfo$genes
  }
  else
  {
    numGenes <- ncol(attractorInfo$attractors[[1]])
    geneNames <- colnames(attractorInfo$attractors[[1]])
  } 
  
  if (missing(subset))
      subset <- seq_along(attractorInfo$attractors)
  else
    if (any(subset > length(attractorInfo$attractors)))
      stop("You specified an attractor index that is greater than the total number of attractors in 'subset'!")

  # escape "_" in LaTeX
  genes = gsub("_", "\\_", attractorInfo$stateInfo$genes)
  
  # determine list of genes to be plotted
  whichFixed <- which(attractorInfo$stateInfo$fixedGenes != -1)
  if (plotFixed | (length(whichFixed) == 0))
    plotIndices <- seq_len(numGenes)
  else
    plotIndices <- (seq_len(numGenes))[-whichFixed]
  
  if (inherits(attractorInfo,"AttractorInfo"))
  {
    # convert decimal state numbers to binary state matrices (one for each attractor)
    binMatrices <- lapply(attractorInfo$attractors,function(attractor)
            {
              res <- matrix(apply(attractor$involvedStates,2,function(state)
                dec2bin(state,numGenes)[plotIndices]),nrow=length(plotIndices))
            })

    # count the numbers of attractors with equal lengths
    attractorLengths <- sapply(attractorInfo$attractors,function(attractor)
                               {
                                  if (is.null(attractor$initialStates))
                                  # simple attractor
                                    ncol(attractor$involvedStates)
                                  else
                                  # complex attractor => extra treatment
                                    -1
                               })
  }
  else
  {
    binMatrices <- lapply(attractorInfo$attractors, t)
    attractorLengths <- sapply(binMatrices, ncol)
  }
  
  lengthTable <- table(attractorLengths)
  lengthTable <- lengthTable[as.integer(names(lengthTable)) != -1]
  
  # Open output file, and print header
  sink(file)
  cat("% Please include packages tabularx and colortbl in your master document:\n",
      "% \\usepackage{tabularx,colortbl}\n\n\n",sep="")
      
  res <- lapply(seq_along(lengthTable),function(i)
  # accumulate all attractors with equal length in one matrix and plot them
  {
     len <- as.integer(names(lengthTable)[i])
     attractorIndices <- intersect(which(attractorLengths == len), subset)
     if (length(attractorIndices) > 0)
     {
      # build accumulated matrix     
      totalMatrix <- c()
      for (mat in binMatrices[attractorIndices])
      {
        totalMatrix <- cbind(totalMatrix,mat)
      }
      rownames(totalMatrix) <- geneNames[plotIndices]
      colnames(totalMatrix) <- sapply(attractorIndices,function(i)paste("Attr",i,".",seq_len(len),sep=""))
    
      if(length(grouping)>0)
      {
         # reorder genes according to the supplied groups
        totalMatrix <- totalMatrix[unlist(grouping$index),]
        separationPositions <- c(1,cumsum(sapply(grouping$index,length)+1))
      }
      else
        separationPositions <- c()
    
      # output table header
      cat("\\begin{table}[ht]\n",
           "\\begin{center}\n",
           "\\caption{",
           title, "Attractors with ",len," state(s)}\n",
           "\\begin{tabularx}{\\linewidth}{l", 
         paste(rep(paste(rep(">{\\centering\\arraybackslash}X", 
              len), collapse = " "),length(intersect(which(attractorLengths == len),subset))),collapse="|"), 
      "}\\hline\n",
           sep="")
    
      cat("\t&\t",paste(paste("\\multicolumn{",len,"}{c}{Attr. ",intersect(which(attractorLengths == len),subset),"}",
            sep=""),collapse="\t&\t"),"\\\\\n")    
    
      # output active and inactive states
      if (reverse)
        indices <- rev(seq_len(nrow(totalMatrix)))
      else
        indices <- seq_len(nrow(totalMatrix))
  
      for(j in indices)
      {
        separator <- which(separationPositions==j)
        if (length(separator) != 0)
        {
          cat("\\hline \\multicolumn{",ncol(totalMatrix) + 1,"}{c}{",grouping$class[separator],"}\\\\ \\hline \n",sep="")
        }
        cat("\\textbf{",rownames(totalMatrix)[j],"}\t&\t",sep="")
        for(i in seq_len(ncol(totalMatrix)))
        {
          if(totalMatrix[j,i] == 1)
            cat("\\cellcolor",onColor,"1",sep="")
          else
            cat("\\cellcolor",offColor,"0",sep="")
          if (i < ncol(totalMatrix))
            cat("\t&\t")
        }
        cat("\\\\\n")
      }
    
      # output frequency of attractor (basin size / number of states)
      if (inherits(attractorInfo,"AttractorInfo"))
      {
        if (is.null(attractorInfo$stateInfo$table))
            freq <- rep(NA, length(attractorIndices))
        else
          freq <- round(sapply(attractorInfo$attractors[attractorIndices],
              function(attractor)attractor$basinSize/ncol(attractorInfo$stateInfo$table)) * 100,2)
      }
      else
      {
        if (!is.null(attractorInfo$graph))
        {
          freq <- round(sapply(attractorIndices, 
                        function(i)sum(attractorInfo$graph$attractorAssignment == i)/
                        nrow(attractorInfo$graph)) * 100,2)
        }
        else
          freq <- rep(NA, length(attractorIndices))
      }

      if (!isTRUE(all(is.na(freq))))
      {
        cat("\\hline Freq.\t&\t",paste(paste("\\multicolumn{",len,"}{c}{",freq,"\\%}",
              sep=""),collapse="\t&\t"),"\\\\\n")
      }

      cat("\\hline\\end{tabularx}\n\\end{center}\n",
          "\\end{table}\n\n",sep="")

      totalMatrix
    }
  })
  
  # return a list of accumulated matrices
  sink()
  names(res) <- names(lengthTable)
  return(res)
}

# Perform a Markov chain simulation with <numIterations> iterations/matrix multiplications on <network>.
# If <startStates> is supplied, probabilities for all other start states will be set to 0.
# All probabilities below <cutoff> are regarded as zero. 
markovSimulation <- function(network, numIterations=1000, startStates=list(), cutoff=0.001, returnTable=TRUE)
{
  stopifnot(inherits(network,"ProbabilisticBooleanNetwork") 
            | inherits(network,"BooleanNetwork"))

  if (sum(network$fixed == -1) > 32)
    stop("A Markov chain simulation with more than 32 non-fixed genes is not supported!")

  # the C code requires all interactions to be coded into one vector:

  if (inherits(network,"BooleanNetwork"))
  # deterministic network
  {
    # Assemble all input gene lists in one list <inputGenes>, and remember the split positions in <inputGenePositions>.
    inputGenes <- as.integer(unlist(lapply(network$interactions,function(interaction)interaction$input)))
    inputGenePositions <- as.integer(cumsum(c(0,sapply(network$interactions,
                                     function(interaction)length(interaction$input)))))

    # Do the same for the transition functions.
    transitionFunctions <- as.integer(unlist(lapply(network$interactions,function(interaction)interaction$func)))
    transitionFunctionPositions <- as.integer(cumsum(c(0,sapply(network$interactions,
                                              function(interaction)length(interaction$func)))))
    probabilities <- as.double(rep(1.0,length(network$genes)))
    functionAssignment <- as.integer(0:(length(network$genes)-1))
  }
  else
  # probabilistic network
  {
    wrongProb <- sapply(network$interactions,function(interaction)
                                    abs(1.0-sum(sapply(interaction,function(func)func$probability))) > 0.0001)
    if (any(wrongProb))
      stop(paste("The probabilities of gene(s) ",paste(network$genes[wrongProb],collapse=", ")," do not sum up to 1!",sep=""))

    # Assemble all input gene lists in one list <inputGenes>, and remember the split positions in <inputGenePositions>.
    inputGenes <- as.integer(unlist(lapply(network$interactions,function(interaction)lapply(interaction,function(singleFunc)singleFunc$input))))
    inputGenePositions <- as.integer(cumsum(c(0,unlist(lapply(network$interactions,
             function(interaction)lapply(interaction,function(singleFunc)length(singleFunc$input)))))))

    # Do the same for the transition functions.  
    transitionFunctions <- as.integer(unlist(lapply(network$interactions,function(interaction)lapply(interaction,function(singleFunc)singleFunc$func))))
  
    transitionFunctionPositions <- as.integer(cumsum(c(0,unlist(lapply(network$interactions,
             function(interaction)lapply(interaction,function(singleFunc)length(singleFunc$func)))))))

    probabilities <- as.double(unlist(lapply(network$interactions,function(interaction)lapply(interaction,function(singleFunc)singleFunc$probability))))

    functionAssignment <- as.integer(unlist(mapply(function(index,interaction)rep(index,length(interaction)),0:(length(network$interactions )- 1),
                              network$interactions)))
  }

  # check whether the start states are allowed
  # by comparing the values of the fixed genes
  fixedGenes <- which(network$fixed != -1)

  if (length(startStates) > 0)
  {
    statesValid <- sapply(startStates,function(state)
                      {
                        isTRUE(all(state[fixedGenes] == network$fixed[fixedGenes]))
                      })
    if (!isTRUE(all(statesValid)))
      warning("Some of the supplied start states did not match the restrictions of the fixed genes and were removed!")

    startStates <- startStates[statesValid]
  }

  convertedStartStates <- NULL

  if (length(startStates) > 0)
    convertedStartStates <- sapply(startStates,function(x)bin2dec(x,length(network$genes)))

  on.exit(.C("freeAllMemory", PACKAGE = "BoolNet"))
  # call C code
  res <- .Call("markovSimulation_R",
      inputGenes,inputGenePositions,
      transitionFunctions,transitionFunctionPositions,
      as.integer(network$fixed),
      functionAssignment,
      probabilities,
      as.integer(numIterations),
      convertedStartStates,
      as.double(cutoff),
      as.integer(returnTable),
      PACKAGE="BoolNet")

  if (length(network$genes) %% 32 == 0)
    numElementsPerEntry <- as.integer(length(network$genes) / 32)
  else
    numElementsPerEntry <- as.integer(length(network$genes) / 32  + 1)

  # build result matrix
  reachedStates <- data.frame(t(sapply(res$states,function(state)dec2bin(state,length(network$genes)))),res$probabilities)
  colnames(reachedStates) <- c(network$genes,"Probability") 
  
  if (returnTable)
  {
    initialStates <- matrix(res$initialStates,nrow=numElementsPerEntry)
    nextStates <- matrix(res$nextStates,nrow=numElementsPerEntry)
    res <- list(reachedStates=reachedStates,
                table=list(initialStates=initialStates,nextStates=nextStates,probabilities=res$transitionProbabilities),
                genes=network$genes)
  }
  else
  {
    res <- list(reachedStates=reachedStates,
                table=NULL,
                genes=network$genes)
  }
              
  class(res) <- "MarkovSimulation"
  return(res)
}

generateRandomNKNetwork <- function(n,k,topology=c("fixed","homogeneous","scale_free"),
          linkage=c("uniform","lattice"),
          functionGeneration=c("uniform","biased"),
          validationFunction, failureIterations=10000,
          simplify=FALSE, noIrrelevantGenes=TRUE, 
          readableFunctions=FALSE,
          d_lattice=1, zeroBias=0.5, gamma=2.5, approx_cutoff=100)
{
  k_i_vec <- switch(match.arg(topology),
      fixed = {
          if (length(k) == n)
            k
          else
          if (length(k) == 1)
            rep(k,n)
          else
            stop("k must have 1 or n element(s)!")
        },
      homogeneous = round(rpois(n,k)),
      scale_free = rzeta(n,k,gamma=gamma,approx_cutoff=approx_cutoff),
      stop("'topology' must be one of \"fixed\",\"homogeneous\"")
      )
      
  k_i_vec[k_i_vec > n] <- n
  
  if ((zeroBias == 0 | zeroBias == 1) && noIrrelevantGenes && any(k_i_vec > 0))
    stop("If setting 'zeroBias' to 0 or 1, you must set 'noIrrelevantGenes' to FALSE!")
  
  geneNames <- paste("Gene",seq_len(n),sep="")
  
  if (!missing(validationFunction) && !is.null(validationFunction))
    validationFunction <- match.fun(validationFunction)
  else
    validationFunction <- NULL    
      
  interactions <- mapply(function(i,k_i)
      {
        if (k_i == 0)
        {
          genes <- 0
          func <- round(runif(min=0,max=1,n=1))
        }
        else
        {
          table <- allcombn(2,k_i) - 1
          genes <- switch(match.arg(linkage,c("uniform","lattice")),
            uniform = sample(seq_len(n),k_i,replace=FALSE),
            lattice = {
                region <- c(max(1,round(i - k_i*d_lattice)):max(1,i-1),
                      min(n,i+1):min(n,round(i + k_i*d_lattice)))
                
                sample(region,k_i,replace=FALSE)
              },
            stop("'linkage' must be one of \"uniform\",\"lattice\""))

            containsIrrelevant <- TRUE
            validFunction <- TRUE
            
            counter <- 0
            while((!validFunction) || containsIrrelevant)
            {
                tryCatch(
                {
                  functionGeneration <- match.fun(functionGeneration)
                },
                error = function(e){}
                )
                if (is.function(functionGeneration))
                  func <- functionGeneration(input=genes)
                else
                  func <- switch(match.arg(functionGeneration,c("uniform","biased")),
                      uniform = round(runif(min=0,max=1,n=2^k_i)),
                      biased = as.integer(runif(min=0,max=1,n=2^k_i) > zeroBias),
                      stop("'functionGeneration' must be one of \"uniform\",\"biased\" or a function!"))

                if (noIrrelevantGenes)
                {
                  dropGenes <- apply(table,2,function(gene)
                  # determine all genes that have no influence on the results,
                  # i.e. the result column is equal for 0 values and 1 values
                          {
                            (identical(func[gene==1],
                                  func[gene==0]))
                          })                                    
                  containsIrrelevant <- (sum(dropGenes) > 0)                  
                }
                else
                  containsIrrelevant <- FALSE
                
                if (!is.null(validationFunction))
                  validFunction <- validationFunction(table, func)
                  
                counter <- counter + 1
                if (counter > failureIterations)
                  stop(paste("Could not find a transition function matching the restrictions of",
                             "\"validationFunction\" in", failureIterations, " runs.",
                             "Change \"validationFunction\" or increase \"failureIterations\"!"))
            }
        }
        return(list(input=genes,func=func,
          expression=getInteractionString(readableFunctions,
                         func,geneNames[genes])))
      },seq_len(length(k_i_vec)),k_i_vec,SIMPLIFY=FALSE)
  fixed <- sapply(interactions,function(i)
      {
        if (i$input[1] == 0)
          i$func[1]
        else
          -1
      })

  names(fixed) <- geneNames
  names(interactions) <- geneNames
  
  net <- list(genes=geneNames,interactions=interactions,fixed=fixed)
  class(net) <- "BooleanNetwork"
  if (simplify)
    net <- simplifyNetwork(net,readableFunctions)
  return(net)  
}

generateCanalyzing <- function(input) 
{
 k <- length(input)
 table <- allcombn(2, k) - 1
 canalyzingInput <- sample(1:k,size=1)
 res <- round(runif(min = 0, 
                    max = 1, n = 2^k))
 
 res[table[,canalyzingInput] == sample(c(0,1),size=1)] <- sample(c(0,1), size=1)
 return(res)
}

generateNestedCanalyzing <- function(input) 
{
 k <- length(input)
 table <- allcombn(2, k) - 1
 remainingIndices <- 1:(2^k)
 res <- rep(sample(c(0,1), size=1),2^k)
 
 for (canalyzingInput in sample(1:k,size=k,replace=FALSE))
 {
   newIndices <- which(table[remainingIndices,canalyzingInput] == sample(c(0,1),size=1))
   res[remainingIndices[newIndices]] <- sample(c(0,1), size=1)
   remainingIndices <- setdiff(remainingIndices, remainingIndices[newIndices])
 }
 return(res)
}

# Convert a BooleanNetwork object 
# <network> to a SymbolicBooleanNetwork object
truthTableToSymbolic <- function(network, generateDNFs=FALSE)
{
  stopifnot(inherits(network, "BooleanNetwork"))
  
  res <- list(genes = network$genes,
              fixed = as.integer(network$fixed),
              interactions = lapply(network$interactions, function(int)
              {
                func <- tryCatch(
                {
                  if (generateDNFs != FALSE)
                    parseBooleanFunction(getDNF(int$func, network$genes[int$input], mode=generateDNFs),
                                              network$genes)
                  else                                              
                    parseBooleanFunction(int$expression,
                                              network$genes)
                },
                error =  function(e)
                {
                  parseBooleanFunction(getDNF(int$func, network$genes[int$input], mode=generateDNFs),
                                            network$genes)
                })
                return(func)
              }))

  names(res$fixed) <- res$genes
  res$internalStructs <- .Call("constructNetworkTrees_R",res)
  res$timeDelays <- apply(sapply(res$interactions,maxTimeDelay,genes=res$genes),1,max)
              
  class(res) <- "SymbolicBooleanNetwork"
  return(res)
}

# Several binarization methods for time series. <measurements> is a list of matrices with the
# genes in the rows, each specifying one time series.
# If <method> is "kmeans", k-means binarization is used.
# <nstart> and <iter.max> are the corresponding parameters for k-means. This clustering
# is employed for binarization. 
# If <method> is "edgeDetector", an edge detector is used for binarization.
# If <method> is "edgeDetector" and <edge> is "firstEdge",
# the first "significant" edge in the sorted data is the threshold for the binarization.
# If <edge> is "maxEdge", the algorithm searches for the edge with the highest gradient. 
# With the <scaling> factor, the size of the first edge can be adapted.
# If <method> is "scanStatistic", a scan statistic is used to binarize the data
# If <dropInsignificant> is true, insignificant genes (that are not recommended by the statistic) are removed.
# Returns a list containing the binarized matrix list and (for k-means) a vector
# of thresholds and (for scan statistic) a vector specifying genes to remove.
binarizeTimeSeries <- function(measurements, method=c("kmeans","edgeDetector","scanStatistic"), nstart=100, iter.max=1000, edge=c("firstEdge","maxEdge"), scaling=1, windowSize=0.25, sign.level=0.1, dropInsignificant=FALSE)
{
  if (!is.null(dim(measurements)))
    fullData <- measurements
  else
  # in case of list, paste all matrices before clustering
  {
    fullData <- measurements[[1]]
    for (m in measurements[-1])
    {
      fullData <- cbind(fullData,m)
    }
  }
      
  #switch between the different methods
  switch(match.arg(method),
    kmeans={
      cluster <- apply(fullData,1,function(gene)
        # cluster data using k-means
      {
        cl_res <- kmeans(gene, 2, nstart=nstart,iter.max=iter.max)
    
        if (cl_res$centers[1] > cl_res$centers[2])
        # exchange clusters if necessary, so that smaller numbers
        # are binarized to 0, and larger numbers are binarized to 1
          group <- abs(cl_res$cluster-2)
        else
          group <- cl_res$cluster-1
    
           
        # calculate the binarization threshold
        threshold <-  min(cl_res$centers) + dist(cl_res$centers)[1]/2
        list(bin=group, threshold=threshold)
      })
  
  	
      if (is.null(dim(measurements)))
      # split up the collated binarized measurements into a list of matrices of the original size
      {
        startIndex <- 0
        binarizedTimeSeries <- lapply(measurements,function(m)
              {  
                currentSplit <- (startIndex+1):(startIndex+ncol(m))
                startIndex <<- startIndex + ncol(m)
                t(sapply(cluster,function(cl)
                  cl$bin[currentSplit]))          
              })
      }
      else
      {
        binarizedTimeSeries <-   t(sapply(cluster,function(cl)
                cl$bin))
      }
       #colnames(binarizedTimeSeries)<-colnames(fullData)

      return(list(binarizedMeasurements=binarizedTimeSeries,
            thresholds=sapply(cluster,function(cl)cl$threshold)))

    
    },
    edgeDetector={
      #switch between the different edgedetectors
      switch(match.arg(edge),
        firstEdge={
          cluster <- apply(fullData,1,function(gene)
          # cluster data using edgedetector
          {  
            cl_res <- edgeDetector(gene,scaling,edge="firstEdge")
    
            list(bin=cl_res$bindata,thresholds=cl_res$thresholds)
          })
          
          if (is.null(dim(measurements)))
      		# split up the collated binarized measurements into a list of matrices of the original size
      	  {
       			startIndex <- 0
        		binarizedTimeSeries <- lapply(measurements,function(m)
              	{  
                	currentSplit <- (startIndex+1):(startIndex+ncol(m))
                	startIndex <<- startIndex + ncol(m)
                	t(sapply(cluster,function(cl)
                  	cl$bin[currentSplit]))          
              	})
              	threshlist<-sapply(cluster,function(cl) cl$thresholds)
      	  }
      	  else
      	  {
        	 binarizedTimeSeries <-   t(sapply(cluster,function(cl)
             cl$bin))
             threshlist<-sapply(cluster,function(cl) cl$thresholds)
     	  }

      	  
             return(list(binarizedMeasurements=binarizedTimeSeries,thresholds= threshlist))
          },
        maxEdge={
          cluster <- apply(fullData,1,function(gene)
          # cluster data using edgedetector
          {
            cl_res <- edgeDetector(gene,edge="maxEdge")
    
            list(bin=cl_res$bindata,thresholds=cl_res$thresholds)
          })
          
          if (is.null(dim(measurements)))
      		# split up the collated binarized measurements into a list of matrices of the original size
      	   {
       			startIndex <- 0
        		binarizedTimeSeries <- lapply(measurements,function(m)
              	{  
                	currentSplit <- (startIndex+1):(startIndex+ncol(m))
                	startIndex <<- startIndex + ncol(m)
                	t(sapply(cluster,function(cl)
                  	cl$bin[currentSplit]))          
              	})
              	threshlist<-sapply(cluster,function(cl) cl$thresholds)
      	  	}
      		else
      		{
        		binarizedTimeSeries <-   t(sapply(cluster,function(cl)
                cl$bin))
                threshlist<-sapply(cluster,function(cl) cl$thresholds)
     		}

                
          	return(list(binarizedMeasurements=binarizedTimeSeries,thresholds= threshlist))
                  
         },
            
        
        	stop("'method' must be one of \"firstEdge\",\"maxEdge\"")
       	 )
      

    	},
    	scanStatistic={
    		cluster <- apply(fullData,1,function(gene)
          	# cluster data using scanStatistic
          	{  
            	cl_res <- scanStatistic(gene,windowSize,sign.level)
    
            	list(bin= cl_res$bindata,thresholds=cl_res$thresholds,reject=cl_res$reject)
          	})
          	significant <- sapply(cluster,function(cl)(cl$reject==FALSE))
          	# remove not recommended genes    
          	if (dropInsignificant)
          	{	
          		significant <- sapply(cluster,function(cl)(cl$reject==FALSE))
          		cluster <- cluster[significant]
          	
          	}
          
          	if (is.null(dim(measurements)))
      			# split up the collated binarized measurements into a list of matrices of the original size
      	   	{
       			startIndex <- 0
        		binarizedTimeSeries <- lapply(measurements,function(m)
              	{  
                	currentSplit <- (startIndex+1):(startIndex+ncol(m))
                	startIndex <<- startIndex + ncol(m)
                	t(sapply(cluster,function(cl)
                  	cl$bin[currentSplit]))          
              	})
              	rejectlist<-sapply(cluster,function(cl) cl$reject)
              	threshlist<-sapply(cluster,function(cl) cl$thresholds)
      	  	}
      	  	else
      		{
        		binarizedTimeSeries <-   t(sapply(cluster,function(cl)
                cl$bin))
                rejectlist<-sapply(cluster,function(cl) cl$reject)
                threshlist<-sapply(cluster,function(cl) cl$thresholds)
              
     		}
     		if(any(rejectlist))
     		{
     		 	warning("The following genes show a uniform behaviour and should possibly be excluded from binarization:", paste(rownames(fullData)[!significant],collapse=" "))
     		}
     		       		          
                          
        return(list(binarizedMeasurements=binarizedTimeSeries,thresholds=threshlist,reject=rejectlist))
    	
    },
    stop("'method' must be one of \"kmeans\",\"edgeDetector\",\"scanStatistic\"")
    )
    

  }

# Creates a deterministic Boolean network from a Probabilistic Boolean network
# by choosing the functions specified in <functionIndices> from the
# network <probabilisticNetwork>.
chooseNetwork <- function(probabilisticNetwork,functionIndices,dontCareValues=NULL,readableFunctions=FALSE)
{
  stopifnot(inherits(probabilisticNetwork,"ProbabilisticBooleanNetwork") 
        | inherits(probabilisticNetwork,"BooleanNetworkCollection"))

  if (length(functionIndices) != length(probabilisticNetwork$genes))
    stop("Please provide a vector of function indices for each gene!")

  if (inherits(probabilisticNetwork,"ProbabilisticBooleanNetwork"))
  {
    interactions <- mapply(function(interaction,index)
          {
            list(input=interaction[[index]]$input,
              func=interaction[[index]]$func,
              expression=interaction[[index]]$expression)
          },
          probabilisticNetwork$interactions,functionIndices,SIMPLIFY=FALSE)
  }
  else
  {
    if (!is.null(dontCareValues) && 
        length(dontCareValues) == length(probabilisticNetwork$genes) &&
        is.null(names(dontCareValues)))
          names(dontCareValues) <- probabilisticNetwork$genes
    interactions <- mapply(function(interaction,index,gene)
          {
            func <- interaction[[index]]$func
            dcPos <- which(func == -1)
            
            if (length(dcPos) > 0)
            {
              if (is.null(dontCareValues) || is.null(dontCareValues[[gene]]))
                stop(paste("No values for the \"don't care\" ",
                           "entries were specified for gene \"",
                           gene,"\"!",sep=""))

              if (!all(dontCareValues[[gene]] %in% c(0,1)))
                stop(paste("Invalid values for \"don't care\" entries specified for gene \"",
                           gene,"\"!",sep=""))
                           
              if (length(dontCareValues[[gene]]) != length(dcPos))
                stop(paste("There must be exactly ",length(dcPos),
                           " value(s) for \"don't care\" entries in the function for gene \"",
                           gene,"\"!",sep=""))

              func[dcPos] <- dontCareValues[[gene]]
              expression <- getInteractionString(readableFunctions,
                                                 func,
                                                 probabilisticNetwork$genes[interaction[[index]]$input])
            }
            else
              expression <- interaction[[index]]$expression

            list(input=interaction[[index]]$input,
              func=func,
              expression=expression)
          },
          probabilisticNetwork$interactions,
          functionIndices,
          probabilisticNetwork$genes,SIMPLIFY=FALSE)

  }
  
  res <- list(genes=probabilisticNetwork$genes,
      interactions=interactions,
      fixed=probabilisticNetwork$fixed)
  class(res) <- "BooleanNetwork"
  return(res)
}

# Distribution function of the Zeta distribution
dzeta <- function(k,gamma=2.5,approx_cutoff=100)
{
  zeta_approx <- sum(sapply(seq_len(approx_cutoff),function(x)1/(x^gamma)))
  sapply(k,function(k_i)
  {
    1/(k_i^gamma * zeta_approx)
  })
}

# Quantile function of the Zeta distribution
qzeta <- function(p,maxK,gamma=2.5,approx_cutoff=100)
{
  vals <- cumsum(dzeta(seq_len(maxK),gamma,approx_cutoff))
  sapply(p,function(p_i)
  {
    indices <- which(vals > p_i)
    if (length(indices) == 0)
      return(NA)
    else
      return(indices[1])
  })
}

# Draw random numbers from the Zeta distribution
rzeta <- function(n,maxK,gamma=2.5,approx_cutoff=100)
{
  maxP <- cumsum(dzeta(seq_len(maxK),gamma,approx_cutoff))
  p <- runif(min=0,max=maxP,n=n)
  return(qzeta(p,maxK,gamma,approx_cutoff))
}

# Convert a SymbolicBooleanNetwork object 
# <network> to a BooleanNetwork object
symbolicToTruthTable <- function(network)
{
  stopifnot(inherits(network, "SymbolicBooleanNetwork"))
  res <- list(genes = network$genes,
              fixed = network$fixed,
              interactions = lapply(network$interactions, function(int)
              {
                newInt <- .Call("getTruthTable_R", int, length(network$genes))
                names(newInt) <- c("input","func")
                newInt$expression <- stringFromParseTree(int)
                return(newInt)
              }))
  class(res) <- "BooleanNetwork"
  return(res)
}

# Parse all nodes in <rootNode> 
# (i.e., all nodes of a certain type),
# and write them to a list
parseGeneAttrs <- function(rootNode)
{
    i <- 0
    res <- xmlApply(rootNode,function(child)
        {
            i <<- i + 1
            attrs <- xmlAttrs(child)
            
            name <- unname(attrs["name"])
            if (name == "" | is.na(name))
              name <- paste(xmlName(child),i,sep="_")
            
            # find the "simulationLogic" node, which contains
            # the necessary logic information
            logic <- xmlFindNode(child,"simulationLogic")
            
            initVal <- NULL
            if (is.null(logic))
            {
                warning(paste("Gene ",name," does not specify a simulation logic, assuming AND!",sep=""))
                logic <- "AND"
            }
            else
            {
              # try to find initial values in the simulation parameters
              logic <- logic[[1]]
              params <- xmlFindNode(logic,"simParameters")
              if (!is.null(params))
              {
                for (param in xmlChildren(params[[1]]))
                {
                  pa <- xmlAttrs(param)
                  if (pa["name"] == "initVal")
                  {
                    initVal <- as.numeric(pa["value"])
                    if (initVal != 0 & initVal != 1)
                    # only 0 and 1 are allowed for the initialization
                    {
                       warning("Initial value for gene ",name," is neither 0 nor 1 and is ignored!",sep="")
                       initVal <- NULL
                    }
                    break
                  }
                }
              }
              
              # get the Boolean function of the node
              logic <- unname(xmlAttrs(logic)["type"])
            }
            
            list(id=unname(attrs["id"]),
                 name=adjustGeneNames(name),logic=logic,initVal=initVal,inputs=NULL)
        })     
}

# Parse all types of nodes (genes, signals, etc)
parseAllGenes <- function(rootNode)
{
  genes <- unlist(xmlApply(rootNode,parseGeneAttrs),recursive=FALSE)
  names(genes) <- lapply(genes,function(gene)gene$id)
  return(genes)
}

# Parse the links in the children of <node> and insert them
# into the input lists of the genes
parseAllLinks <- function(rootNode,geneList)
{
  for (link in xmlChildren(rootNode))
  {
      attrs <- xmlAttrs(link)
      
      if (is.na(attrs["sign"]))
      # a link with unspecified direction (inhibition or activation) was found
      {
        warning(paste("The link between \"",geneList[[attrs["src"]]]$name,
                      "\" and \"",geneList[[attrs["targ"]]]$name,
                      "\" is not specified as an enhancer or suppressor and is therefore ignored!",sep=""))
        
      }
      else
      {
        geneList[[attrs["targ"]]]$input[[length(geneList[[attrs["targ"]]]$input) + 1]] <- 
            list(input=unname(attrs["src"]),sign=attrs["sign"])
      }    
  }
  return(geneList)
}

# Load a BioTapestry file (*.btp) from <file>
# and convert it to a Boolean network
loadBioTapestry <- function(file, symbolic=FALSE)
{
    doc <- xmlRoot(xmlTreeParse(file))
    
    # detect the "genome" XML node which holds the nodes and links of the network
    genome <- xmlFindNode(doc,"genome",throwError=TRUE)[[1]]
    
    # find the "nodes" node that contains all genes/inputs/etc
    nodes <- xmlFindNode(genome,"nodes",throwError=TRUE)[[1]]
    
    # read in genes
    geneList <- parseAllGenes(nodes)
    
    # find the "links" node that contains the links/dependencies
    links <- xmlFindNode(genome,"links",throwError=TRUE)[[1]]
    
    # add links to gene list
    geneList <- parseAllLinks(links,geneList)
    
    # build up network structure
    genes <- unname(sapply(geneList,function(gene)gene$name))
    
    geneIds <- names(geneList)
    
    fixed <- rep(-1L,length(genes))
    
    i <- 0
    interactions <- lapply(geneList,function(gene)
        {
            i <<- i + 1
            input <- sapply(gene$input,function(inp)
            {
                which(geneIds == inp$input) 
            })
            
            if (length(input) == 0)
            {
                if (!is.null(gene$initVal))
                # input-only gene without fixed value (=> depending on itself)
                {
                    input <- 0
                    func <- gene$initVal
                    fixed[i] <<- func
                    expr <- func
                } 
                else
                # input-only gene with fixed value
                {
                    input <- i
                    func <- c(0,1)
                    expr <- gene$name
                }
            }    
            else
            # dependent gene
            {
                # determine signs/negations of genes
                inputSigns <- sapply(gene$input,function(inp)inp$sign)  
        
                if (!symbolic || gene$logic == "XOR")
                {
                  tt <- as.matrix(allcombn(2,length(input)) - 1)
                              
                  func <- as.integer(switch(gene$logic,
                      AND = {
                              # calculate truth table for AND
                              apply(tt,1,function(assignment)
                              {
                                  res <- 1
                                  for (i in seq_along(assignment))
                                  {
                                      if (inputSigns[i] == "+")
                                      res <- res & assignment[i]
                                      else
                                      res <- res & !assignment[i]
                                      if (!res)
                                      break
                                  }
                                  res
                              })
                          },
                      OR = {
                              # calculate truth table for OR
                              apply(tt,1,function(assignment)
                              {
                                  res <- 0
                                  for (i in seq_along(assignment))
                                  {
                                      if (inputSigns[i] == "+")
                                      res <- res | assignment[i]
                                      else
                                      res <- res | !assignment[i]
                                      if (res)
                                      break
                                  }
                                  res
                              })
                          },
                      XOR = {
                              # calculate truth table for XOR
                              apply(tt,1,function(assignment)
                              {
                                  res <- assignment[1]
                                  for (i in 2:length(assignment))
                                  {
                                      res <- xor(res,assignment[i])
                                  }
                                  res
                              })
                          },
                      stop(paste("Unknown Boolean operator \"",
                                gene$logic,"\"!",sep=""))
                  ))
                }                
                # get string representation of the input gene literals
                literals <- mapply(function(gene,sign)
                                    {
                                    if (sign == "+")
                                        gene
                                    else
                                        paste("!",gene,sep="")
                                    }, genes[input], inputSigns)
                
                # get string representation of function                    
                expr <- switch(gene$logic,
                    AND = {
                            paste(literals,collapse=" & ")
                        },
                    OR =  {
                            paste(literals,collapse=" | ")
                        },
                    XOR = {
                            getDNF(func,genes[input]) 
                          }      
                        )
            }
            
            if (symbolic)
              return(parseBooleanFunction(expr,varNames=genes))
            else
              return(list(input=input,func=func,expression=expr))
        })
        
    names(interactions) <- genes
    fixed <- as.integer(fixed)
    names(fixed) <- genes
        
    net <- list(genes=genes,interactions=interactions,fixed=fixed)
    
    if (symbolic)
    {
      net$timeDelays <- apply(sapply(net$interactions,maxTimeDelay,genes=net$genes),1,max)
      net$internalStructs <- .Call("constructNetworkTrees_R",net)
      class(net) <- "SymbolicBooleanNetwork"
    }
    else    
      class(net) <- "BooleanNetwork"
    return(net)
}

# Plot a wiring graph of the network <network> with the supplied
# graphical parameters.
# Requires igraph.
# Returns the igraph structure representing the wiring graph. 
plotNetworkWiring <- function(network,layout=layout.fruchterman.reingold,plotIt=TRUE,...)
{
  stopifnot(inherits(network,"ProbabilisticBooleanNetwork") | inherits(network,"BooleanNetworkCollection")
            | inherits(network,"BooleanNetwork") | inherits(network,"SymbolicBooleanNetwork"))
  
  if (installed.packages()["igraph","Version"] < package_version("0.6"))
    bias <- 1
  else
    bias <- 0 
    
  edgeList <- c()
  
  # construct list of edges from interactions

  if (inherits(network,"BooleanNetwork"))
  # deterministic network
  {
    for (i in seq_along(network$genes))
    {
      if (network$interactions[[i]]$input[1] != 0)
      # no edges for constant genes
      {
        edgeList <- rbind(edgeList,
                  cbind(network$interactions[[i]]$input,
                  rep(i,length(network$interactions[[i]]$input))))
      }
    }
  }
  else
  if (inherits(network,"SymbolicBooleanNetwork"))
  # symbolic network
  {
    inputs <- lapply(network$interactions, getInputs, index=TRUE)
    for (i in seq_along(network$genes))
    {
      edgeList <- rbind(edgeList,
                  cbind(inputs[[i]],
                  rep(i,length(inputs[[i]]))))
    }
  }  
  else
  # probabilistic network
  {
    for (i in seq_along(network$genes))
    {
      for (j in seq_along(network$interactions[[i]]))
      {
        if (network$interactions[[i]][[j]]$input[1] != 0)
        # no edges for constant genes
        {
          edgeList <- rbind(edgeList,
                            cbind(network$interactions[[i]][[j]]$input,
                            rep(i,length(network$interactions[[i]][[j]]$input))))
        }
      }
    }
  }

  # build graph from edge list
  res <- graph.data.frame(edgeList-bias,directed=TRUE,vertices=as.data.frame((seq_along(network$genes)) - bias))
  res <- set.vertex.attribute(res,"name",value=network$genes)
  
  args <- list(...)
  
  # check for certain graphical parameters in ... 
  # that have different default values in this plot
  if (is.null(args$vertex.color))
    args$vertex.color <- "grey"
    
  if (is.null(args$edge.arrow.size))
    args$edge.arrow.size <- 0.5
    
  if (is.null(args$vertex.label.cex))
    args$vertex.label.cex <- 0.7

  if (is.null(args$vertex.size))
    args$vertex.size <- 18    

  if (plotIt)
  {
    plot(res,vertex.label=network$genes,vertex.label.cex=args$vertex.label.cex,
         vertex.size=args$vertex.size,vertex.color=args$vertex.color,
         edge.arrow.size=args$edge.arrow.size,
         layout=layout,...)
  }
  return(invisible(res))
}

# Custom print function for class AttractorInfo
print.SymbolicSimulation <- function(x, activeOnly = FALSE, sequences=FALSE, 
                                     graph=FALSE, attractors=TRUE, ...)
{
  cat("Simulation of a symbolic Boolean network\n")
  
  if (!is.null(x$sequences))
  {
    if (sequences)
    {
      cat(sprintf("Sequences for %d start states:\n", length(x$sequences)))
      
      if (!activeOnly)
        print(x$sequences)
      else
        for (i in seq_along(x$sequences))
        {
          cat("[[",i,"]]\n",sep="")
          seq <- x$sequences[[i]]
          cat(paste(apply(seq,1,function(r)
          {
            if (any(r == 1))
              paste(colnames(seq)[which(r == 1)],collapse=", ")
            else
              "--"
          }), collapse="\n"))
          cat("\n\n")
        }
    }
    else
      cat(sprintf("Sequences for %d start states (print with sequences=TRUE to show them)\n", 
                  length(x$sequences)))
    cat("\n")
  }
  if (!is.null(x$graph))
  {
      if (graph)
    {
      cat(sprintf("Graph containing %d state transitions:\n", nrow(x$graph)))
      print(x$graph, activeOnly=activeOnly, ...)
    }
    else
      cat(sprintf("Graph containing %d state transitions (print with graph=TRUE to show them)\n", 
                  nrow(x$graph)))
    cat("\n")                  
  }
  
  if (!is.null(x$attractors))
  {
    if (attractors)
    {
      cat(sprintf("%d Attractors:\n", length(x$attractors)))
      
      for (i in seq_along(x$attractors))
      {
        if (is.null(x$graph))
          printSynchronousAttractor(x$attractors[[i]], i, activeOnly=activeOnly)
        else
          printSynchronousAttractor(x$attractors[[i]], i, 
                                    sum(x$graph$attractorAssignment == i), activeOnly=activeOnly)
      }
    }      
    else
        cat(sprintf("%d Attractors (print with attractors=TRUE to show them)\n", 
                    length(x$attractors)))  
  }                    
  return(invisible(x))
}

# Determine the basin of attraction of attractor <attractorNo> in <attractorInfo>.
getBasinOfAttraction <- function(attractorInfo,attractorNo)
{
  stopifnot(inherits(attractorInfo,"AttractorInfo") || inherits(attractorInfo,"SymbolicSimulation"))

  if (missing(attractorNo) || attractorNo <= 0 || attractorNo > length(attractorInfo$attractors))
    stop("Please provide a valid attractor number!")
  
  table <- getTransitionTable(attractorInfo)
  return(table[which(table$attractorAssignment == attractorNo),,drop=FALSE])
}

generateState <- function(network, specs, default=0)
{
  stopifnot(inherits(network,"ProbabilisticBooleanNetwork")
            || inherits(network,"BooleanNetwork")
            || inherits(network,"SymbolicBooleanNetwork"))
  
  if (!all(names(specs) %in% network$genes))
    stop(paste("Undefined gene names:",
               paste(setdiff(names(specs), network$genes), collapse=", ")))
  
  if (!all(unlist(specs) %in% c(0,1)))
    stop("Please provide only Boolean values!")
 
 lengths <- unique(sapply(specs, length))
 if (length(lengths) > 1) 
    stop("The number of specifications for each gene must be the same!")
 
 if (lengths == 1)
 {   
   state <- rep(default, length(network$genes))
   names(state) <- network$genes
   state[names(specs)] <- specs
 }
 else
 {
   state <- matrix(rep(default, length(network$genes) * lengths), nrow=lengths)
   colnames(state) <- network$genes
   
   for (i in seq_along(specs))
    state[,names(specs)[i]] <- specs[[i]]
 }
 return(state)                
}

# Simplify a Boolean network by dropping all input genes that are not relevant 
# for the output
simplifyNetwork <- function(network,readableFunctions=FALSE)
{
  stopifnot(inherits(network,"BooleanNetwork") | inherits(network,"ProbabilisticBooleanNetwork")
          | inherits(network,"BooleanNetworkCollection"))
  
  if (inherits(network,"BooleanNetwork"))
  {
    network$interactions <- mapply(function(interaction, j)
          {
            if (interaction$input[1] != 0)
            # no constant gene
            {
        
              table <- allcombn(2,length(interaction$input)) - 1
        
              dropGenes <- apply(table,2,function(gene)
              # drop all genes that have no influence on the results,
              # i.e. the result column is equal for 0 values and 1 values
                  {
                    (identical(interaction$func[gene==1],
                         interaction$func[gene==0]))
                  })
              if (sum(!dropGenes) == 0)
              {
                network$fixed[j] <<- interaction$func[1]
                interaction$input <- 0
                interaction$func <- interaction$func[1]
              }
              else
              if (sum(dropGenes) > 0)
              {
                # update network
                network$fixed[j] <<- -1
                dropFunctionIndices <- unlist(sapply(seq_along(dropGenes),function(i)
                      {
                        if(dropGenes[i])
                          which(table[,i] == 0)
                        else
                          NULL
                      }))
                interaction$input <- interaction$input[!dropGenes]
                interaction$func <- interaction$func[-dropFunctionIndices]
                interaction$expression <- 
                  getInteractionString(readableFunctions,
                           interaction$func,
                           network$genes[interaction$input])
              }
            }
            interaction
          }, network$interactions, seq_along(network$interactions), SIMPLIFY=FALSE)
  }
  else
  {
    network$interactions <- lapply(network$interactions,function(gene)
                                   lapply(gene,function(interaction)
      {
        if (interaction$input[1] != 0)
        # no constant gene
        {

          table <- allcombn(2,length(interaction$input)) - 1
  
          dropGenes <- apply(table,2,function(gene)
          # drop all genes that have no influence on the results,
          # i.e. the result column is equal for 0 values and 1 values
              {
                (identical(interaction$func[gene==1],
                     interaction$func[gene==0]))
              })
          if (sum(!dropGenes) == 0)
          {
            interaction$input <- 0
            interaction$func <- interaction$func[1]
          }
          else
          if (sum(dropGenes) > 0)
          {
            # update network
            dropFunctionIndices <- unlist(sapply(seq_along(dropGenes),function(i)
                  {
                    if(dropGenes[i])
                      which(table[,i] == 0)
                    else
                      NULL
                  }))
            interaction$input <- interaction$input[!dropGenes]
            interaction$func <- interaction$func[-dropFunctionIndices]
            interaction$expression <- 
              getInteractionString(readableFunctions,
                       interaction$func,
                       network$genes[interaction$input])
          }
        }
        interaction
      }))
  }
  return(network)
}

# Calculate the Gini index of <x>
Gini <- function(x) 
{
    n <- length(x)
    x <- sort(x)
    1/(n-1)*(n+1-2*(sum((n+1-seq_len(n))*x)/sum(x)))
}

# Compare the Hamming distances of the successor states of states and their perturbed copies.
testTransitionRobustness <- function(network, accumulate=TRUE, params=list())
{
  res <- do.call("perturbTrajectories", c(list(network=network, measure="hamming"), params))
  if (accumulate)
    return(res$value)
  else
    return(res$stat)
}

# Test attractor robustness by searching the original attractor
# in <params$numSamples> perturbed copies of <network> or by perturbing the state trajectories
# and checking whether the attractors change.
testAttractorRobustness <- function(network, accumulate=TRUE, params=list())
{
  origAttrs <- getAttractors(network,canonical=TRUE)

  # decode parameters
  if (is.null(params$perturb))
    perturb <- "functions"
  else
    perturb <- params$perturb
    
  
  if (params$perturb == "trajectories")
  {
    params$perturb <- NULL
    
    if (!is.null(params$copies))
    {
      params$numSamples <- params$copies
      params$copies <- NULL
    }
    res <- do.call("perturbTrajectories", c(list(network=network, measure="attractor"), params))
    if (accumulate)
      return(res$value)
    else
      return(res$stat)
  }  
  else 
  {
    if (is.null(params$method))
      method <- "bitflip"
    else
      method <- params$method
      
    if (is.null(params$maxNumBits))
      maxNumBits <- 1
    else
      maxNumBits <- params$maxNumBits
      
    if (is.null(params$numStates))
      numStates <- max(1,2^length(network$genes)/100)
    else
      numStates <- params$numStates
      
    if (is.null(params$simplify))
      simplify <- (perturb[1] == "states")
    else
      simplify <- params$simplify
    
    if (is.null(params$readableFunctions))
      readableFunctions <- FALSE
    else
      readableFunctions <- params$readableFunctions
      
    if (is.null(params$excludeFixed))
      excludeFixed <- TRUE
    else
      excludeFixed <- params$excludeFixed

    if (!is.null(params$numSamples))
      copies <- params$numSamples
    else    
    if (is.null(params$copies))
      copies <- 100
    else
      copies <- params$copies
    
    perturbationResults <- unlist(sapply(seq_len(copies),function(copy)
      {
        # get attractors of perturbed network
        perturbedAttrs <- getAttractors(perturbNetwork(network,
                                                       perturb=perturb,
                                                       method=method,
                                                       maxNumBits=maxNumBits,
                                                       numStates=numStates,
                                                       simplify=simplify,
                                                       readableFunctions=readableFunctions,
                                                       excludeFixed=excludeFixed))
        
        # try to find original attractors in perturbed network                                               
	      attractorIndices <- sapply(origAttrs$attractors,function(attractor1)
				  {
					  index <- which(sapply(perturbedAttrs$attractors,function(attractor2)
						  {
							  identical(attractor1,attractor2)
						  }))
					  if (length(index) == 0)
						  NA
					  else
						  index
				  })
	      return(sum(!is.na(attractorIndices)))
	    }))

	  if (accumulate)
	    # return overall percentage of found attractors
    	return(sum(perturbationResults)/(length(origAttrs$attractors) * copies) * 100)
    else
      # return percentage of found attractors for each run
      return(perturbationResults/(length(origAttrs$attractors)) * 100)
  }
}

# Calculate the in-degrees of states in the network.
# If <accumulate> is true, the in-degrees are accumulated
# using the Gini coefficient
testIndegree <- function(network,accumulate=TRUE,params)
{
  attr <- getAttractors(network)
  graph <- plotStateGraph(attr,plotIt=FALSE)
  
  # calculate in-degree using igraph
  if (accumulate)
    # accumulate using Gini index
    return(Gini(degree(graph,mode="in",loops=TRUE)))
  else
    # return the raw degrees
    return(degree(graph,mode="in",loops=TRUE))
}

# Calculate the Kullback-Leibler distance of
# two distributions <x> and <y>.
# <bins> is the number of bins used for discretization.
# <minVal> is the minimum value to be used instead of zero
kullbackLeiblerDistance <- function(x,y,bins=list(),minVal=0.00001)
{
  x <- cut(x,breaks=bins,include.lowest=T,right=F)
  y <- cut(y,breaks=bins,include.lowest=T,right=F)

  tx <- table(x)
  ty <- table(y)
  
  tx <- tx/length(x)
  ty <- ty/length(y)
  
  tx[tx < minVal] <- minVal
  ty[ty < minVal] <- minVal
    
  return(sum(tx*(log(tx/ty))))  
}

# Generic function to test properties of <network> against random networks.
# <numRandomNets> specifies the number of random networks to generate.
# <testFunction> is the name of a function that returns a distribution of test values or a test statistic for each network,
# which receives <testFunctionParams> as optional parameters.
# If <accumulation> is "characteristic", the test function must return a characteristic/test statistic value for each value, and
# a histogram of these value is plotted with a line for the original network.
# If <accumulation> is "kullback_leibler", a histogram of the Kullback-Leibler distances of the test value distribution for the original network
# and the random networks is plotted.
# <sign.level> is the desired significance level for <network> in comparison to the random networks.
# <functionGeneration>,<simplify>,<noIrrelevantGenes>,<validationFunction>,<failureInterations,
# <d_lattice>,<zeroBias> are the corresponding parameters of generateRandomNKNetwork().
# <title> is the title of the plot, <xlab> is its x axis caption, <breaks> is the corresponding histogram parameter, and ... supplies further
# graphical parameters
testNetworkProperties <- function(network, numRandomNets=100, testFunction="testIndegree",
                                  testFunctionParams=list(),
                                  accumulation=c("characteristic","kullback_leibler"),
                                  alternative=c("greater","less"),
                                  sign.level=0.05, drawSignificanceLevel=TRUE,
                                  klBins,klMinVal=0.00001,
                                  linkage=c("uniform","lattice"),
                                  functionGeneration=c("uniform","biased"),
                                  validationFunction, failureIterations=10000,
                                  simplify=FALSE, noIrrelevantGenes=TRUE,
                                  d_lattice=1, zeroBias=0.5, 
                                  title="", xlab, xlim, breaks=30, 
                                  ...)
{
  stopifnot(inherits(network,"BooleanNetwork") || inherits(network,"SymbolicBooleanNetwork"))

  if (is.character(testFunction))
    testFunctionName <- testFunction
  else
    testFunctionName <- ""
  
  testFunction <- match.fun(testFunction)
  
  accumulate <- (match.arg(accumulation) == "characteristic")
  origResult <- testFunction(network,accumulate,testFunctionParams)

  numGenes <- length(network$interactions)
  
  if (inherits(network,"SymbolicBooleanNetwork"))
    inputGenes <- sapply(network$interactions,function(interaction)length(getInputs(interaction)))
  else
    inputGenes <- sapply(network$interactions,function(interaction)length(interaction$input))
  
  if (missing(validationFunction))
    validationFunction <- NULL
      
  randomResults <- lapply(seq_len(numRandomNets),function(i)
                   {      
                      randomNet <- generateRandomNKNetwork(n=numGenes,
                                                           k=inputGenes,
                                                           topology="fixed",
                                                           linkage=linkage,
                                                           functionGeneration=functionGeneration,
                                                           validationFunction=validationFunction,
                                                           failureIterations=failureIterations,
                                                           simplify=simplify,
                                                           noIrrelevantGenes=noIrrelevantGenes,
                                                           d_lattice=d_lattice,
                                                           zeroBias=zeroBias)
                      randomRes <- testFunction(randomNet,accumulate,testFunctionParams)
                      return(randomRes)
                   })
  if (accumulate)
    randomResults <- unlist(randomResults)                 
  
  args <- list(...)
                   
  res <- switch(match.arg(accumulation,c("characteristic","kullback_leibler")),
    characteristic = {
                  # get one value for each random network, and plot a histogram of these values
                  if (missing(xlab))
                  {                    
                    xlab <- switch(testFunctionName,
                      testIndegree = "Gini index of state in-degrees",
                      testAttractorRobustness = "% of identical attractors",
                      testTransitionRobustness = "Normalized Hamming distance",
                      "accumulated results"
                    )
                  }
                  if (missing(xlim))
                  {                    
                    #xlim <- switch(testFunctionName,
                    #  testIndegree = c(0,1),
                    #  testAttractorRobustness = c(0,100),
                    #  c(min(c(origResult,randomResults)),
                    #        max(c(origResult,randomResults)))
                    #)
                    xlim <- range(c(origResult, randomResults))
                  }
                  
                  alternative <- match.arg(alternative, c("greater","less"))
                  
                  # calculate p-value
                  
                  if (alternative == "greater")
                    pval <- sum(randomResults < origResult) / length(randomResults)
                  else                    
                    pval <- sum(randomResults > origResult) / length(randomResults)
                    
                  # plot histogram
                  if (testFunctionName == "testIndegree" | testFunctionName == "testAttractorRobustness")
                  {
                    r <- hist(randomResults,xlim=xlim,xlab=xlab,main=title,xaxt="n",...)
                    axis(side=1,at=seq(xlim[1],xlim[2],length.out=11))
                  }
                  else
                  {
                    # plot with default axis
                    r <- hist(randomResults,xlim=xlim,xlab=xlab,main=title,...)
                  }
                            
                  # plot result for original network          
                  abline(v=origResult,col="red")
                  
                  if (alternative == "greater")
                    text(x=origResult,pos=2,y=max(r$counts)*0.75,
                         labels=paste("> ",round(pval * 100),"%\nof random results",sep=""), 
                         col="red",cex=0.75)
                  else
                    text(x=origResult,pos=4,y=max(r$counts)*0.75,
                         labels=paste("< ",round(pval * 100),"%\nof random results",sep=""), 
                         col="red",cex=0.75)
                       
                  if (drawSignificanceLevel)
                  # plot line for significance level
                  {
                    if (alternative == "greater")
                    {
                      quant <- quantile(randomResults,1.0-sign.level)
                      abline(v=quant,col="blue")
                      text(x=quant,pos=2,y=max(r$counts)*0.85,
                         labels=paste((1.0-sign.level) * 100,"% quantile",sep=""), 
                         col="blue",cex=0.75)                    
                    }
                    else
                    {
                      quant <- quantile(randomResults,sign.level)
                      abline(v=quant,col="blue")
                      text(x=quant,pos=4,y=max(r$counts)*0.85,
                         labels=paste(sign.level * 100,"% quantile",sep=""), 
                         col="blue",cex=0.75)
                    }
                  }
                  list(hist=r,pval=1.0-pval,significant=(1.0-pval<=sign.level))
                },
    kullback_leibler = 
                {
                  # a distribution of values is returned for each network,
                  # plot the Kullback-Leibler distances to the original network
                  
                  if (missing(xlab))
                    xlab <- "Kullback-Leibler distance"
                    
                  if (missing(klBins))
                  {
                     bins <- unique(c(origResult,unlist(randomResults)))
                     bins <- c(bins,max(bins) + 1)
                  }
                  else
                  {
                    bins <- unique(c(origResult,unlist(randomResults)))

                    if (klBins < length(bins))
                      bins <- seq(min(bins),max(bins),length.out=klBins+1)
                    else
                      bins <- c(bins,max(bins) + 1)
                  }
                    
                  vals <- sapply(randomResults,function(results)
                                  kullbackLeiblerDistance(origResult,results,bins=bins,minVal=klMinVal))

                  r <- hist(vals,xlab=xlab,main=title,breaks=breaks,...)
                  list(hist=r)
                },
    stop("'accumulation' must be one of \"characteristic\",\"kullback_leibler\""))   
  return(res)
}

# Plots a graph of the transitions in a probabilistic Boolean network.
# <markovSimulation> is the result of a Markov simulation with returnTable=TRUE.
# If stateSubset is specified, only transitions between states in the set are considered.
# If <drawProbabilities> is true, the edges are annotated with probabilities.
# If <drawStateLabels> is true, the states are annotated with their gene values.
# <layout> specifies the igraph layout to be used.
# If <plotIt> is false, only the graph is returned, and nothing is plotted.
# ... specifies further parameters to igraph.
plotPBNTransitions <- function(markovSimulation,stateSubset,
                               drawProbabilities=TRUE,drawStateLabels=TRUE,
                               layout=layout.fruchterman.reingold,
                               plotIt=TRUE,...)
{
  stopifnot(inherits(markovSimulation,"MarkovSimulation"))

  if (is.null(markovSimulation$table))
    stop(paste("The supplied simulation result does not contain transition information.",
               "Please re-run markovSimulation() with returnTable=TRUE!"))
  
  # assemble edges from table
  edgeMatrix <- data.frame(apply(markovSimulation$table$initialStates,2,
                           function(x)paste(dec2bin(x,length(markovSimulation$genes)),collapse="")),
                           apply(markovSimulation$table$nextStates,2,
                           function(x)paste(dec2bin(x,length(markovSimulation$genes)),collapse="")))
                           
  if (!missing(stateSubset))
  {    
    # determine edges to be excluded based on the subset
    stateSubset <- sapply(stateSubset,function(x)paste(x,collapse=""))
    keepIndices <- apply(edgeMatrix,1,function(row)
                        {
                          (length(intersect(row,stateSubset)) == length(unique(row)))
                        })
    
    # drop edges                   
    edgeMatrix <- edgeMatrix[keepIndices,]
    probabilities <- markovSimulation$table$probabilities[keepIndices]
  }
  else
    probabilities <- markovSimulation$table$probabilities
  
  # determine set of vertices
  vertices <- as.data.frame(as.character(unique(c(as.character(edgeMatrix[,1]),
                            as.character(edgeMatrix[,2])))))

  # build graph                             
  graph <- graph.data.frame(edgeMatrix,vertices=vertices,directed=TRUE)
  
  if (drawProbabilities)
    graph <- set.edge.attribute(graph,"label",value=paste("    ",probabilities))
  if (drawStateLabels)
    label <- as.character(vertices[,1])
  else
    label <- NA
    
    
  if (plotIt)
  {
    # set default values for further graphical parameters
    args <- list(...)
  
    if (is.null(args$vertex.size))
      args$vertex.size <- 2

    if (is.null(args$edge.arrow.mode))
      args$edge.arrow.mode <- 0

    if (is.null(args$vertex.label.cex))
      args$vertex.label.cex <- 0.75
      
    if (is.null(args$edge.label.cex))
      args$edge.label.cex <- 0.75      

    if (is.null(args$vertex.label.dist))
      args$vertex.label.dist <- 1 

    if (is.null(args$vertex.color))
      args$vertex.color <- "grey"

    if (is.null(args$edge.label.color))
      args$edge.label.color <- "green"

    if (is.null(args$edge.arrow.size))
      args$edge.arrow.size <- 0.5 

    # plot it       
    plot(graph,vertex.label=label,layout=layout,vertex.label.cex=args$vertex.label.cex,
         vertex.size=args$vertex.size,vertex.color=args$vertex.color,
         vertex.label.dist = args$vertex.label.dist,
         edge.arrow.size=args$edge.arrow.size,
         edge.label.color=args$edge.label.color,
         edge.label.cex=args$edge.label.cex,...)
  }
  
  # return the igraph object
  return(invisible(graph))
}

# Custom print function for class BooleanStateInfo
print.BooleanStateInfo <- function(x, activeOnly=FALSE, ...)
{
  # Create a TransitionTable object and print it
  cat("Transition table of Boolean network\n\n")
   
  transitionTable <- .internal.getTransitionTable(x)
  
  print(transitionTable, activeOnly=activeOnly, ...)
}

# Custom print function for class MarkovSimulation
print.MarkovSimulation <- function(x, activeOnly = FALSE, ...)
{
  genes <- x$genes
  numGenes <- length(genes)

  cat("States reached at the end of the simulation:\n")
  if (activeOnly)
  {
    reachedStates <- data.frame(apply(x$reachedStates[,seq_len(numGenes)],1,function(state)
                         {
                            r <- paste(genes[which(state == 1)],collapse=", ")
                            if (r == "")
                              r <- "--"
                            r
                         }),x$reachedStates$Probability)
    colnames(reachedStates) <- c("Active genes", "Probability")                        

    print(reachedStates)                           
  }
  else
  {
    print(x$reachedStates)
  }

  if (!is.null(x$table))
  {
                           
    cat("\nProbabilities of state transitions in the network:\n")
    transitionProbs <- getTransitionProbabilities(x)

    colIndices <- c(1,numGenes,numGenes + 1, 2*numGenes, 
                    2*numGenes + 1)
                      
    if(activeOnly)
    {
      inputStates <- apply(transitionProbs,1,function(row)
                          {
                            r <- paste(genes[which(row[colIndices[1]:colIndices[2]] == 1)],collapse=", ")
                            if (r == "")
                              r <- "--"
                            r
                          })
      outputStates <- apply(transitionProbs,1,function(row)
                            {
                              r <- paste(genes[which(row[colIndices[3]:colIndices[4]] == 1)],collapse=", ")
                              if (r == "")
                                r <- "--"
                              r
                            })
      colWidth <- max(c(sapply(inputStates,nchar),sapply(outputStates,nchar)))
      align <- "left"
    }
    else
    {
        inputStates <- apply(transitionProbs,1,function(row)
                          paste(row[colIndices[1]:colIndices[2]],collapse=""))
        outputStates <- apply(transitionProbs,1,function(row)
                          paste(row[colIndices[3]:colIndices[4]],collapse=""))  
        colWidth <- numGenes
        align <- "right"                  
    }

    binMatrix <- data.frame(inputStates,outputStates,
                            transitionProbs[,colIndices[5]])                    
                    
    cat(format("State",width=max(7,colWidth),justify=align),"    ",
        format("Next state",width=max(11,colWidth + 2),justify=align),
        format("Probability",width=13,justify="right"),"\n",sep="")                    
                    
    apply(binMatrix,1,function(row)
    {
      # paste all states of input and output into one string, and put out all columns of the table in a
      # formatted way
      cat(format(row[1],width=max(7,colWidth),justify=align),
          " => ",
          format(row[2],width=max(11,colWidth + 2),justify=align),
          format(row[3],width=13,justify="right"),"\n",sep="")
    })  
  }
  return(invisible(x))
}

sequenceToLaTeX <- function(network, startState, includeAttractorStates = c("all","first","none"),
                            sequence, title="", grouping = list(), plotFixed = TRUE,
                            onColor="[gray]{0.9}",offColor="[gray]{0.6}", highlightAttractor=TRUE,
                            reverse=FALSE, 
                            file="sequence.tex")
{
  if (!missing(network))
  {
    stopifnot(inherits(network,"BooleanNetwork") || inherits(network,"SymbolicBooleanNetwork"))
    if (missing(startState) || !missing(sequence))
      stop("Either \"network\" and \"startState\" or \"sequence\" must be provided!")
    
    sequence <- getPathToAttractor(network = network,
                                   state = startState, 
                                   includeAttractorStates = includeAttractorStates)

    numGenes <- length(network$genes)                                   
    whichFixed <- which(network$fixed != -1)
    if (plotFixed | (length(whichFixed) == 0))
      plotIndices <- seq_len(numGenes)
    else
      plotIndices <- seq_len(numGenes)[-whichFixed]
    
    attractor <- attributes(sequence)$attractor  
    sequence <- sequence[,plotIndices]
    attributes(sequence)$attractor <- attractor                                  
  }
  else
  {
    if (missing(sequence) || !missing(startState))
        stop("Either \"network\" and \"startState\" or \"sequence\" must be provided!")
    
    attractor <- attributes(sequence)$attractor          
  }
  
  # escape "_" in LaTeX
  genes = gsub("_", "\\_", colnames(sequence))
  
  # determine list of genes to be plotted
  
  # Open output file, and print header
  sink(file)
  cat("% Please include packages tabularx and colortbl in your master document:\n",
      "% \\usepackage{tabularx,colortbl}\n\n\n",sep="")
      
  totalMatrix <- t(sequence)
  colnames(totalMatrix) <- seq_len(ncol(totalMatrix))
  
  if(length(grouping)>0)
  {
     # reorder genes according to the supplied groups
    totalMatrix <- totalMatrix[unlist(grouping$index),]
    separationPositions <- c(1,cumsum(sapply(grouping$index,length)+1))
  }
  else
    separationPositions <- c()

  if (highlightAttractor && !is.null(attractor))
  {
      header <- paste(paste(rep(">{\\centering\\arraybackslash}X", 
                            min(attractor) - 1), collapse = " "),
                      "|",
                      paste(rep(">{\\centering\\arraybackslash}X", 
                            length(attractor)), collapse = " "),
                      "|", sep="")
  }
  else
    header <- paste(rep(">{\\centering\\arraybackslash}X", 
          ncol(totalMatrix), collapse = " "))
  # output table header
  cat("\\begin{table}[ht]\n",
       "\\begin{center}\n",
       "\\caption{",title,"}\n",
       "\\begin{tabularx}{\\linewidth}{l", 
       header, 
  "}\\hline\n", sep="")
       
  cat("\\textbf{Time}\t&\t",paste(seq_len(ncol(totalMatrix)),collapse="\t&\t"),"\\\\")
  
   if(length(grouping) == 0)
     cat("\\hline\n")
   else
     cat("\n")  

  # output active and inactive states
  if (reverse)
    indices <- rev(seq_len(nrow(totalMatrix)))
  else
    indices <- seq_len(nrow(totalMatrix))
  
  for(j in indices)
  {
    separator <- which(separationPositions==j)
    if (length(separator) != 0)
    {
      cat("\\hline \\multicolumn{",ncol(totalMatrix) + 1,
          "}{c}{",grouping$class[separator],"}\\\\ \\hline \n",sep="")
    }
    cat("\\textbf{",rownames(totalMatrix)[j],"}\t&\t",sep="")
    for(i in seq_len(ncol(totalMatrix)))
    {
      if(totalMatrix[j,i] == 1)
        cat("\\cellcolor",onColor,"1",sep="")
      else
        cat("\\cellcolor",offColor,"0",sep="")
      if (i < ncol(totalMatrix))
        cat("\t&\t")
    }
    cat("\\\\\n")
  }
  
  cat("\\hline")
  if (highlightAttractor && !is.null(attractor))
  {    
    cat("\\multicolumn{",min(attractor),
        "}{c|}{}\t&\t\\multicolumn{",length(attractor),
        "}{c|}{Attractor}\\\\\\cline{",min(attractor) + 1, "-", max(attractor) + 1, "}\n",sep="")
  }

  cat("\\end{tabularx}\n\\end{center}\n",
      "\\end{table}\n\n",sep="")

  sink()
  # return the matrix  
  return(totalMatrix)
}

simulateSymbolicModel <- function(network, 
                          method=c("exhaustive","random","chosen","sat.exhaustive","sat.restricted"),
						  startStates=NULL, 
						  returnSequences=(!(match.arg(method) %in% c("sat.exhaustive", "sat.restricted"))),
						  returnGraph=(!(match.arg(method) %in% c("sat.exhaustive", "sat.restricted"))),
						  returnAttractors=TRUE,
						  maxTransitions=Inf,
						  maxAttractorLength=Inf,
						  canonical=TRUE)
{
  stopifnot(inherits(network,"SymbolicBooleanNetwork"))
  if (is.null(network$internalStructs) || checkNullPointer(network$internalStructs))
  # refresh internal tree representations if necessary
    network$internalStructs = .Call("constructNetworkTrees_R",network);
  
  if (!is.null(maxAttractorLength) && is.infinite(maxAttractorLength))
    maxAttractorLength <- NULL
  
  if (length(method) > 1)
  # if no method is supplied, infer method from the type of <startStates>
  {
    if (!is.null(maxAttractorLength))
      method <- "sat.restricted"
    else
    if (length(startStates) == 0)
    {
      method <- "exhaustive"
    }
    else
    if (is.numeric(startStates))
    {
      if (length(startStates) > 1)
        stop("Please supply either the number of start states or a list of start states in startStates!")
      else
        method <- "random"
    }
    else
    if (is.list(startStates))
      method <- "chosen"
    else
      stop("Please supply either the number of start states or a list of start states in startStates!")
  }
  
  method <- match.arg(method, c("exhaustive","random","chosen","sat.exhaustive","sat.restricted"))
  
  if (method == "random")
  {
    if (startStates > 2 ^ (sum(network$timeDelays[network$fixed == -1])))
          # more start states than in the full network
     {
              method <- "exhaustive"
              warning(paste("The number of random states is set larger than the total",
                            "number of states. Performing an exhaustive search!"))
     }                            
  }
    
  if (method %in% c("sat.exhaustive", "sat.restricted"))
  {
    if (!is.null(maxAttractorLength))
      maxAttractorLength <- as.integer(maxAttractorLength)
    else
	if (method == "sat.restricted")
		stop("maxAttractorLength must be set for method=\"sat.restricted\"!")
  
    if (returnSequences)
    {
      warning("Sequences cannot be returned for method=\"sat.exhaustive\" and method=\"sat.restricted\"!")
      returnSequences <- FALSE
    }
    if (returnGraph)
    {
      warning("Graph cannot be returned for method=\"sat.exhaustive\" and method=\"sat.restricted\"!")
      returnGraph <- FALSE
    }
  }
  else
  if (method == "exhaustive")
  {
    startStates <- NULL
    convertedStates <- NULL
  }
  else
  if (method == "chosen")
  {
    convertedStates <- 
      lapply(startStates, function(state)
      {
        if (!is.null(dim(state)))
        {
          if (ncol(state) != length(network$genes))
            stop(paste("\"startStates\" must be either a list of vectors with one value for each gene,",
                       "or a list of matrices with the genes in the columns and multiple predecessor states in the rows!"))
          return(as.integer(as.matrix(state[nrow(state):1,])))
        }
        else
        if (!is.numeric(state) || length(state) != length(network$genes))
          stop(paste("\"startStates\" must be either a list of vectors with one value for each gene,",
                     "or a list of matrices with the genes in the columns and multiple predecessor states in the rows!"))
         return(as.integer(state))
      })
  }
  else
    convertedStates <- as.integer(startStates)
  
  if (maxTransitions == 0)
    warning("\"maxTransitions\" is set to 0, which disables the transition limit!")
  else
  if (is.infinite(maxTransitions))
    maxTransitions <- 0
    
  on.exit(.C("freeAllMemory", PACKAGE = "BoolNet"))
  
  if (method %in% c("sat.exhaustive","sat.restricted"))  
    res <- .Call("symbolicSATSearch_R", network$internalStructs, maxAttractorLength, method == "sat.restricted")
  else
    res <- .Call("simulateStates_R",
                 network$internalStructs, 
                 convertedStates, 
                 #as.integer(length(startStates)), 
                 as.integer(maxTransitions),
                 as.logical(returnSequences),
                 as.logical(returnGraph),
                 as.logical(returnAttractors))
 
  ret <- list()
  if (returnSequences)
  { 
    ret[["sequences"]] <- list()
  }
  
  if (returnGraph)
  {
    initialStates <- matrix(res[[2]][[1]], ncol=length(network$genes), byrow=TRUE)
    colnames(initialStates) <- paste("initialState.",network$genes, sep="")
    
    nextStates <- matrix(res[[2]][[2]], ncol=length(network$genes), byrow=TRUE)
    colnames(nextStates) <- paste("nextState.",network$genes, sep="")
    
    attractorAssignment <- data.frame(res[[2]][[3]] + 1)
    attractorAssignment[attractorAssignment == 0] <- NA
    colnames(attractorAssignment) <- "attractorAssignment"
    graph <- data.frame(unique(cbind(initialStates, nextStates, attractorAssignment)))
    class(graph) <- c("TransitionTable","data.frame")
    ret[["graph"]] <- graph
  }
  
  if (returnAttractors)
  {
    attractors <- lapply(res[[3]], function(x)
    {
      att <- matrix(x, ncol=length(network$genes), byrow=TRUE)
      colnames(att) <- network$genes
      
      if (canonical)
      {
        smallestIndex <- -1
        smallestVal  <- rep(Inf, ncol(att))
        for (i in seq_len(nrow(att)))
        # iterate over elements of encoded state
        {
		  equal <- TRUE
          for (j in seq(ncol(att),by=-1,length.out=ncol(att)))
          {
            if (att[i,j] < smallestVal[j])
            # determine new minimum
            {
			  equal <- FALSE				
              smallestVal <- att[i,]
              smallestIndex <- i
              break
            }
            else
			if (att[i,j] > smallestVal[j])
			{
				equal <- FALSE        
				break
			}
          }
		  if (equal && i != smallestIndex)
		  {
			  if (i - smallestIndex < (smallestIndex + nrow(att) - i) %% nrow(att))
			  {
				  smallestVal <- att[i,]
				  smallestIndex <- i
			  }
		  }
        }
      }
      if (smallestIndex != 1)
        # rearrange matrix
        att <- rbind(att[smallestIndex:nrow(att),,drop=FALSE],
                     att[seq_len(smallestIndex-1),,drop=FALSE])
      
      as.data.frame(att)
    })
    ret[["attractors"]] <- attractors
  }
  
  if (returnSequences)
  {               
    maxDelay <- max(network$timeDelays)
    
    if (returnAttractors)
    {
      ret[["attractorAssignment"]] <- res[[4]] + 1
      ret[["attractorAssignment"]][ret[["attractorAssignment"]] == 0] <- NA
    }
    
    sequences <- mapply(function(seq, att)
    {
      seq <- matrix(seq, ncol=length(network$genes), byrow=TRUE)
      colnames(seq) <- network$genes
      rownames(seq) <- paste("t =", (1:nrow(seq)) - maxDelay)
      seq <- as.data.frame(seq)
      
      # add attractor information to sequence
      if (!is.na(att))
        attributes(seq)$attractor <-
          (nrow(seq) - nrow(ret[["attractors"]][[att]]) + 1):nrow(seq)
      seq
    }, res[[1]], ret[["attractorAssignment"]], SIMPLIFY=FALSE)
    ret[["sequences"]] <- sequences
  }
  class(ret) <- "SymbolicSimulation"  
  return(ret)
}

# Encode a vector of binary values <bin> with <len> bits
# to a decimal number
bin2dec <- function(bin,len)
{
  if (len %% 32 == 0)
    numElts <- as.integer(len / 32)
  else
    numElts <- as.integer(len / 32) + 1

  dec = rep(0,numElts)
  
  dec = .C("bin2decC",as.integer(dec),as.integer(bin),as.integer(len))[[1]]
}

# Decode the <len> low-order bits of <dec> to a vector of binary values,
# where the first entry is the least significant bit
dec2bin <- function(dec,len)
{
  bin = rep(0,len)
  
  bin = .C("dec2binC",as.integer(bin),as.integer(dec),as.integer(len),NAOK=TRUE)[[1]]
}

# Generate a list of all assignments of n variables with N possible values
allcombn <- function(N,n)
{
  rownum = N^n
  sapply(n:1,function(i)
    {
            rep(seq_len(N),each=N^(i-1),len=rownum)
          })
}

# Get DNF representation of a truth table <truthTable>
# using the gene names in <genes>. 
# If <mode> is "canonical", build a canonical DNF.
# If <mode> is "short", join terms to reduce the DNF
getDNF <- function(truthTable, genes, mode = c("short","canonical"))
{
  if (mode[1] == TRUE)
    mode <- (if (length(genes) <= 12) "short" else "canonical")
    
  mode <- match.arg(mode, c("short","canonical"))
  # check for constant functions
  if (isTRUE(all.equal(truthTable,rep(0,length(truthTable)))))
    return("0")
  else
  if (isTRUE(all.equal(truthTable,rep(1,length(truthTable)))))
    return("1")
  
  # generate truth table
  entries <- allcombn(2,length(genes)) - 1
  colnames(entries) <- genes

 
  if (mode == "short")
  {
    # heuristic minimization
    
    # the 1 terms that need to be covered
    uncoveredEntries <- which(truthTable == 1)

    # current conjunction list
    conjunctions <- list()  
    
    while (length(uncoveredEntries) > 0)
    {
      # take an uncovered entry and expand it
      currentEntry <- entries[uncoveredEntries[1],]
          
      for (gene in genes)
      # test for each gene whether it can be eliminated from the term
      {
        geneIdx <- which(names(currentEntry) == gene)
        candidate <- currentEntry[-geneIdx]
        condition <- rep(TRUE,length(truthTable))
        for (i in seq_along(candidate))
        {
          condition <- condition & (entries[,names(candidate)[i]] == candidate[i])
        }
        
        if (length(unique(truthTable[condition])) == 1)
          # eliminate gene
          currentEntry <- currentEntry[-geneIdx]
      }
      
      # determine which truth table result entries are now covered
      eliminatedEntries <- rep(TRUE,length(truthTable))
      for (i in seq_along(currentEntry))
      {
        eliminatedEntries <- eliminatedEntries & 
                             (entries[,names(currentEntry)[i]] == currentEntry[i])
      }
      uncoveredEntries <- setdiff(uncoveredEntries, which(eliminatedEntries))
      
      # remember conjunction
      conjunctions <- c(conjunctions, list(currentEntry))
    }
    return(paste(paste("(",sapply(conjunctions, function(conj)
    {
      paste(mapply(function(gene, val)
      {
        if (val == 1)
          return(gene)
        else
          return(paste("!",gene,sep=""))
      }, names(conj), conj), collapse=" & ")
    }), ")", sep=""), collapse=" | "))
  }
  else
  {
    # canonical DNF
    conjunctions <- apply(entries[truthTable==1,,drop=FALSE],1,function(conj)
    {
      paste("(",paste(sapply(seq_along(conj),function(lit)
      {
        if (conj[lit])
          genes[lit]
        else
          paste("!",genes[lit],sep="")
      }),collapse=" & "),")",sep="")
    })
    return(paste(conjunctions[conjunctions != ""],collapse = " | "))
  }
}

# Retrieves a string representation of an interaction function by either
# building a DNF (if <readableFunction> is false)
# or returning an unspecific function description 
# (if <readableFunction> is true or "canonical" or "short").
# <truthTable> contains the result column of the interaction's truth table, and
# <genes> contains the names of the involved genes.
getInteractionString <- function(readableFunctions,truthTable,genes)
{
  if (readableFunctions != FALSE)
    getDNF(truthTable,genes, readableFunctions)
  else
  {
    if (isTRUE(all.equal(truthTable,rep(0,length(truthTable)))))
      return("0")
    else
    if (isTRUE(all.equal(truthTable,rep(1,length(truthTable)))))
      return("1")
    else
    {
      truthTable <- sapply(truthTable,function(x)
      {
        if (x == 0)
          "0"
        else
        if (x == 1)
          "1"
        else  
          "*"
      })
      paste("<f(",
        paste(genes,collapse=","),"){",
        paste(truthTable,collapse=""),"}>", sep="")
    }
  }    
}

# Reorders a matrix of states <stateMatrix> (with each column being one state)
# in a canonical way such that the "smallest" state is the first.
# This makes attractor representations unique.
canonicalStateOrder <- function(stateMatrix)
{
  smallestIndex <- -1
  smallestVal <- rep(Inf,nrow(stateMatrix))
  for (i in seq_len(ncol(stateMatrix)))
  # iterate over states
  {
    for (j in seq_len(nrow(stateMatrix)))
    # iterate over elements of encoded state
    {
      if (stateMatrix[j,i] < smallestVal[j])
      # determine new minimum
      {
        smallestVal <- stateMatrix[,i]
        smallestIndex <- i
        break
      }
      else
      {
        if (stateMatrix[j,i] > smallestVal[j])
          break
      }
    }
  }
  if (smallestIndex != 1)
    # rearrange matrix
    return(cbind(stateMatrix[,smallestIndex:ncol(stateMatrix),drop=FALSE],
           stateMatrix[,seq_len(smallestIndex-1),drop=FALSE]))
  else
    return(stateMatrix)
}

# Find a child node named <name>
# of <node>, or return NULL if such a node
# does not exist.
# If <throwError>, throw an error if the node does not exist.
xmlFindNode <- function(node,name,throwError=FALSE)
{
    indices <- which(xmlSApply(node, xmlName) ==  name)
    if (length(indices) == 0)
    {
      if (throwError)
        stop(paste("Node \"",name,"\" is required, but missing!", sep=""))
      else
        return(NULL)
    }
    return(xmlChildren(node)[indices])
}

# Remove leading and trailing whitespace characters from <string>
trim <- function(string)
{
  string <- gsub("^[ \t]+", "", string)
  string <- gsub("[ \t]+$", "", string)
  return(string)
}

# Generate an interaction by parsing <expressionString>
# and building the corresponding truth table.
# Here, <genes> is a list of all genes in the network.
# Returns an interaction as used in the BooleanNetwork class.
generateInteraction <- function(expressionString, genes)
{
  res <- .Call("getTruthTable_R", parseBooleanFunction(expressionString, genes), length(genes))
  names(res) <- c("input","func")
  res$expression <- expressionString
  return(res)
}

# Create a DNF from a count predicate 
# ("maj","sumis","sumgt","sumlt").
# <tree> is the expression tree of the predicate
# Returns an expression tree in DNF
expandCountPredicate <- function(tree)
{
  if (tree$operator == "maj")
  {
    if (length(tree$operands) %%2 == 0)
      count <- length(tree$operands) / 2
    else
      count <- floor(length(tree$operands) / 2)
       
    operands <- tree$operands      
  }
  else
  {
    count <- tree$operands[[length(tree$operands)]]$value
    operands <- tree$operands[-length(tree$operands)]
  }
  table <- as.matrix(allcombn(2,length(operands)) - 1)
  tableRes <- as.integer(apply(table,1,function(x)
  {
    switch(tree$operator,
      "maj" = sum(x)  > count,
      "sumgt" = sum(x)  > count,
      "sumlt" = sum(x)  < count,
      "sumis" = sum(x) == count
      )
  }))
  return(parseBooleanFunction(getDNF(tableRes, sapply(operands, stringFromParseTree))))
}

# Get the inputs for an expression tree <tree>.
# If <index> is TRUE, the indices are returned instead of the gene names.
# Returns a vector of gene names.
getInputs <- function(tree, index=FALSE)
{
 res <- switch(tree$type,
    operator = 
    {
      unique(unlist(lapply(tree$operands, getInputs, index=index)))
    },    
    atom = 
    {
      if (index)
        tree$index
      else
        tree$name
    },   
    constant =  
    {
      NULL
    }
  )                     
  return(res) 
}

# Determine the maximum time delays for the genes <genes> 
# in a symbolic expression tree <tree>.
# Returns a vector of time delays for the genes.
maxTimeDelay <- function(tree, genes)
{
  res <- switch(tree$type,
    operator = 
    {
      timeDelays <- sapply(tree$operands, maxTimeDelay, genes=genes)
      
      if (!is.null(dim(timeDelays)))
        apply(timeDelays,1,max)
      else
        timeDelays
    },    
    atom = 
    {
      res <- rep(1,length(genes))
      names(res) <- genes
      res[tree$index] <- -tree$time
      res
    },   
    constant =  
    {
      res <- rep(1,length(genes))
      names(res) <- genes
      res
    }
  )                     
  return(res)  
}

# Convert <geneNames> into valid identifiers
# by replacing special characters
# Returns a vector of adjusted gene names.
adjustGeneNames <- function(geneNames)
{
  # eliminate special characters
  res <- gsub("[^a-zA-Z0-9_]+","_",geneNames)
  # ensure that identifier does not start with a number
  res <- gsub("(^[0-9][a-zA-Z0-9_]*)$","_\\1", res)
  names(res) <- geneNames
  return(res)
}

# Check whether the internal C pointer <ptr> is null
checkNullPointer <- function(ptr)
{
  return(.Call("checkNullPointerC",ptr))
}

# Print a synchronous attractor specified by a data frame <attractor>
# The attractor has the index <index> and a basin consisting of <basinSize> states.
# If <activeOnly> is true, states are represented as a list of active genes (otherwise binary vectors).
printSynchronousAttractor <- function(attractor, index, basinSize=NA, activeOnly=FALSE)
{
  if (is.na(basinSize))
    cat("Attractor ",index," is a simple attractor consisting of ", 
        nrow(attractor),
        " state(s)",sep="")
  else
    cat("Attractor ",index," is a simple attractor consisting of ", 
        nrow(attractor),
        " state(s) and has a basin of ", 
        basinSize, " state(s)",sep="")
    
  # print a graphical representation of the attractor cycle
  if (activeOnly)
  {
    cat(".\nActive genes in the attractor state(s):\n")
    
    for (j in seq_len(nrow(attractor)))
    {
      
      state <- paste(colnames(attractor)[which(attractor[j,] == 1)],collapse=", ")
      if (state == "")
        state <- "--"
      cat("State ", j, ": ", state, "\n", sep="")
    }
    cat("\n")
  }
  else
  {
    cat(":\n\n")
    numGenes <- ncol(attractor)
    cat(" |--<", paste(rep("-",numGenes-1), collapse=""),"|\n", sep="")
    cat(" V ", paste(rep(" ",numGenes-1), collapse=""),"  |\n", sep="")
    for (j in seq_len(nrow(attractor)))
    {
      cat(" ", as.integer(attractor[j,]),"   |\n",sep="")
      #cat(" | ", paste(rep(" ",numGenes-1), collapse=""),"  |\n", sep="")
    }
    cat(" V ", paste(rep(" ",numGenes-1), collapse=""), "  |\n", sep="")
    cat(" |-->", paste(rep("-",numGenes-1), collapse=""), "|\n\n", sep="")
  }
  if (!activeOnly)
   cat("\nGenes are encoded in the following order: ",
       paste(colnames(attractor), collapse=" "), "\n\n", sep="")
}

getAttractorSequence <- function(attractorInfo, attractorNo)
{
  stopifnot(inherits(attractorInfo,"AttractorInfo") || inherits(attractorInfo,"SymbolicSimulation"))
  
  if (inherits(attractorInfo,"SymbolicSimulation"))
  {
     if (is.null(attractorInfo$attractors))
      stop(paste("This SymbolicSimulation structure does not contain attractor information.",
           "Please re-run simulateSymbolicModel() with returnAttractors=TRUE!"))
     if (missing(attractorNo) || attractorNo <= 0 || attractorNo > length(attractorInfo$attractors))
      stop("Please provide a valid attractor number!")
      
     return(attractorInfo$attractors[[attractorNo]])                
  }
  else
  {
    if (missing(attractorNo) || attractorNo <= 0 || attractorNo > length(attractorInfo$attractors))
      stop("Please provide a valid attractor number!")
      
    if (!is.null(attractorInfo$attractors[[attractorNo]]$initialStates))
      stop("A sequence can be obtained for synchronous attractors only!")
    
    numGenes <- length(attractorInfo$stateInfo$genes)
    
    # decode binary representation of states involved in the attractors
    binMatrix <- t(matrix(apply(attractorInfo$attractors[[attractorNo]]$involvedStates,2,function(state)
                          dec2bin(state,numGenes)),nrow=numGenes))
                          
    colnames(binMatrix) <- attractorInfo$stateInfo$genes
    return(as.data.frame(binMatrix))
  }
}

getPathToAttractor <- function(network, state, includeAttractorStates=c("all","first","none"))
{
  stopifnot(inherits(network,"BooleanNetwork") || inherits(network,"SymbolicBooleanNetwork") 
            || inherits(network, "AttractorInfo"))
  
  includeAttractorStates <- match.arg(includeAttractorStates, c("all","first","none"))
  
  if (inherits(network,"SymbolicBooleanNetwork"))
  {
    sim <- simulateSymbolicModel(network, startStates=list(state))
    res <- data.frame(sim$sequences[[1]])
    
    attractorStates <- nrow(res) - seq(length.out=nrow(sim$attractors[[1]]),by=-1,to=1) + 1
    
    if (includeAttractorStates == "first")
    {
      res <- res[seq_len(attractorStates[1]),,drop=FALSE]
      attractorStates <- attractorStates[1]
    }
    else
    if (includeAttractorStates == "none")
    {
      res <- res[-attractorStates,,drop=FALSE]
      attractorStates <- NULL
    }
    attributes(res)$attractor <- attractorStates
  }
  else
  {  
    if (inherits(network,"BooleanNetwork"))
    {
      table <- getTransitionTable(getAttractors(network, startStates=list(state)))
    }
    else
    {
      if (is.null(network$stateInfo$table))
        stop(paste("This AttractorInfo structure does not contain transition table information.",
             "Please re-run getAttractors() with a synchronous search and returnTable=TRUE!"))
      table <- getTransitionTable(network)
    }    
    
    numGenes <- (ncol(table) - 2) / 2
    initialStates <- apply(table[,seq_len(numGenes),drop=FALSE],1,function(x)paste(x,collapse=""))
    
    currentState <- state
    
    res <- data.frame(matrix(state,nrow=1))
    
    attractorStart <- NA
    stateCount <- 1
    repeat
    {
      currentStateIdx <- which(initialStates == paste(currentState,collapse=""))

      if (length(currentStateIdx) == 0)
        stop(paste("Could not find state",paste(currentState,collapse=""),"in the transition table!"))

      if (table[currentStateIdx,"transitionsToAttractor"] == 0 && is.na(attractorStart))
        attractorStart <- stateCount

      # stop depending on "includeAttractorStates" option
      if ((includeAttractorStates == "all" && stateCount == nrow(table))
          || (includeAttractorStates == "first" && table[currentStateIdx,"transitionsToAttractor"] == 0)
          || (includeAttractorStates == "none" && table[currentStateIdx,"transitionsToAttractor"] <= 1))
        break
        
      currentState <- as.integer(table[currentStateIdx,(numGenes+1):(2*numGenes)])
      res <- rbind(res,currentState)
      
      stateCount <- stateCount + 1
    }
    
    if (!is.na(attractorStart))
      attractorIdx <- seq(attractorStart, nrow(table), by=1)
    else
      attractorIdx <- NULL
    
    # special case: start state is attractor state and we do not want to include attractor states
    # => return empty data frame
    if (includeAttractorStates == "none" && table[currentStateIdx,"transitionsToAttractor"] == 0)
    {
      res <- data.frame(matrix(nrow=0,ncol=numGenes))
    }
    else
      if (length(attractorIdx) > 0)
        attributes(res)$attractor <- attractorIdx

    colnames(res) <- sapply(colnames(table)[seq_len(numGenes)],function(n)strsplit(n,".",fixed=TRUE)[[1]][2])
    
    rownames(res) <- NULL
  }
  return(res)
}

# Sets the genes in <fixIndices> to the values in <values>
# and returns the customized copy of <network>
fixGenes <- function(network,fixIndices,values)
{
  stopifnot(inherits(network,"BooleanNetwork") | inherits(network,"SymbolicBooleanNetwork") | 
            inherits(network,"ProbabilisticBooleanNetwork"))

  if (length(fixIndices) != length(values) && length(values) != 1)
    stop("fixIndices and values must have the same number of elements, or values must have 1 element!")

  if (any(is.na(network$fixed[fixIndices])))
    stop("fixIndices contains invalid indices!")
    
  if (any(values != 0 & values != 1 & values != -1))
    stop("Please supply only 0, 1, or -1 in values!")

  network$fixed[fixIndices] <- as.integer(values)
  
  if (inherits(network,"SymbolicBooleanNetwork"))
    network$internalStructs = .Call("constructNetworkTrees_R",network);
    
  return(network)
}

# Provides several methods to binarize a vector consisting of real values.
# The methods are edge-detector-based. With <edge>="firstEdge",
# the first "significant" edge in the sorted data is the threshold for the binarization.
# With the <scaling> factor, the size of the first edge can be adapted.
# The "maxEdge" method searches for the edge with the highest gradient.  
edgeDetector <- function(vector,scaling=1,edge=c("firstEdge","maxEdge"))
{  
  distance<-c()
  binarizeddata<-vector
	
  #sort data
  sortedvector<-sort(vector)
  
  #distance calculation
  for(i in seq_len(length(vector)-1))
  {
    distance[i]<-sortedvector[i+1]-sortedvector[i]  
  }
  
  switch(match.arg(edge),
    #the index of the first edge with distance[i]>threshold is determined
    firstEdge=
      {
        threshold<-scaling*((sortedvector[length(vector)]-sortedvector[1])/((length(vector)-1)))
        index <- 0
        for(i in seq_len(length(vector)-1))
        {  
          if(distance[i]>threshold)
          {
            index<-i
            break  
          }
        }
      },

    #the index of the edge with the maximal gradient is determined
    maxEdge=
      {
       index <- which.max(distance)
      },
    stop("'method' must be one of \"firstEdge\",\"maxEdge\"")
    )
  
  #based on the edge index, the binarization is performed  
  if(index!=0)
  {
    for(i in seq_len(length(vector)))
    {  
      if(vector[i]>=sortedvector[index+1])
        binarizeddata[i]<-1
      else
        binarizeddata[i]<-0
        
    }
    threshold=(sortedvector[index+1]+sortedvector[index])/2
    return(list(bindata=binarizeddata,thresholds=as.numeric(threshold)))
  }
  else
  #if no edge was found, a vector consisting of zeros is returned
  {
  	binarizeddata<-(sapply(vector,function(x) 0))  
    return(list(bindata=binarizeddata,thresholds=NA))
  }  
  
}

generateRandomStartStates <- function(network, n)
{
  mat <- matrix(nrow=n,ncol=length(network$genes))
  fixedPositions <- which(network$fixed != -1)
  nonFixedPositions <- which(network$fixed == -1)
  
  if (n > (2 ^ length(nonFixedPositions)))
    stop("The number of states to generate exceeds the total number of possible states!")

  if (length(fixedPositions) != 0)
    # fill fixed positions with the corresponding values
    mat[,fixedPositions] <- sapply(fixedPositions,function(x)
            rep(network$fixed[x],n))

  if (n != 2 ^ length(nonFixedPositions))
  {  
    # generate other positions randomly
    mat[,nonFixedPositions] <- round(runif(n=n*length(nonFixedPositions)))
  }
  else
  {
    mat[,nonFixedPositions] <- allcombn(2,length(nonFixedPositions)) - 1
  }
  
  # eliminate duplicates
  mat <- unique(mat)

  while (nrow(mat) != n)
  # if duplicates were removed, generate new states until the
  # desired number of states is reached
  {
    vec <- rep(0,length(network$genes))
    if (length(fixedPositions) != 0)
      # fill fixed positions
      vec[fixedPositions] <- sapply(fixedPositions,
                  function(x)network$fixed[x])
    # generate other positions randomly
    vec[nonFixedPositions] <- round(runif(n=length(nonFixedPositions)))
    mat <- unique(rbind(mat,vec))
  }
  #cat("Using initial states:\n")

  # print states and form a list
  res <- lapply(1:nrow(mat),function(i)
    {
      #cat(paste(mat[i,],collapse=""),"\n",sep="")
      mat[i,]
    })
  #cat("\n")
return(res);
}

# Reconstruct a Boolean network from a transition table or a list of time series in <measurements>.
# If <method> is "bestfit", Lähdesmäki's best-fit extension algorithm is called.
# If <method> is "reveal", Liang's REVEAL algorithm is called.
# <maxK> specifies the maximum number of input genes of a function.
# If <readableFunctions> is true, DNF representations of the functions are generated.
reconstructNetwork <- function(measurements,method=c("bestfit","reveal"),maxK=5,
                               requiredDependencies = NULL, excludedDependencies = NULL, perturbations=NULL,
                               readableFunctions=FALSE, allSolutions=FALSE, returnPBN=FALSE)
{
  if (maxK < 0)
    stop("maxK must be >= 0!")

  # determine method to use
  meth <- switch(match.arg(method,c("bestfit","reveal")),
      bestfit=0,
      reveal=1,
      stop("'method' must be one of \"bestfit\",\"reveal\""))
      
  perturbationMatrix <- NULL     
      
  if (inherits(measurements,"TransitionTable"))
  # preprocess transition table and call algorithm
  {
    numGenes <- (ncol(measurements) - 2) / 2
    
    if (numGenes < maxK)
    {
      maxK <- numGenes
      warning(paste("maxK was chosen greater than the total number of input genes and reset to ",numGenes,"!",sep=""))
    }
    
    if (!is.null(perturbations))
    { 
      if (!all(perturbations %in% c(0,1,-1,NA)))
        stop("The perturbation matrix may only contain the values 0,1,-1 and NA!")
      
      perturbations[is.na(perturbations)] <- -1 
        
      if (is.null(dim(perturbations)))
        perturbations <- matrix(perturbations,ncol=1)
        
      if (ncol(perturbations) != 1 && ncol(perturbations) != nrow(measurements))
        stop(paste("There must be either one global vector of perturbations, or a matrix",
                   "containing exact one column for each time series!"))
     
      if (nrow(perturbations) != numGenes)
        stop("The perturbation matrix must have exactly the same number of rows as the measurements!")
        
      if (ncol(perturbations) == 1)
      {
        perturbationMatrix <- matrix(-1,nrow=numGenes,ncol=nrow(measurements))
        for (j in seq_len(nrow(measurements)))
          perturbationMatrix[,j] <- perturbations[,1]
      }
      else
        perturbationMatrix <- as.matrix(perturbations)
    }   
    
    inputStates <- as.integer(t(as.matrix(measurements[,seq_len(numGenes)])))
    outputStates <- as.integer(t(as.matrix(measurements[,(numGenes+1):(2*numGenes)])))
    numStates <- nrow(measurements)
    genenames <- sapply(colnames(measurements)[seq_len(numGenes)],function(x)strsplit(x,".",fixed=TRUE)[[1]][2])    
  }
  else
  # the measurements are time series
  {
    if (!is.null(dim(measurements)))
    # only one time series => create list
      measurements <- list(measurements)
      
    numGenes <- nrow(measurements[[1]])
    
    if (numGenes < maxK)
    {
      maxK <- numGenes
      warning(paste("maxK was chosen greater than the total number of input genes and reset to ",numGenes,"!",sep=""))
    }

    if (is.null(perturbations) && !is.null(measurements$perturbations))
    {
      perturbations <- measurements$perturbations
      measurements$perturbations <- NULL
    }
    
    if (!is.null(perturbations))
    { 
      if (!all(perturbations %in% c(0,1,-1,NA)))
        stop("The perturbation matrix may only contain the values 0,1,-1 and NA!")
      
      perturbations[is.na(perturbations)] <- -1    
      
      if (is.null(dim(perturbations)))
        perturbations <- matrix(perturbations,ncol=1)

      if (ncol(perturbations) != 1 && ncol(perturbations) != length(measurements))
        stop(paste("There must be either one global vector of perturbations, or a matrix",
                   "containing exact one column for each time series!"))                   
     
      if (nrow(perturbations) != numGenes)
        stop("The perturbation matrix must have exactly the same number of rows as the measurements!")
    }
    
    perturbationMatrix <- c()
    
    genenames <- rownames(measurements[[1]])
    if (is.null(genenames))
      genenames <- paste("Gene",seq_len(numGenes))
    inputStates <- c()
    outputStates <- c()
    
    for (i in seq_along(measurements))
    # iterate over all time series and build state vectors
    {
      measurement <- measurements[[i]]
      if (numGenes != nrow(measurement))
        stop("All measurement matrices must contain the same genes!")
      inputStates <- c(inputStates,as.integer(as.matrix(measurement[,1:(ncol(measurement)-1)])))
      outputStates <- c(outputStates,as.integer(as.matrix(measurement[,2:ncol(measurement)])))

      if (!is.null(perturbations))
      {
        for (j in seq_len(ncol(measurement) - 1))
        {
          if (ncol(perturbations) == 1)
            perturbationMatrix <- cbind(perturbationMatrix, perturbations[,1])
          else
            perturbationMatrix <- cbind(perturbationMatrix, perturbations[,i])
        }
      }
    }
    numStates <- as.integer(length(inputStates) / numGenes)
  }
  
  if (is.null(requiredDependencies))
    requiredDepMatrix <- NULL
  else
  {
    if (is.null(names(requiredDependencies)))
      names(requiredDependencies) <- genenames
    
    if (length(union(names(requiredDependencies),genenames)) != length(genenames))
      stop("The required dependencies must consist of gene names that are comprised in the measurements!")

    requiredDepMatrix <- matrix(0,nrow=numGenes,ncol=numGenes)
    colnames(requiredDepMatrix) <- genenames
    rownames(requiredDepMatrix) <- genenames
    
    maxRequired <- 0      
    for (i in seq_along(requiredDependencies))
    {
      maxRequired <- max(maxRequired, length(requiredDependencies[i]))
      for (el in requiredDependencies[i])
        requiredDepMatrix[el,names(requiredDependencies)[i]] <- 1
    }
    
    if (maxRequired > maxK)
    {
      warning(paste("The number of required dependencies is greater than maxK! Setting maxK to ",                  maxRequired, "!", sep=""))
      maxK <- maxRequired
    } 
    
    requiredDepMatrix <- as.integer(requiredDepMatrix) 
  }
  
  if (is.null(excludedDependencies))
    excludedDepMatrix <- NULL
  else
  {
    if (is.null(names(excludedDependencies)))
      names(excludedDependencies) <- genenames
    
    if (length(union(names(excludedDependencies),genenames)) != length(genenames))
      stop("The excluded dependencies must consist of gene names that are comprised in the measurements!")

    excludedDepMatrix <- matrix(0,nrow=numGenes,ncol=numGenes)
    colnames(excludedDepMatrix) <- genenames
    rownames(excludedDepMatrix) <- genenames
          
    for (i in seq_along(excludedDependencies))
    {
      gene <- names(excludedDependencies)[i]
      if (!is.null(requiredDependencies) && 
          !is.null(requiredDependencies[[gene]]))
      {
        conflicts <- intersect(requiredDependencies[[gene]],
                               excludedDependencies[[gene]])
        
        if (length(conflicts) > 0)
          stop(paste("For gene ",gene,
                      ", potential inputs were specified both as required and excluded dependencies: ",
                      paste(conflicts, collapse=", "), sep=""))
      }
      for (el in excludedDependencies[i])
        excludedDepMatrix[el,names(excludedDependencies)[i]] <- 1
    }
    excludedDepMatrix <- as.integer(excludedDepMatrix)
  }  
  
  if (!is.null(perturbations))
    perturbationMatrix <- as.integer(perturbationMatrix)
  
  on.exit(.C("freeAllMemory", PACKAGE = "BoolNet"))
  # call C code
  res <- .Call("reconstructNetwork_R",
      inputStates,
      outputStates,
      perturbationMatrix,
      as.integer(numStates),
      requiredDepMatrix,
      excludedDepMatrix,
      as.integer(maxK),
      as.integer(meth),
      as.integer(allSolutions),
      as.integer(returnPBN))
  
  
  if (any(sapply(res,function(interaction)length(interaction)==0)))
  # some function lists are empty
    warning("Some functions could not be inferred. Possibly the input data is noisy or maxK was chosen too small!")
    
  # prepare result object
  res <- list(genes=genenames,
              interactions=lapply(res,function(gene)
                    lapply(gene,function(interaction)
                    {
                      interaction$expression <- 
                                getInteractionString(readableFunctions,
                                           interaction$func,
                                           genenames[interaction$input])
                      if (returnPBN)
                        interaction$probability <- 1.0/length(gene)
                        
                      interaction
                    })),
              fixed=sapply(res,function(gene)
                  {
                    if (length(gene) == 0)
                      -1
                    else
                    if (gene[[1]]$input[1] == 0)
                      gene[[1]]$func[1]
                    else
                      -1
                  }))
  
  names(res$interactions) <- res$genes
  names(res$fixed) <- res$genes
  
  if (returnPBN)
    class(res) <- "ProbabilisticBooleanNetwork"
  else
    class(res) <- "BooleanNetworkCollection"
  
  
  if (allSolutions)
  # simplify functions and remove duplicates
  {
    res <- simplifyNetwork(res)
    res$interactions <- lapply(res$interactions,function(interaction)
                               {
                                 duplicates <- duplicated(sapply(interaction,function(func)func$expression))
                                 return(interaction[!duplicates])
                               })
  }
  
  return(res)
}

# Check whether N is an integer using
# a regular expression.
check.integer <- function(N)
{
    !length(grep("[^[:digit:]]", as.character(N)))
}

# Parse an SBML species list in <rootNode>.
# Returns a list containing a map of gene ids to gene names,
# a vector specifying which genes are fixed
# and a vector of initial levels.
parseSBMLSpecies <- function(rootNode)
{
  genes <- c()
                  
  fixed <- c()
  
  initialLevels <- c()

  # iterate over species
  for (species in xmlChildren(rootNode))
  {
    attrs <- xmlAttrs(species)
    
    # if available, use the "name" attribute as the gene name
    # (and replace unsuitable characters by underscores)
    # otherwise use the gene id.
    if (is.na(attrs["name"]) || attrs["name"] == "")
      # for better compatibility with CoLoMoTo, 
      # remove "s_" at the beginning of identifiers
      name <- sub("^s_","",attrs["id"])
    else
      name <- gsub("[^a-zA-Z0-9_]+","_",attrs["name"])
     
    # gene names must be unique 
    if (name %in% genes)
    {
      suffix <- 1
      while (paste(name,suffix,sep="_") %in% genes)
        suffix <- suffix + 1
        
      warning(paste("Duplicate gene \"",name,"\", renaming to ",
                    name,"_",suffix,"!",sep=""))
                    
      name <- paste(name,suffix,sep="_")
    }
    
    # Reject logical networks with more than two values for a gene
    if (!is.na(attrs["maxLevel"]) && as.integer(attrs["maxLevel"]) > 1)
      stop(paste("BoolNet supports only binary genes, but gene \"",
                  name,"\" has a maximum level of ",attrs["maxLevel"],"!", sep=""))
    
    # build a lookup table id -> gene name  
    genes[attrs["id"]] <- adjustGeneNames(name)
    
    if (!is.na(attrs["constant"]) && tolower(attrs["constant"]) == "true")
    # if the gene is constant, save its initial level in the "fixed" vector
    {
      #if (is.na(attrs["initialLevel"]))
        #stop(paste("Gene \"", name, "\" is constant, but no initial level is supplied!", sep=""))
      #  warning(paste("Gene \"", name, "\" is constant, but no initial level is supplied! Assuming an input!", sep=""))
      
      fixed[name] <- TRUE 
    }
    else
    # this gene is not constant
      fixed[name] <- FALSE
          
    initialLevels[name] <- as.integer(attrs["initialLevel"])
  }
  return(list(genes = genes, fixed = fixed, initialLevels = initialLevels))
}

# Parse a list of transitions in <rootNode>.
# Here, <genes> is the result of parseSBMLSpecies()
# containing a map of gene ids/assignments, a vector
# specifying fixed genes and a vector of initial levels.
# Returns a list of interactions in the format 
# of class BooleanNetwork
parseSBMLTransitions <- function(rootNode, genes, symbolic)
{ 
  # iterate over all transitions 
  transitions <- xmlApply(rootNode,function(transition)
  {
    # parse inputs
    inputList <- xmlFindNode(transition, "listOfInputs")
    if (is.null(inputList))
    {
      inputs <- c()
    }
    else
    {
      inputList <- inputList[[1]]
      inputs <- c()
      inputThresholds <- c()
      
      # iterate over inputs
      for (input in xmlChildren(inputList))
      {
        attrs <- xmlAttrs(input)
        id <- attrs["qualitativeSpecies"]

        # verify gene name in species list
        if (is.na(genes$genes[id]))
          stop(paste("Unknown input \"",id,"\"!",sep=""))
        else
          inputs <- union(inputs, id)
        
        # check whether attributes of the input are compatible 
        # with Boolean logic
        if (tolower(attrs["transitionEffect"]) != "none")
          stop(paste("Transition effect for input gene \"",id,"\" is \"",
               attrs["transitionEffect"],"\", expected \"none\"!", sep=""))

        if (!is.na(attrs["thresholdLevel"]) && !is.na(attrs["id"]))
        # if a threshold level has been specified, save it with the 
        # corresponding ID for later use in the MathML terms
        {
          inputThresholds[attrs["id"]] <- as.integer(attrs["thresholdLevel"])
          
          if (inputThresholds[attrs["id"]] > 1)
            stop("Threshold levels must be 0 or 1!")        
        }
          
      }
    }
    
    # parse outputs
    outputList <- xmlFindNode(transition, "listOfOutputs")
    outputs <- c()
    
    if (!is.null(outputList))
    {
      outputList <- outputList[[1]]
      for (output in xmlChildren(outputList))
      # iterate over outputs
      {
        attrs <- xmlAttrs(output)
        id <- attrs["qualitativeSpecies"]
        
        # verify gene list in species list
        if (is.na(genes$genes[id]))
          stop(paste("Unknown output ",id,"!",sep=""))
        else  
          outputs <- union(outputs, id)
        
        # check whether attributes of the output are compatible 
        # with Boolean logic
        if (tolower(attrs["transitionEffect"]) != "assignmentlevel")
          stop(paste("Transition effect for output gene \"",id,"\" is \"",
               attrs["transitionEffect"],"\", expected \"assignmentLevel\"!", sep=""))
               
        if (!is.na(attrs["outputLevel"]))
          stop("Output levels for transitions are not supported in Boolean models!")
      }
    }
    
    # parse function terms
    functionTermList <- xmlFindNode(transition, "listOfFunctionTerms", throwError=TRUE)[[1]]
    transitionFunction <- parseSBMLFunctionTerms(functionTermList, 
                                                 genes$genes[inputs],
                                                 inputThresholds)
    
    return(list(inputs=inputs, outputs=outputs, transitionFunction=transitionFunction))
  })
  
  # now convert the read data to the BoolNet interaction format
  interactions <- lapply(names(genes$genes), function(gene)
  {
    # identify the transitions linked to each gene
    linkedTransitions <- which(sapply(transitions, 
                                      function(transition)gene %in% transition$output))
    
    if (length(linkedTransitions) == 0)
    # no transitions are assigned to this gene
    {
      if (!genes$fixed[genes$gene[gene]] || is.na(genes$initialLevels[genes$gene[gene]]))
      {
        if (is.na(genes$initialLevels[genes$gene[gene]]))
        {
          # Assume an input if the gene has no transition function and no initial value
          warning(paste("There is no transition and no initial level for gene \"",
                     gene,"\"! Assuming an input!",sep=""), call.=FALSE)
                     
        if (symbolic)
          return(parseBooleanFunction(genes$gene[gene]))
        else
          return(list(input=which(names(genes$genes) == gene), 
                      func=c(0,1), 
                      expression=genes$gene[gene]))
        }
        else
        if (!genes$fixed[genes$gene[gene]])
          warning(paste("There is no transition for the non-constant gene \"",
                     gene,"\"! Setting it to a constant ", 
                     genes$initialLevels[genes$gene[gene]], "!" ,sep=""), call.=FALSE)
      }
      
      # build a constant interaction
      
      if (symbolic)
        return(parseBooleanFunction(as.character(genes$initialLevels[genes$gene[gene]])))
      else
        return(list(input=0, 
                    func=genes$initialLevels[genes$gene[gene]], 
                    expression=as.character(genes$initialLevels[genes$gene[gene]])))
    }
    else
    {
    
      if (length(linkedTransitions) > 1)
      # multiple transitions per gene are not allowed, as these may be conflicting
      {
        stop(paste("Gene \"",gene,"\" is affected by multiple transitions!",sep=""))
      }
      
      if (genes$fixed[genes$gene[gene]])
      # a constant gene should not be the output of a transition
        stop(paste("Gene \"",gene,"\" has been specified as constant, but there is a transition!",sep=""))

      if (symbolic)
      {
        # parse the Boolean expression, and construct a symbolic expression tree
        return(parseBooleanFunction(transitions[[linkedTransitions]]$transitionFunction, genes$genes))
      }
      else
      {
        # parse the Boolean expression, and generate an interaction with 
        # the corresponding truth table
        return(generateInteraction(transitions[[linkedTransitions]]$transitionFunction, 
                                   #genes$genes[transitions[[linkedTransitions]]$input], 
                                   genes$genes))
      }
    }
  })
  names(interactions) <- genes$genes
  return(interactions)
}

# Parse a list of function terms in <rootNode>, where <genes> specifies
# the assignment of gene identifiers to gene names,
# and <inputThresholds> specifies the assignment of threshold identifiers
# to values (see also parseMathML()).
# Returns a single character string representing the function term 
# as an R expression.
parseSBMLFunctionTerms <- function(rootNode, genes, inputThresholds)
{
  # iterate over function terms
  functionTerms <- xmlApply(rootNode, function(term)
  {
    attrs <- xmlAttrs(term)
    
    outputLevel <- as.integer(attrs["resultLevel"])
    
    if (outputLevel > 1)
      stop("The result level of a function term must be 0 or 1!")
    
    if (tolower(xmlName(term)) == "defaultterm")
    # this is the default term => no expression
    {
      return(list(term="",outputLevel=outputLevel))
    }
    else
    # parse the MathML expression in the function term    
    {
      math <- xmlFindNode(term, "math", throwError=TRUE)[[1]][[1]]
              
      return(list(term=parseMathML(math, genes, inputThresholds), 
                  outputLevel=outputLevel))
    }
  })
  
  # build lists of terms with result 0 (negative terms)
  # and terms with result 1 (positive term)
  posTerms <- c()
  negTerms <- c()
  defaultValue <- NA
  
  for (term in functionTerms)
  {
    if (term$term != "")
    {
      if (term$outputLevel == 0)
      {
        negTerms <- c(negTerms, term$term)
      }
      else
      {
        posTerms <- c(posTerms, term$term)
      }
    }
    else
      defaultValue <- term$outputLevel

  }
  
  if (is.na(defaultValue))
    stop("Missing default term in transition!")
  
  if (defaultValue == 0)
  # if the default is 0, the result is a DNF of the positive terms
  {
    if (length(posTerms) > 0)
      totalTerm <- paste(posTerms, collapse=" | ")
    else
      totalTerm <- "0"
    
    if (length(negTerms) > 0)
    # if the default value is 0, additional negative terms are ignored, 
    # as they should be part of the default value 
    # (otherwise this is a contradiction to the positive terms)
      warning("Potentially contradictory terms in a transition have been ignored!")
  }
  else
  # if the default value is 1, the result is a negated DNF of the negative terms
  {
    if (length(negTerms) > 0)
      totalTerm <- paste("!(",paste(negTerms, collapse=" | "),")",sep="")
    else
      totalTerm <- "1"
      
    if (length(posTerms) > 0)
    # if the default value is 1, additional positive terms are ignored, 
    # as they should be part of the default value 
    # (otherwise this is a contradiction to the negative terms)    
      warning("Potentially contradictory terms in a transition have been ignored!")
  }
  return(totalTerm)
}

# Recursively parse the MathML expression in <rootNode>.
# Here, valid identifiers are the gene names in <names(genes)>
# and the input thresholds in <names(inputThresholds)>,
# which are replaced by the corresponding values.
# Returns an R expression string representing the MathML expression.
parseMathML <- function(rootNode, genes, inputThresholds)
{
  name <- xmlName(rootNode)
  if (name == "apply")
  # a bracket
  {
    operator <- xmlName(xmlChildren(rootNode)[[1]])
    
    children <- sapply(xmlChildren(rootNode)[-1],parseMathML, genes, inputThresholds)

    if (operator == "and" || operator == "times")
    {
      # treat "and" and "times" equally, but warn    
      if (operator == "times")
        warning("Interpreting \"times\" operator as a logical \"and\"!")
        
      return(paste("(",paste(children,
                   collapse = " & "),")",sep=""))
    }
    else
    if (operator == "or" || operator == "plus")
    {
      # treat "or" and "plus" equally, but warn    
      if (operator == "plus")
        warning("Interpreting \"plus\" operator as a logical \"or\"!")
        
      return(paste("(",paste(children,
                   collapse = " | "),")",sep=""))
    }
    else
    if (operator == "xor")
    {
      # convert XOR to a DNF by pasting all odd entries in the truth table
      tt <- allcombn(2, length(children)) - 1
      tt <- apply(tt,1,function(x)sum(x) %/% 2  == 1)
      return(paste("(",getDNF(tt, children),")",sep=""))
    }
    else
    if (operator %in% c ("eq", "neq", "gt", "lt", "geq", "leq"))
    {
      # comparison operators have to be converted to Boolean logic
      
      if (length(children) != 2)
        stop(paste("Operator \"",operator,"\" requires two operands!",sep=""))

      # check which of the child expressions are constant
      isConst <- sapply(children,function(x)
                        {
                          check.integer(x)
                        })
                        
      if (all(isConst))
      # two constants are compared (this does not really make sense!)
      {
        children <- as.integer(children)
        return(as.integer(switch(operator,
          eq = {children[1] == children[2]},
          neq = {children[1] != children[2]},
          gt = {children[1] > children[2]},
          lt = {children[1] < children[2]},
          geq = {children[1] >= children[2]},
          leq = {children[1] <= children[2]}
        )))
      }
      else
      if (any(isConst))
      # one constant and one variable are compared
      {
        constChild <- as.integer(children[isConst])
        varChild  <- children[!isConst]
        return(switch(operator,
          eq = 
          {
            if (constChild == 1)
              varChild
            else
              paste("!",varChild,sep="")
          },
          neq = 
          {
            if (constChild == 0)
              varChild
            else
              paste("!",varChild,sep="")
          },
          gt = 
          {
            if (constChild == 1)
              "0"
            else
              varChild
          },
          lt = 
          {
            if (constChild == 0)
              "0"
            else
              paste("!",varChild,sep="")
          },
          geq = 
          {
            if (constChild == 0)
              "1"
            else
              varChild
          },
          leq = 
          {
            if (constChild == 1)
              "1"
            else
              paste("!",varChild,sep="")
          }
        ))
      }
      else
      # two variables are compared
      {
        return(switch(operator,
          eq = 
          {
            paste("((",children[1]," & ",children[2],") | ",
                  "(!",children[1]," & !",children[2],"))", sep="")
          },
          neq = 
          {
            paste("((",children[1]," & !",children[2],") | ",
                  "(!",children[1]," & ",children[2],"))", sep="")
          },
          gt = 
          {
            paste("(",children[1]," & !",children[2],")", sep="")
          },
          lt = 
          {
            paste("(!",children[1]," & ",children[2],")", sep="")
          },
          geq = 
          {
            paste("(",children[1], " | !",children[2],")", sep="")
          },
          leq = 
          {
            paste("(!",children[1], " | ",children[2],")", sep="")
          }
        ))
      }
    }
    else
    if (operator == "not")
    {
      if (length(children) > 2)
        stop("Multiple arguments supplied to unary operator \"neg\"!")
      return(paste("!",children,
                   sep=""))
    }
    else
    # an unsupported symbol has been specified
      stop(paste("Unsupported math symbol: ",
                 operator,"!",sep=""))
  }
  else
  if (name == "ci")
  # this is a gene identifier or a threshold level
  {
    id <- trim(xmlValue(rootNode))
    if (!(id %in% names(genes)))
    {
      if (!(id %in% names(inputThresholds)))
      # neither threshold identifier nor gene
        stop(paste("Unspecified input \"",id,"\" in transition function!",sep=""))
      else
      # this is a threshold identifier
        return(inputThresholds[id])
    }
    # this is a gene
    return(genes[id])  
  }
  else
  if (name == "cn")
  # this is a constant
  {
    # convert value and ensure it is Boolean
    attrs <- xmlAttrs(rootNode)
    
    if (!is.null(attrs) && !is.na(attrs["type"]) && tolower(attrs["type"]) != "integer")
      stop("\"cn\" nodes must be of type \"integer\"!")
      
    val <- trim(xmlValue(rootNode))
    
    if (!check.integer(val) || !(as.integer(val) %in% c(0,1)))
      stop("\"cn\" nodes must be 0 or 1!")
    
    return(val)  
  }
  else
  if (name == "true")
    return("1")
  else
  if (name == "false")
    return("0")
  else
  # an unsupported symbol has been specified
    stop(paste("Unsupported math symbol: ",
               name,"!",sep=""))
}

# Import the sbml-qual document <file>
loadSBML <- function(file, symbolic=FALSE)
{
  # load XML document  
  doc <- xmlRoot(xmlParse(file))
  
  # remove comments from the document
  suppressWarnings(comments <- getNodeSet(doc,"//comment()"))
  
  if (length(comments) > 0)
    removeNodes(comments)
  
  # do various checks to ensure this is an sbml-qual document
  if (xmlName(doc) != "sbml")
    stop("Not an SBML document!")
    
  if (as.integer(xmlAttrs(doc)["level"]) > 3 || 
      as.integer(xmlAttrs(doc)["version"]) > 1)
      warning("This import is designed for documents up to SBML level 3 version 1!")
  
  if (is.null(xmlNamespaces(doc)$qual))
    stop("This document does not import the sbml-qual package!")
  
  model <- xmlFindNode(doc, "model", throwError=TRUE)[[1]]
  
  # compartments are ignored
  
  # parse species
  species <- xmlFindNode(model, "listOfQualitativeSpecies", throwError=TRUE)[[1]]
  genes <- parseSBMLSpecies(species)
  
  # parse transitions  
  transitions <- xmlFindNode(model, "listOfTransitions", throwError=TRUE)[[1]]
  interactions <- parseSBMLTransitions(transitions, genes, symbolic)
  
  if (symbolic)
  {
    delays <- apply(sapply(interactions,maxTimeDelay,genes=genes$genes),1,max)

    fixed <- as.integer(rep(-1L,length(genes$genes)))
    names(fixed) <- genes$genes
    
    res <- list(genes = genes$genes, interactions=interactions, fixed=fixed)
    res$internalStructs <- .Call("constructNetworkTrees_R",res)
    res$timeDelays <- delays
    
    class(res) <- "SymbolicBooleanNetwork"
  }
  else
  {
    # create BooleanNetwork structure
    res <- list(genes = genes$genes,
                fixed = sapply(interactions,function(i)
                              {
                                if (i$input[1] == 0)
                                  i$func[1]
                                else
                                  -1
                              }),
                interactions = interactions)
    class(res) <- "BooleanNetwork"
  }
  return(res)              
}

print.BooleanNetworkCollection <- function(x, ...)
{
  print.ProbabilisticBooleanNetwork(x, ...)
}

# Custom print function for class ProbabilisticBooleanNetwork
print.ProbabilisticBooleanNetwork <- function(x, ...)
{
  cat("Probabilistic Boolean network with",length(x$genes),"genes\n\n")
  cat("Involved genes:\n",paste(x$genes,collapse=" "),"\n\n",sep="")
  cat("Transition functions:\n")
    
  mapply(function(gene,interaction)
    {
      cat("\nAlternative transition functions for gene ",gene,":\n",sep="")
      # print original expressions read from the files (if available)
      lapply(interaction,function(func)
      {
        cat(gene," = ",func$expression,sep="")
        
        if (!is.null(func$probability) || !is.null(func$error))
        {
          cat(" (")
          
          if (!is.null(func$probability))
          {
            cat(" probability: ",func$probability,sep="")
            if (!is.null(func$error))
              cat(", ")
          }  
          if (!is.null(func$error))
            cat("error: ",func$error,sep="")
          
          cat(")")
        }
        cat("\n")
      })
    },  
    x$genes,x$interactions)

  if (sum(x$fixed != -1) > 0)
  {
    cat("\nKnocked-out and over-expressed genes:\n")
    mapply(function(gene,fixed)
      {
        if (fixed != -1)
          cat(gene," = ",fixed,"\n",sep="")
      },
      x$genes,x$fixed)
  }  
    
  return(invisible(x))
}

# Load a network in a specified rule description language
# from file <file>.
# <bodySeparator> is the character that separates targets and factors
# <lowercaseGenes> specifies whether gene names are converted to lower case
loadNetwork <- function(file, bodySeparator=",", lowercaseGenes=FALSE, symbolic=FALSE) 
{
  func <- readLines(file,-1)
  
  # strip comments
  func <- gsub("#.*", "", trim(func))
  func <- func[nchar(func) > 0]
  
  # check header
  if (length(func) == 0)
    stop("Header expected!")
  
  header <- func[1]
  header <- tolower(trim(strsplit(header, bodySeparator)[[1]]))
  
  if (length(header) < 2 || header[1] != "targets" || !(header[2] %in% c("functions","factors")) ||
      (length(header) == 3 && header[3] != "probabilities"))
      stop(paste("Invalid header:", func[1]))

  func <- func[-1]      
  
  if (lowercaseGenes)
    func <- tolower(func)
  
  # Replace all invalid characters to be able to load most networks
  func <- gsub("[^\\[\\]a-zA-Z0-9_\\|\\&!\\(\\) \t\\-+=.,]+","_", func, perl=TRUE)

  tmp <-  unname(lapply(func,function(x)
  # split strings at separators that are NOT
  # inside a bracket block
  {
    bracketCount <- 0
    lastIdx <- 1

    chars <- strsplit(x,split="")[[1]]
    
    res <- c()
    
    if (length(chars) > 0)
    {
      for (i in seq_along(chars))
      {
        if (chars[i] == "(")
          bracketCount <- bracketCount + 1
        else
        if (chars[i] == ")")
          bracketCount <- bracketCount -1
        else
        if (chars[i] == bodySeparator && bracketCount == 0)
        {
          res <- c(res, trim(paste(chars[lastIdx:(i-1)],collapse="")))
          lastIdx <- i + 1
        }
      }
      res <- c(res, trim(paste(chars[lastIdx:length(chars)],collapse="")))
    }
    return(res)
  }))
  targets <- sapply(tmp,function(rule)trim(rule[1]))
  
  for (target in targets)
  {
    if (regexec("^[a-zA-Z_][a-zA-Z0-9_]*$", target)[[1]] == -1)
      stop(paste("Invalid gene name:",target))
  }
   
  
  factors <- sapply(tmp,function(rule)trim(rule[2]))
  
  temporal <- length(grep("timeis|timelt|timegt|\\[|\\]", factors, ignore.case=TRUE) > 0)
  if (temporal && !symbolic)
  {
    warning("The network contains temporal elements. This requires loading the model with symbolic=TRUE!")
    symbolic <- TRUE
  }
  
  
  probabilities <- sapply(tmp,function(rule)
                          {
                            if (length(rule) >= 3)
                              as.numeric(trim(rule[3]))
                            else
                              1.0
                          })
  
  factors.tmp <- lapply(factors,matchNames)

  # create list of all gene names in both sides of the functions
  genes <- unique(c(targets,unname(unlist(factors.tmp))))
  isProbabilistic <- (length(unique(targets)) < length(targets))
                          
  if (symbolic)
  {
    if (isProbabilistic)
      stop("Probabilistic networks cannot be loaded with symbolic=TRUE!")
                                
    interactions <- lapply(factors, function(rule)parseBooleanFunction(rule, genes))
    
    onlyInputs <- setdiff(genes,targets)
    if(length(onlyInputs) > 0)
    # some genes are only used as inputs, but are not involved in the network
    # -> create dummy input and function
    {
      for(gene in onlyInputs)
      {
        warning(paste("There is no transition function for gene \"",
                       gene,"\"! Assuming an input!",sep=""))
        
        interactions[[gene]] = parseBooleanFunction(gene, genes)
      }
    }
    
    delays <- apply(sapply(interactions,maxTimeDelay,genes=genes),1,max)
    names(interactions) <- genes
    
    fixed <- as.integer(sapply(interactions, function(int)
                        {
                          if (int$type == "constant")
                            int$value
                          else
                            -1L  
                        }))
    names(fixed) <- genes
    
    res <- list(genes = genes, interactions=interactions, fixed=fixed)
    res$internalStructs <- .Call("constructNetworkTrees_R",res)
    res$timeDelays <- delays
    
    class(res) <- "SymbolicBooleanNetwork"
    return(res)
  }
  else
  {
    # extract "real" gene names from the list, drop constants
    #suppressWarnings(genes <- genes[is.na(as.integer(genes))])

    fixed <- rep(-1,length(genes))
    names(fixed) <- genes

    interactions <- list()

    for(i in seq_along(targets))
    {
      target <- targets[i]
      interaction <- generateInteraction(factors[i], genes);

      if (isProbabilistic)
      {
       interaction$probability <- probabilities[i]
       interactions[[target]][[length(interactions[[target]]) + 1]] <- interaction
      }
      else
      {
        if (length(interaction$func) == 1)
        # this is a constant gene => fix it
        {
          fixed[target] <- interaction$func
        }
        interactions[[target]] <- interaction
      }
    }

    onlyInputs <- setdiff(genes,targets)
    if(length(onlyInputs) > 0)
    # some genes are only used as inputs, but are not involved in the network
    # -> create dummy input and function
    {
      for(gene in onlyInputs)
      {
        warning(paste("There is no transition function for gene \"",
                       gene,"\"! Assuming an input!",sep=""))
        if (isProbabilistic)
          interactions[[gene]][[1]] = list(input = length(interactions)+1,func=c(0,1),
          									  expression = gene, probability=1.0)
        else
          interactions[[gene]] = list(input = length(interactions)+1,func=c(0,1),
          							expression = gene)
      }
    }
    
    if (isProbabilistic)
    {
      wrongProb <- sapply(interactions,function(interaction)
                   {
                      abs(1.0-sum(sapply(interaction,function(func)func$probability))) > 0.0001
                   })
      if (any(wrongProb))
        stop(paste("The probabilities of gene(s) ",paste(genes[wrongProb],collapse=", ")," do not sum up to 1!",sep=""))
    }  

    res <- list(interactions = interactions,
                genes = genes,
                fixed = fixed)
    
    if (isProbabilistic)
      class(res) <- "ProbabilisticBooleanNetwork"
    else
      class(res) <- "BooleanNetwork"
    return(res)

  }
}

matchNames <- function(rule)
{
  regexpr <- "([_a-zA-Z][_a-zA-Z0-9]*)[,| |\\)|\\||\\&|\\[]"
  rule <- paste(gsub(" ", "", rule, fixed=TRUE)," ",sep="")
  res <- unique(unname(sapply(regmatches(rule,gregexpr(regexpr, rule))[[1]], 
                    function(m) 
                    {
                      sapply(regmatches(m,regexec(regexpr,m)),function(x)x[2])
                    })))
  
  # remove operators
  isOp <- sapply(res, function(x)
          {
            tolower(x) %in% c("all", "any", 
                              "sumis", "sumgt", "sumlt", 
                              "maj", "timegt", "timelt", "timeis")
          })
                      
  return(res[!isOp])
}

# Summarize information on a state <state>, specified as a vector of
# Boolean values. Information is taken from the state tables in <attractorInfo>.
# The function outputs the corresponding attractor, the distance to the basin, 
# and the next state.
getStateSummary <- function(attractorInfo,state)
{
  stopifnot(inherits(attractorInfo,"AttractorInfo") | inherits(attractorInfo,"SymbolicSimulation"))
  
  if (inherits(attractorInfo,"SymbolicSimulation"))
  {
    if (is.null(attractorInfo$graph))
      stop(paste("This SymbolicSimulation structure does not contain transition table information.",
           "Please re-run simulateSymbolicModel() with returnGraph=TRUE!"))
    
    geneCols <- setdiff(colnames(attractorInfo$graph),c("attractorAssignment","transitionsToAttractor"))
    numGenes <- (length(geneCols)) / 2
    
    stateIndices <- apply(attractorInfo$graph[,seq_len(numGenes),drop=FALSE], 1, function(x)
                    {
                      all(as.integer(x) == as.integer(state))
                    })
           
    return(attractorInfo$graph[stateIndices,,drop=FALSE])
  }
  else
  {
    if (length(state) != length(attractorInfo$stateInfo$genes))
      stop("State must have one element for each gene!")
    
    if (is.null(attractorInfo$stateInfo$table))
      stop(paste("This AttractorInfo structure does not contain transition table information.",
             "Please re-run getAttractors() with a synchronous search and returnTable=TRUE!"))

    if (!is.null(attractorInfo$stateInfo$initialStates))
    {
      stateNo <- bin2dec(state,length(state))
      
      # search element in initial state index
      stateNo <- which(apply(attractorInfo$stateInfo$initialStates,2,function(x)
          {
            isTRUE(all.equal(x,stateNo))
          }))
    }
    else
    {
      # find out the decimal number of the state which is used in the
      # transition table
      shortenedState <- state[attractorInfo$stateInfo$fixedGenes == -1]
      
      # simply use <stateNo>-th elements if initial states are not provided    
      stateNo <- bin2dec(shortenedState,length(shortenedState)) + 1
    }
    
    # coerce summary into a one-row dataframe, i.e. create a
    # TransitionTable object with one element
    res <- as.data.frame(as.list(unlist(list(state,
          dec2bin(attractorInfo$stateInfo$table[,stateNo],
            length(attractorInfo$stateInfo$genes)),
          attractorInfo$stateInfo$attractorAssignment[stateNo],
          attractorInfo$stateInfo$stepsToAttractor[stateNo]))))
          
    colnames(res) <- c(paste("initialState.",attractorInfo$stateInfo$genes,sep=""),
           paste("nextState.",attractorInfo$stateInfo$genes,sep=""),
           "attractorAssignment","transitionsToAttractor")  
    class(res) <- c("TransitionTable","data.frame")
    
    return(res)
  }  
}

# Export a state table in <stateGraph> to a Pajek graph
toPajek <- function (stateGraph, file="boolean.net", includeLabels=FALSE, ...) 
{
  args <- list(...)
  
  if (!is.null(args$attractorInfo))
  {
    warning("The parameter \"attractorInfo\" is deprecated. Use \"stateGraph\" instead!")
    stateGraph <- args$attractorInfo
  }
  
  if (!inherits(stateGraph,"TransitionTable"))
    stateGraph <- getTransitionTable(stateGraph)
  
  geneCols <- setdiff(colnames(stateGraph),c("attractorAssignment","transitionsToAttractor"))
  numGenes <- (length(geneCols)) / 2
  
  from <- apply(stateGraph[,1:numGenes,drop=FALSE],1,paste,collapse="")
  to <- apply(stateGraph[,((numGenes+1):(2*numGenes)),drop=FALSE],1,paste,collapse="")
  vertices <- unique(c(from,to))
  vertexIndices <- seq_along(vertices)
  names(vertexIndices) <- vertices
  
  from <- vertexIndices[from]
  to <- vertexIndices[to]

 
  sink(file)
  cat("*Vertices ", length(vertices), "\r\n", sep = "")
  if (includeLabels)
  {
    for (i  in seq_along(vertices))
      cat(i," \"",vertices[i],"\"\r\n",sep="")
  }
  cat("*Arcs\r\n")

  for (i in seq_along(from))
    cat(from[i]," ",to[i]," 1\r\n",sep="")
  sink()
}

# Save <network> to <file> in the BoolNet file format.
# If <generateDNFs> is true or a mode of getDNF(), 
# a new symbolic representation for the interactions
# is generated on the basis of the truth tables (in disjunctive normal form).
# If <generateDNFs> is false, the $expression elements of the interactions are exported.
# If <saveFixed> is true, constant transition functions are exported for fixed genes
# instead of their true transition functions
saveNetwork <- function(network, file, generateDNFs = FALSE, saveFixed = TRUE)
{
  stopifnot(inherits(network,"BooleanNetwork") || inherits(network,"SymbolicBooleanNetwork") ||
            inherits(network,"ProbabilisticBooleanNetwork"))
  expressions <- NULL
  
  if (inherits(network,"BooleanNetwork"))
  {
    if (generateDNFs == FALSE)
    # Check whether all interactions have suitable string representations
    {
      tryCatch(
      {
         # parse the string representations
         lapply(network$interactions,
                function(int)parseBooleanFunction(int$expression))
      },
      error=function(e)
      {
        warning(paste("The transition functions of this network did not contain valid symbolic expressions!",
                "Generating DNF representations from the truth tables!"))
        # There was an error parsing a string representation => generate DNFs              
        generateDNFs <<- TRUE
      })
    }
    
    if (generateDNFs != FALSE)
    {
      expressions <- mapply(function(interaction, gene, fixed)
      {
        if (!saveFixed || fixed == -1)
        {
          table <- allcombn(2, length(interaction$input)) - 1
          expression <- getDNF(interaction$func, 
                               network$genes[interaction$input],
                               generateDNFs)
          return(paste(gene, ", ", expression, sep=""))
        }
        else
          return(paste(gene, ", ", fixed, sep=""))
      }, network$interactions, network$genes, network$fixed)
    }
    else
    {
      expressions <- mapply(function(interaction, gene, fixed)
      {
        if (!saveFixed || fixed == -1)
          return(paste(gene, ", ", interaction$expression, sep=""))
        else
          return(paste(gene, ", ", fixed, sep=""))
      }, network$interactions, network$genes, network$fixed)
    }
    sink(file)
    cat("targets, factors\n")
  }
  else
  if (inherits(network,"SymbolicBooleanNetwork"))
  {
    expressions <- mapply(function(interaction, gene, fixed)
    {
      if (!saveFixed || fixed == -1)
        return(paste(gene, ", ", stringFromParseTree(interaction), sep=""))
      else
        return(paste(gene, ", ", fixed, sep=""))
    }, network$interactions, network$genes, network$fixed)
    sink(file)
    cat("targets, factors\n")
  }
  else
  {
    if (generateDNFs == FALSE)
    # Check whether all interactions have suitable string representations
    {
      tryCatch(
      {
         # parse the string representations
         lapply(network$interactions,
                function(int)lapply(int, function(func)parseBooleanFunction(func$expression)))
      },
      error=function(e)
      {
        warning(paste("The transition functions of this network did not contain valid symbolic expressions!",
                "Generating DNF representations from the truth tables!"))
        # There was an error parsing a string representation => generate DNFs              
        generateDNFs <<- TRUE
      })
    }
    
    if (generateDNFs != FALSE)
    {
      expressions <- mapply(function(interaction, gene, fixed)
      {
        if (!saveFixed || fixed == -1)
        {
          lapply(interaction, function(func)
          {
              table <- allcombn(2, length(func$input)) - 1
              expression <- getDNF(func$func, 
                                   network$genes[func$input],
                                   generateDNFs)
              return(paste(gene, ", ", expression, ", ", func$probability, sep=""))
          })
        }
        else
          return(paste(gene, ", ", fixed, ", 1", sep=""))
      }, network$interactions, network$genes, network$fixed)
    }
    else
    {
      expressions <- mapply(function(interaction, gene, fixed)
      {
        if (!saveFixed || fixed == -1)
        {
          lapply(interaction, function(func)
           {
             return(paste(gene, ", ", func$expression, ", ", func$probability, sep=""))
           })
         }
         else
           return(paste(gene, ", ", fixed, ", 1", sep=""))
         
      }, network$interactions, network$genes, network$fixed)
    }
    expressions <- unlist(expressions)
    sink(file)
    cat("targets, factors, probabilities\n")
  }
  cat(paste(expressions, collapse="\n"),"\n",sep="")
  sink()
}

stateTransition <- function(network,state,type=c("synchronous","asynchronous","probabilistic"),
                            geneProbabilities, chosenGene, chosenFunctions, timeStep=0)
{
  stopifnot(inherits(network,"BooleanNetwork") | inherits(network,"SymbolicBooleanNetwork") 
            | inherits(network,"ProbabilisticBooleanNetwork"))

  type <- match.arg(type, c("synchronous","asynchronous","probabilistic"))
  if (inherits(network,"SymbolicBooleanNetwork"))
  {
    if (type != "synchronous")
      stop("Only synchronous updates are allowed for symbolic networks!")
      
    if (!is.null(dim(state)))
    {
      if (ncol(state) != length(network$genes))
        stop(paste("\"state\" must be either a vector with one value for each gene,",
                   "or a matrix with the genes in the columns and multiple predecessor states in the rows!"))
      state <- as.integer(as.matrix(state[nrow(state):1,]))
    }
    else
    {
      if (!is.numeric(state) || length(state) != length(network$genes))
        stop(paste("\"state\" must be either a vector with one value for each gene,",
                   "or a matrix with the genes in the columns and multiple predecessor states in the rows!"))
      state <- as.integer(state)
    }

    if (is.null(network$internalStructs) || checkNullPointer(network$internalStructs))
    # refresh internal tree representations if necessary
      network$internalStructs = .Call("constructNetworkTrees_R",network);
      
    on.exit(.C("freeAllMemory", PACKAGE = "BoolNet"))  
    res <- .Call("symbolicStateTransition_R",
                 network$internalStructs, 
                 state,
                 as.integer(timeStep))
  }
  else
  {
    if (length(state) != length(network$genes))
        stop("The state must consist of exactly one value for each gene!")
            
    nonFixedIndices = (network$fixed == -1)

    res <- state
    
    if (inherits(network,"BooleanNetwork") & type ==  "probabilistic")
      type <- "synchronous"
      
    if (!inherits(network,"BooleanNetwork") & length(type) == 3)
      type <- "probabilistic"
    
    if (type ==  "probabilistic")
    {
      if (missing(chosenFunctions) || is.null(chosenFunctions))
      {
        chosenFunctions <- sapply(network$interactions,function(gene)
                                  {
                                    distr <- c(0,cumsum(sapply(gene,function(func)func$probability)))
                                    r <- runif(n=1)
                                    idx <- 0
                                    for (i in seq_along(gene))
                                    {
                                      if (r > distr[i] & r <= distr[i+1])
                                      {
                                        idx <- i
                                        break
                                      }
                                    }
                                    idx
                                  })
      }
      else
        if (length(chosenFunctions) != length(network$genes))
          stop("Please provide a function index for each gene!")    

      res[nonFixedIndices] <- mapply(function(i,f)
                { 
                  if(network$interactions[[i]][[f]]$input[1] == 0)
                  # this is a constant gene with no transition function
                    return(network$interactions[[i]][[f]]$func[1])
                  input = state[network$interactions[[i]][[f]]$input]
                  return(network$interactions[[i]][[f]]$func[bin2dec(rev(input),length(input)) + 1])
                }, which(nonFixedIndices), chosenFunctions[nonFixedIndices])                                        
    }
    else
    {
      if (!inherits(network,"BooleanNetwork"))
        stop("Please choose type=\"probabilistic\" for a probabilistic Boolean network!")
      changeIndices <- switch(type,
        synchronous = which(nonFixedIndices),
        asynchronous =
        {
          if (missing(chosenGene) || is.null(chosenGene))
          {
            if (missing(geneProbabilities) || is.null(geneProbabilities))
              sample(which(nonFixedIndices),1)
            else
            {
              if (length(geneProbabilities) != length(network$genes))
                stop("Please supply exactly one probability for each gene!")
                
              if (abs(1.0 - sum(geneProbabilities)) > 0.0001)
                stop("The supplied gene probabilities do not sum up to 1!")
                
              
              if (sum(geneProbabilities[nonFixedIndices]) < 0.0001)
              	stop("There is no non-fixed gene with a probability greater than 0!")  
                
              geneProbabilities[nonFixedIndices] <- geneProbabilities[nonFixedIndices]/sum(geneProbabilities[nonFixedIndices])
              
              if (sum(nonFixedIndices) != length(network$genes))
                geneProbabilities[!nonFixedIndices] <- 0 
              distr <- c(0,cumsum(geneProbabilities))
              r <- runif(n=1)
              idx <- 0
              for (i in seq_along(network$genes))
              {
                if (r > distr[i] & r <= distr[i+1])
                {
                  idx <- i
                  break
                }
              }
              idx
            }
          }
          else
          {
            if (is.character(chosenGene))
            {
              chosenGeneIdx <- which(network$genes == chosenGene)
              if (length(chosenGeneIdx) == 0)
                stop(paste("Gene \"",chosenGene,"\" does not exist in the network!",sep=""))
              chosenGeneIdx
            }
            else
              chosenGene
          }
      })

      res[changeIndices] <- sapply(changeIndices,function(i)
      {    
        if(network$interactions[[i]]$input[1] == 0)
        # this is a constant gene with no transition function
          return(network$interactions[[i]]$func[1])
        input = state[network$interactions[[i]]$input]
        return(network$interactions[[i]]$func[bin2dec(rev(input),length(input)) + 1])
      })
    }

    res[!nonFixedIndices] = network$fixed[!nonFixedIndices]
  }
  
  names(res) <- network$genes
  return(res)
}

# Indent <string> using <count> tabs.
indent <- function(string,count)
{
  if (count == 0)
    return(string)
  return(paste(paste(rep("\t",count),collapse=""), string, sep=""))
}

# Export a Boolean network <network> to an sbml-qual file <fileName>.
# If <generateDNFs>, a new symbolic representation for the interactions
# is generated on the basis of the truth tables (in disjunctive normal form).
# Otherwise, the $expression elements of the interactions are parsed.
# If <saveFixed> is true, constant transition functions are exported for fixed genes
# instead of their true transition functions
toSBML <- function(network, file, generateDNFs=FALSE, saveFixed = TRUE)
{
  symbolic <- inherits(network,"SymbolicBooleanNetwork")
  stopifnot(inherits(network,"BooleanNetwork") || symbolic)
    
  if (symbolic)
  {
    if (any(network$timeDelays > 1))
      stop("SBML does not support networks with time delays!")
      
    parseTrees <- network$interactions
  }
  else
  {
    parseTrees <- NULL
    
    if (generateDNFs == FALSE)
    # Check whether all interactions have suitable string representations
    {
      tryCatch(
      {
         # parse the string representations
         parseTrees <- lapply(network$interactions,
                             function(int)parseBooleanFunction(int$expression))
      },
      error=function(e)
      {
        warning(paste("The transition functions of this network did not contain valid symbolic expressions!",
                "Generating DNF representations from the truth tables!"))

        # There was an error parsing a string representation => generate DNFs              
        generateDNFs <<- TRUE
      })
    }
    
    if (generateDNFs != FALSE)
    # build new representations of the functions in disjunctive normal form
    {
      network$interactions <- lapply(network$interactions, function(interaction)
      {
        table <- allcombn(2, length(interaction$input)) - 1
        interaction$expression <- getDNF(interaction$func, 
                                         network$genes[interaction$input],
                                         generateDNFs)
        return(interaction)
      })
      
      # parse the DNF representations
      parseTrees <- lapply(network$interactions,
                          function(int)parseBooleanFunction(int$expression))
    }
    
    names(parseTrees) <- network$genes
  }  
  # generate a network identifier from the file name
  id <- sub(".sbml", "", basename(file), fixed=TRUE)
  id <- gsub("[^a-zA-Z0-9_]+","_",id)
  
  # open a string connection
  output <- NULL
  f <- textConnection("output",  encoding="UTF-8", open="w", local=TRUE)
  
  # write document header
  cat(file=f, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
  cat(file=f, "<sbml xmlns=\"http://www.sbml.org/sbml/level3/version1/core\" level=\"3\" version=\"1\" xmlns:qual=\"http://www.sbml.org/sbml/level3/version1/qual/version1\" qual:required=\"true\">\n")
  cat(file=f, "\t<model id=\"", id, "\">\n", sep="")
  
  # write default compartment
  cat(file=f, "\t\t<listOfCompartments>\n")
  cat(file=f, "\t\t\t<compartment id=\"default\" constant=\"true\"/>\n")
  cat(file=f, "\t\t</listOfCompartments>\n")
  
  # write genes
  cat(file=f, "\t\t<qual:listOfQualitativeSpecies>\n")
  for (gene in network$genes)
  {
    if ((saveFixed && network$fixed[gene] != -1) || 
        (!symbolic && network$interactions[[gene]]$input[1] == 0))
    {
      if (saveFixed && network$fixed[gene] != -1)
        level <- network$fixed[gene]
      else
        level <- network$interactions[[gene]]$func[1]
        
      cat(file=f, "\t\t\t<qual:qualitativeSpecies qual:compartment=\"default\"",
                  " qual:constant=\"true\" qual:id=\"", gene,
                  "\" qual:name=\"", gene,
                  "\" qual:initialLevel=\"", level,
                  "\" qual:maxLevel=\"", level, 
                  "\"/>\n", sep="")
    }
    else
      cat(file=f, "\t\t\t<qual:qualitativeSpecies qual:compartment=\"default\"",
                  " qual:constant=\"false\" qual:id=\"", gene, 
                  "\" qual:name=\"", gene, "\" qual:maxLevel=\"1\"/>\n", sep="")
  }
  cat(file=f, "\t\t</qual:listOfQualitativeSpecies>\n")
  
  # write transition functions
  cat(file=f, "\t\t<qual:listOfTransitions>\n")
  
  for (gene in network$genes)
  {
    if ((!saveFixed || network$fixed[[gene]] == -1)  && 
        (symbolic || network$interactions[[gene]]$input[1] != 0))
    {
      cat(file=f, "\t\t\t<qual:transition qual:id=\"tr_", gene,
          "\" qual:name=\"Interactions targeting ", gene, "\">\n", sep="")
      cat(file=f, "\t\t\t\t<qual:listOfInputs>\n")
      
      if (symbolic)
        inputs <- getInputs(network$interactions[[gene]])
      else
        inputs <- network$genes[network$interactions[[gene]]$input]
      
      for (input in inputs)
        cat(file=f, "\t\t\t\t\t<qual:input qual:qualitativeSpecies=\"", input,
            "\" qual:transitionEffect=\"none\"/>\n", sep="")
      cat(file=f, "\t\t\t\t</qual:listOfInputs>\n")
      cat(file=f, "\t\t\t\t<qual:listOfOutputs>\n")
      cat(file=f, "\t\t\t\t\t<qual:output qual:qualitativeSpecies=\"", gene,
          "\" qual:transitionEffect=\"assignmentLevel\"/>\n", sep="")
      cat(file=f, "\t\t\t\t</qual:listOfOutputs>\n")
      
      cat(file=f, "\t\t\t\t<qual:listOfFunctionTerms>\n")
      cat(file=f, "\t\t\t\t\t<qual:functionTerm qual:resultLevel=\"1\">\n")
      cat(file=f, "\t\t\t\t\t\t<math xmlns=\"http://www.w3.org/1998/Math/MathML\">\n")

      parseTree <- parseTrees[[gene]]
      cat(file=f, MathMLFromParseTree(parseTree, indentLevel=7))

      cat(file=f, "\t\t\t\t\t\t</math>\n")
      cat(file=f, "\t\t\t\t\t</qual:functionTerm>\n")
      cat(file=f, "\t\t\t\t\t<qual:defaultTerm qual:resultLevel=\"0\"/>\n")
      cat(file=f, "\t\t\t\t</qual:listOfFunctionTerms>\n")
      cat(file=f, "\t\t\t</qual:transition>\n")
    }
  }
  # finish document
  cat(file=f, "\t\t</qual:listOfTransitions>\n")
  cat(file=f, "\t</model>\n")
  cat(file=f, "</sbml>\n")
  close(f)
  
  # open file and write the complete XML string
  f <- file(file, encoding="UTF-8", open="w")
  cat(file=f,output,sep="\n")
  close(f)  
}

# Build a MathML representation of a parse tree <tree>
# that represents a symbolic Boolean expression.
# Indentation of the XML nodes starts with <indentLevel>.
# Returns a string with MathML code.
MathMLFromParseTree <- function(tree,indentLevel=0)
{
  res <- switch(tree$type,
    operator = 
    {
      if (tree$operator %in% c("timeis","timegt","timelt"))
        stop(sprintf("Operator \"%s\" not supported in SBML!", tree$operator));
      if (tree$operator %in% c("maj","sumis","sumlt","sumgt"))
      # convert special predicates to general Boolean formulae
        tree <- expandCountPredicate(tree)
        
      if (tree$negated)
        paste(indent("<apply>\n", indentLevel),
              indent("<not/>\n", indentLevel+1),
              indent("<apply>\n" ,indentLevel+1),
              indent({if (tree$operator=="|" || tree$operator=="any")
              {"<or/>\n"} else{"<and/>\n"}}, indentLevel+2),
              paste(sapply(tree$operands,MathMLFromParseTree, indentLevel+2), 
                    collapse=""),
              indent("</apply>\n", indentLevel+1), 
              indent("</apply>\n", indentLevel), sep="")
      else
        paste(indent("<apply>\n", indentLevel),
              indent({if (tree$operator=="|" || tree$operator=="any")
              {"<or/>\n"} else{"<and/>\n"}}, indentLevel+1),
              paste(sapply(tree$operands, MathMLFromParseTree, indentLevel+1), 
                    collapse=""),
              indent("</apply>\n", indentLevel), sep="")
    },
    atom =  
    {
      if ((tree$name == "0" && !tree$negated) || (tree$name == "1" && tree$negated))
        indent("<cn type=\"integer\">0</cn>\n", indentLevel)
      else
      if ((tree$name == "1" && !tree$negated) || (tree$name == "0" && tree$negated))
        indent("<cn type=\"integer\">1</cn>\n", indentLevel)
      else      
        paste(indent("<apply>\n",indentLevel),
              indent("<eq/>\n", indentLevel+1),
              indent(paste("<ci>",tree$name,"</ci>\n",sep=""),indentLevel+1),
              indent(paste("<cn type=\"integer\">", {if (tree$negated) 0 else 1},
                     "</cn>\n", sep=""), indentLevel+1),
              indent("</apply>\n",indentLevel), sep="")
    })
  return(res)
}

# Retrieves the transition table of an AttractorInfo structure
# in a readable way
getTransitionTable <- function(attractorInfo)
{
  stopifnot(inherits(attractorInfo,"AttractorInfo") || inherits(attractorInfo,"SymbolicSimulation"))
  
  if (inherits(attractorInfo,"SymbolicSimulation"))
  {
    if (is.null(attractorInfo$graph))
      stop(paste("This SymbolicSimulation structure does not contain transition table information.",
           "Please re-run simulateSymbolicModel() with returnGraph=TRUE!"))
    return(attractorInfo$graph)           
  }
  else
  if (is.null(attractorInfo$stateInfo$table))
    stop(paste("This AttractorInfo structure does not contain transition table information.",
           "Please re-run getAttractors() with a synchronous search and returnTable=TRUE!"))

  return(.internal.getTransitionTable(attractorInfo$stateInfo))
}

# Internally used to extract the information from the <stateInfo> subcomponent
# of AttractorInfo
.internal.getTransitionTable <- function(stateInfo)
{
  fixedGenes <- which(stateInfo$fixedGenes != -1)
  nonFixedGenes <- which(stateInfo$fixedGenes == -1)
  
  if (!is.null(stateInfo$initialStates))
  {
    initialStates <- t(apply(stateInfo$initialStates,2,dec2bin,length(stateInfo$genes)))
  }
  else
  {
    initialStates <- matrix(ncol=length(stateInfo$genes),nrow=2^length(nonFixedGenes))
  
    # encode the initial states
    # first, encode the changing part by calculating all combinations
    temp <- allcombn(2,length(nonFixedGenes)) - 1
    initialStates[,nonFixedGenes] <- temp[,ncol(temp):1]
  
    if (length(fixedGenes) > 0)
    # if there are fixed genes, encode their values
      initialStates[,fixedGenes] <- sapply(stateInfo$fixedGenes[fixedGenes],
                   function(value)rep(value,2^length(nonFixedGenes)))
  }
                   
  # build return structure of class TransitionTable
  res <- data.frame(initialStates,
        t(apply(stateInfo$table,2,dec2bin,length(stateInfo$genes))),
        stateInfo$attractorAssignment,stateInfo$stepsToAttractor)
  colnames(res) <- c(paste("initialState.",stateInfo$genes,sep=""),
         paste("nextState.",stateInfo$genes,sep=""),
         "attractorAssignment","transitionsToAttractor")
  class(res) <- c("TransitionTable","data.frame")
  return(res)
}

# Custom print function for class TransitionTable
print.TransitionTable <- function(x, activeOnly=FALSE, ...)
{

  geneCols <- setdiff(colnames(x),c("attractorAssignment","transitionsToAttractor"))
  numGenes <- (length(geneCols)) / 2

  colIndices <- c(1,numGenes,numGenes + 1, 2*numGenes);
  
  if ("attractorAssignment" %in% colnames(x))
    colIndices <- c(colIndices, 2*numGenes + 1)
    
  if ("transitionsToAttractor" %in% colnames(x))
    colIndices <- c(colIndices, 2*numGenes + 2)    
      
  genes <- sapply(colnames(x)[seq_len(numGenes)],function(n)strsplit(n,".",fixed=TRUE)[[1]][2])
  
  if(activeOnly)
  {
    inputStates <- apply(x,1,function(row)
                        {
                          r <- paste(genes[which(row[colIndices[1]:colIndices[2]] == 1)],collapse=", ")
                          if (r == "")
                            r <- "--"
                          r
                        })
    outputStates <- apply(x,1,function(row)
                          {
                            r <- paste(genes[which(row[colIndices[3]:colIndices[4]] == 1)],collapse=", ")
                            if (r == "")
                              r <- "--"
                            r
                          })
    colWidth <- max(c(sapply(inputStates,nchar),sapply(outputStates,nchar)))
    align <- "left"
  }
  else
  {
      inputStates <- apply(x,1,function(row)
                        paste(row[colIndices[1]:colIndices[2]],collapse=""))
      outputStates <- apply(x,1,function(row)
                        paste(row[colIndices[3]:colIndices[4]],collapse=""))  
      colWidth <- numGenes
      align <- "right"                  
  }
  
  binMatrix <- cbind(inputStates,outputStates)
  
   if ("attractorAssignment" %in% colnames(x))
    binMatrix <- cbind(binMatrix, x[,colIndices[5]])
    
   if ("transitionsToAttractor" %in% colnames(x))
    binMatrix <- cbind(binMatrix, x[,colIndices[6]])

  binMatrix <- as.data.frame(binMatrix)

  cat(format("State",width=max(7,colWidth),justify=align),"    ",
      format("Next state",width=max(11,colWidth + 2),justify=align),
      if ("attractorAssignment" %in% colnames(x))
      {
        format("Attr. basin",width=13,justify="right")
      }
      else
        "",
      if ("transitionsToAttractor" %in% colnames(x))
      { 
        format("# trans. to attr.",width=19,justify="right")
      }
      else
        "",
      "\n",sep="")
      
  apply(binMatrix,1,function(row)
  {
    # paste all states of input and output into one string, and put out all columns of the table in a
    # formatted way
    cat(format(row[1],width=max(7,colWidth),justify=align),
        " => ",
        format(row[2],width=max(11,colWidth + 2),justify=align),
        if ("attractorAssignment" %in% colnames(x))
        { 
          format(row[3],width=13,justify="right")
        }
        else 
          "",
        if ("transitionsToAttractor" %in% colnames(x))
        { 
            format(row[4],width=19,justify="right")
        }
        else
        "",
        "\n",sep="")
  })  
  if (!activeOnly)
    cat("\nGenes are encoded in the following order: ",
        paste(genes,collapse=" "),"\n",sep="")
    
  return(invisible(x))
}

# Randomly perturb a supplied network.
# <perturb="functions"> perturbs the functions associated with the genes directly.
# <perturb="states"> perturbs a maximum of <numStates> states 
# in the transition table resulting from the functions.
# <method="bitflip"> randomly flips up to <maxNumBits> in the functions or states.
# <method="shuffle"> randomly permutes the bits in the functions or states.
# If <simplify> is set, the perturbed network is simplified to remove irrelevant input functions.
# If <excludeFixed> is set, fixed genes are excluded from the perturbations and stay as they are.
perturbNetwork <- function(network,perturb=c("functions","transitions"),method=c("bitflip","shuffle"),
      simplify=(perturb[1]!="functions"),readableFunctions=FALSE,excludeFixed=TRUE,
      maxNumBits=1,numStates=max(1,2^length(network$genes)/100))
{
  stopifnot(inherits(network,"BooleanNetwork") | inherits(network,"ProbabilisticBooleanNetwork"))

  fixedGenes <- which(network$fixed != -1)

  if (length(perturb) == 1 && perturb == "states")
  {
    warning("perturb=\"states\" is deprecated. Use perturb=\"transitions\" instead!")
    perturb <- "transitions"
  }
  
  if (inherits(network,"BooleanNetwork"))
  # deterministic network
  {
    switch(match.arg(perturb,c("functions","transitions")),
      functions=
        switch(match.arg(method,c("bitflip","shuffle")),
        bitflip =
          {
            # choose the function to be perturbed
            if (length(fixedGenes) > 0 & excludeFixed)
              functionIdx <- sample((seq_along(network$interactions))[-fixedGenes],size=1)
            else
              functionIdx <- sample(seq_along(network$interactions),size=1)
          
            # choose the indices of the truth table to be flipped  
            flipIndices <- sample(seq_along(network$interactions[[functionIdx]]$func),
                                  size=runif(n=1,min=1,
                                  max=min(maxNumBits,
                                  length(network$interactions[[functionIdx]]$func))),
                                  replace=FALSE)
            # flip the bits
            network$interactions[[functionIdx]]$func[flipIndices] <- 
              as.integer(!network$interactions[[functionIdx]]$func[flipIndices])
            network$interactions[[functionIdx]]$expression <-
              getInteractionString(readableFunctions,
                     network$interactions[[functionIdx]]$func,
                     network$genes[network$interactions[[functionIdx]]$input])
          },    
        shuffle=
          {
            # choose the function to be perturbed
            if (length(fixedGenes) > 0 & excludeFixed)
              functionIdx <- sample((seq_along(network$interactions))[-fixedGenes],size=1)
            else
              functionIdx <- sample(seq_along(network$interactions),size=1)

            # draw a random permutation of bit positions
            flipIndices <- sample(seq_along(network$interactions[[functionIdx]]$func),
                  size=length(network$interactions[[functionIdx]]$func),
                  replace=FALSE)
                
            # permute the bits
            network$interactions[[functionIdx]]$func <- 
              network$interactions[[functionIdx]]$func[flipIndices]
            network$interactions[[functionIdx]]$expression <-
              getInteractionString(readableFunctions,
                     network$interactions[[functionIdx]]$func,
                     network$genes[network$interactions[[functionIdx]]$input])
                    
          },
        stop("'method' must be one of \"bitflip\",\"shuffle\"")),
    
    transitions =
      {  
        # turn off gene fixing - otherwise reverse-engineering of the transition table is not possible
      
        oldFixed <- network$fixed
        network$fixed <- rep(-1,length(network$genes))
        names(network$fixed) <- network$genes
      
        # calculate transition table
        table <- t(sapply(getAttractors(network)$stateInfo$table,dec2bin,length(network$genes)))
      
        # determine the states to be perturbed
        statesToChange <- sample(seq_len(nrow(table)),min(numStates,nrow(table)),replace=FALSE)
      
        lapply(statesToChange,function(state)
          {
            # choose the indices of the states that are allowed to be changed
            flipIndices <- seq_along(network$genes)
            if (length(fixedGenes) > 0 & excludeFixed)
              flipIndices <- flipIndices[-fixedGenes]
            
            switch(match.arg(method,c("bitflip","shuffle")),
            bitflip =
              {
                # choose the actual indices to be changed
                flipIndex <- sample(flipIndices,
                                    size=runif(n=1,min=1,
                                    max=min(maxNumBits,
                                    length(flipIndices))),
                                    replace=FALSE)
                    
                # flip the bits at these positions
                table[state,flipIndex] <<- 
                  as.integer(!table[state,flipIndex])
        
              },
            shuffle =
              {
                # determine a permutation of the bit indices
                flipIndex <- sample(flipIndices,
                                    size=length(flipIndices),
                                    replace=FALSE)
                    
                # permute the state
                table[state,] <<- 
                  as.integer(table[state,flipIndex])
              },
              stop("'method' must be one of \"bitflip\",\"shuffle\"")        
            )
          NULL})
        
        # restore network by assigning the columns of the state table to the corresponding genes
        network$interactions <- apply(table,2,function(gene)
                {
                  input = seq_along(network$genes)
                
                  # encoding is reversed in the transition table
                  input <- input[length(input):1]
                  list(input=input,
                       func=gene,
                       expression= getInteractionString(readableFunctions,
                       gene,
                       network$genes[input]))
                })
        # reactivate fixed genes
        network$fixed <- oldFixed
      
      
      },  
      stop("'perturb' must be one of \"functions\",\"transitions\""))
  }
  else
  # probabilistic network
  {
    if (match.arg(perturb) != "functions")
      stop("In probabilistic Boolean networks, only perturb=functions is allowed!")

      switch(match.arg(method,c("bitflip","shuffle")),
      bitflip =
        {
          # choose the gene and the function to be perturbed
      
          if (length(fixedGenes) > 0 & excludeFixed)
            geneIdx <- sample((seq_along(network$interactions))[-fixedGenes],size=1)
          else
            geneIdx <- sample(seq_along(network$interactions),size=1)
                
          functionIdx <- sample(seq_along(network$interactions[[geneIdx]]),size=1)

          # choose the indices of the truth table to be flipped  
          flipIndices <- sample(seq_along(network$interactions[[geneIdx]][[functionIdx]]$func),
                              size=runif(n=1,min=1,
                              max=min(maxNumBits,
                              length(network$interactions[[geneIdx]][[functionIdx]]$func))),
                              replace=FALSE)
          # flip the bits
          network$interactions[[geneIdx]][[functionIdx]]$func[flipIndices] <- 
            as.integer(!network$interactions[[geneIdx]][[functionIdx]]$func[flipIndices])
          network$interactions[[geneIdx]][[functionIdx]]$expression <-
            getInteractionString(readableFunctions,
                                 network$interactions[[geneIdx]][[functionIdx]]$func,
                                 network$genes[network$interactions[[geneIdx]][[functionIdx]]$input])
        },    
      shuffle=
        {
          # choose the function to be perturbed
          if (length(fixedGenes) > 0 & excludeFixed)
            geneIdx <- sample((seq_along(network$interactions))[-fixedGenes],size=1)
          else
            geneIdx <- sample(seq_along(network$interactions),size=1)
                
          functionIdx <- sample(seq_along(network$interactions[[geneIdx]]),size=1)


          # draw a random permutation of bit positions
          flipIndices <- sample(seq_along(network$interactions[[geneIdx]][[functionIdx]]$func),
                                size=length(network$interactions[[geneIdx]][[functionIdx]]$func),
                                replace=FALSE)
                
          # permute the bits
          network$interactions[[geneIdx]][[functionIdx]]$func <- 
            network$interactions[[geneIdx]][[functionIdx]]$func[flipIndices]
          network$interactions[[geneIdx]][[functionIdx]]$expression <-
            getInteractionString(readableFunctions,
                                 network$interactions[[geneIdx]][[functionIdx]]$func,
                                 network$genes[network$interactions[[geneIdx]][[functionIdx]]$input])
                  
        },
      stop("'method' must be one of \"bitflip\",\"shuffle\""))
    
  }
  # simplify the network if necessary
  if (simplify)
    network <- simplifyNetwork(network,readableFunctions)
  return(network)
}

# Builds a matrix of transitions and their probabilities from <markovSimulation>
getTransitionProbabilities <- function(markovSimulation)
{
  stopifnot(inherits(markovSimulation,"MarkovSimulation"))
  if (is.null(markovSimulation$table))
    stop(paste("The supplied simulation result does not contain transition information.",
               "Please re-run markovSimulation() with returnTable=TRUE!"))
  
  initialStates <- t(apply(markovSimulation$table$initialStates,2,dec2bin,length(markovSimulation$genes)))

  nextStates <- t(apply(markovSimulation$table$nextStates,2,dec2bin,length(markovSimulation$genes)))

  idx <- order(apply(initialStates,1,function(x)paste(x,collapse="")))
  
  res <- data.frame(initialStates,nextStates,markovSimulation$table$probabilities)
  
  res <- res[idx,]
  
  rownames(res) <- NULL  
  colnames(res) <- c(paste("initialState.",markovSimulation$genes,sep=""),
                     paste("nextState.",markovSimulation$genes,sep=""),
                     "probability")
  return(res)
}

# Split up <expression> into symbols.
# Returns a vector of symbols.
scan <- function(expression, lowerCase=FALSE)
{
  if (lowerCase)
    expression <- tolower(expression)
  
  # add extra whitespace to brackets and operators
  
  expression <- gsub("(\\(|\\)|\\[|\\]|\\||\\&|\\!|\\=|\\.\\.|[a-zA-Z0-9_]*)"," \\1 ", expression)

  # strip multiple whitespace characters
  expression <- gsub("[ ]+", " ", expression)
  expression <- gsub("^[ ]+", "", expression)
  expression <- gsub("[ ]+$", "", expression)
  
  # split up at whitespace positions
  res <- strsplit(expression, " ", fixed=TRUE)[[1]]
  return(res)
}

# Parse a Boolean function in <expression>,
# and build a corresponding parse tree.
parseBooleanFunction <- function(expression, varNames=c(), lowerCase=FALSE)
{
  minArgCounts <- c("&"=2,"|"=2, "all" = 1, "any" = 1, "maj" = 1, 
                    "sumis" = 2, "sumgt" = 2, "sumlt" = 2,
                    "timeis" = 1, "timelt" = 1, "timegt" = 1)
  maxArgCounts <- c("&"=Inf,"|"=Inf,"all" = Inf, "any" = Inf, "maj" = Inf, 
                    "sumis" = Inf, "sumgt" = Inf, "sumlt" = Inf,
                    "timeis" = 1, "timelt" = 1, "timegt" = 1)
  
  isBooleanArgument <- function(arg)
  {
    return (arg$type %in% c("atom","operator") ||
            arg$type == "constant" && arg$value %in% c(0,1))
  }
  
  checkPredicate <- function(pred)
  {
    if (pred$operator %in% c("&","|","maj"))
    {
      for (el in pred$operands)
      {
        if (!isBooleanArgument(el))
        {
          stop(paste("Invalid argument to \"",pred$operator,"\": ", stringFromParseTree(el),sep=""))
        }
      }
    }
    else
    if (pred$operator %in% c("sumis","sumgt","sumlt"))
    {
      for (el in pred$operands[-length(pred$operands)])
      {
        if (!isBooleanArgument(el))
        {
          stop(paste("Invalid argument to \"",pred$operator,"\": ", stringFromParseTree(el),sep=""))
        }
      }
      if (pred$operands[[length(pred$operands)]]$type != "constant")
        stop(paste("Last argument of \"",pred$operator,"\" must be an integer value!"))
    }
    else
    if (pred$operator %in% c("timeis"))
    {
      if (pred$operands[[1]]$type != "constant" || pred$operands[[1]]$value < 0)
        stop(paste("Argument of \"",pred$operator,"\" must be a non-negative integer value!"))
    }
  }
  
  # internal function to step forward one symbol
  advanceSymbol <- function(errorIfEnd = TRUE)
  {
    pos <<- pos + 1
    #print(symbols[-(1:pos-1)])    
    if (pos > length(symbols))
      if (errorIfEnd)
        stop("Unexpected end of expression!")
      else
        return(NA)
    else
      return(symbols[pos])
  }
  
  getCurrentSymbol <- function()
  {
    #print(symbols[-(1:pos-1)])
    if (pos > length(symbols))
      return(NA)
    else
      return(symbols[pos])
  }
  
  # internal function to preview the next symbol
  previewSymbol <- function()
  {
    if (pos + 1 > length(symbols))
      return(NA)
    else
      return(symbols[pos + 1])
  }
  
  getSymbolPos <- function()
  {
    return(pos)
  }
  
  setSymbolPos <- function(newPos)
  {
    pos <<- newPos
  }
  
  # internal function to parse a sub-expression in brackets
  parseExpression <- function(temporalVariables = c())
  {
    operators <- c()
    children <- c()
    symb <- advanceSymbol()
    
    while (TRUE)
    {
      if (symb == "(")
        # a new sub-expression
        children[[length(children)+1]] <- parseExpression(temporalVariables=temporalVariables)
      else
      if (symb == "!")
        # a negation
        children[[length(children)+1]] <- parseNegation(temporalVariables=temporalVariables)
      else
      if (regexpr("^[a-zA-Z0-9_]*$",symb) == -1)
        # something unexpected happened
        stop(paste("Unexpected symbol:",symb))
      else
        children[[length(children)+1]] <- parseAtomOrPredicate(symb, temporalVariables=temporalVariables)
      
      symb <- advanceSymbol(FALSE)  
      
      if (is.na(symb) || symb %in%  c(")",","))
        # end of expression was reached      
        break
        
      if (!symb %in% c("&","|"))
        stop("Operator expected!")
      
      operators <- c(operators,symb)
      
      symb <- advanceSymbol()
    }
    
    if (length(children) == 1)
    {
      # an operator with only one child
      return(children[[1]])
    }
    else
    if (length(unique(operators)) == 1)
    {
      # an n-ary operator
      res <- list(type="operator",operator=operators[1],
                  negated=FALSE, operands=children)
                  
      checkPredicate(res)                  
      return(res)
    }
    else
    # a mixture of multiple operators => ensure correct precedence
    {
      i <- 1
      startPos <- NA
      operators <- c(operators, "|")
      operands <- list()

      while (i <= length(operators))
      # identify AND operators and move them to a subtree
      {
        if (operators[i] == "&" && is.na(startPos))
          # start of a conjunction
          startPos <- i
        if (operators[i] == "|" && !is.na(startPos))
        # end of a conjunction => create subtree
        {
          subOp <- list(type="operator", operator="&",
                        negated=FALSE, operands=children[startPos:i])
          operands[[length(operands)+1]] <- subOp
          startPos <- NA
        }
        else
        if (is.na(startPos))
          operands[[length(operands)+1]] <- children[[i]]
        i <- i + 1  
      }
      res <- list(type="operator",operator="|",
                  negated=FALSE, operands=operands)
      checkPredicate(res)                
      return(res)
    }
  }
  
  # internal function to parse atoms or temporal predicates
  parseAtomOrPredicate <- function(name, temporalVariables=c())
  {
    if (regexpr("^[0-9]*$",name) != -1 || name %in% c("true","false"))
    # a (Boolean or integer) constant
    {
      if (name == "true")
        value <- 1L
      else
      if (name == "false")
        value <- 0L
      else
        value <- as.integer(name)
         
      return(list(type="constant",
                  value=value,
                  negated=FALSE))
    } 
    if (tolower(name) %in% names(minArgCounts))
    # a temporal predicate
    {
      name <- tolower(name)
      # translate all into & and any into |
      operator <- switch(name,
                         all = "&",
                         any = "|",
                         name)

      nextSym <- previewSymbol()
      if (!is.na(nextSym) && nextSym == "[")
      # a temporal iterator definition follows
      {
        # discard bracket
        advanceSymbol()
      
        varName <- advanceSymbol()
        
        if (regexpr("^[a-zA-Z0-9_]*$",varName) == -1)
          # something unexpected happened
          stop(paste("Unexpected symbol:",varName))
        
        symb <- advanceSymbol()   
        if (symb != "=")
        {
          stop(paste("Expected \"=\" instead of \"",symb,"\" in temporal predicate!"))
        }
        
        # parse start time point
        startTime <- parseTimePoint(temporalVariables=temporalVariables, TRUE)
        symb <- getCurrentSymbol()
        
        if (symb != "..")
        {
          stop(paste("Expected \"..\" instead of \"",symb,"\" in temporal predicate!"))
        }

        # parse end time point
        endTime <- parseTimePoint(temporalVariables=temporalVariables, TRUE)        
        symb <- getCurrentSymbol()
                    
        if (symb != "]")
            stop(paste("Unexpected symbol:",symb))
        
      }
      else
      # no temporal iterator defined
      {
        startTime <- 0
        endTime <- 0
        varName <- NULL
      }            

      symb <- advanceSymbol()      
      if (symb != "(")
          stop(paste("Unexpected symbol:",symb))

      subElements <- c()
            
      while(TRUE)
      # parse parameters of predicate
      {
        oldPos <- getSymbolPos()
        
        for (time in startTime:endTime)
        # expand arguments according to temporal range
        {
          tempVars <- temporalVariables
          
          if (!is.null(varName))
          {
            if (length(tempVars) > 0 && !is.na(tempVars[varName]))
            {
              stop(paste("Duplicate variable ",varName,"! Please use only unique variable names!",sep=""))
            }
            tempVars[varName] <- time
          }
          # re-parse the argument with a new time point => reset parser position
          setSymbolPos(oldPos)          
          subElement <- parseExpression(temporalVariables=tempVars)
          
          subElements <- c(subElements, list(subElement))  
          if (subElement$type == "constant")
            break
        }
        
        subElements <- unique(subElements)
        
        symb <- getCurrentSymbol()
        if (is.na(symb))
          stop("Unexpected end of expression!")
        
        if (symb == ")")
          break
          
        if (symb != ",")
          stop ("Expected \",\" or \")\"!")
        
      }
      
      # check if number of argument matches
      if (length(subElements) < minArgCounts[name] || length(subElements) > maxArgCounts[name])
        stop(paste("Invalid number of arguments to ",name,": ",length(subElements),sep=""))
      
      res <- list(type="operator",operator=operator,
                  negated=FALSE, operands=subElements)

      # check if argument types match
      checkPredicate(res)
                  
      return(res)
    }
    else
    # an atom with or without temporal information
    {
      nextSym <- previewSymbol()
      if (!is.na(nextSym) && nextSym == "[")
      # the atom has a time point specification
      {
        symb <- advanceSymbol()
        time <- parseTimePoint(temporalVariables=temporalVariables)
        symb <- getCurrentSymbol()        
                  
        if (symb != "]")
          stop(paste("Unexpected symbol:",symb))
      }
      else
      # no time point specified => default to previous time point
        time <- -1L
        
      if (length(varNames) > 0)
      # check if variable is known
      {
        varIndex <- which(varNames == name)
        if (length(varIndex) == 0)
          stop(paste("Unknown variable:",name))
        negated <- FALSE
        
        return(list(type="atom",
                    name=name,
                    index=as.integer(varIndex),
                    time=as.integer(time), 
                    negated=negated))
          
      }
      else
      {
          return(list(type="atom",
                      name=name,
                      time=as.integer(time), 
                      negated=FALSE))
      }

    }
  }
  
  # internal function to parse a time specification
  parseTimePoint <- function(temporalVariables=c(), allowPositive=FALSE)
  {
    symb <- advanceSymbol()
    time <- 0
    while (TRUE)
    {
      sign <- 1
      
      if (symb == "+")
      # ignore positive sign
        symb <- advanceSymbol()
      else
      if (symb == "-")
      # set multiplier to negative sign
      {
        sign <- -1
        symb <- advanceSymbol()
      }
      
        
      if (symb %in% names(temporalVariables))
      # a variable was found => add its value
      {
        time <- time + sign * temporalVariables[symb]
        symb <- advanceSymbol()
      }      
      else
      # this has to be an integer time value
      {
        suppressWarnings(addTime <- as.integer(symb))
        
        if (is.na(addTime))
        {
          stop(paste("Invalid time specification: ",symb, 
                     ". Time specifications must be integers <= -1)", sep=""))
        }
        time <- time + sign * addTime
        symb <- advanceSymbol()
      }      
      if (symb %in% c("]",".."))
        break
    }
    
    if (time > -1 && !allowPositive)
    {
      stop(paste("Invalid time specification: ",time, 
                 ". Time specifications must be integers <= -1)", sep=""))
    }
    return(as.integer(time))
  }
  
  # Internal function to parse a negation
  parseNegation <- function(temporalVariables=c())
  {
    symb <- advanceSymbol()
    if (symb == "(")
    {
      res <- parseExpression(temporalVariables)
    }
    else
    if (symb %in% c(")","&","|","]","[","="))
      stop(paste("Unexpected symbol:",symb))
    else
      res <- parseAtomOrPredicate(symb, temporalVariables=temporalVariables)
    res$negated <- !res$negated
    return(res)
  }

  tryCatch(
  {
    symbols <- scan(expression, lowerCase=lowerCase)
    
    pos <- 0
    res <- parseExpression()
  },
  error = function(e)
  {
    stop(sprintf("Error parsing expression \"%s\": %s", expression, e$message))
  })
  if (!isBooleanArgument(res))
    stop(paste("Non-Boolean formula ",stringFromParseTree(res),"!",sep=""))
  
  return(res)
}

# Regenerate the expression string 
# from the parse tree <tree>
stringFromParseTree <- function(tree)
{
  res <- switch(tree$type,
    operator = 
    {
      if (tree$operator %in% c("|","&"))
      {
        paste({if (tree$negated) "!" else ""},
              "(",
              paste(sapply(tree$operands,stringFromParseTree), 
                    collapse=paste(" ",tree$operator," ",sep="")),
              ")", sep="")
      }
      else
      {
        paste({if (tree$negated) "!" else ""},
              tree$operator,"(",paste(sapply(tree$operands,stringFromParseTree), collapse=", "),")",sep="")
      }
    },    
    atom = paste({if (tree$negated) "!" else ""},
                 tree$name,
                 {if (tree$time != -1) paste("[",tree$time,"]",sep="") else ""},
                 sep=""),    
    constant = paste({if (tree$negated) "!" else ""},
                     tree$value,
                     sep="")
  )                     
  return(res)  
}

# Plots a graph that visualizes the state transitions and attractor basins. <attractorInfo> is an object
# of class AttractorInfo. This requires the igraph package.
# If <highlightAttractors> is set, attractor edges are drawn bold.
# If <colorBasins> is true, each basin is drawn in a different color. 
# Colors can be provided in <colorSet>.
# <layout> specifies the graph layouting function.
# If <piecewise> is true, subgraphs are layouted separately.
# <basin.lty> and <attractor.lty> specify the line types used to draw states in the basins
# and in the attractors (if <highlightAttractor> is set).
# If <plotIt> is not set, only the igraph object is returned, but no graph is plotted.
# ... provides further graphical parameters for the plot.
# Returns an object of class igraph
plotStateGraph <- function(stateGraph,
                           highlightAttractors = TRUE,
                           colorBasins = TRUE,
                           colorSet,
                           drawLegend = TRUE,
                           drawLabels = FALSE,
                           layout = layout.kamada.kawai,
                           piecewise = FALSE,
                           basin.lty = 2,
                           attractor.lty = 1,
                           plotIt = TRUE,
                           colorsAlpha = c(colorBasinsNodeAlpha = .3,
                                           colorBasinsEdgeAlpha = .3,
                                           colorAttractorNodeAlpha = 1,
                                           colorAttractorEdgeAlpha = 1),
                           ...)
{
  stopifnot(inherits(stateGraph,"AttractorInfo") || 
              inherits(stateGraph,"TransitionTable") || 
              inherits(stateGraph,"SymbolicSimulation"))
  
  args <- list(...)
  
  if (!is.null(args$attractorInfo))
  {
    warning("The parameter \"attractorInfo\" is deprecated. Use \"stateGraph\" instead!")
    stateGraph <- args$attractorInfo
  }
  if(is.null(colorsAlpha) | (length(colorsAlpha) != 4)) {
    warning("colorsAlpha parameter not properly specified. Parameter will be set to opaque values (1,1,1,1).")
    colorsAlpha <- c(1,1,1,1)
  }
  if (any(colorsAlpha < 0 | colorsAlpha > 1)) {
    warning("colorsAlpha parameters are not in range [0,1] - they will be normalized.")
    colorsAlpha <- colorsAlpha/sum(colorsAlpha)
  }
  
  if (installed.packages()["igraph","Version"] < package_version("0.6"))
    bias <- 1
  else
    bias <- 0
  
  symbolic <- FALSE
  if (inherits(stateGraph,"AttractorInfo"))
  {
    stateGraph <- getTransitionTable(stateGraph)
  } else if (inherits(stateGraph,"SymbolicSimulation")) {
    symbolic <- TRUE
    if (is.null(stateGraph$graph))
      stop(paste("This SymbolicSimulation structure does not contain transition table information.",
                 "Please re-run simulateSymbolicModel() with returnGraph=TRUE!"))
    stateGraph <- stateGraph$graph
  }
  
  geneCols <- setdiff(colnames(stateGraph), 
                      c("attractorAssignment","transitionsToAttractor"))
  numGenes <- (length(geneCols)) / 2
  
  from <- apply(stateGraph[ , 1:numGenes, drop=FALSE], 1, paste, collapse="")
  to <- apply(stateGraph[ , ((numGenes+1):(2*numGenes)), drop=FALSE], 1, paste, collapse="")
  vertices <- unique(c(from, to))
  edges <- data.frame(from, to)
  
  res <- graph.data.frame(edges, vertices = as.data.frame(vertices), directed=TRUE)
  res <- set.vertex.attribute(res, "name", value = vertices)    
  
  if ("attractorAssignment" %in% colnames(stateGraph))
    attractorAssignment <- stateGraph$attractorAssignment
  else
  {
    attractorAssignment <- c()
    colorBasins <- FALSE
    drawLegend <- FALSE
  }
  
  if ("transitionsToAttractor" %in% colnames(stateGraph))
    attractorIndices <- to[stateGraph$transitionsToAttractor == 0]
  else
  {
    if (highlightAttractors) {
      warning("The parameter \"highlightAttractors\" is set to true although not enough information is available in stateGraph. Highlightning of attractors will be set to FALSE.") 
    }
    attractorIndices <- c()
    highlightAttractors <- FALSE
  }
  
  # determine nodes and edges that belong to attractors
  
  
  # set default edge width and line type
  res <- set.edge.attribute(res, "width" , value = 0.8)
  res <- set.edge.attribute(res, "lty", value = basin.lty)
  
  if (highlightAttractors)
  {
    attractorEdgeIndices <- which(apply(edges, 1 , function(edge){
      return( (edge[1] %in% attractorIndices) & (edge[2] %in% attractorIndices) )
    })) - bias
    # set different edge width and line type for attractor edges
    res <- set.edge.attribute(res, "width", index = attractorEdgeIndices, value = 2)
    res <- set.edge.attribute(res, "lty", index = attractorEdgeIndices, value = attractor.lty)
  }
  
  if (missing(colorSet))
  {
    # define default colors
    colorSet <- c("blue","green","red","darkgoldenrod","gold","brown","cyan",
                  "purple","orange","seagreen","tomato","darkgray","chocolate",
                  "maroon","darkgreen","gray12","blue4","cadetblue","darkgoldenrod4",
                  "burlywood2")
  }
  
  # check for certain graphical parameters in ... 
  # that have different default values in this plot
  if (is.null(args$vertex.size))
    args$vertex.size <- 2
  
  if (is.null(args$edge.arrow.mode))
    args$edge.arrow.mode <- 2
  
  if (is.null(args$edge.arrow.size))
    args$edge.arrow.size <- 0.3
  if (is.null(args$vertex.label.cex))
    args$vertex.label.cex <- 0.5
  
  if (is.null(args$vertex.label.dist))
    args$vertex.label.dist <- 1
  
  attractors <- unique(attractorAssignment)
  attractors <- attractors[!is.na(attractors)]
  
  if (colorBasins)
  {  
    res <- set.edge.attribute(res, "color", value = "darkgrey")
    for (attractor in attractors)
    {
      # determine nodes and edges belonging to the basin of <attractor>
      attractorGraphIndices <- NULL
      basinIndices <- which(attractorAssignment == attractor)
      if(!is.null(stateGraph$transitionsToAttractor)) {
        attractorGraphIndices <- intersect(basinIndices, which(stateGraph$transitionsToAttractor == 0))
        basinIndices <- base::setdiff(basinIndices, attractorGraphIndices)
      }
      

      if (!symbolic)
      {
        # change vertex color
        res <- set.vertex.attribute(res, "color", basinIndices - bias, 
                                    value = adjustcolor(colorSet[(attractor-1) %% length(colorSet) + 1], 
                                                        alpha.f = colorsAlpha[1]))
        res <- set.vertex.attribute(res, "frame.color", basinIndices - bias, 
                                    value = adjustcolor("black", 
                                                        alpha.f = colorsAlpha[1]))
        if(!is.null(attractorGraphIndices)) {
          res <- set.vertex.attribute(res, "color", attractorGraphIndices - bias, 
                                      value = adjustcolor(colorSet[(attractor-1) %% length(colorSet) + 1], 
                                                          alpha.f = colorsAlpha[3]))
          res <- set.vertex.attribute(res, "frame.color", attractorGraphIndices - bias, 
                                      value = adjustcolor("black", 
                                                          alpha.f = colorsAlpha[3]))  
        }
        
        if (drawLabels)
          res <- set.vertex.attribute(res,"label.color",basinIndices - bias,
                                      value=colorSet[(attractor-1) %% length(colorSet) + 1])
      }
      
      # change edge color
      res <- set.edge.attribute(res, "color", index = basinIndices - bias,
                                value = adjustcolor(colorSet[(attractor-1) %% length(colorSet) + 1], alpha.f = colorsAlpha[2]))
      if(!is.null(attractorGraphIndices)) {
        res <- set.edge.attribute(res, "color", index = attractorGraphIndices - bias,
                            value = adjustcolor(colorSet[(attractor-1) %% length(colorSet) + 1], alpha.f = colorsAlpha[4]))
      }
    }
  }
  
  if(plotIt)
  {   
    if (drawLabels)
      labels <- vertices
    else
      labels <- NA
    if (piecewise)
      layout <- piecewise.layout(res, layout)
    
    if (symbolic)
      autocurve.edges(res)
    
    do.call("plot",c(list(res),args,"vertex.label"=list(labels), "layout"=list(layout)))
    #plot(res,vertex.size=args$vertex.size,layout=layout,
    #     edge.arrow.mode=args$edge.arrow.mode,
    #     vertex.label=labels,vertex.label.cex=args$vertex.label.cex,
    #     vertex.label.dist=args$vertex.label.dist,
    #     ...)
    if (colorBasins & drawLegend)
      legend(x="bottomleft",pch=15,ncol=1,
             
             col=colorSet[attractors-1 %% length(colorSet) + 1],
             legend = paste("Attractor",seq_along(attractors)),
             cex=0.5)
  }
  return(invisible(res))
}

# determination of P(k,N,w)
pval <- function(k,N,w)
{	
	
	return((k/w-N-1)*b(k,N,w)+2*Gb(k,N,w))
		
}

# helper function
b<-function(k,N,w)
{
	return(choose(N,k)*w^k*(1-w)^(N-k))	
}

# helper function
Gb<-function(k,N,w)
{	
	sum<-0
	for(i in k:N)
	{	
		sum <- sum + b(i,N,w)	
	}
	
	return(sum)
}

# If two significant overlapping windows were found, these windows are
# merged. If the windows do not overlap, two different windows are stored 
# in a list 
listadapt <- function(lcur,lnew)
{
	if(length(lcur)==0)
	{
		lcur=lnew	
		return(lcur)
	}
	else
	{		
		if(lnew[[1]][1]<=lcur[[length(lcur)]][2])
		{
			lcur[[length(lcur)]][2]<-lnew[[1]][2]
			if(lcur[[length(lcur)]][3]>lnew[[1]][3])
			{
				lcur[[length(lcur)]][3] <- lnew[[1]][3]
			}
			return(lcur)
		}	
		else
		{
			lcur<-append(lcur,lnew)
			return(lcur)
		}
			
		
	}
	
}
# This method searches for data accumulations by shifting a window with
# window size <w> across the data and deciding at each position if there 
# is a data accumulation. To test this, a scan statistic with significance
# level <sign.level> is used. 
scanStatistic <- function(vect, w=0.25, sign.level=0.1)
{	
	temp<-vect
	vect <-unlist(vect)
	vsort <- sort(vect)
	N <- length(vect)
	range <- (max(vect)) - (min(vect))
	windowsize <- range*w	
	N <- length(vect)
	binarizeddata<-temp
	res<-list()
	lcur<-list()
	
	# shift a fixed window over the data
	# the window is moved from point to point
	for(i in seq_along(vect))
	{	
		start <- vsort[i]
		stop <- vsort[i] + windowsize
		
		k <- length(vect[(vect >= start) & (vect <= stop)])
		
		p <- pval(k,N,w)
		
		if(p>1)
		{
			p=0.99	
		}
		
		if(p<=sign.level & p>0 & k >= (N*w-1) & k > 2)
		{
			res <- listadapt(res,list(c(start,stop,p)))
		}
		
	}
	
	
	# if no accumulation for a fixed <sign.level> was found, the 
	# binarization is rejected, and we search for a accumulation
	# with a higher sign.level.
	if(length(res)==0)
	{ 
		while(TRUE)
		{
			sign.level=sign.level+0.05
			
			if(sign.level>2)
			{
				binarizeddata<-(sapply(vect,function(x) 0))  
				return(list(bindata=binarizeddata,thresholds=NA,reject=TRUE))
    			
			}
		
			for(i in seq_along(vect))
			{
				start <- vsort[i]
				stop <- vsort[i] + windowsize
		
				k <- length(vect[(vect >= start) & (vect <= stop)])
		
				p <- pval(k,N,w)
				
				if(p>1)
				{
					p=0.99	
				}
				
				if(p<=sign.level & p>0 & k >= (N*w-1) & k > 2)
				{	
					#res <- append(res,list(c(start=start,stop=stop,pval=p)))
					res <- listadapt(res,list(c(start,stop,p)))
				}
		
			}
			if(length(res)!=0)
				break
				
				
			
		}
		reject<-TRUE	
		
	}
	else
	{
		reject<-FALSE
	}
	
	
	# search the window with the smallest sign.level.
	# this window is used for the binarization
	min=1000
	ind=0
	for(i in seq_along(res))
	{
		if(res[[i]][3]<min)
		{
			ind=i	
			min=res[[i]][3]
		}
	}
	
	# are more points on the left or on the right side
	# of the window? Based on this, the binarization is performed
	bigger <- length(vect[vect > res[[ind]][2]]) 
	smaller <- length(vect[vect < res[[ind]][1]])
	
	if(bigger > smaller)
	{
		threshold<-res[[ind]][2]
		
		small<-tail(vsort[vsort<=threshold],n=1)
		big<-vsort[vsort>threshold][1]
		
		thres<-(big+small)/2
		
		for(i in seq_along(vect))
		{
			if(vect[i]<=threshold)
			{
				binarizeddata[i]<-0
			}
			else
			{
				binarizeddata[i]<-1
			}
		}	
			
	}
	else
	{
		threshold<-res[[ind]][1]
		
		small<-tail(vsort[vsort<threshold],n=1)
		big<-vsort[vsort>=threshold][1]		
		
		thres<-(big+small)/2
		
		for(i in seq_along(vect))
		{
			if(vect[i]>=threshold)
			{
				binarizeddata[i]<-1
			}
			else
			{
				binarizeddata[i]<-0
			}
		}
		
	}
	
	return(list(bindata=binarizeddata,thresholds=as.numeric(thres),reject=reject))
	
}

# Custom print function for class AttractorInfo
print.AttractorInfo <- function(x, activeOnly = FALSE, ...)
{
  numGenes <- length(x$stateInfo$genes)
  attractors <- x$attractors
  
  for (i in seq_along(attractors))
  {
    if (is.null(attractors[[i]]$initialStates))
    # simple attractor
    {
      printSynchronousAttractor(getAttractorSequence(x, i), i, 
                                attractors[[i]]$basinSize, activeOnly=activeOnly)
    }
    else
    {
         # print general information on the attractor
      cat("Attractor ",i," is a complex/loose attractor consisting of ",ncol(attractors[[i]]$involvedStates),
        " state(s) and ",ncol(attractors[[i]]$initialStates), " transition(s)",sep="")
      
      if (activeOnly)
      {
        cat(".\nActive genes in the state transitions: \n")
        initialStates <- t(apply(attractors[[i]]$initialStates,2,function(state)
                               dec2bin(state,numGenes)))
        nextStates <- t(apply(attractors[[i]]$nextStates,2,function(state)
                               dec2bin(state,numGenes)))
                               
        binMatrix <- data.frame(initialStates,nextStates)                                 

        apply(binMatrix,1,function(row)
        {
          state1 <- paste(x$stateInfo$genes[which(row[seq_len(numGenes)] == 1)],collapse=", ")
          
          if (state1 == "")
            state1 <- "--"
          
          state2 <- paste(x$stateInfo$genes[which(row[seq_len(numGenes) + numGenes] == 1)],collapse=", ")
          
          if (state2 == "")
            state2 <- "--"
          
          cat(state1," => ",state2,"\n",sep="")
        })

      }
      else
      { 
        cat(":\n\n")
        initialStates <- apply(attractors[[i]]$initialStates,2,function(state)
                               paste(dec2bin(state,numGenes),collapse=""))
        nextStates <- apply(attractors[[i]]$nextStates,2,function(state)
                               paste(dec2bin(state,numGenes),collapse=""))

        binMatrix <- data.frame(initialStates,nextStates)

        apply(binMatrix,1,function(row)
        {         
            cat(row[1]," => ",row[2],"\n",sep="")
        })
        cat("\nGenes are encoded in the following order: ",paste(x$stateInfo$genes,collapse=" "),"\n\n",sep="")
      }      
    }
  }  
  return(invisible(x))
}

# Custom print function for class BooleanNetwork
print.BooleanNetwork <- function(x, ...)
{
  cat("Boolean network with",length(x$genes),"genes\n\n")
  cat("Involved genes:\n",paste(x$genes,collapse=" "),"\n\n",sep="")
  cat("Transition functions:\n")
  
  mapply(function(gene,interaction)
    {
      # print original expressions read from the files (if available)
      cat(gene," = ",interaction$expression,"\n",sep="")
    },  
    x$genes,x$interactions)
  
  if (sum(x$fixed != -1) > 0)
  {
    cat("\nKnocked-out and over-expressed genes:\n")
    mapply(function(gene,fixed)
      {
        if (fixed != -1)
          cat(gene," = ",fixed,"\n",sep="")
      },
      x$genes,x$fixed)
  }
    
  return(invisible(x))  
}

print.SymbolicBooleanNetwork <- function(x,...)
{
  cat("Symbolic representation of a Boolean network\n\n");
  cat("Transition functions:\n")
  for (gene in x$genes)
  {
    cat(gene, " = ", stringFromParseTree(x$interactions[[gene]]), "\n", sep="")
  } 
  
  if (sum(x$fixed != -1) > 0)
  {
    cat("\nKnocked-out and over-expressed genes:\n")
    mapply(function(gene,fixed)
      {
        if (fixed != -1)
          cat(gene," = ",fixed,"\n",sep="")
      },
      x$genes,x$fixed)
  }
    
}

plotSequence <- function (network, startState, includeAttractorStates = c("all", "first", "none"),
                          sequence, title = "", mode = c("table", "graph"),
                          plotFixed = TRUE, grouping = list(),
                          onColor = "#4daf4a", offColor = "#e41a1c",
                          layout, drawLabels = TRUE, drawLegend = TRUE,
                          highlightAttractor = TRUE, reverse = FALSE,
                          ### new parameters
                          borderColor = "black", eps=0.1,
                          attractor.sep.lwd = 2, attractor.sep.col = "blue", ...)
{
  if (!missing(network)) {
    stopifnot(inherits(network, "BooleanNetwork") || inherits(network,
                                                              "SymbolicBooleanNetwork"))
    if (missing(startState) || !missing(sequence))
      stop("Either \"network\" and \"startState\" or \"sequence\" must be provided!")
    sequence <- getPathToAttractor(network = network, state = startState, includeAttractorStates = includeAttractorStates)
    numGenes <- length(network$genes)
    if (match.arg(mode, c("table", "graph")) == "table") {
      whichFixed <- which(network$fixed != -1)
      if (plotFixed | (length(whichFixed) == 0))
        plotIndices <- seq_len(numGenes)
      else plotIndices <- seq_len(numGenes)[-whichFixed]
      attractor <- attributes(sequence)$attractor
      sequence <- sequence[, plotIndices, drop = FALSE]
      attributes(sequence)$attractor <- attractor
    }
  }
  else {
    if (missing(sequence) || !missing(startState))
      stop("Either \"network\" and \"startState\" or \"sequence\" must be provided!")
  }
  switch(match.arg(mode, c("table", "graph")), table = {
    totalMatrix <- t(sequence)
    if (length(grouping) > 0) 
      totalMatrix = totalMatrix[unlist(grouping$index), , drop = FALSE]
    if (is.null(colnames(totalMatrix))) {
      colnames(totalMatrix) <- seq_len(ncol(totalMatrix)) 
    }
    else if (length(grep("t = ", colnames(totalMatrix)) == ncol(totalMatrix))) {
      colnames(totalMatrix) <- sapply(colnames(totalMatrix),
                                      gsub, 
                                      pattern = "t = ", replacement = "", fixed = TRUE)
    }
    if (!reverse) 
      totalMatrix <- totalMatrix[nrow(totalMatrix):1, , drop = F]
    plot(0, type="n", 
         xlim = c(0, ncol(totalMatrix)), ylim = c(-2, nrow(totalMatrix)+1), 
         xlab = "", ylab = "", axes = FALSE, main = title, ...)
    
    len <- ncol(totalMatrix)
    xStart <- 1
    unitFactor <- (len - (2 * eps))/len
    
    axis(3, eps, "t = ", 
         lty = "blank", yaxt = "s", xaxt = "s", xaxs = "i", pos = nrow(totalMatrix)+1)
    axis(3, eps + (1:ncol(totalMatrix))*unitFactor - unitFactor/2, ((1:ncol(totalMatrix)) - 1),
         lty = "blank", yaxt = "s", xaxt = "s", xaxs = "i", pos = nrow(totalMatrix)+1)
    axis(2, seq_len(nrow(totalMatrix)) - 0.5, rownames(totalMatrix),     yaxt = "s", xaxt = "s", xaxs = "i", las = 2)
    
    startX <- eps
    for (i in seq_len(ncol(totalMatrix))) {

      for (j in seq_len(nrow(totalMatrix))) {
        rectCol <- ifelse(totalMatrix[j, i], onColor, offColor)
        rect(startX, j - 1, startX + unitFactor, j, col = rectCol, border = borderColor, lwd = 2)
        
      }
      startX <- startX + unitFactor
      if (i %% len == 0) {
        startX <- startX + 2*eps
      }
    }

    if (length(grouping) > 0) {
      if(!is.null(grouping$class)) {
        sepPos = cumsum(sapply(grouping$index, length))
        abline(h = sepPos[-length(sepPos)], col = "black", lwd = 3)
        text(ncol(totalMatrix)/2, sepPos - 0.5, grouping$class, cex = 0.9)
      }
    }
    if (!is.null(attributes(sequence)$attractor) && highlightAttractor) {
      attrStart <- min(attributes(sequence)$attractor) - 1
      #lines(x = c(attrStart, attrStart), y = c(-1, nrow(totalMatrix)) + 0.5)
      #abline(v =  eps+attrStart*unitFactor, lwd = attractor.sep.lwd, col = attractor.sep.col)
      rect(xleft = eps+attrStart*unitFactor, xright = eps+attrStart*unitFactor,
           ybottom = -0.5, ytop = nrow(totalMatrix)+0.75, lwd = attractor.sep.lwd, border = attractor.sep.col)
      arrows(x0 =  eps+attrStart*unitFactor, y0 = nrow(totalMatrix)+0.5,
             x1 = ncol(totalMatrix) - eps, y1 = nrow(totalMatrix)+0.5,
             length = 0.1, angle = 20, code = 3, col = attractor.sep.col)
      xpd.prev <- par()$xpd
      par(xpd=T)
      text(eps+attrStart*unitFactor, nrow(totalMatrix)+1, "Attractor", pos = 4)
      par(xpd=xpd.prev)
    }
    #if (drawLegend) legend(x = "bottomright", pch = c(15, 15), col = c(onColor, offColor), legend = c("active", "inactive"), cex = 0.7, horiz = T)
    if (drawLegend) legend(x = 0, y = -2, pch = c(15, 15), col = c(onColor, offColor), legend = c("active", "inactive"), cex = 0.7, horiz = T, xpd=T)
    #return(totalMatrix[network$genes,])
    return(totalMatrix)
  }, graph = {
    if (installed.packages()["igraph", "Version"] < package_version("0.6")) bias <- 1 else bias <- 0
    args <- list(...)
    if (is.null(args$vertex.size)) args$vertex.size <- 2
    if (is.null(args$edge.arrow.mode)) args$edge.arrow.mode <- 0
    if (is.null(args$rescale)) args$rescale <- !missing(layout)
    if (is.null(args$vertex.label.cex)) args$vertex.label.cex <- 0.75
    if (is.null(args$vertex.label.dist)) args$vertex.label.dist <- 0.25
    if (is.null(args$vertex.label.degree)) args$vertex.label.degree <- -pi/2
    if (is.null(args$vertex.color)) args$vertex.color <- "grey"
    if (is.null(args$edge.arrow.size)) args$edge.arrow.size <- 0.5
    if (missing(layout)) layout <- matrix(c(seq(-1, 1, length.out = nrow(sequence)),
                                            rep(0, nrow(sequence))), ncol = 2)
    states <- apply(sequence, 1, paste, collapse = "")
    nodes <- data.frame(seq_along(states), stringsAsFactors = FALSE)
    lastAttractorEdgeIndex <- NULL
    if (length(states) > 1) {
      initialStates <- 1:(length(states) - 1)
      nextStates <- 2:length(states)
      edgeMatrix <- data.frame(initialStates, nextStates)
      if (!is.null(attributes(sequence)$attractor)) {
        attractorEdge <- c(max(attributes(sequence)$attractor),
                           min(attributes(sequence)$attractor))
        if (all(attractorEdge <= length(states))) {
          edgeMatrix <- rbind(edgeMatrix, attractorEdge)
          lastAttractorEdgeIndex <- nrow(edgeMatrix)
        }
      }
    } else {
      edgeMatrix <- data.frame(matrix(nrow = 0, ncol = 2))
    }
    graph <- graph.data.frame(edgeMatrix, vertices = nodes,
                              directed = TRUE)
    if (drawLabels) labels <- states else labels <- NA
    if (highlightAttractor && !is.null(attributes(sequence)$attractor)) {
      attractorEdgeIndices <- intersect(seq_len(nrow(edgeMatrix)),
                                        c(attributes(sequence)$attractor, nrow(edgeMatrix))) -
        bias
      graph <- set.edge.attribute(graph, "width", index = attractorEdgeIndices,
                                  value = 3)
    }
    if (!is.null(lastAttractorEdgeIndex)) {
      graph <- set.edge.attribute(graph, "curved", value = 0)
      graph <- set.edge.attribute(graph, "curved", 
                                  index = lastAttractorEdgeIndex - bias, value = 0.5)
    }
    plot(graph, layout = layout, vertex.label = labels,
         vertex.label.cex = args$vertex.label.cex, vertex.size = args$vertex.size,
         vertex.color = args$vertex.color, vertex.label.dist = args$vertex.label.dist,
         vertex.label.degree = args$vertex.label.degree,
         edge.arrow.size = args$edge.arrow.size, rescale = args$rescale,
         main = title, ...)
    return(graph)
  })
}

# Generate time series from a Boolean network <network>.
# <numSeries> is the number of start states for which time series are generated.
# <numMeasurements> is the number of time points for each time series.
# <type> is the type of transitions used (synchronous or asynchronous)
# If <type> is "asynchronous", <geneProbabilities> describes the 
# transition probabilities for the genes.
# If <noiseLevel> is not 0, Gaussian noise is added to the result.
generateTimeSeries <- function(network, numSeries, numMeasurements, 
                            type = c("synchronous","asynchronous","probabilistic"),
                            geneProbabilities, 
                            perturbations=0,
                            noiseLevel = 0.0)
{
  symbolic <- inherits(network, "SymbolicBooleanNetwork")

  if (missing(geneProbabilities))
    geneProbabilities <- NULL
  
  if (perturbations > 0)
  {
    perturbationMatrix <- sapply(1:numSeries, function(x)
                                 {
                                   p <- rep(NA, length(network$genes))
                                   p[sample(1:length(network$genes),size=perturbations)] <- 
                                      sample(c(0,1),size=perturbations,replace=TRUE)
                                   
                                   return(p)
                                 })
    rownames(perturbationMatrix) <- network$genes                                 
  }
    
  ts <- lapply(seq_len(numSeries), function(i)
  {
    if (symbolic)
    {
      if (perturbations > 0)
      {
        fixedIdx <- which(perturbationMatrix[,i] != -1)
        network <- fixGenes(network, fixedIdx, perturbationMatrix[fixedIdx,i])
      }
      
      res <- t(simulateSymbolicModel(network, startStates=1, 
                                     returnAttractors=FALSE, 
                                     returnGraph=TRUE, 
                                     returnSequences=TRUE)$sequences[[1]])
    }
    else
    {
      startState <- round(runif(length(network$genes)))
      
      if (perturbations > 0)
      {
        fixedIdx <- which(perturbationMatrix[,i] != -1)
        network <- fixGenes(network, fixedIdx, perturbationMatrix[fixedIdx,i])
        startState[fixedIdx] <- perturbationMatrix[fixedIdx,i]
      }
      
      res <- startState
      for (j in 2:numMeasurements)
      {
        startState <- stateTransition(network, startState, 
                                      type=type, geneProbabilities=geneProbabilities)
        res <- cbind(res,startState)
      }
    }    
    colnames(res) <- NULL

    if (noiseLevel != 0)
    {
      res <- res + matrix(rnorm(mean=0, sd=noiseLevel, n = length(res)), nrow=nrow(res))
    }
    
    rownames(res) <- network$genes
    colnames(res) <- seq_len(ncol(res))
    return(res)
  })
  
  if (perturbations > 0)
    ts$perturbations <- perturbationMatrix
  
  return(ts)
}

# Plot state tables of all attractors in <attractorInfo>.
# Genes are grouped according to <grouping>.
# An additional title can be supplied in <title>.
# If <plotFixed> is set, fixed variables are included in the plot.
# <onColor> and <offColor> specify the colors of ON/1 and OFF/0 states.
# TODO: remove old definition from comments
#plotAttractors <- function (attractorInfo, subset, title = "", mode=c("table","graph"),
#                            grouping = list(), plotFixed = TRUE, onColor="green",offColor="red",
#                            layout=layout.circle, drawLabels=TRUE, drawLegend=TRUE, ask=TRUE, 
#                            reverse=FALSE, ...) 
#{}
plotAttractors <- function (attractorInfo, subset, 
                            title = "", 
                            mode = c("table", "graph"),
                            grouping = list(), 
                            plotFixed = TRUE,
                            onColor = "#4daf4a", offColor = "#e41a1c",
                            layout = layout.circle,
                            drawLabels = TRUE, drawLegend = TRUE, ask = TRUE, reverse = FALSE, 
                            # new parameters
                            borderColor = "black",
                            eps = 0.1, allInOnePlot = FALSE, ...)
{
  stopifnot(inherits(attractorInfo, "AttractorInfo") || inherits(attractorInfo,
                                                                 "SymbolicSimulation"))
  if (inherits(attractorInfo, "AttractorInfo")) {
    numGenes <- length(attractorInfo$stateInfo$genes)
    geneNames <- attractorInfo$stateInfo$genes
  }
  else {
    numGenes <- ncol(attractorInfo$attractors[[1]])
    geneNames <- colnames(attractorInfo$attractors[[1]])
  }
  if (missing(subset))
    subset <- seq_along(attractorInfo$attractors)
  else if (any(subset > length(attractorInfo$attractors)))
    stop("You specified an attractor index that is greater than the total number of attractors in 'subset'!")
  res <- switch(match.arg(mode, c("table", "graph")), table = {
    whichFixed <- which(attractorInfo$stateInfo$fixedGenes != -1)
    if (plotFixed | (length(whichFixed) == 0)) {
      plotIndices <- seq_len(numGenes)
    } else {
      plotIndices <- seq_len(numGenes)[-whichFixed]
    }
    if (inherits(attractorInfo, "AttractorInfo")) {
      binMatrices <- lapply(attractorInfo$attractors, function(attractor) {
        res <- matrix(apply(attractor$involvedStates, 2, function(state) dec2bin(state, numGenes)[plotIndices]), nrow = length(plotIndices))
      })
      attractorLengths <- sapply(attractorInfo$attractors, function(attractor) {
        ifelse (is.null(attractor$initialStates), ncol(attractor$involvedStates), -1)
      })
    } else {
      binMatrices <- lapply(attractorInfo$attractors, t)
      attractorLengths <- sapply(binMatrices, ncol)
    }
    lengthTable <- table(attractorLengths)
    lengthTable <- lengthTable[as.integer(names(lengthTable)) != -1]
    oldAsk <- par("ask")
    oldmfrow <- NULL
    if (allInOnePlot) {
      oldmfrow <- par()$mfrow
      #par(mfrow=c(1,length(table(sapply(sapply(attractorInfo$attractors, "[[", "involvedStates", simplify=F), ncol)))))
      par(mfrow=c(1,length(lengthTable)))
    }
    res <- lapply(seq_along(lengthTable), function(i) {
      len <- as.integer(names(lengthTable)[i])
      attractorIndices <- intersect(which(attractorLengths == len), subset)
      if (length(attractorIndices) > 0) {
        cnt <- length(attractorIndices)
        totalMatrix <- c()
        for (mat in binMatrices[attractorIndices]) {
          totalMatrix <- cbind(totalMatrix, mat)
        }
        rownames(totalMatrix) <- geneNames[plotIndices]
        colnames(totalMatrix) <- sapply(attractorIndices, function(i) paste("Attr", i, ".", seq_len(len), sep = ""))
        if (length(grouping) > 0) totalMatrix <- totalMatrix[unlist(grouping$index), , drop = FALSE]
        par(ask = ask && i > 1 && dev.interactive())
        if (reverse) {
          plot(c(), c(),
               xlim = c(0, len *cnt), ylim = c(-2, nrow(totalMatrix) + 1),
               xlab = "", ylab = "", axes = FALSE,
               main = paste(title, "\nAttractors with ", len, " state(s)", sep = ""))
        } else {
          plot(c(), c(),
               xlim = c(0, len * cnt), ylim = c(nrow(totalMatrix) + 1, -2),
               xlab = "", ylab = "", axes = FALSE,
               main = paste(title, "\nAttractors with ", len," state(s)", sep = ""))
        }
        #rect(par("usr")[1], par("usr")[3], par("usr")[2], par("usr")[4], col = "grey")
        axis(2, seq_len(nrow(totalMatrix)) - 0.5, rownames(totalMatrix), yaxt = "s", las = 2)
        xStart <- 1
        unitFactor <- (len - (2 * eps))/len
        startX <- eps
        for (i in seq_len(ncol(totalMatrix))) {
          for (j in seq_len(nrow(totalMatrix))) {
            rectCol <- ifelse(totalMatrix[j, i], onColor, offColor)
            rect(startX, j - 1, startX + unitFactor, j, col = rectCol, border = borderColor, lwd = 2)
          }
          startX <- startX + unitFactor
          if (i %% len == 0) {
            startX <- startX + 2*eps
          }
        }
        sep = seq(0, ncol(totalMatrix), by = len)
        #abline(v = sep[-1], col = par()$bg, lwd = 3)
        if (inherits(attractorInfo, "AttractorInfo")) {
          if (is.null(attractorInfo$stateInfo$table)) {
            freq <- rep(NA, length(attractorIndices))
          }
          else {
            freq <- round(100 * sapply(attractorInfo$attractors[attractorIndices], function(attractor) {
              attractor$basinSize/ncol(attractorInfo$stateInfo$table)}), 2)
          }
        } else {
          if (!is.null(attractorInfo$graph)) {
            freq <- round(sapply(attractorIndices, function(i) sum(attractorInfo$graph$attractorAssignment == i)/nrow(attractorInfo$graph)) * 100, 2)
          } else {
            freq <- rep(NA, length(attractorIndices))
          }
        }
        if (!isTRUE(all(is.na(freq)))) {
          text(sep[seq_along(sep) - 1] + len/2, ifelse(reverse, -nrow(totalMatrix)-1.5, 1) + rep(1 + nrow(totalMatrix), ncol(totalMatrix)), paste(freq, "%", sep = ""), cex = 0.75, font = 3)
        }
        if (length(grouping) > 0) {
          if(!is.null(grouping$class)) {
            sepPos = cumsum(sapply(grouping$index, length))
            abline(h = sepPos[-length(sepPos)], col = "black", lwd = 3)
            text(ncol(totalMatrix)/2, sepPos - 0.5, grouping$class, cex = 0.9)
          }
        }
        #if (drawLegend) legend(x = "bottomright", pch = c(15, 15), col = c(onColor, offColor), legend = c("active", "inactive"), cex = 0.7, horiz = T, xpd = T)
        if (drawLegend) legend(x = 0, y = ifelse(reverse, -nrow(totalMatrix)-1, 2) + nrow(totalMatrix), pch = c(15, 15), col = c(onColor, offColor), legend = c("active", "inactive"), cex = 0.7, horiz = T, xpd=T)
        totalMatrix
      }
    })
    par(ask = oldAsk)
    if(!is.null(oldmfrow)) par(mfrow = oldmfrow)
    names(res) <- names(lengthTable)
    return(res)
    
  }, graph = {
    args <- list(...)
    if (is.null(args$vertex.size)) args$vertex.size <- 2
    if (is.null(args$edge.arrow.mode)) args$edge.arrow.mode <- 0
    if (is.null(args$vertex.label.cex)) args$vertex.label.cex <- 0.75
    if (is.null(args$vertex.label.dist)) args$vertex.label.dist <- 1
    if (is.null(args$vertex.color)) args$vertex.color <- "grey"
    if (is.null(args$edge.arrow.size)) args$edge.arrow.size <- 0.5
    lapply(attractorInfo$attractors[subset], function(attractor) {
      if (inherits(attractorInfo, "AttractorInfo")) {
        nodes <- data.frame(apply(attractor$involvedStates, 2, function(state) paste(dec2bin(state, numGenes), collapse = "")), stringsAsFactors = FALSE)
        if (!is.null(attractor$initialStates)) {
          initialStates <- apply(attractor$initialStates, 2, function(state) paste(dec2bin(state, numGenes), collapse = ""))
          nextStates <- apply(attractor$nextStates, 2, function(state) paste(dec2bin(state, numGenes), collapse = ""))
        } else {
          initialStates <- apply(attractor$involvedStates, 2, function(state) paste(dec2bin(state, numGenes), collapse = ""))
          if (length(initialStates) == 1) nextStates <- initialStates else nextStates <- initialStates[c(2:length(initialStates), 1)]
        }
      } else {
        initialStates <- apply(attractor, 1, paste, collapse = "")
        nextStates <- apply(attractor[c(2:nrow(attractor), 1), , drop = FALSE], 1, paste, collapse = "")
        nodes <- data.frame(unique(c(initialStates, nextStates)), stringsAsFactors = FALSE)
      }
      edgeMatrix <- data.frame(initialStates, nextStates)
      graph <- graph.data.frame(edgeMatrix, vertices = nodes, directed = TRUE)
      if (drawLabels) labels <- nodes[, 1] else labels <- NA
      args$layout <- layout
      args$x <- graph
      args$vertex.label <- labels
      args$main <- title
      do.call("plot", args)
      graph
    })
  })
}

# Identify attractors in a Boolean network.
# <network> is a BooleanNetwork/SymbolicBooleanNetwork structure specifying the network.
# <method> specifies what kind of search is conducted: "exhaustive" performs a full search over all states,
# "random" generates <startStates> random initial states, and "chosen" uses the states supplied in <startStates>. 
# "sat.exhaustive" and "sat.restricted" start a SAT-based attractor search.
# <genesON> and <genesOFF> are lists of genes to be set to ON/1 or OFF/0 respectively.
# If <canonical> is true, states in the attractors are reordered such that the "smallest" state is the first
# <randomChainLength> is the number of random transitions performed for the identification of an asynchronous attractor
# If <avoidSelfLoops> is true, loops to the same state are eliminated from asynchronous attractors.
# <geneProbabilities> optionally specifies the probabilities of choosing a gene for an asynchronous update.
# <maxAttractorLength> specifies the maximum attractor length for method="sat.restricted" and the initial search length for method="sat.exhaustive".
# if <returnTable> is true, the transition table is included in the result.
getAttractors <- function (network, type=c("synchronous","asynchronous"), 
         method=c("exhaustive","sat.exhaustive","sat.restricted","random","chosen"), startStates=list(),
         genesON = c(), genesOFF = c(), canonical=TRUE,
         randomChainLength = 10000, avoidSelfLoops = TRUE, 
         geneProbabilities = NULL, 
		 maxAttractorLength=Inf, 
         returnTable=TRUE) 
{
  stopifnot(inherits(network,"BooleanNetwork") || inherits(network,"SymbolicBooleanNetwork"))
  
  symbolic <- inherits(network,"SymbolicBooleanNetwork")
  
  nonFixedPositions <- which(network$fixed == -1)
  
  type <- match.arg(type, c("synchronous","asynchronous"))
  
  if (type == "asynchronous")
  {
    if (symbolic)
      stop("Only synchronous updates are allowed for symbolic networks!")
    
    if (length(method) == 1 & match.arg(method) == "exhaustive")
      stop("Asynchronous attractor search cannot be performed in exhaustive search mode!")
    
    if (length(geneProbabilities) > 0 )
    {
      if (length(geneProbabilities) != length(network$genes))
        stop("Please supply exactly one probability for each gene!")
      if (abs(1.0 - sum(geneProbabilities)) > 0.0001)
        stop("The supplied gene probabilities do not sum up to 1!")
    }
    
  }
  
  if (!is.null(maxAttractorLength) && is.infinite(maxAttractorLength))
    maxAttractorLength <- NULL
  
  if (length(method) > 1)
  # if no method is supplied, infer method from the type of <startStates>
  {
    if (type == "asynchronous" & length(startStates) == 0)
    {
      startStates <- max(round(2 ^ sum(network$fixed == -1) / 20), 5)
      method = "random"
    }
    else
    if (is.numeric(startStates))
    {
      if (length(startStates) > 1)
        stop("Please supply either the number of start states or a list of start states in startStates!")
      else
        method <- "random"
    }
    else
    if (is.list(startStates) & (length(startStates) > 0))
      method <- "chosen"
    else
    if (!is.null(maxAttractorLength))
      method <- "sat.restricted"
    else
      method <- "exhaustive"
  }
 
  # fix genes according to genesON and genesOFF
  if (length(genesON) > 0) 
  {
    network <- fixGenes(network,genesON,1)
  }
  if (length(genesOFF) > 0) 
  {
    network <- fixGenes(network,genesOFF,0)
  }
    
  if (symbolic)
  {
    return(simulateSymbolicModel(network, 
                                 method=method, 
                                 startStates=startStates,
                                 maxAttractorLength=maxAttractorLength,
                                 returnGraph=returnTable && !(match.arg(method) %in% c("sat.exhaustive", "sat.restricted")),
                                 returnSequences=FALSE,
                                 returnAttractors=TRUE,
                                 canonical=canonical))
  }
  else
  {
    method <- match.arg(method,c("exhaustive","sat.exhaustive","sat.restricted","random","chosen"))

	if (method == "sat.restricted" && is.null(maxAttractorLength))
		stop("maxAttractorLength must be set for method=\"sat.restricted\"!")
	
    if ((length(network$genes) > 29) && (method == "exhaustive") && (type == "synchronous"))
    {
      method <- "sat.exhaustive"
      warning("An exhaustive state space search is restricted to networks with at most 29 genes. Switching to the SAT-based exhaustive search, which supports more genes, but does not return a transition table!")
    }
    else
    if (method %in% c("sat.exhaustive", "sat.restricted") && type != "synchronous")
      stop("SAT-based search can only be used for synchronous networks!")
    
    startStates <- switch(method,
      exhaustive = list(),
      sat.exhaustive = list(),
	  sat.restricted = list(),
      random = {
          if (!is.numeric(startStates))
            stop("Please supply the number of random states in startStates!")

          if (startStates > (2 ^ length(nonFixedPositions)))
          # more start states than in the full network
          {
            if (type == "synchronous")
            {
              list()
              warning(paste("The number of random states is set larger than the total",
                            "number of states. Performing an exhaustive search!"))
            }
            else
            {
              warning(paste("The number of random states is set larger than the total ",
                          "number of states! The maximum number of different states is ",2 ^ length(nonFixedPositions)),"!",sep="")
              startStates = 2 ^ length(nonFixedPositions)            
            }     
          }
          # generate random matrix
          generateRandomStartStates(network, startStates)

         },
      chosen = {
          if (!is.list(startStates) | length(startStates) == 0)
            stop("No start states supplied!")
          if (!all(sapply(startStates,function(x)(length(x) == length(network$genes)))))
            stop(paste("Please provide binary vectors with",length(network$genes),
              "elements in startStates!"))
    
            fixedGenes <- which(network$fixed != -1)
            statesValid <- sapply(startStates,function(state)
                                  {
                                    isTRUE(all(state[fixedGenes] == network$fixed[fixedGenes]))
                                  })
        startStates <- startStates[statesValid]
            if (!any(statesValid))
              stop("None of the supplied start states matched the restrictions of the fixed genes!")
            if (!all(statesValid))
              warning("Some of the supplied start states did not match the restrictions of the fixed genes and were removed!")    
              
          startStates
         }
    )
    
    if (!is.null(maxAttractorLength))
    {
      if (!(method %in% c("sat.exhaustive", "sat.restricted")))
        stop("maxAttractorLength can only be used with method=\"sat.exhaustive\" or method=\"sat-res.ricted\"!")
      maxAttractorLength <- as.integer(maxAttractorLength)
    }
    specialInitialization <- NULL
    
    convertedStartStates <- NULL

    if (length(startStates) > 0)
      convertedStartStates <- sapply(startStates,function(x)bin2dec(x,length(network$genes)))

    # the C code requires all interactions to be coded into one vector:
    # Assemble all input gene lists in one list <inputGenes>, and remember the split positions in <inputGenePositions>.
    inputGenes <- as.integer(unlist(lapply(network$interactions,function(interaction)interaction$input)))
    inputGenePositions <- as.integer(cumsum(c(0,sapply(network$interactions,
             function(interaction)length(interaction$input)))))

    # Do the same for the transition functions.
    transitionFunctions <- as.integer(unlist(lapply(network$interactions,function(interaction)interaction$func)))
    transitionFunctionPositions <- as.integer(cumsum(c(0,sapply(network$interactions,
                function(interaction)length(interaction$func)))))

   searchType <- switch(type,
      synchronous = if (method == "sat.exhaustive") 2 else if (method == "sat.restricted") 3 else 0,
      asynchronous = 1)
    
    on.exit(.C("freeAllMemory", PACKAGE = "BoolNet"))
    # Call the C code
    result <- .Call("getAttractors_R",inputGenes,inputGenePositions,
          transitionFunctions,transitionFunctionPositions,
          as.integer(network$fixed),
          as.integer(convertedStartStates),
          as.integer(searchType),
          as.double(geneProbabilities),
          as.integer(randomChainLength),
          as.integer(avoidSelfLoops),
          as.integer(returnTable),
          maxAttractorLength,
          PACKAGE="BoolNet")
    
    if (is.null(result))
      stop("An error occurred in external C code!")
    
    if (length(result$attractors) == 0)
      stop("getAttractors() was not able to identify any attractors! Please check the supplied parameters and restart!")
    
    if (length(network$genes) %% 32 == 0)
      numElementsPerEntry <- as.integer(length(network$genes) / 32)
    else
      numElementsPerEntry <- as.integer(length(network$genes) / 32  + 1)
    
    if (!is.null(result$stateInfo))
    {
      result$stateInfo$table <- matrix(result$stateInfo$table,nrow=numElementsPerEntry)
    
      if (!is.null(result$stateInfo$initialStates))
        result$stateInfo$initialStates <- matrix(result$stateInfo$initialStates,nrow=numElementsPerEntry)
    }
    
    for (i in seq_len(length(result$attractors)))
    {
      result$attractors[[i]]$involvedStates <- matrix(result$attractors[[i]]$involvedStates,nrow=numElementsPerEntry)
      if (canonical)
      # reorder states
        result$attractors[[i]]$involvedStates <- canonicalStateOrder(result$attractors[[i]]$involvedStates)
        
      if (!is.null(result$attractors[[i]]$initialStates))
        result$attractors[[i]]$initialStates <- matrix(result$attractors[[i]]$initialStates,nrow=numElementsPerEntry)
      if (!is.null(result$attractors[[i]]$nextStates))
        result$attractors[[i]]$nextStates <- matrix(result$attractors[[i]]$nextStates,nrow=numElementsPerEntry)
        
      if (result$attractors[[i]]$basinSize == 0)
        result$attractors[[i]]$basinSize <- NA
    }
    
    # order attractors according to their lengths
    attractorLengths <- sapply(result$attractors,function(attractor)ncol(attractor$involvedStates))  
    reordering <- order(attractorLengths)
    result$attractors <- result$attractors[reordering]
    
    if (!is.null(result$stateInfo))
    {
      inverseOrder <- sapply(seq_along(reordering),function(x)which(reordering == x))    
      result$stateInfo$attractorAssignment <- inverseOrder[result$stateInfo$attractorAssignment]
    }
    
    # extend the resulting structure by additional information, and assign a class      
    result$stateInfo$genes <- network$genes
    result$stateInfo$fixedGenes <- network$fixed

    if (!is.null(result$stateInfo$table))
      class(result$stateInfo) <- "BooleanStateInfo"
    class(result) <- "AttractorInfo"
    return(result)
  }
}

\documentclass[a4paper]{article}

\SweaveOpts{keep.source=TRUE}
%\VignetteIndexEntry{Detailed introduction to all major features of BoolNet}

\usepackage{a4wide}

\usepackage{graphicx}
\usepackage{amsmath}
\usepackage{hyperref}

\usepackage{listings}
\lstset{breaklines=true, breakautoindent=false, breakindent=0pt, columns=fullflexible,keepspaces=true, basicstyle=\small\ttfamily}

\setlength{\parindent}{0em}
\setlength{\parskip}{0.2em}

\title{BoolNet package vignette}
\author{Christoph M\"ussel, Martin Hopfensitz, Hans A. Kestler}

\widowpenalty=10000
\clubpenalty=10000

\hyphenation{me-thods pro-per-ties re-pre-sen-ta-tion}

\begin{document}
\SweaveOpts{concordance=TRUE}
\maketitle
\tableofcontents

\clearpage
\section{Introduction}

\texttt{BoolNet} is an R package that provides tools for assembling, analyzing and visualizing synchronous and asynchronous Boolean networks as well as probabilistic Boolean networks. This document gives an introduction to the usage of the software and includes examples for use cases.

\texttt{BoolNet} supports four types of networks:
\begin{description}
\item[Synchronous Boolean networks]{ consist of a set of Boolean variables 
\[
X = \left\{X_1, \ldots, X_n \right\} 
\]
and a set of transition functions 
\[
F=\left\{ f_{1},\ldots,f_{n}\right\},
\] 
one for each variable. These transition functions map an input of the Boolean variables in $X$ to a Boolean value ($0$ or $1$). We call a Boolean vector $\mathbf{x}(t) = \left(x_1(t), \ldots, x_n(t) \right)$ the {\em state} of the network at time $t$. Then, the next state of the network  $\mathbf{x}(t)$ is calculated by applying {\em all} transition functions $f_i(\mathbf{x}(t-1))$.

In a biological context, genes can be modeled as Boolean variables ({\em active/expressed} or {\em inactive/not expressed}), and the transition functions model the dependencies among these genes. In the synchronous model, the assumption is that all genes are updated at the same time. This simplification facilitates the analysis of the networks.}

\item[Asynchronous Boolean networks]{ have the same structure as synchronous Boolean networks. Yet, at each point of time $t$, only {\em one} of the transition functions $f_i \in F$ is chosen at random, and the corresponding Boolean variable is updated. This corresponds to the assumption that in a genetic network, gene expression levels are likely to change at different points of time. In the most common model, the gene to be updated is chosen uniformly among all genes. Moreover, \texttt{BoolNet} supports specifying non-uniform update probabilities for the genes.}

\item[Probabilistic Boolean networks (PBN)]{ allow for specifying more than one transition function per variable/gene. Each of these functions has a probability to be chosen, where the probabilities of all functions for one variable sum up to 1. Formally
\[
F=\left\{\left\{\left(f_{11}, p_{11}\right), \ldots, \left(f_{1k_1}, p_{1k_1}\right)\right\}, \ldots,  \left\{\left(f_{n1}, p_{n1}\right), \ldots, \left(f_{nk_n}, p_{nk_n}\right)\right\}\right\}
\]

where $k_i$ is the number of alternative transition functions for variable $i$, and $p_{ij}$ is the probability that function $j$ is chosen for variable $i$. A state transition is performed by selecting one function for each gene based on the probabilities and applying the chosen functions synchronously.}

\item[Temporal Boolean networks]{ are Boolean networks that incorporate temporal predicates and discrete time delays. Here, the next state $x(t)$ may not only depend on $x(t-1)$, but can depend on any predecessor state $x(t - \Delta), \Delta \in \{1, 2, \ldots\}$. Furthermore, $x(t)$ may also directly depend on the time step $t$ itself.
}

\end{description}

In \texttt{BoolNet}, there are different structure classes representing these network types:
\begin{description}
\item[\emph{BooleanNetwork} objects]{ contain synchronous and asynchronous Boolean networks. Here, the transition functions are internally represented as truth tables.}
\item[\emph{ProbabilisticBooleanNetwork} objects]{ encode Probabilistic Boolean networks. They use a truth table representation as well.}
\item[\emph{SymbolicBooleanNetwork} objects]{ represent synchronous and temporal Boolean networks. They encode Boolean functions in a symbolic form, i.e. as expression trees.}
\end{description}
As we have seen, the networks are represented in two different forms: The truth table representation, which basically maps inputs to the corresponding output values, is usually the most efficient representation for synchronous, asynchronous and probabilistic networks and uses a very fast simulator. However, this representation grows exponentially with the number of inputs and is therefore inappropriate for networks with a high number of inputs. This is particularly the case for temporal networks, where each unique time delay for a gene encodes an input. Hence, temporal networks are represented by directly encoding the corresponding Boolean expressions and use a different simulator. As synchronous Boolean networks are a special case of temporal networks (with all time delays being 1), these networks can also be represented as \emph{SymbolicBooleanNetwork} objects.

The package provides several methods of constructing networks:
Networks can be loaded from files in which human experts describe the dependencies between the genes. Furthermore, they can be reconstructed from time series of gene expression measurements. It is also possible to generate random networks. This can be helpful for the identification of distinct properties of biological networks by comparison to random structures. The different methods of assembling networks are described in Section~\ref{sec:assemblingnetworks}.

In Section~\ref{sec:networkanalysis}, tools for the analysis and visualization of network properties are introduced. For synchronous, asynchronous and temporal Boolean networks, the most important tool is the identification of attractors. Attractors are cycles of states and are assumed to be associated with the stable states of cell function. 
Another possibility of identifying relevant states is the included Markov chain simulation. This method is particularly suited for probabilistic networks and calculates the probability that a state is reached after a certain number of iterations. To test the robustness of structural properties of the networks to noise and mismeasurements, the software also includes extensive support for perturbing networks. In this way, it is possible to test these properties in noisy copies of a biological network.

In Section~\ref{sec:importexport}, the interaction of \texttt{BoolNet} with related software is described. In particular, the import from and export to SBML is discussed. Also, the necessary steps to import networks import networks from BioTapestry and to export networks to Pajek are outlined. 

For the examples in the following sections, we assume that the \texttt{BoolNet} package has been properly installed into the R environment. This can be done by typing
<<eval=FALSE>>=
install.packages("BoolNet")
@
into the R console or by the corresponding menu entries in an R GUI. For some of the plots, the \texttt{igraph} package is required and must be installed in your R environment as well. This is analogous to installing \texttt{BoolNet}. For the BioTapestry and SBML import, the \texttt{XML} package must be installed.
After installation, the \texttt{BoolNet} package can be loaded via
<<>>=
library(BoolNet)
@


\section{Assembling networks}\label{sec:assemblingnetworks}

\subsection{Assembling a network from expert knowledge}

A major advantage of Boolean networks is the fact that natural-language statements can easily be transferred into this representation. This allows researchers for building Boolean networks entirely from expert knowledge, for example by collecting statements on gene dependencies from literature and expressing them as Boolean rules. 

\texttt{BoolNet} is able to read in networks consisting of such rule sets in a standardized text file format. In such a file, each line consists of a target gene and an update rule, usually separated by a comma. Optionally, it is also possible to add a probability for the rule if the file describes a probabilistic network. The first line of such a file is a header

\begin{samepage}
\begin{verbatim}
targets, factors
\end{verbatim}

or

\begin{verbatim}
targets, factors, probabilities
\end{verbatim}
\end{samepage}

To illustrate the process of transforming natural-language statements into Boolean rules, we take a look at the mammalian cell cycle network introduced by Faur\'e et al. \cite{faure06}.  In Table 1 of this paper, the authors list natural-language statements of gene dependencies and the corresponding Boolean expressions. The following rules are taken from this table.

For gene CycD, Faur\'e et al. state:
\begin{quote}
\textit{CycD is an input, considered as constant.}
\end{quote}
Transforming this into a Boolean rule is rather simple: CycD does not change its value, which means that its value after a transition only depends on its previous value. Thus, the transition rule is

\begin{verbatim}
CycD, CycD
\end{verbatim} 
 
 Gene Rb has a more complex description:
 
 \begin{quote}
 \textit{Rb is expressed in the absence of the cyclins, which inhibit it by phosphorylation [...]; it can be expressed in the presence of CycE or CycA if their
inhibitory activity is blocked by p27.} 
 \end{quote}
 
 As a general rule, inhibition can be represented by a Boolean negation. In the \texttt{BoolNet} file format, a negation is expressed by the \texttt{!} character. The referenced cyclins comprise the genes CycA, CycB, CycD, and CycE. If {\em all} these genes are absent, Rb is expressed -- i.e. if CycA is not expressed {\em and} CycB is not expressed {\em and} CycD is not expressed {\em and} CycE is not expressed. A logical AND is embodied by the \texttt{\&} character. Consequently, the first part of the rule is
 \begin{verbatim}
 ! CycA & ! CycB & ! CycD & ! CycE
 \end{verbatim}
In combination with the above statement, the fact that Rb can be expressed in the presence of CycE and CycA if p27 is active means that CycB and CycD must not be active. Thus, the second part of the rule is
\begin{verbatim}
p27 & ! CycB & ! CycD 
\end{verbatim}
This statement is an exception (or alternative) to the first statement; this can be expressed as a logical OR, for which the \texttt{|} character is used.

The complete rule for gene Rb is thus
\begin{verbatim}
Rb, (! CycA & ! CycB & ! CycD & ! CycE) | (p27 & ! CycB & ! CycD)
\end{verbatim}

After processing all genes in the table in this way, we get the following network description:

\begin{samepage}
\begin{footnotesize}
\begin{verbatim}
targets, factors
CycD, CycD
Rb, (! CycA & ! CycB & ! CycD & ! CycE) | (p27 & ! CycB & ! CycD)
E2F, (! Rb & ! CycA & ! CycB) | (p27 & ! Rb & ! CycB)
CycE, (E2F & ! Rb)
CycA, (E2F & ! Rb & ! Cdc20 & ! (Cdh1 & UbcH10)) | (CycA & ! Rb & ! Cdc20 & ! (Cdh1 & UbcH10))
p27, (! CycD & ! CycE & ! CycA & ! CycB) | (p27 & ! (CycE & CycA) & ! CycB &! CycD)
Cdc20, CycB
Cdh1,(! CycA & ! CycB) | (Cdc20) | (p27 & ! CycB)
UbcH10, ! Cdh1 | (Cdh1 & UbcH10 & (Cdc20 | CycA | CycB))
CycB, ! Cdc20 & ! Cdh1
\end{verbatim} 
\end{footnotesize}
\end{samepage}

Now save this description to a file ``cellcycle.txt'' in your R working directory. The network can be loaded via
<<eval=FALSE>>=
cellcycle <- loadNetwork("cellcycle.txt")
@
The result is an object of class \emph{BooleanNetwork} containing a truth table representation of the network. 

The same network is also included in \texttt{BoolNet} as an example and can be accessed via
<<>>=
data(cellcycle)
@

\texttt{BoolNet} also provides several convenience operators that can be used to express complex Boolean functions in a compact way, e.g.
\begin{itemize}
\item \texttt{maj(a, b, ...)} is 1 if the majority of its operands are 1. Similarly, \texttt{sumgt(a, b, ..., N)} is 1 if more than $N$ operands are 1, \texttt{sumlt(a, b, ..., N)} is 1 if less than $N$ operands are~1, and \texttt{sumis(a, b, ..., N)}  is 1 if exactly $N$ operands are 1.
\item \texttt{all(a, b, ...)} is 1 if all its operands are 1 (i.e. a logical AND), and \texttt{any(a, b, ...)} is~1 if at least one of its operands is 1 (i.e. a logical OR).
\end{itemize}

The cell cycle network is a classical Boolean network, where each transition function only depends on the previous state of the network. E.g., \verb+CycB, ! Cdc20 & ! Cdh1+ can be written formally as $CycB(t) = \neg Cdc20(t-1)  \wedge \neg Cdh1(t-1)$. As already discussed before, \texttt{BoolNet} also incorporates several temporal extensions. For example, a transition function can also depend on the states of genes at earlier time points:
\begin{verbatim}
a, b[-3] & b[-2] & b
\end{verbatim}
is 1 if b has been active in the previous three time steps. The operators described above can also incorporate time ranges. The previous statement can be written in a more compact way using the \texttt{all} operator:
\begin{verbatim}
a, all[d=-3..-1](b[d])
\end{verbatim}
This defines a time delay variable \texttt{d} that can be used for time specifications inside the operator. It can also be used in arithmetic operations. E.g.,
\begin{verbatim}
a, all[d=-3..-1](b[d] & c[d-1])
\end{verbatim}
specifies that a is active if b has been active in the previous three time steps and c has been active at time $t-4$, $t-3$ and $t-2$. 

Apart from relative time specifications, the \texttt{BoolNet} network language also incorporates predicates that depend on the absolute time, i.e. the number of time steps that have elapsed since the initial state. 

\begin{samepage}
For example,
\begin{verbatim}
a, timeis(3)
\end{verbatim}
specifies that \texttt{a} is active at time step 3 and inactive at all other time steps. Similarly, the predicates \texttt{timelt} and \texttt{timegt} evaluate to 1 before and after the specified time point respectively.
\end{samepage}

As the above examples do not cover all possibilities of the network description language, a full language specification is provided in Section~\ref{sec:appendix}.

For temporal networks, \texttt{BoolNet} uses a special symbolic simulator that represents the functions as expression trees, whereas the standard simulator is based on a truth table representation. These simulators are discussed in Section~\ref{sec:networkanalysis}. As synchronous Boolean networks are a special case of temporal networks, they can also be simulated with the symbolic simulator. When a network is loaded from a file using \texttt{loadNetwork()}, the user can specify the parameter \texttt{symbolic=TRUE} to load it in form of a \emph{SymbolicBooleanNetwork} object instead of a \emph{BooleanNetwork} object. The same parameter is also available for the import functions discussed in Section~\ref{sec:importexport}. Temporal networks can only be loaded with \texttt{symbolic=TRUE}, as \texttt{BoolNet} cannot represent them as truth tables.

\begin{sloppypar}
As many network generation and modification routines (such as random network generation and network reconstruction that are discussed in the following sections) internally use the truth table representation, there are conversion routines \texttt{truthTableToSymbolic()} and \texttt{symbolicToTruthTable()} that convert synchronous Boolean networks of class \emph{BooleanNetwork} in a truth table representation to networks of class \emph{SymbolicBooleanNetwork} in a symbolic representation and vice versa. For more details, please refer to the manual.
\end{sloppypar}

\subsection{Reconstructing a network from time series}

An entirely different approach of assembling a network is to infer rules from series of expression measurements of the involved genes over time. For example, microarray experiments can be conducted at different points of time to cover the expression levels of different cell states. To reconstruct networks from such data, \texttt{BoolNet} includes two  
reconstruction algorithms for synchronous Boolean networks, Best-Fit Extension~\cite{laehdesmaeki03} and REVEAL \cite{liang98}. REVEAL requires the inferred functions to match the input time series perfectly, hence it is not always able to reconstruct networks in the presence of noisy and inconsistent measurements. Best-Fit Extension retrieves a set of functions with minimum error on the input and is thus suited for noisy data. 

In the following, we introduce a tool chain for the reconstruction of a Probabilistic Boolean Network from time series using Best-Fit extension.

Microarray measurements are usually represented as matrices of real-valued numbers which, for example, quantify the expression levels of genes. \texttt{BoolNet} includes a real-valued time series of gene measurements  from a project to analyze the yeast cell cycle \cite{spellman98} which can be loaded using
<<>>=
data(yeastTimeSeries)
@
This data contains four preselected genes and a series of 14 measurements for each of these genes.

\begin{sloppypar}
In a first step, the real-valued dataset has to be converted to binary data as required by the reconstruction algorithm. \texttt{BoolNet} offers several binarization algorithms in the function \texttt{binarizeTimeSeries()}. We here employ the default method which is based on $k$-means clustering (with $k=2$ for active and inactive):
\end{sloppypar}
<<>>=
binSeries <- binarizeTimeSeries(yeastTimeSeries)
@
The returned structure in \texttt{binSeries} has an element \texttt{\$binarizedMeasurements} containing the binary time series, and, depending on the chosen binarization method, some other elements describing parameters of the binarization.

To reconstruct the network from this data, we call the Best-Fit Extension algorithm:
<<>>=
net <- reconstructNetwork(binSeries$binarizedMeasurements, 
                          method="bestfit", 
                          maxK=4)
@
Here, \texttt{maxK} is the maximum number of input genes for a gene examined by the algorithm. The higher this number, the higher is the runtime and memory consumption of the reconstruction.

\begin{samepage}
We can now take a look at the network using
<<>>=
net
@
\end{samepage}

\begin{sloppypar}
The dependencies among the genes in the network can be visualized using the \texttt{plotNetworkWiring()} function. In this graph, each gene corresponds to a vertex, and the inputs of transition functions correspond to edges.
\end{sloppypar}
<<eval=FALSE>>=
plotNetworkWiring(net)
@
plots a graph similar to that at the top of Figure~\ref{fig:wiring}. To use this function, you must install the \texttt{igraph} package.

\begin{figure}[p]
\centering
\includegraphics[width=0.5\linewidth]{wiring1}
\includegraphics[width=0.5\linewidth]{wiring2}
\caption{The wiring graph of the reconstructed network without prior knowledge (top) and with the inclusion of prior knowledge (bottom). Each node of the graph represents one gene, and each arrow represents a gene dependency.}
\label{fig:wiring}
\end{figure}

A network that involved the same genes was examined by Kim et al. \cite{kim07}. When comparing the wiring graph of our reconstructed network with the reference network presented in Figure~2 of this paper, one observes a very high similarity between the two networks. However, the reconstructed network comprises too many links for the gene Sic1: The reference network does not contain a self-regulation of Sic1 or regulation of Sic1 by Fkh2. If it is known in advance that these regulations are not plausible, such prior knowledge can be supplied to the reconstruction algorithm:

<<>>=
net <- reconstructNetwork(binSeries$binarizedMeasurements, 
                          method="bestfit", 
                          maxK=4, 
                          excludedDependencies = list("Sic1" = c("Sic1", "Fkh2")))
@


The wiring of the reconstruction with prior knowledge is shown at the bottom of Figure~\ref{fig:wiring}. We can see that the two false links are now eliminated. Similar to \texttt{excludedDependencies}, there is also a parameter \texttt{requiredDependencies} that specifies dependencies that must be included in the network.

When \texttt{reconstructNetwork()} discovers multiple functions for a gene with the minimum error on the input data, it includes all of these functions as alternative functions with equal probability. Consequently, the function returns a \texttt{ProbabilisticBooleanNetwork} structure.

If you would like to obtain a \texttt{BooleanNetwork} object with only one function per gene from a probabilistic network, you can extract such a network by telling the software which of the functions you would like to use. This can be done by specifying the indices of the functions to extract:
<<>>=
net <- reconstructNetwork(binSeries$binarizedMeasurements, 
method="bestfit", maxK=4)
functionIndices <- c(1,2,3,2) #select function index for each regulatory component
dontCareDefaults <- lapply(seq_along(net$interactions), function(idx) rep(F, sum(net$interactions[[idx]][[functionIndices[idx]]]$func == -1))) #determine number of don't care values for each selected function and set them to 0
names(dontCareDefaults) <- net$genes
singleNet <- chooseNetwork(net, functionIndices, dontCareValues = dontCareDefaults)
@
In case of don't care values in reconstructed functions, it is possible to set them to 0 or 1 per default. The result is a Boolean network that is created by extracting the first function of gene Fkh2, the second function of genes Swi5 and Clb1, and the third function of gene Sic1 from the above probabilistic network:
<<>>=
singleNet
@

\begin{sloppypar}
\texttt{BoolNet} also supports the generation of artificial time series from existing networks: The \texttt{generateTimeSeries()} function generates a set of time series from a network using random start states and optionally adds Gaussian noise.
\end{sloppypar}
<<echo=FALSE,results=hide>>=
set.seed(3176)
@
<<>>=
series <- generateTimeSeries(cellcycle, 
                             numSeries=100, 
                             numMeasurements=10, 
                             noiseLevel=0.1)
@
generates a list of 100 time series by calculating 10 consecutive transitions from 100 randomly chosen network states in the mammalian cell cycle network. The series are subject to Gaussian noise with a standard deviation of 0.1, such that the result is a list of real-valued matrices.

We can now binarize these simulated measurements and try to reconstruct the original network:
<<>>=
binSeries <- binarizeTimeSeries(series, method="kmeans")
net <- reconstructNetwork(binSeries$binarizedMeasurements, method="bestfit")
net
@

Obviously, the number of generated time series is still to small to reconstruct the network unambiguously. However, the result comes close to the original network. We see that the functions of the network are not fully specified: At some positions, there are asterisks (\texttt{*}) denoting a \emph{don't care} value. This means that functions with a 0 and a 1 at this position match the time series equally well. That is, a partially defined function with $m$ asterisks corresponds to $2^m$ fully defined Boolean functions. It is also possible to generate these fully defined functions instead of the partially defined function by setting the parameter \texttt{returnPBN} to true (which was the behaviour prior to \texttt{BoolNet} version 2.0). For many \emph{don't care} values, this may consume a high amount of memory and computation time. 

\texttt{generateTimeSeries()} can also generate time series with artificial knock-outs and overexpressions:
<<echo=FALSE,results=hide>>=
set.seed(4463)
@
<<>>=
series <- generateTimeSeries(cellcycle, 
                             numSeries=10, 
                             numMeasurements=10, 
                             perturbations=1,
                             noiseLevel=0.1)
@
specifies that each generated time series is generated from a network where one randomly selected gene is artificially knocked down (constantly 0) or overexpressed (constantly 1). These perturbations are returned in an additional element \texttt{perturbations}. 
\begin{samepage}
<<<>>=
series$perturbations
@
\end{samepage}
Here, each column corresponds to the perturbations applied in one series. A value of 1 denotes an overexpression, a value of 0 denotes a knock-out, and an N/A value means that no perturbation was applied to this gene.

The \texttt{reconstructNetwork()} function also supports the reconstruction from such perturbation experiments if it is known which genes were perturbed. First, we store the series and the perturbation matrix in separate variables and binarize the data as before:
<<>>=
perturbations <- series$perturbations
series$perturbations <- NULL

binSeries <- binarizeTimeSeries(series, method="kmeans")
@
Now, we can reconstruct the network by specifying the \texttt{perturbations} parameter:
<<>>=
net <- reconstructNetwork(binSeries$binarizedMeasurements, 
                          method="bestfit", 
                          perturbations=perturbations)
net
@
As we generated only 10 series in this case, the reconstructed network is much more incomplete than in the previous reconstruction. 

In biological settings, perturbation experiments are probably one of the most frequent ways of exploring the behaviour of a regulatory network, as it is much easier to obtain various different responses by applying perturbations than by just measuring the wild type behaviour. 

\subsection{Creating random networks}

To study structural properties of Boolean networks and to determine the specific properties of biological networks in comparison to arbitrary networks, it is often desirable to generate artificial networks. \texttt{BoolNet} comprises a facility for the generation of random $N$-$K$ networks \cite{kauffman69, kauffman93}. In the standard $N$-$K$ networks, $N$ is the total number of genes, and $K$ is the number of input genes for each gene transition function. Such a network can be generated using
<<>>=
net <- generateRandomNKNetwork(n=10, k=3)
@
This creates a network with 10 genes, each of which has a transition function that depends on 3 genes and whose output is generated uniformly at random. Similarly, one can also specify different numbers of input genes for each gene:
<<>>=
net <- generateRandomNKNetwork(n=10, k=c(1,2,3,1,3,2,3,2,1,1))
@
\texttt{BoolNet} does not only support this standard case, but allows for different methods of choosing the numbers of input genes (parameter \texttt{topology}), the input genes themselves (parameter \texttt{linkage}), and the transition functions (parameter \texttt{functionGeneration}). In the following, some examples are presented. 

The command
<<>>=
net <- generateRandomNKNetwork(n=20, k=20, topology="scale_free")
@
determines the numbers of input genes by drawing values from the scale-free Zeta distribution \cite{aldana03}. According to this distribution, most transition functions will have a small number of input genes, but a few transition functions may depend on a high number of genes. The shape of the Zeta distribution can be customized using an additional parameter \texttt{gamma}, which potentially increases the number of input genes when chosen small and vice versa.

<<>>=
net <- generateRandomNKNetwork(n=10, k=3, linkage="lattice")
@
creates a network in which the transition functions of the genes depend on a choice of genes with adjacent indices \cite{aldana03_2}. This leads to networks with highly interdependent genes.

It is also possible to influence the truth tables of the functions in several ways. The parameter \texttt{zeroBias} changes the ratio of 1 and 0 returned by the functions: 
<<>>=
net <- generateRandomNKNetwork(n=10, 
                               k=3, 
                               functionGeneration="biased", 
                               zeroBias=0.75)
@
generates a network in which the outcome of a transition function is 0 for around 75\% of the inputs.

A more intricate way of influencing the function generation is the specification of generation functions. Generation functions explicitly generate functions according to a specific function class. 

\emph{Canalyzing functions} are assumed to occur frequently in biological systems~\cite{kauffman04}. A canalyzing function has the property that one input can determine the output value on its own, i.e. if this input is either active or inactive, the output of the function is always the same. \emph{Nested canalyzing functions} are a recursive definition of canalyzing functions, where the part of the function that does not depend on the canalyzing input is a canalyzing function for another input. Such functions can be generated using the built-in generation functions \texttt{generateCanalyzing()} and \texttt{generateNestedCanalyzing()}:
<<>>=
net1 <- generateRandomNKNetwork(n=10, 
                                k=3,
                                functionGeneration=generateCanalyzing, 
                                zeroBias=0.75)
net2 <- generateRandomNKNetwork(n=10, 
                                k=3, 
                                functionGeneration=generateNestedCanalyzing, 
                                zeroBias=0.75)
@
It is also possible to define own generation functions: A generation function receives a vector \texttt{input} of input gene indices as a parameter and returns a truth table result column with \verb+2^length(input)+ values representing the function.

If no explicit generation scheme is known for the function class of interest, validation functions can be used instead of generation functions. Validation functions verify whether randomly generated functions belong to a specific function class and reject invalid functions. Naturally, this is much less efficient than generating appropriate functions directly.
An example is the generation of monotone functions, which are also thought to be biologically plausible. These function account for the assumption that a transcription factor usually either activates or inhibits a specific target gene, but does not change the type of regulation depending on other factors~\cite{maucher11}. We can specify a simple validation function that checks whether a Boolean function is monotone:

\begin{samepage}
<<>>=
isMonotone <- function(input, func)
{
  for (i in seq_len(ncol(input)))
  # check each input gene
  {
    groupResults <- split(func, input[,i])
    if (any(groupResults[[1]] < groupResults[[2]]) && 
        any(groupResults[[1]] > groupResults[[2]]))
      # the function is not monotone
      return(FALSE)
  }
  return(TRUE)
}
@
\end{samepage}
\begin{sloppypar}
Here, \texttt{input} is a matrix containing the input part of the transition table, and \texttt{func} is the output of the Boolean function. In a monotone function, the values of the target may only change in one direction when switching one input. Hence, a function for which switching the value of an input gene sometimes switches the target from active to inactive, but also sometimes switches it from inactive to active is not monotone. This is validated by comparing the two groups of transition table entries for which the current input is active and inactive respectively.

If a validation function is supplied to \texttt{generateRandomNKNetwork()}, the generator generates Boolean functions until either the validation function returns \texttt{TRUE} or the maximum number of iterations (specified by the parameter \texttt{failureIterations}) is reached, in which case it fails.
\end{sloppypar}

<<>>=
net <- generateRandomNKNetwork(n=10,
                               k=3, 
                               validationFunction="isMonotone", 
                               failureIterations=1000)
@
creates a network with 10 genes in which all functions have 3 inputs, of which at least one is canalyzing.

By default, \texttt{generateRandomNKNetwork()} creates functions that cannot be simplified, i.e. that do not contain any genes that are irrelevant for the outcome of the function. If desired, this behaviour can be changed by setting \texttt{noIrrelevantGenes} to \texttt{FALSE}.

The presented parameters can be combined, and there are further options and parameters, so that a broad variety of networks with different structural properties can be generated. For a full reference of the possible parameters, please refer to the manual.

\subsection{Knock-out and overexpression of genes}

\texttt{BoolNet} allows for temporarily knocking out and overexpressing genes in a network without touching the transition functions. This means that genes can be set to a fixed value, and in any calculation on the network, this fixed value is taken instead of the value of the corresponding transition function. Knocked-out and overexpressed genes speed up the analysis of the network, as they can be ignored in many calculations. For example, to knock out CycD in the mammalian cell cycle network, we call
<<>>=
data(cellcycle)
knockedOut <- fixGenes(cellcycle, "CycD", 0)
@
or alternatively use the gene index
<<>>=
knockedOut <- fixGenes(cellcycle, 1, 0)
@
This sets the gene constantly to 0. To over-express the gene (i.e. to fix it to 1), the corresponding call is
<<>>=
overExpressed <- fixGenes(cellcycle, "CycD", 1)
@
The command
<<>>=
originalNet <- fixGenes(knockedOut, "CycD", -1)
@
reactivates the gene (for both knock and overexpression) and resets the network to its original state. 

The function also accepts multiple genes in a single call, such as
<<>>=
newNet <- fixGenes(cellcycle, c("CycD","CycE"), c(0,1))
@
which knocks out CycD and overexpresses CycE.

\section{Network analysis}\label{sec:networkanalysis}

\subsection{Simulation of state transitions}

To simulate a state transition and identify successor states of a given state, \texttt{BoolNet} includes the function \texttt{stateTransition()}. The function supports transitions for all four types of networks.

The following code performs a synchronous state transition for the state in which all genes are set to 1 on the mammalian cell cycle network:
<<>>=
data(cellcycle)
stateTransition(cellcycle, rep(1,10))
@

To calculate all state transitions in a synchronous network until an attractor is reached, you can call
<<>>=
path <- getPathToAttractor(cellcycle, rep(0,10))
path
@
The returned matrix consists of the subsequent states until an attractor is reached. Depending on the optional parameter \texttt{includeAttractorStates}, the sequence comprises all attractor states, only the first attractor state or none of the attractor states.

A sequence can be visualized by plotting a table of state changes:
<<eval=FALSE>>=
plotSequence(sequence=path)
@

\begin{figure}[t]
\centering
\includegraphics[width=0.6\linewidth]{sequence}
\caption{Visualization of a sequence of states in the mammalian cell cycle network. The columns of the table represent consecutive states of the time series. The last state is the steady-state attractor of the network.}
\label{fig:sequence}
\end{figure}

The result is depicted in Figure~\ref{fig:sequence}. \texttt{plotSequence()} also includes a shortcut that calculates the sequence directly if a network and a start state are supplied. It also provides an alternative way of visualizing the sequence as a state transition by setting \texttt{mode="graph"}.

The function
<<eval=FALSE>>=
sequenceToLaTeX(sequence=path, file="sequence.tex")
@
creates a \LaTeX\ table similar to \texttt{plotSequence()} function document.

In many cases, start states are defined by a set of active genes. Instead of supplying a full state vector, one can also supply only these active genes using the \texttt{generateState()} function.
<<>>=
startState <- generateState(cellcycle, specs=c("CycD"=1,"CycA"=1))
stateTransition(cellcycle,startState)
@
\begin{sloppypar}
calculates a state transition starting from a state where only the genes \emph{CycD} and \emph{CycA} are active, while all other genes are inactive (which is controlled by the \texttt{default} parameter of \texttt{generateState()}). 
\end{sloppypar}

For temporal Boolean networks (objects of class \emph{SymbolicBooleanNetworks}), the above functions can be utilized mostly in the same way. We demonstrate this using a small temporal network example of the IGF (Insulin-like growth receptor) pathway that is included in the package and can be loaded via
<<>>=
data(igf)
@
The model illustrates the activation and feedback inhibition of the PI3K-Akt-mTOR signalling cascade through IGF and IRS.

A state transition from the initial state in which the trigger of the pathway -- IGF -- is active, can be performed using
<<>>=
startState <- generateState(igf, specs=c("IGF"=1))
stateTransition(igf, startState)
@
The IGF network incorporates time delays of up to 3. Therefore, the last three states have to be known to calculate a successor state. If only a single state is supplied -- as above -- the function assumes that the state was the same before. This can be seen when calculating the sequence to the attractor using
<<>>=
getPathToAttractor(network=igf,state=startState)
@

Here, the states at $t=-2, \ldots 0$ are the same as the generated start state.
If it is required to specify multiple predecessor states here, a matrix of states can be supplied instead of a vector. E.g.,  
<<>>=
startState <- generateState(igf, specs=list("IGF"=c(0,0,1)))

startState
@
specifies that the first two states ($t=-2, t=-1$) should have all genes inactive, while IGF is activated only at $t=0$. This time, we plot the sequence instead of printing it:
\begin{figure}[bt]
\centering
\includegraphics[width=0.6\linewidth]{sequence_igf}
\caption{Visualization of a sequence of states in the IGF network. The columns of the table represent consecutive states of the time series. After activation of IGF, the attractor consisting of 14 states is entered immediately. This attractor represents the activation and inactivation of the PI3K-Akt-mTOR signalling cascade through IGF and IRS.}
\label{fig:sequence_igf}
\end{figure}

<<eval=FALSE>>=
plotSequence(network=igf, startState=startState)
@
The result is depicted in Figure~\ref{fig:sequence_igf}.


\texttt{stateTransition()} can also perform asynchronous updates. A random asynchronous transition is performed using
<<echo=FALSE>>=
set.seed(54321)
@
<<>>=
stateTransition(cellcycle, rep(1,10), type="asynchronous")
@
In this case, the fifth gene, CycA, was chosen at uniformly at random and updated. 

\begin{samepage}
We can also specify non-uniform probabilities for the genes, for example
<<echo=FALSE>>=
set.seed(4321)
@
<<>>=
stateTransition(cellcycle, rep(1,10), type="asynchronous", 
geneProbabilities=c(0.05,0.05,0.2,0.3,0.05,0.05,0.05,0.05,0.1,0.1))
@
This obviously increases probabilities for the genes 3 and 4 (E2F and CycE) to be chosen. In this case, CycE was chosen for the update.
\end{samepage}

\enlargethispage{0.5cm}
Sometimes you do not want a random update at all, but would like to specify which gene should be chosen for the update. This is possible via
<<>>=
stateTransition(cellcycle, rep(1,10), type="asynchronous", 
chosenGene="CycE")
@

In probabilistic Boolean networks, a state transition is performed by choosing one of the alternative functions for each gene and applying this set of functions to the current state. The following performs a state transition with a randomly chosen set of functions on the artificial probabilistic Boolean network taken from \cite{shmulevich02} with 3 genes, starting from state (0,1,1):
<<echo=FALSE>>=
set.seed(432)
@
<<>>=
data(examplePBN)
stateTransition(examplePBN, c(0,1,1), type="probabilistic")
@
You may get a different result, as the functions are chosen randomly according to the probabilities stored in the network. If you would like to execute a specific set of transition functions, you can supply this in an additional parameter:
<<>>=
stateTransition(examplePBN, c(0,1,1), type="probabilistic", 
chosenFunctions=c(2,1,2))
@
This call uses the second function for gene x1 and x3 and the first function for gene x2.  

\subsection{Identification of attractors}

Attractors are stable cycles of states in a Boolean network. As they comprise the states in which the network resides most of the time, attractors in models of gene-regulatory networks are expected to be linked to phenotypes \cite{kauffman93,li04}. Transitions from all states in a Boolean network eventually lead to an attractor, as the number of states in a network is finite. All states that lead to a certain attractor form its {\em basin of attraction}. \texttt{BoolNet} is able to identify attractors in synchronous and asynchronous Boolean networks. There are three types of attractors in these networks:
\begin{description}
\item[Simple attractors]{occur in synchronous and temporal Boolean networks and consist of a set of states whose synchronous transitions form a cycle.}
\item[Complex or loose attractors]{are the counterpart of simple attractors in asynchronous networks. As there is usually more than one possible transition for each state in an asynchronous network, a complex attractor is formed by two or more overlapping loops. Precisely, a complex attractor is a set of states in which all asynchronous state transitions lead to another state in the set, and a state in the set can be reached from all other states in the set.}
\item[Steady-state attractors]{are attractors that consist of only one state. All transitions from this state result in the state itself. These attractors are the same both for synchronous and asynchronous update of a network. Steady states are a special case of both simple attractors and complex attractors.}
\end{description}

The \texttt{getAttractors()} function incorporates several methods for the identification of attractors in synchronous and asynchronous networks. We present these methods using the included mammalian cell cycle network as an example. This network has one steady-state attractor, one simple synchronous attractor consisting of 7 states, and one complex asynchronous attractor with 112 states (see \cite{faure06}).

We first demonstrate the use of exhaustive synchronous search. This means that the software starts from all possible states of the network and performs synchronous state transitions until a simple or steady-state attractor is reached. 

\pagebreak[4]
<<eval=FALSE>>=
data(cellcycle)
attr <- getAttractors(cellcycle)
attr
@
\begin{lstlisting}[linewidth=.9\linewidth]
<<echo=FALSE,results=tex>>=
attr <- getAttractors(cellcycle)
attr
@
\end{lstlisting}


Typing \texttt{attr} calls a special print method that presents the attractor in a human-readable way. Here, a state in an attractor is represented by a binary vector, where each entry of the vector codes for one gene. An alternative is to print only the names of the active genes (i.e., the genes that are set to 1) instead of the full vector by calling the \texttt{print()} method explicitly with a changed parameter:
<<eval=FALSE>>=
print(attr, activeOnly=TRUE)
@
\begin{lstlisting}[linewidth=.9\linewidth]
<<echo=FALSE,results=tex>>=
print(attr, activeOnly=TRUE)
@
\end{lstlisting}

We can see that the search identified both synchronous attractors. 

\begin{sloppypar}
The \texttt{AttractorInfo} structure stores the attractors in an encoded form. The function \texttt{getAttractorSequence()} can be used to obtain the sequence of states that constitute a specific synchronous attractor as a table:
\end{sloppypar}
<<>>=
getAttractorSequence(attr, 2)
@
retrieves the states that make up the second (i.e., the 7-state attractor) as a data frame with the genes in the columns and the successive states in the rows.

The advantage of the exhaustive search method is that the complete transition table is calculated and stored in the return value. This table stores information that is used by a number of analysis methods described below.

You can extract the transition table in a data frame and print it out using
<<eval=FALSE>>=
tt <- getTransitionTable(attr)
tt
@
\begin{verbatim}
     State      Next state  Attr. basin  # trans. to attr.
0000000000 =>   0110010111            1                  4
[...]
1111111111 =>   1000001110            2                  1

Genes are encoded in the following order: CycD Rb E2F CycE 
CycA p27 Cdc20 Cdh1 UbcH10 CycB
\end{verbatim}
In the printed table, the first column denotes the initial state, the second column contains the state after the transition, the first column contains the number of the attractor that is finally reached from this state, and the fourth column lists the number of state transitions required to attain this attractor.

A table of the same structure is returned by 
<<eval=FALSE>>=
getBasinOfAttraction(attr, 1)
@
which extracts all states from the transition table that belong to the basin of attraction of attractor one (i.e., whose attractor number in column 3 is 1).

If you are interested in information on a single state (here: the state with all genes set to 1), you can type
<<eval=FALSE>>=
getStateSummary(attr, c(1,1,1,1,1,1,1,1,1,1))
@
\begin{verbatim}
     State      Next state  Attr. basin  # trans. to attr.
1111111111 =>   1000001110            2                  1

Genes are encoded in the following order: CycD Rb E2F CycE 
CycA p27 Cdc20 Cdh1 UbcH10 CycB
\end{verbatim}

The visualization function \texttt{getStateGraph()} makes use of the transition table as well: It plots a transition graph in which the basins of attraction are drawn in different colors, and the attractors are highlighted.
The result of
<<eval=FALSE>>=
plotStateGraph(attr)
@
is depicted at the top of Figure~\ref{fig:stategraph}. The blue basin belongs to attractor 1, and the green basin belongs to attractor 2.

The above call does not ensure that the basins of attraction are clearly separated in the plot. If this is desired, one can choose to use a piecewise layout, which means that the layouting function is applied separately to each basin of attraction, and the basins are drawn side by side.
The result of
<<eval=FALSE>>=
plotStateGraph(attr, piecewise=TRUE)
@
is depicted at the bottom of Figure~\ref{fig:stategraph}.

\begin{figure}[p]
\centering
\includegraphics[width=0.7\linewidth]{stategraph1}
\includegraphics[width=0.7\linewidth]{piecewisestategraph}
\caption{The state graph of the mammalian cell cycle network using the regular layout (top) and using a piecewise layout (bottom). Each node represents a state of the network, and each arrow is a state transition. The colors mark different basins of attraction. Attractors are highlighted using bold lines.}
\label{fig:stategraph}
\end{figure}


Exhaustive search consumes a high amount of time and memory with increasing size of the network, which makes it intractable for large networks (\texttt{BoolNet} currently supports networks with up to 29 genes for exhaustive search due to memory restrictions in {\em R}). Therefore, \texttt{BoolNet} also allows for heuristic search of attractors, which works for larger networks as well. Heuristic synchronous search starts from a predefined small set of states and identifies the attractors to which state transitions from these states lead. The start states can either be supplied, or they can be calculated randomly.

<<eval=FALSE>>=
attr <- getAttractors(cellcycle, method="random", startStates=100)
@
chooses 100 random start states for the heuristic search and usually identifies both attractors. 

<<eval=FALSE>>=
attr <- getAttractors(cellcycle, 
                      method="chosen", 
                      startStates=list(rep(0,10),rep(1,10)))
@
starts from the states \texttt{(0,0,0,0,0,0,0,0,0,0)} and \texttt{(1,1,1,1,1,1,1,1,1,1)} and again identifies both synchronous attractors.

\begin{sloppypar}
For the previous calls, only the subset of the transition table traversed by the heuristic is returned. This means that there is no guarantee that, e.g. \texttt{getBasinOfAttraction()} returns the complete basin of attraction of an attractor in heuristic mode.
\end{sloppypar}

Synchronous attractors can be visualized by plotting a table of changes of gene values in the states of the attractor: 
<<eval=FALSE>>=
plotAttractors(attr, subset=2)
@
plots the state changes of the simple attractor with 7 states, as depicted in Figure~\ref{fig:attractor1}. Similarly,
<<eval=FALSE>>=
attractorsToLaTeX(attr, subset=2, file="attractors.tex")
@
exports the same state table to a \LaTeX\ document.

\begin{figure}[t]
\centering
\includegraphics[width=0.65\linewidth]{attractor1}
\caption{Visualization of the state changes in an attractor. The columns of the table represent consecutive states of the attractor.}
\label{fig:attractor1}
\end{figure}

To identify asynchronous attractors, another special heuristic algorithm is included. This algorithm again starts from a small subset of states and makes a number of random transitions to reach an attractor with a high probability. After that, a validation step is performed to analyze whether a complex attractor has been identified.

The command
<<results=hide>>=
attr <- getAttractors(cellcycle, 
                      type="asynchronous",
                      method="random", 
                      startStates=500)
@
conducts an asynchronous search with 500 random start states on the mammalian cell cycle network.
In this case, the algorithm has identified both the steady-state attractor and the complex attractor:

\enlargethispage{0.5cm}
<<eval=FALSE>>=
attr
@
\begin{verbatim}
Attractor 1 is a simple attractor consisting of 1 state(s):

 |--<---------|
 V            |
 0100010100   |
 V            |
 |-->---------|

Genes are encoded in the following order: CycD Rb E2F CycE 
CycA p27 Cdc20 Cdh1 UbcH10 CycB

Attractor 2 is a complex/loose attractor consisting of 112 state(s) 
and 338 transition(s):

1011101111 => 1011101110
[...]
1000000000 => 1010000000

Genes are encoded in the following order: CycD Rb E2F CycE 
CycA p27 Cdc20 Cdh1 UbcH10 CycB
\end{verbatim}

For the complex attractor, the involved transitions are printed out. By default, the algorithm tries to avoid self-loops, i.e. transitions that lead to the same state again. This means that self-loop transitions are only allowed if there is no other transition that leads to a different state. If you would like to allow the algorithm to enter self-loops even if transitions to different states are possible, you can call
<<eval=FALSE>>=
attr <- getAttractors(cellcycle, 
                      type="asynchronous",
                      method="random", 
                      startStates=500, 
                      avoidSelfLoops=FALSE)
@
In the resulting complex attractor with 112 states, there are 450 transitions instead of 338 transitions, which is due to the additional self-loops. 

The asynchronous heuristic search does not return a transition table, such that the above analysis methods cannot be applied here.


\begin{figure}[tb]
\centering
\includegraphics[width=0.55\linewidth]{attractor2}
\caption{Graph representation of the complex attractor in the mammalian cell cycle network. Each node represents a state of the complex attractor, and each arrow represents a state transition.}
\label{fig:attractor2}
\end{figure}

As there are multiple possible transitions for each state, complex attractors cannot be visualized as in Figure~\ref{fig:attractor1}. For this reason, \texttt{plotAttractors()} supports a graph mode that visualizes the transitions among the states in the attractor:
<<eval=FALSE>>=
plotAttractors(attr, subset=2, mode="graph", drawLabels=FALSE)
@
plots the 112-state attractor as depicted in Figure~\ref{fig:attractor2}. We omit the state labels (i.e. the gene values) due to the high number of states. This plot again requires the \texttt{igraph} package.

\begin{sloppypar}
Although \texttt{getAttractors()} can also be applied to temporal networks and other networks that are in a symbolic representation (i.e. \emph{SymbolicBooleanNetwork} objects), this function is only a shortcut to the  simulation function \texttt{simulateSymbolicModel()} in this case. It is advised to use \texttt{simulateSymbolicModel()} directly , as it provides more options. For the temporal model of the IGF pathway included in \texttt{BoolNet}, an exhaustive simulation can be performed as follows:
\end{sloppypar}

<<eval=FALSE>>=
sim <- simulateSymbolicModel(igf)
sim
@
\begin{lstlisting}[linewidth=.9\linewidth]
<<echo=FALSE,results=tex>>=
sim <- simulateSymbolicModel(igf)
sim
@
\end{lstlisting}

By default, the result object of class \emph{SymbolicSimulation} comprises several components:
\begin{itemize}
\item A list of sequences \texttt{sequences} from each start state to the corresponding attractor. If this component is not desired, the parameter \texttt{returnSequences} can be set to false.
\item A graph structure \texttt{graph} that comprises all traversed state transitions. If this component is not desired, the parameter \texttt{returnGraph} can be set to false.
\item The identified attractors \texttt{attractors}. If this component is not desired, the parameter \texttt{returnAttractors} can be set to false.
\end{itemize}
In this case, the network has two attractors: A steady state describes the inactive state of the pathway. The circular attractor describes the activation and inactivation of the PI3K-Akt-mTOR signalling cascade initiated by IGF. 

All visualization and analysis function described above can also be applied to the simulation results obtained by \texttt{simulateSymbolicModel}. For example, the cascade attractor can be visualized via

<<eval=FALSE>>=
plotAttractors(sim, subset=2)
@
Similarly, 
<<eval=FALSE>>=
plotStateGraph(sim)
@
plots the state transition graph of the network. Unlike in classical synchronous networks, a state can have multiple successor states (outgoing edges) in temporal networks, as a state transition may also depend on the history of states before the current state.
The two plots are shown in Figure~\ref{fig:temporal_plots}.

\begin{figure}[p]
\centering
\includegraphics[width=0.6\linewidth]{attractor3}
\includegraphics[width=0.6\linewidth]{stategraph2}
\caption{Top: Visualization of an attractor that describes the activation and inactivation of the PI3K-Akt-mTOR signalling cascade through IGF and IRS. The columns of the table represent consecutive states of the attractor. On top, the percentage of states leading to the attractor is supplied.\newline Bottom: The state transition graph of the IGF pathway network. Each node represents a state of the network, and each arrow is a state transition. The colors mark different basins of attraction. }
\label{fig:temporal_plots}
\end{figure}

For temporal networks, it is often infeasible to perform an exhaustive search, as the search space is not only exponential in the number of genes, but also in the time delays. Hence, the search can also be restricted to randomly generated or prespecified start states similarly to \texttt{getAttractors()}. If time delays of more than 1 are included in the network, not only single start states are generated, but the required history of states is generated as well. For example,
<<echo=FALSE>>=
set.seed(43851)
@
<<>>=
sim <- simulateSymbolicModel(igf, method="random", startStates=2)
@
generates two start state matrices, each comprising 3 states (the maximum delay of the IGF network), and uses them as the basis for a simulation. This can be seen when examining the sequences to the attractors:
<<>>=
sim$sequences
@
Both sequences comprise start states $t=-2$, $t=-1$ and $t=0$.

\begin{sloppypar}
Classical synchronous Boolean networks can also be simulated using the symbolic simulator \texttt{simulateSymbolicModel()} if they are in a symbolic form. However, in most cases, \texttt{getAttractors()} will be faster and will consume less memory for synchronous networks without temporal elements. Only if the number of inputs to genes is very high and exhaustive simulation is not required, it may be advisable to use the symbolic simulator.
\end{sloppypar}

\subsection{Markov chain simulations}

Another way of identifying relevant states in Boolean networks are Markov chain simulations. Instead of identifying cycles explicitly, these simulations calculate the probability that a certain state is reached after a predefined number of iterations. Of course, states in an attractor have a high probability of being reached if the number of iterations is chosen large enough. Markov chain simulations for probabilistic Boolean networks were introduced by Shmulevich et al. \cite{shmulevich02}. As a special case of probabilistic Boolean networks, these simulations are also suited for synchronous Boolean networks.

The following performs a Markov experiment with the predefined number of 1000 iterations on the example PBN described in \cite{shmulevich02}:

\enlargethispage{-0.5cm}
<<>>=
data(examplePBN)
sim <- markovSimulation(examplePBN)
sim
@

Only states with a non-zero probability are listed in the two tables. The first table shows the states that are reached after 1000 iterations. The second table is a transition table annotated with transition probabilities. This table can be suppressed by the parameter \texttt{returnTable=FALSE}. The results correspond exactly to those in \cite{shmulevich02}.

If the transition table is included in the simulation results, we can plot a graph of the network:
<<eval=FALSE>>=
plotPBNTransitions(sim)
@
This graph is displayed in Figure~\ref{fig:pbntransitions}. The vertices are the states of the graph. The edges represent transitions and are annotated with the corresponding transition probabilities. For this plot, the \texttt{igraph} package must be installed.

\begin{figure}[h]
\centering
\includegraphics[width=0.65\linewidth]{pbntransitions}
\caption{State transition graph of the example probabilistic Boolean network included in \texttt{BoolNet}. Each node represents a state of the network, and each arrow is a possible state transition, annotated by the transition probability.}
\label{fig:pbntransitions}
\end{figure}

We can also use Markov chain simulations to identify the attractor states in the mammalian cell cycle network:
\begin{samepage}
<<>>=
data(cellcycle)
sim <- markovSimulation(cellcycle, 
                        numIterations=1024, 
                        returnTable=FALSE)
sim
@
\end{samepage}
We set the maximum number of iterations to 1024, which is the number of states in the network. In a deterministic network, this guarantees that all states are found.

The fourth state in the returned table is the steady-state attractor identified previously. It has a probability of 0.5, as the basin of attraction is exactly half of the states. The other 7 states belong to the simple synchronous attractor.

It is also possible to restrict the simulation to a certain set of input states instead of using all possible input states. In the following example, we only consider the state with all genes set to 1, and identify the state belonging to the steady-state attractor again:
\begin{samepage}
<<>>=
sim <- markovSimulation(cellcycle, 
                        numIterations=1024,
                        returnTable=FALSE, 
                        startStates=list(rep(1,10)))
sim
@
\end{samepage}

\subsection{Robustness assessment}

 A biological network is assumed to be robust to small amounts of noise. The plausibility of network models is therefore often assessed by testing its robustness to noise and mismeasurements. Typically, artificial noise is applied, and its influence on the behaviour of a network is measured.
 
There are two major ways of applying random noise: Either the current state of a network in a simulation can be perturbed, or the network structure itself can be perturbed. \texttt{BoolNet} includes functions for both types of robustness assessment.

The function \texttt{perturbTrajectories} measures the influence of noise that is applied to the current network state. It generates a set of initial states and creates perturbed copies of these states by randomly flipping bits. It then measures the influence of the flips on the further dynamic behaviour of the network. For example

<<echo=FALSE>>=
set.seed(3361)
@
<<>>=
data(cellcycle)
r <- perturbTrajectories(cellcycle, 
                         measure="hamming", 
                         numSamples=100, 
                         flipBits=1)
@
randomly generates 100 states and 100 copies with one bitflip and performs a single state transition for each state. It then measures the normalized Hamming distance (the fraction of different bits) between each state and the corresponding perturbed copy. A robust network is assumed to yield a low Hamming distance.
The average distance can be viewed by typing
<<>>=
r$value
@
A related measure is the average sensitivity. This measure assesses only a single transition function and counts the number of successor states that differ between the original states and the perturbed copies for the corresponding gene.
E.g.,
<<>>=
r <- perturbTrajectories(cellcycle, 
                         measure="sensitivity", 
                         numSamples=100, 
                         flipBits=1, 
                         gene="CycE")
r$value
@
measures the average sensitivity of the transition function for gene CycE.

The long-term behaviour can be evaluated by comparing the attractors that are reached from the initial states and their perturbed copies. 
<<>>=
r <- perturbTrajectories(cellcycle, 
                         measure="attractor", 
                         numSamples=100, 
                         flipBits=1)
r$value
@
measures the fraction of pairs of states and perturbed copies that yield the same attractors. It can be assumed that small changes in the state should not influence the long-term behaviour of the network, and hence the attractors should mostly be the same. 

The second class of perturbations adds random noise to the network itself. This is implemented in the \texttt{perturbNetwork()} function. Unlike \texttt{perturbTrajectories()}, this function does not perform any simulations, but returns a perturbed copy of the network that can be analyzed further.

\texttt{BoolNet} includes a set of different perturbation options that can be combined. For example,
<<>>=
perturbedNet <- perturbNetwork(cellcycle, 
                               perturb="functions", 
                               method="bitflip")
@
chooses a function of the network at random and flips a single bit in this function. By setting the parameter \texttt{maxNumBits}, you can also flip more than one bit at a time.

Instead of flipping bits,
<<>>=
perturbedNet <- perturbNetwork(cellcycle, 
                               perturb="functions", 
                               method="shuffle")
@
randomly permutes the output values of the chosen transition functions. This preserves the numbers of 0s and 1s, but may change the Boolean function completely.
These kinds of perturbations are supported for synchronous and asynchronous networks as well as for probabilistic networks.

For synchronous networks, a further perturbation mode is available:
<<>>=
perturbedNet <- perturbNetwork(cellcycle, 
                               perturb="transitions", 
                               method="bitflip", 
                               numStates=10)
@
Here, \texttt{BoolNet} calculates the complete transition table of the network and then flips a single bit in 10 states of the transition table. From this modified table, a network is reconstructed. Changes of this type only affect a few states (which might not be the case when perturbing the functions directly as above), but possibly several of the transition functions. As in the previous examples, it is also possible to modify the number of bits to be flipped or to choose \texttt{method="shuffle"}.

\begin{samepage}
A detailed listing of a perturbation experiment is shown below. In this experiment, 1000 perturbed copies of the cell cycle network are created, and the occurrences of the original synchronous attractors are counted in the perturbed copies. This is similar to the simulation in \texttt{perturbTrajectories()} with \texttt{measure="attractor"}. However, instead of comparing the outcomes of different initial states in the same network, it compares all attractors of different perturbed networks to all attractors of the original network by exhaustive search. 

\begin{footnotesize}\label{alg:perturbation}
<<eval=FALSE>>=
# Perform a robustness test on a network
# by counting the numbers of perturbed networks
# containing the attractors of the original net

library(BoolNet)

# load mammalian cell cycle network
data(cellcycle)

# get attractors in original network
attrs <- getAttractors(cellcycle, canonical=TRUE)

# create 1000 perturbed copies of the network and search for attractors
perturbationResults <- sapply(1:1000, function(i)
{
  # perturb network and identify attractors
  perturbedNet <- perturbNetwork(cellcycle, perturb="functions", method="bitflip")
  perturbedAttrs <- getAttractors(perturbedNet, canonical=TRUE)
  
  # check whether the attractors in the original network exist in the perturbed network
  attractorIndices <- sapply(attrs$attractors,function(attractor1)
        {
          index <- which(sapply(perturbedAttrs$attractors, function(attractor2)
            {
              identical(attractor1, attractor2)
            }))
          if (length(index) == 0)
            NA
          else
            index
        })
  return(attractorIndices)
})

# perturbationResults now contains a matrix
# with the first 2 columns specifying the indices or the 
# original attractors in the perturbed network 
# (or NA if the attractor was not found) and the next 2 
# columns counting the numbers of states
# in the basin of attraction (or NA if the attractor was not found)

# measure the total numbers of occurrences of the original attractors in the perturbed copies
numOccurrences <- apply(perturbationResults[seq_along(attrs$attractors),,drop=FALSE], 1,
                      function(row)sum(!is.na(row)))

# print original attractors
cat("Attractors in original network:\n")
print(attrs)

# print information
cat("Number of occurrences of the original attractors",
  "in 1000 perturbed copies of the network:\n")
for (i in 1:length(attrs$attractors))
{
  cat("Attractor ",i,": ",numOccurrences[i],"\n",sep="")
}
@
\end{footnotesize}
\end{samepage}

\pagebreak[4]
\begin{samepage}
The results of such an experiment could look like this:

\begin{verbatim}
Attractors in original network:
Attractor 1 is a simple attractor consisting of 1 state(s) 
and has a basin of 512 state(s):

 [...]
 
Attractor 2 is a simple attractor consisting of 7 state(s) 
and has a basin of 512 state(s):

[...]

Number of occurrences of the original attractors in 1000 
perturbed copies of the network:
Attractor 1: 622
Attractor 2: 589
\end{verbatim}
\end{samepage}
We see that the steady-state attractor is slightly more robust to perturbations than the simple attractor with 7 states, as it can be identified in a higher number of perturbed copies.

\subsection{Identifying specific properties of biological networks}

The described robustness measures could also be used to identify specific properties of real-world networks in comparison to arbitrary (random) networks. For example, one could assume that attractors in biological networks are more robust to perturbations than attractors in random networks with a similar structure, as they should be capable of compensating for small dysfunctions of their components. Similarly to the above code, one could execute a number of random perturbations on the biological network and measure the percentage of original attractors found in the perturbed copies.  Afterwards, one could repeat this process on a number of randomly generated networks -- i.e., generate perturbed copies from each of the random networks, and measure the percentage of attractors in the copies. If the percentage of the biological network is higher than most of the percentages of the random network, this suggests that the biological network exhibits a higher robustness. This is a kind of computer-intensive test.

\texttt{BoolNet} comprises a generic facility for such computer-intensive tests. This facility already includes several tests (mainly for synchronous Boolean networks) and can be extended by custom test functions. The outlined example of attractor robustness is one of the integrated functions:
<<eval=FALSE>>=
data(cellcycle)
res <- testNetworkProperties(cellcycle, 
                      numRandomNets=100, 
                      testFunction="testAttractorRobustness", 
                      testFunctionParams = list(copies=100, perturb="functions"))
@
creates a set of 100 random networks (each with the same number of input genes for the functions as the cell cycle network) and creates 100 perturbed copies for each of these networks and for the cell cycle network by applying \texttt{perturbNetwork()} with \texttt{perturb="functions"}. For each network, it then calculates the percentage of attractors that can still be found in the perturbed copies.

The function plots a histogram of this robustness measures of the random networks (see Figure~\ref{fig:robustness}, top panel). The corresponding value of the original cell cycle network is plotted as a red line, and the 95\% quantile is plotted as a blue line.

\begin{figure}[p]
\centering
\includegraphics[width=0.6\linewidth]{attractor_robustness}
\includegraphics[width=0.6\linewidth]{transition_robustness}
\caption{Top: Attractor robustness of randomly generated networks (histogram) in comparison to the mammalian cell cycle network (red line).\newline
Bottom: Normalized Hamming distance of randomly generated networks (histogram) in comparison to the mammalian cell cycle network (red line).
}
\label{fig:robustness}
\end{figure}

We can see that the average percentage of found attractors is significantly higher in the biological network with a $p$-value of 0.01.

\begin{sloppypar}
It is also possible to perturb the states instead of the networks themselves by setting \texttt{perturb} to \texttt{"trajectories"}. In this case, the function applies \texttt{perturbTrajectories()} with \texttt{measure="attractor"} to the biological network and the randomly generated networks. It then tests whether the fraction of state pairs that yield the same attractor is higher in the biological network than in the randomly generated networks.

The second built-in test function also tests the robustness of the network behaviour by perturbing the network states: \texttt{testTransitionRobustness()} applies \texttt{perturbTrajectories()} with \texttt{measure="hamming"} to each network. It then checks whether random bit flips yield a higher Hamming distance of the successor states in randomly generated networks than in the biological model, i.e. whether noise in the states influences the randomly generated networks stronger than the biological model. In contrast to the previous measures for which a greater value was assumed in the biological model, the Hamming distance is assumed to be smaller. Hence, we must specify the test alternative as \texttt{alternative="less"}.
\end{sloppypar}

<<eval=FALSE>>=
testNetworkProperties(cellcycle, 
                      numRandomNets=100,
                      testFunction="testTransitionRobustness",
                      testFunctionParams=list(numSamples=100),
                      alternative="less")  
@


The results are shown in the bottom panel of Figure~\ref{fig:robustness}. Again, the result is highly significant (indeed, the Hamming distances are \emph{always} lower in the biological network), which means that the biological network is considerably more robust to noise in the states than the randomly generated models.

Another network property can also be tested using a built-in function: When looking at the state graph of a biological network (which can be generated using \texttt{plotStateGraph()}), it can often be observed that many state transitions lead to the same successor states, which means that the dynamics of the network quickly concentrate on a few states after a number of state transitions. We call the number of states whose synchronous state transitions lead to a state $s$ the {\em in-degree} of state $s$. We expect the biological network to have a few states with a high in-degree and many states with a low in-degree. A characteristic to summarize the in-degrees is the Gini index, which is a measure of inhomogeneity. If all states have an in-degree of 1, the Gini index is 0; if all state transitions lead to only one state, the Gini index is 1.
<<eval=FALSE>>=
testNetworkProperties(cellcycle, 
                      numRandomNets=100, 
                      testFunction="testIndegree")
@

plots an histogram of Gini indices in 100 random networks and draws the Gini index of the cell cycle network as a red line, as depicted in the top panel of Figure~\ref{fig:indegree}.


\begin{figure}[p]
\centering
\includegraphics[width=0.6\linewidth]{indegree}
\includegraphics[width=0.6\linewidth]{indegree_kl}
\caption{Top: Gini indices of state in-degrees of randomly generated networks (histogram) in comparison to the mammalian cell cycle network (red line)\newline
Bottom: Kullback-Leibler distances of in-degrees of the mammalian cell cycle network and 100 random networks.}
\label{fig:indegree}
\end{figure}

The histogram shows that the Gini index of the in-degrees is always higher in the biological network. This is probably due to the special structure of functions in biological networks.

Instead of accumulating the in-degrees using the Gini index, it is also possible to compare the distributions of the in-degrees across the networks. For this purpose, the Kullback-Leibler distances of the in-degrees of the supplied network and each of the random networks are calculated and plotted in a histogram. The Kullback-Leibler distance (also called relative entropy) is an asymetric measure of similarity of two distributions \cite{cover91}. If the distributions are equal, the Kullback-Leibler distance is 0, otherwise it is greater than 0. 
<<eval=FALSE>>=
testNetworkProperties(cellcycle, 
                      numRandomNets=100, 
                      testFunction="testIndegree", 
                      accumulation="kullback_leibler")
@
results in the plot displayed in the bottom panel of Figure~\ref{fig:indegree}.

It is possible to switch between the histogram of an accumulated characteristic (e.g. the Gini index) and the histogram of the Kullback-Leibler distances for all tests. 

\begin{sloppypar}
You can also easily implement your own tests. To do this, the only thing you have to do is implement a custom testing function that replaces \texttt{testIndegree()} or \texttt{testAttractorRobustness()}. Testing functions have the following signature:
\end{sloppypar}
\begin{verbatim}
function(network, accumulate=TRUE, params)
\end{verbatim}
The first parameter is the network that should be tested. The parameter \texttt{accumulate} specifies whether a single characteristic value (e.g., the Gini index of the in-degrees) should be calculated, or whether a distribution of values (e.g., a vector of all in-degrees) should be returned. The third parameter is a list of further arguments needed by your function. 

If, for example, we would like to compare the sizes of the basins of attractions of synchronous attractors in biological and random networks, we would write a function like this:
\begin{samepage}
<<>>=
testBasinSizes <- function(network, accumulate=TRUE, params)
{
  attr <- getAttractors(network)
  basinSizes <- sapply(attr$attractors, function(a)
                      { 
                         a$basinSize
                      })
   if (accumulate)
     return(mean(basinSizes))
   else
     return(basinSizes)                  
}
@
\end{samepage}
This function calculates the mean basin size as a characteristic value if accumulation is required, or returns the sizes of all basins of attraction in a vector otherwise. It does not need any further parameters in \texttt{params}.

Now, we can start a test using
<<eval=FALSE>>=
testNetworkProperties(cellcycle, 
                      numRandomNets=100,
                      testFunction="testBasinSizes",
                      xlab="Average size of basins of attraction")
@
to produce the plot shown in Figure~\ref{fig:basinsize}. Apparently, the average basin sizes do not differ as much as the built-in test characteristics between the random networks and the cell cycle network.

\begin{figure}[h]
\centering
\includegraphics[width=0.6\linewidth]{basinsize}
\caption{A custom test statistic measuring the basin sizes on randomly generated networks (histogram) and the mammalian cell cycle network (red line).}
\label{fig:basinsize}
\end{figure}

\begin{sloppypar}
By writing custom test functions, you can extend the test facility to perform a wide variety computer-intensive test. Of course, it is also possible to plot the Kullback-Leibler distances with such new methods by using \verb+accumulation="kullback_leibler"+.

\texttt{testNetworkProperties()} accepts most of the parameters of \texttt{generateRandomNKNetwork()}. If necessary, you can generate more specialized kinds of random networks which resemble the original network in certain aspects, for example by specifying a generation function or by setting a proportion of 0 and 1 in the function outputs similar to the original network using \verb+functionGeneration="biased"+. 
\end{sloppypar}

\clearpage
\section{Import and export}\label{sec:importexport}

\subsection{Saving networks in the \texttt{BoolNet} file format}

Corresponding to the \texttt{loadNetwork()} command, a network can be saved using \texttt{saveNetwork()}. This stores the network in the network file format described in Section~\ref{sec:appendix} and can be applied to all types of networks supported by \texttt{BoolNet}. For example, the cell cycle network can be saved using
<<eval=FALSE>>=
saveNetwork(cellcycle, file="cellcycle.txt")
@
\begin{sloppypar}
The function stores the expressions that describe the transition functions. In some cases, there may not always be a valid symbolic description of the networks (e.g. for networks returned by \texttt{generateRandomNKNetwork()} when the \texttt{readableFunctions} parameter was not set). In this case, \texttt{saveNetwork()} can generate symbolic representations of the transition functions in Disjunctive Normal Form (DNF):
\end{sloppypar}
<<>>=
net <- generateRandomNKNetwork(n=10, k=3, readableFunctions=FALSE)
saveNetwork(net, file="randomnet.txt", generateDNF=TRUE)
@
The \texttt{generateDNF} parameter can also be used to detail which type of DNF formulae should be exported: \texttt{generateDNF="canonical"} exports canonical DNF formulae. \texttt{generateDNF="short"} minimizes the canonical functions by joining terms. By simply setting \texttt{generateDNF=TRUE}, formulae with up to 12 inputs are minimized, and formulae with more than 12 inputs are exported in a canonical form, as a minization is very time-consuming in this case.

\subsection{Import from and export to SBML}

\begin{sloppypar}
\texttt{BoolNet} provides an interface to the widely used Systems Biology Markup Language (SBML) via the import function \texttt{loadSBML()} and the export function \texttt{saveSBML()}. As the core SBML does not fully support Boolean models, import and export of SBML models is based on the \texttt{sbml-qual} package which extends SBML by several qualitative modeling approaches, such as general logical models and Petri nets. For a full description of \texttt{sbml-qual}, refer to \href{http://sbml.org/Documents/Specifications/SBML_Level_3/Packages/Qualitative_Models_(qual)}{http://sbml.org/Documents/Specifications/SBML\_Level\_3/Packages/Qualitative\_Models\_(qual)}.
\end{sloppypar}

\texttt{BoolNet} only supports a subset of \texttt{sbml-qual}. It can read and write logical models with two possible values for each state, which are equivalent to Boolean networks. Logical models with more than two values for a gene or Petri nets cannot currently be handled by \texttt{BoolNet}.

An export to SBML is usually not associated with any loss of information. For example, we can write the cell cycle network to a file and re-import it into \texttt{BoolNet}:

\begin{footnotesize}
<<eval=FALSE>>=
toSBML(cellcycle, file="cellcycle.sbml")
sbml_cellcycle <- loadSBML("cellcycle.sbml")
sbml_cellcycle
@
\end{footnotesize}

Apart from some additional brackets, the re-imported network coincides with the original network. 

Similar to the \texttt{saveNetwork()} function, \texttt{toSBML()} exports a symbolic representation of the network transition functionss, which may not always be available. As for \texttt{saveNetwork()}, there is a parameter \texttt{generateDNF} that can be set to generate a symbolic representation in Disjunctive Normal Form from the truth tables.

\subsection{Importing networks from BioTapestry}

BioTapestry is a widely-used application for visual modeling of gene-regulatory networks \cite{longabaugh05}. It can be freely accessed at \url{http://www.biotapestry.org}. Although its primary purpose is visualization, the software supports specifying logical functions for the genes. \texttt{BoolNet} can read in the top-level (``Full genome'') plot of a BioTapestry file (*.btp) and convert it into a Boolean network.

As an example, we assume the following BioTapestry model with 5 genes (2~inputs and 3~dependent genes):

\begin{center}
\includegraphics[width=0.8\linewidth]{biotap_model}
\end{center}

The corresponding BioTapestry file is included in \texttt{BoolNet}. You can determine its path using
<<eval=FALSE>>=
system.file("doc/example.btp", package="BoolNet")
@
\label{cmd:example.btp}
to access it in BioTapestry or \texttt{BoolNet}.

For the import, \texttt{BoolNet} needs to know the type of influence a gene has on another gene. Therefore, imported networks should only use links that are either enhancers or repressors. Neutral links are ignored in the import.

We now set further simulation parameters for the model. These parameters are imported by \texttt{BoolNet} to construct the functions of the Boolean network. First, we want to change the function of Gene~2 to \texttt{OR}. Right-click on Gene~2 and choose \texttt{Simulation Properties...}. 

\begin{center}
\includegraphics[width=0.8\linewidth]{biotap_sim_properties}
\end{center}

In the properties dialog, choose the \texttt{Logic} tab, and select \texttt{OR} for the logical function.

\begin{center}
\includegraphics[width=0.8\linewidth]{biotap_logic}
\end{center}

Now set the function of Gene~1 to \texttt{XOR} (exclusive or) in the same way.

\begin{samepage}
You can also specify initial values for constant genes, i.e., genes with no input links. Choose the simulation properties of Input~1, and change to the \texttt{Parameters} tab. Choose \texttt{initVal} and set it to 1. 

\begin{center}
\includegraphics[width=0.8\linewidth]{biotap_initval}
\end{center}
\end{samepage}

Press Return to store the result, and exit the dialog with \texttt{OK}. This will create a fixed gene with value 1 (i.e., an over-expressed gene) in the \texttt{BoolNet} import. Note that values other than 0 and 1 are ignored by the import, as well as initialization values for non-constant genes.

We assume that you save the network to a file ``example.btp'' in your working directory. In \texttt{R}, type
<<echo=FALSE>>=
net <- loadBioTapestry(system.file("doc/example.btp", package="BoolNet"))
@
<<eval=FALSE>>=
net <- loadBioTapestry("example.btp")
@
to import the network. Alternatively, replace the file name by the command on page~\pageref{cmd:example.btp} to use the file in the package if you do not want to create the file yourself.

The imported network looks like this:
<<eval=FALSE>>=
net
@
\begin{verbatim}
Boolean network with 5 genes

Involved genes:
Input 1 Input 2 Gene 1 Gene 2 Gene 3
\end{verbatim}
\pagebreak[4]
\begin{verbatim}
Transition functions:
Input 1 = 1
Input 2 = Input 2
Gene 1 = (!Gene 1 & !Input 1 & Input 2) | (!Gene 1 & Input 1 & !Input 2) 
         | (Gene 1 & !Input 1 & !Input 2) | (Gene 1 & Input 1 & Input 2)
Gene 2 = Gene 1 & Gene 3 & !Input 2
Gene 3 = Gene 1 | Gene 2

Knocked-out and over-expressed genes:
Input 1 = 1
\end{verbatim}

We can see that Input~1 is specified as an over-expressed constant gene. Input~2 is modeled as depending only on itself, i.e. it keeps its initial value. Gene~1 is a representation of the XOR function in Disjunctive Normal Form (DNF), using only logical ANDs, logical ORs, and negations. Gene~2 and Gene~3 consist of conjunctions and disjunctions of their inputs respectively. In addition to this textual description, we can visually verify the network by plotting its wiring:
<<eval=FALSE>>=
plotNetworkWiring(net)
@
The resulting plot is shown in Figure~\ref{fig:wiring_biotap}.
\begin{figure}[h]
\centering
\includegraphics[width=0.6\linewidth]{wiring_biotap}
\caption{The wiring graph of the imported network specified in BioTapestry.}
\label{fig:wiring_biotap}
\end{figure}

You can now use the imported network just like any other network in \texttt{BoolNet}.

\clearpage

\subsection{Exporting network simulations to Pajek}

For further analysis, network simulations can be exported to Pajek, a Windows application that provides visualization and analysis methods for graph structures \cite{batagelij98}. For more information on Pajek, please refer to \url{http://pajek.imfm.si/doku.php}.

The export function writes the state transition graph to a Pajek file (*.net). This requires a synchronous exhaustive attractor search in \texttt{BoolNet} to build the full transition table. 

To export the mammalian cell cycle network to Pajek, call
<<eval=FALSE>>=
data(cellcycle)
attr <- getAttractors(cellcycle)
toPajek(attr, file="cellcycle.net")
@

This will export the graph of the state transitions, which is usually sufficient for plotting. If you want to include the state information (i.e., the gene assignment vectors), call
<<eval=FALSE>>=
toPajek(attr, file="cellcycle.net", includeLabels=TRUE)
@

Now, start Pajek, load the network with \texttt{File | Network | Read}, and check out the tools provided by this application. For example, visualizations can be accessed using the menu item \texttt{Draw | Draw}. 

Figure~\ref{fig:pajek} shows a plot of the cell cycle network with the Kamada-Kawai layout (Menu entry \texttt{Layout | Energy | Kamada-Kawai | Separate Components}).

\begin{figure}[h]
\centering
\includegraphics[width=0.75\linewidth]{pajek}
\caption{A visualization of the mammalian cell cycle network in Pajek.}
\label{fig:pajek}
\end{figure}

\clearpage

\bibliographystyle{plain}
\bibliography{BoolNet_package_vignette}

\section{Appendix}\label{sec:appendix}

\subsection{Network file format}

This section provides a full language description for the network file format of \texttt{BoolNet}. The language is described in Extended Backus-Naur Form (EBNF).

For synchronous, asynchronous and probabilistic Boolean networks, the supported format is as follows:

\begin{verbatim}
Network           = Header Newline {Rule Newline | Comment Newline};
Header            = "targets" Separator "factors";
Rule              = GeneName Separator BooleanExpression [Separator Probability];
Comment           = "#" String;
BooleanExpression = GeneName 
                    | "!" BooleanExpression 
                    | "(" BooleanExpression ")" 
                    | BooleanExpression " & " BooleanExpression  
                    | BooleanExpression " | " BooleanExpression;
                    | "all(" BooleanExpression {"," BooleanExpression} ")"
                    | "any(" BooleanExpression {"," BooleanExpression} ")"
                    | "maj(" BooleanExpression {"," BooleanExpression} ")"
                    | "sumgt(" BooleanExpression {"," BooleanExpression} "," Integer ")"
                    | "sumlt(" BooleanExpression {"," BooleanExpression} "," Integer ")";
GeneName          = ? A gene name from the list of involved genes ?;
Separator         = ",";
Integer           = ? An integer value?;
Probability       = ? A floating-point number ?;
String            = ? Any sequence of characters (except a line break) ?;
Newline           = ? A line break character ?;
\end{verbatim}

\pagebreak[4]
The extended format for temporal networks includes additional time specifications and temporal predicates is defined as follows:

\begin{verbatim}
Network               = Header Newline
                        {Function Newline | Comment Newline};
Header                = "targets" Separator "factors";
Function              = GeneName Separator BooleanExpression;
Comment               = "#" String;
BooleanExpression     = GeneName 
                        | GeneName TemporalSpecification 
                        | BooleanOperator 
                        | TemporalOperator
BooleanOperator       =   BooleanExpression 
                        | "!" BooleanExpression 
                        | "(" BooleanExpression ")" 
                        | BooleanExpression " & " BooleanExpression  
                        | BooleanExpression " | " BooleanExpression;
TemporalOperator      =   "all" [TemporalIteratorDef] 
                                "(" BooleanExpression {"," BooleanExpression} ")"
                        | "any" [TemporalIteratorDef] 
                                "(" BooleanExpression {"," BooleanExpression} ")"
                        | "maj" [TemporalIteratorDef] 
                                "(" BooleanExpression {"," BooleanExpression} ")"
                        | "sumgt" [TemporalIteratorDef] 
                                  "(" BooleanExpression {"," BooleanExpression} "," Integer ")"
                        | "sumlt" [TemporalIteratorDef] 
                                  "(" BooleanExpression {"," BooleanExpression} "," Integer ")"
                        | "timeis" "(" Integer ")"
                        | "timegt" "(" Integer ")"
                        | "timelt" "(" Integer ")";
TemporalIteratorDef   = "[" TemporalIterator "=" Integer ".." Integer "]";
TemporalSpecification = "[" TemporalOperand {"+" TemporalOperand | "-" TemporalOperand} "]";
TemporalOperand       = TemporalIterator | Integer
TemporalIterator      = ? An alphanumeric string ?;
GeneName              = ? A gene name from the list of involved genes ?;
Separator             = ",";
Integer               = ? An integer value?;
String                = ? Any sequence of characters (except a line break) ?;
Newline               = ? A line break character ?;
\end{verbatim}

\end{document}

\documentclass[a4paper]{article}

\SweaveOpts{keep.source=TRUE}
%\VignetteIndexEntry{Detailed introduction to all major features of BoolNet}

\usepackage{a4wide}

\usepackage{graphicx}
\usepackage{amsmath}
\usepackage{hyperref}

\usepackage{listings}
\lstset{breaklines=true, breakautoindent=false, breakindent=0pt, columns=fullflexible,keepspaces=true, basicstyle=\small\ttfamily}

\setlength{\parindent}{0em}
\setlength{\parskip}{0.2em}

\title{BoolNet package vignette}
\author{Christoph M\"ussel, Martin Hopfensitz, Hans A. Kestler}

\widowpenalty=10000
\clubpenalty=10000

\hyphenation{me-thods pro-per-ties re-pre-sen-ta-tion}

\begin{document}
\SweaveOpts{concordance=TRUE}
\maketitle
\tableofcontents

\clearpage
\section{Introduction}

\texttt{BoolNet} is an R package that provides tools for assembling, analyzing and visualizing synchronous and asynchronous Boolean networks as well as probabilistic Boolean networks. This document gives an introduction to the usage of the software and includes examples for use cases.

\texttt{BoolNet} supports four types of networks:
\begin{description}
\item[Synchronous Boolean networks]{ consist of a set of Boolean variables 
\[
X = \left\{X_1, \ldots, X_n \right\} 
\]
and a set of transition functions 
\[
F=\left\{ f_{1},\ldots,f_{n}\right\},
\] 
one for each variable. These transition functions map an input of the Boolean variables in $X$ to a Boolean value ($0$ or $1$). We call a Boolean vector $\mathbf{x}(t) = \left(x_1(t), \ldots, x_n(t) \right)$ the {\em state} of the network at time $t$. Then, the next state of the network  $\mathbf{x}(t)$ is calculated by applying {\em all} transition functions $f_i(\mathbf{x}(t-1))$.

In a biological context, genes can be modeled as Boolean variables ({\em active/expressed} or {\em inactive/not expressed}), and the transition functions model the dependencies among these genes. In the synchronous model, the assumption is that all genes are updated at the same time. This simplification facilitates the analysis of the networks.}

\item[Asynchronous Boolean networks]{ have the same structure as synchronous Boolean networks. Yet, at each point of time $t$, only {\em one} of the transition functions $f_i \in F$ is chosen at random, and the corresponding Boolean variable is updated. This corresponds to the assumption that in a genetic network, gene expression levels are likely to change at different points of time. In the most common model, the gene to be updated is chosen uniformly among all genes. Moreover, \texttt{BoolNet} supports specifying non-uniform update probabilities for the genes.}

\item[Probabilistic Boolean networks (PBN)]{ allow for specifying more than one transition function per variable/gene. Each of these functions has a probability to be chosen, where the probabilities of all functions for one variable sum up to 1. Formally
\[
F=\left\{\left\{\left(f_{11}, p_{11}\right), \ldots, \left(f_{1k_1}, p_{1k_1}\right)\right\}, \ldots,  \left\{\left(f_{n1}, p_{n1}\right), \ldots, \left(f_{nk_n}, p_{nk_n}\right)\right\}\right\}
\]

where $k_i$ is the number of alternative transition functions for variable $i$, and $p_{ij}$ is the probability that function $j$ is chosen for variable $i$. A state transition is performed by selecting one function for each gene based on the probabilities and applying the chosen functions synchronously.}

\item[Temporal Boolean networks]{ are Boolean networks that incorporate temporal predicates and discrete time delays. Here, the next state $x(t)$ may not only depend on $x(t-1)$, but can depend on any predecessor state $x(t - \Delta), \Delta \in \{1, 2, \ldots\}$. Furthermore, $x(t)$ may also directly depend on the time step $t$ itself.
}

\end{description}

In \texttt{BoolNet}, there are different structure classes representing these network types:
\begin{description}
\item[\emph{BooleanNetwork} objects]{ contain synchronous and asynchronous Boolean networks. Here, the transition functions are internally represented as truth tables.}
\item[\emph{ProbabilisticBooleanNetwork} objects]{ encode Probabilistic Boolean networks. They use a truth table representation as well.}
\item[\emph{SymbolicBooleanNetwork} objects]{ represent synchronous and temporal Boolean networks. They encode Boolean functions in a symbolic form, i.e. as expression trees.}
\end{description}
As we have seen, the networks are represented in two different forms: The truth table representation, which basically maps inputs to the corresponding output values, is usually the most efficient representation for synchronous, asynchronous and probabilistic networks and uses a very fast simulator. However, this representation grows exponentially with the number of inputs and is therefore inappropriate for networks with a high number of inputs. This is particularly the case for temporal networks, where each unique time delay for a gene encodes an input. Hence, temporal networks are represented by directly encoding the corresponding Boolean expressions and use a different simulator. As synchronous Boolean networks are a special case of temporal networks (with all time delays being 1), these networks can also be represented as \emph{SymbolicBooleanNetwork} objects.

The package provides several methods of constructing networks:
Networks can be loaded from files in which human experts describe the dependencies between the genes. Furthermore, they can be reconstructed from time series of gene expression measurements. It is also possible to generate random networks. This can be helpful for the identification of distinct properties of biological networks by comparison to random structures. The different methods of assembling networks are described in Section~\ref{sec:assemblingnetworks}.

In Section~\ref{sec:networkanalysis}, tools for the analysis and visualization of network properties are introduced. For synchronous, asynchronous and temporal Boolean networks, the most important tool is the identification of attractors. Attractors are cycles of states and are assumed to be associated with the stable states of cell function. 
Another possibility of identifying relevant states is the included Markov chain simulation. This method is particularly suited for probabilistic networks and calculates the probability that a state is reached after a certain number of iterations. To test the robustness of structural properties of the networks to noise and mismeasurements, the software also includes extensive support for perturbing networks. In this way, it is possible to test these properties in noisy copies of a biological network.

In Section~\ref{sec:importexport}, the interaction of \texttt{BoolNet} with related software is described. In particular, the import from and export to SBML is discussed. Also, the necessary steps to import networks import networks from BioTapestry and to export networks to Pajek are outlined. 

For the examples in the following sections, we assume that the \texttt{BoolNet} package has been properly installed into the R environment. This can be done by typing
<<eval=FALSE>>=
install.packages("BoolNet")
@
into the R console or by the corresponding menu entries in an R GUI. For some of the plots, the \texttt{igraph} package is required and must be installed in your R environment as well. This is analogous to installing \texttt{BoolNet}. For the BioTapestry and SBML import, the \texttt{XML} package must be installed.
After installation, the \texttt{BoolNet} package can be loaded via
<<>>=
library(BoolNet)
@


\section{Assembling networks}\label{sec:assemblingnetworks}

\subsection{Assembling a network from expert knowledge}

A major advantage of Boolean networks is the fact that natural-language statements can easily be transferred into this representation. This allows researchers for building Boolean networks entirely from expert knowledge, for example by collecting statements on gene dependencies from literature and expressing them as Boolean rules. 

\texttt{BoolNet} is able to read in networks consisting of such rule sets in a standardized text file format. In such a file, each line consists of a target gene and an update rule, usually separated by a comma. Optionally, it is also possible to add a probability for the rule if the file describes a probabilistic network. The first line of such a file is a header

\begin{samepage}
\begin{verbatim}
targets, factors
\end{verbatim}

or

\begin{verbatim}
targets, factors, probabilities
\end{verbatim}
\end{samepage}

To illustrate the process of transforming natural-language statements into Boolean rules, we take a look at the mammalian cell cycle network introduced by Faur\'e et al. \cite{faure06}.  In Table 1 of this paper, the authors list natural-language statements of gene dependencies and the corresponding Boolean expressions. The following rules are taken from this table.

For gene CycD, Faur\'e et al. state:
\begin{quote}
\textit{CycD is an input, considered as constant.}
\end{quote}
Transforming this into a Boolean rule is rather simple: CycD does not change its value, which means that its value after a transition only depends on its previous value. Thus, the transition rule is

\begin{verbatim}
CycD, CycD
\end{verbatim} 
 
 Gene Rb has a more complex description:
 
 \begin{quote}
 \textit{Rb is expressed in the absence of the cyclins, which inhibit it by phosphorylation [...]; it can be expressed in the presence of CycE or CycA if their
inhibitory activity is blocked by p27.} 
 \end{quote}
 
 As a general rule, inhibition can be represented by a Boolean negation. In the \texttt{BoolNet} file format, a negation is expressed by the \texttt{!} character. The referenced cyclins comprise the genes CycA, CycB, CycD, and CycE. If {\em all} these genes are absent, Rb is expressed -- i.e. if CycA is not expressed {\em and} CycB is not expressed {\em and} CycD is not expressed {\em and} CycE is not expressed. A logical AND is embodied by the \texttt{\&} character. Consequently, the first part of the rule is
 \begin{verbatim}
 ! CycA & ! CycB & ! CycD & ! CycE
 \end{verbatim}
In combination with the above statement, the fact that Rb can be expressed in the presence of CycE and CycA if p27 is active means that CycB and CycD must not be active. Thus, the second part of the rule is
\begin{verbatim}
p27 & ! CycB & ! CycD 
\end{verbatim}
This statement is an exception (or alternative) to the first statement; this can be expressed as a logical OR, for which the \texttt{|} character is used.

The complete rule for gene Rb is thus
\begin{verbatim}
Rb, (! CycA & ! CycB & ! CycD & ! CycE) | (p27 & ! CycB & ! CycD)
\end{verbatim}

After processing all genes in the table in this way, we get the following network description:

\begin{samepage}
\begin{footnotesize}
\begin{verbatim}
targets, factors
CycD, CycD
Rb, (! CycA & ! CycB & ! CycD & ! CycE) | (p27 & ! CycB & ! CycD)
E2F, (! Rb & ! CycA & ! CycB) | (p27 & ! Rb & ! CycB)
CycE, (E2F & ! Rb)
CycA, (E2F & ! Rb & ! Cdc20 & ! (Cdh1 & UbcH10)) | (CycA & ! Rb & ! Cdc20 & ! (Cdh1 & UbcH10))
p27, (! CycD & ! CycE & ! CycA & ! CycB) | (p27 & ! (CycE & CycA) & ! CycB &! CycD)
Cdc20, CycB
Cdh1,(! CycA & ! CycB) | (Cdc20) | (p27 & ! CycB)
UbcH10, ! Cdh1 | (Cdh1 & UbcH10 & (Cdc20 | CycA | CycB))
CycB, ! Cdc20 & ! Cdh1
\end{verbatim} 
\end{footnotesize}
\end{samepage}

Now save this description to a file ``cellcycle.txt'' in your R working directory. The network can be loaded via
<<eval=FALSE>>=
cellcycle <- loadNetwork("cellcycle.txt")
@
The result is an object of class \emph{BooleanNetwork} containing a truth table representation of the network. 

The same network is also included in \texttt{BoolNet} as an example and can be accessed via
<<>>=
data(cellcycle)
@

\texttt{BoolNet} also provides several convenience operators that can be used to express complex Boolean functions in a compact way, e.g.
\begin{itemize}
\item \texttt{maj(a, b, ...)} is 1 if the majority of its operands are 1. Similarly, \texttt{sumgt(a, b, ..., N)} is 1 if more than $N$ operands are 1, \texttt{sumlt(a, b, ..., N)} is 1 if less than $N$ operands are~1, and \texttt{sumis(a, b, ..., N)}  is 1 if exactly $N$ operands are 1.
\item \texttt{all(a, b, ...)} is 1 if all its operands are 1 (i.e. a logical AND), and \texttt{any(a, b, ...)} is~1 if at least one of its operands is 1 (i.e. a logical OR).
\end{itemize}

The cell cycle network is a classical Boolean network, where each transition function only depends on the previous state of the network. E.g., \verb+CycB, ! Cdc20 & ! Cdh1+ can be written formally as $CycB(t) = \neg Cdc20(t-1)  \wedge \neg Cdh1(t-1)$. As already discussed before, \texttt{BoolNet} also incorporates several temporal extensions. For example, a transition function can also depend on the states of genes at earlier time points:
\begin{verbatim}
a, b[-3] & b[-2] & b
\end{verbatim}
is 1 if b has been active in the previous three time steps. The operators described above can also incorporate time ranges. The previous statement can be written in a more compact way using the \texttt{all} operator:
\begin{verbatim}
a, all[d=-3..-1](b[d])
\end{verbatim}
This defines a time delay variable \texttt{d} that can be used for time specifications inside the operator. It can also be used in arithmetic operations. E.g.,
\begin{verbatim}
a, all[d=-3..-1](b[d] & c[d-1])
\end{verbatim}
specifies that a is active if b has been active in the previous three time steps and c has been active at time $t-4$, $t-3$ and $t-2$. 

Apart from relative time specifications, the \texttt{BoolNet} network language also incorporates predicates that depend on the absolute time, i.e. the number of time steps that have elapsed since the initial state. 

\begin{samepage}
For example,
\begin{verbatim}
a, timeis(3)
\end{verbatim}
specifies that \texttt{a} is active at time step 3 and inactive at all other time steps. Similarly, the predicates \texttt{timelt} and \texttt{timegt} evaluate to 1 before and after the specified time point respectively.
\end{samepage}

As the above examples do not cover all possibilities of the network description language, a full language specification is provided in Section~\ref{sec:appendix}.

For temporal networks, \texttt{BoolNet} uses a special symbolic simulator that represents the functions as expression trees, whereas the standard simulator is based on a truth table representation. These simulators are discussed in Section~\ref{sec:networkanalysis}. As synchronous Boolean networks are a special case of temporal networks, they can also be simulated with the symbolic simulator. When a network is loaded from a file using \texttt{loadNetwork()}, the user can specify the parameter \texttt{symbolic=TRUE} to load it in form of a \emph{SymbolicBooleanNetwork} object instead of a \emph{BooleanNetwork} object. The same parameter is also available for the import functions discussed in Section~\ref{sec:importexport}. Temporal networks can only be loaded with \texttt{symbolic=TRUE}, as \texttt{BoolNet} cannot represent them as truth tables.

\begin{sloppypar}
As many network generation and modification routines (such as random network generation and network reconstruction that are discussed in the following sections) internally use the truth table representation, there are conversion routines \texttt{truthTableToSymbolic()} and \texttt{symbolicToTruthTable()} that convert synchronous Boolean networks of class \emph{BooleanNetwork} in a truth table representation to networks of class \emph{SymbolicBooleanNetwork} in a symbolic representation and vice versa. For more details, please refer to the manual.
\end{sloppypar}

\subsection{Reconstructing a network from time series}

An entirely different approach of assembling a network is to infer rules from series of expression measurements of the involved genes over time. For example, microarray experiments can be conducted at different points of time to cover the expression levels of different cell states. To reconstruct networks from such data, \texttt{BoolNet} includes two  
reconstruction algorithms for synchronous Boolean networks, Best-Fit Extension~\cite{laehdesmaeki03} and REVEAL \cite{liang98}. REVEAL requires the inferred functions to match the input time series perfectly, hence it is not always able to reconstruct networks in the presence of noisy and inconsistent measurements. Best-Fit Extension retrieves a set of functions with minimum error on the input and is thus suited for noisy data. 

In the following, we introduce a tool chain for the reconstruction of a Probabilistic Boolean Network from time series using Best-Fit extension.

Microarray measurements are usually represented as matrices of real-valued numbers which, for example, quantify the expression levels of genes. \texttt{BoolNet} includes a real-valued time series of gene measurements  from a project to analyze the yeast cell cycle \cite{spellman98} which can be loaded using
<<>>=
data(yeastTimeSeries)
@
This data contains four preselected genes and a series of 14 measurements for each of these genes.

\begin{sloppypar}
In a first step, the real-valued dataset has to be converted to binary data as required by the reconstruction algorithm. \texttt{BoolNet} offers several binarization algorithms in the function \texttt{binarizeTimeSeries()}. We here employ the default method which is based on $k$-means clustering (with $k=2$ for active and inactive):
\end{sloppypar}
<<>>=
binSeries <- binarizeTimeSeries(yeastTimeSeries)
@
The returned structure in \texttt{binSeries} has an element \texttt{\$binarizedMeasurements} containing the binary time series, and, depending on the chosen binarization method, some other elements describing parameters of the binarization.

To reconstruct the network from this data, we call the Best-Fit Extension algorithm:
<<>>=
net <- reconstructNetwork(binSeries$binarizedMeasurements, 
                          method="bestfit", 
                          maxK=4)
@
Here, \texttt{maxK} is the maximum number of input genes for a gene examined by the algorithm. The higher this number, the higher is the runtime and memory consumption of the reconstruction.

\begin{samepage}
We can now take a look at the network using
<<>>=
net
@
\end{samepage}

\begin{sloppypar}
The dependencies among the genes in the network can be visualized using the \texttt{plotNetworkWiring()} function. In this graph, each gene corresponds to a vertex, and the inputs of transition functions correspond to edges.
\end{sloppypar}
<<echo=FALSE,results=hide>>=
pdf("wiring1.pdf") 
par(mar = c(1, 1, 1, 1))
set.seed(333)
plotNetworkWiring(net)
dev.off()
@
<<eval=FALSE>>=
plotNetworkWiring(net)
@
plots a graph similar to that at the top of Figure~\ref{fig:wiring}. To use this function, you must install the \texttt{igraph} package.

\begin{figure}[p]
\centering
\includegraphics[width=0.5\linewidth]{wiring1}
\includegraphics[width=0.5\linewidth]{wiring2}
\caption{The wiring graph of the reconstructed network without prior knowledge (top) and with the inclusion of prior knowledge (bottom). Each node of the graph represents one gene, and each arrow represents a gene dependency.}
\label{fig:wiring}
\end{figure}

A network that involved the same genes was examined by Kim et al. \cite{kim07}. When comparing the wiring graph of our reconstructed network with the reference network presented in Figure~2 of this paper, one observes a very high similarity between the two networks. However, the reconstructed network comprises too many links for the gene Sic1: The reference network does not contain a self-regulation of Sic1 or regulation of Sic1 by Fkh2. If it is known in advance that these regulations are not plausible, such prior knowledge can be supplied to the reconstruction algorithm:

<<>>=
net <- reconstructNetwork(binSeries$binarizedMeasurements, 
                          method="bestfit", 
                          maxK=4, 
                          excludedDependencies = list("Sic1" = c("Sic1", "Fkh2")))
@
<<echo=FALSE,results=hide>>=
pdf("wiring2.pdf") 
par(mar = c(1, 1, 1, 1))
set.seed(442)
plotNetworkWiring(net)
dev.off()
@

The wiring of the reconstruction with prior knowledge is shown at the bottom of Figure~\ref{fig:wiring}. We can see that the two false links are now eliminated. Similar to \texttt{excludedDependencies}, there is also a parameter \texttt{requiredDependencies} that specifies dependencies that must be included in the network.

When \texttt{reconstructNetwork()} discovers multiple functions for a gene with the minimum error on the input data, it includes all of these functions as alternative functions with equal probability. Consequently, the function returns a \texttt{ProbabilisticBooleanNetwork} structure.

If you would like to obtain a \texttt{BooleanNetwork} object with only one function per gene from a probabilistic network, you can extract such a network by telling the software which of the functions you would like to use. This can be done by specifying the indices of the functions to extract:
<<>>=
net <- reconstructNetwork(binSeries$binarizedMeasurements, 
method="bestfit", maxK=4)
singleNet <- chooseNetwork(net, c(1,2,3,2))
@
The result is a Boolean network that is created by extracting the first function of gene Fkh2, the second function of genes Swi5 and Clb1, and the third function of gene Sic1 from the above probabilistic network:
<<>>=
singleNet
@

\begin{sloppypar}
\texttt{BoolNet} also supports the generation of artificial time series from existing networks: The \texttt{generateTimeSeries()} function generates a set of time series from a network using random start states and optionally adds Gaussian noise.
\end{sloppypar}
<<echo=FALSE,results=hide>>=
set.seed(3176)
@
<<>>=
series <- generateTimeSeries(cellcycle, 
                             numSeries=100, 
                             numMeasurements=10, 
                             noiseLevel=0.1)
@
generates a list of 100 time series by calculating 10 consecutive transitions from 100 randomly chosen network states in the mammalian cell cycle network. The series are subject to Gaussian noise with a standard deviation of 0.1, such that the result is a list of real-valued matrices.

We can now binarize these simulated measurements and try to reconstruct the original network:
<<>>=
binSeries <- binarizeTimeSeries(series, method="kmeans")
net <- reconstructNetwork(binSeries$binarizedMeasurements, method="bestfit")
net
@

Obviously, the number of generated time series is still to small to reconstruct the network unambiguously. However, the result comes close to the original network. We see that the functions of the network are not fully specified: At some positions, there are asterisks (\texttt{*}) denoting a \emph{don't care} value. This means that functions with a 0 and a 1 at this position match the time series equally well. That is, a partially defined function with $m$ asterisks corresponds to $2^m$ fully defined Boolean functions. It is also possible to generate these fully defined functions instead of the partially defined function by setting the parameter \texttt{returnPBN} to true (which was the behaviour prior to \texttt{BoolNet} version 2.0). For many \emph{don't care} values, this may consume a high amount of memory and computation time. 

\texttt{generateTimeSeries()} can also generate time series with artificial knock-outs and overexpressions:
<<echo=FALSE,results=hide>>=
set.seed(4463)
@
<<>>=
series <- generateTimeSeries(cellcycle, 
                             numSeries=10, 
                             numMeasurements=10, 
                             perturbations=1,
                             noiseLevel=0.1)
@
specifies that each generated time series is generated from a network where one randomly selected gene is artificially knocked down (constantly 0) or overexpressed (constantly 1). These perturbations are returned in an additional element \texttt{perturbations}. 
\begin{samepage}
<<<>>=
series$perturbations
@
\end{samepage}
Here, each column corresponds to the perturbations applied in one series. A value of 1 denotes an overexpression, a value of 0 denotes a knock-out, and an N/A value means that no perturbation was applied to this gene.

The \texttt{reconstructNetwork()} function also supports the reconstruction from such perturbation experiments if it is known which genes were perturbed. First, we store the series and the perturbation matrix in separate variables and binarize the data as before:
<<>>=
perturbations <- series$perturbations
series$perturbations <- NULL

binSeries <- binarizeTimeSeries(series, method="kmeans")
@
Now, we can reconstruct the network by specifying the \texttt{perturbations} parameter:
<<>>=
net <- reconstructNetwork(binSeries$binarizedMeasurements, 
                          method="bestfit", 
                          perturbations=perturbations)
net
@
As we generated only 10 series in this case, the reconstructed network is much more incomplete than in the previous reconstruction. 

In biological settings, perturbation experiments are probably one of the most frequent ways of exploring the behaviour of a regulatory network, as it is much easier to obtain various different responses by applying perturbations than by just measuring the wild type behaviour. 

\subsection{Creating random networks}

To study structural properties of Boolean networks and to determine the specific properties of biological networks in comparison to arbitrary networks, it is often desirable to generate artificial networks. \texttt{BoolNet} comprises a facility for the generation of random $N$-$K$ networks \cite{kauffman69, kauffman93}. In the standard $N$-$K$ networks, $N$ is the total number of genes, and $K$ is the number of input genes for each gene transition function. Such a network can be generated using
<<>>=
net <- generateRandomNKNetwork(n=10, k=3)
@
This creates a network with 10 genes, each of which has a transition function that depends on 3 genes and whose output is generated uniformly at random. Similarly, one can also specify different numbers of input genes for each gene:
<<>>=
net <- generateRandomNKNetwork(n=10, k=c(1,2,3,1,3,2,3,2,1,1))
@
\texttt{BoolNet} does not only support this standard case, but allows for different methods of choosing the numbers of input genes (parameter \texttt{topology}), the input genes themselves (parameter \texttt{linkage}), and the transition functions (parameter \texttt{functionGeneration}). In the following, some examples are presented. 

The command
<<>>=
net <- generateRandomNKNetwork(n=20, k=20, topology="scale_free")
@
determines the numbers of input genes by drawing values from the scale-free Zeta distribution \cite{aldana03}. According to this distribution, most transition functions will have a small number of input genes, but a few transition functions may depend on a high number of genes. The shape of the Zeta distribution can be customized using an additional parameter \texttt{gamma}, which potentially increases the number of input genes when chosen small and vice versa.

<<>>=
net <- generateRandomNKNetwork(n=10, k=3, linkage="lattice")
@
creates a network in which the transition functions of the genes depend on a choice of genes with adjacent indices \cite{aldana03_2}. This leads to networks with highly interdependent genes.

It is also possible to influence the truth tables of the functions in several ways. The parameter \texttt{zeroBias} changes the ratio of 1 and 0 returned by the functions: 
<<>>=
net <- generateRandomNKNetwork(n=10, 
                               k=3, 
                               functionGeneration="biased", 
                               zeroBias=0.75)
@
generates a network in which the outcome of a transition function is 0 for around 75\% of the inputs.

A more intricate way of influencing the function generation is the specification of generation functions. Generation functions explicitly generate functions according to a specific function class. 

\emph{Canalyzing functions} are assumed to occur frequently in biological systems~\cite{kauffman04}. A canalyzing function has the property that one input can determine the output value on its own, i.e. if this input is either active or inactive, the output of the function is always the same. \emph{Nested canalyzing functions} are a recursive definition of canalyzing functions, where the part of the function that does not depend on the canalyzing input is a canalyzing function for another input. Such functions can be generated using the built-in generation functions \texttt{generateCanalyzing()} and \texttt{generateNestedCanalyzing()}:
<<>>=
net1 <- generateRandomNKNetwork(n=10, 
                                k=3,
                                functionGeneration=generateCanalyzing, 
                                zeroBias=0.75)
net2 <- generateRandomNKNetwork(n=10, 
                                k=3, 
                                functionGeneration=generateNestedCanalyzing, 
                                zeroBias=0.75)
@
It is also possible to define own generation functions: A generation function receives a vector \texttt{input} of input gene indices as a parameter and returns a truth table result column with \verb+2^length(input)+ values representing the function.

If no explicit generation scheme is known for the function class of interest, validation functions can be used instead of generation functions. Validation functions verify whether randomly generated functions belong to a specific function class and reject invalid functions. Naturally, this is much less efficient than generating appropriate functions directly.
An example is the generation of monotone functions, which are also thought to be biologically plausible. These function account for the assumption that a transcription factor usually either activates or inhibits a specific target gene, but does not change the type of regulation depending on other factors~\cite{maucher11}. We can specify a simple validation function that checks whether a Boolean function is monotone:

\begin{samepage}
<<>>=
isMonotone <- function(input, func)
{
  for (i in seq_len(ncol(input)))
  # check each input gene
  {
    groupResults <- split(func, input[,i])
    if (any(groupResults[[1]] < groupResults[[2]]) && 
        any(groupResults[[1]] > groupResults[[2]]))
      # the function is not monotone
      return(FALSE)
  }
  return(TRUE)
}
@
\end{samepage}
\begin{sloppypar}
Here, \texttt{input} is a matrix containing the input part of the transition table, and \texttt{func} is the output of the Boolean function. In a monotone function, the values of the target may only change in one direction when switching one input. Hence, a function for which switching the value of an input gene sometimes switches the target from active to inactive, but also sometimes switches it from inactive to active is not monotone. This is validated by comparing the two groups of transition table entries for which the current input is active and inactive respectively.

If a validation function is supplied to \texttt{generateRandomNKNetwork()}, the generator generates Boolean functions until either the validation function returns \texttt{TRUE} or the maximum number of iterations (specified by the parameter \texttt{failureIterations}) is reached, in which case it fails.
\end{sloppypar}

<<>>=
net <- generateRandomNKNetwork(n=10,
                               k=3, 
                               validationFunction="isMonotone", 
                               failureIterations=1000)
@
creates a network with 10 genes in which all functions have 3 inputs, of which at least one is canalyzing.

By default, \texttt{generateRandomNKNetwork()} creates functions that cannot be simplified, i.e. that do not contain any genes that are irrelevant for the outcome of the function. If desired, this behaviour can be changed by setting \texttt{noIrrelevantGenes} to \texttt{FALSE}.

The presented parameters can be combined, and there are further options and parameters, so that a broad variety of networks with different structural properties can be generated. For a full reference of the possible parameters, please refer to the manual.

\subsection{Knock-out and overexpression of genes}

\texttt{BoolNet} allows for temporarily knocking out and overexpressing genes in a network without touching the transition functions. This means that genes can be set to a fixed value, and in any calculation on the network, this fixed value is taken instead of the value of the corresponding transition function. Knocked-out and overexpressed genes speed up the analysis of the network, as they can be ignored in many calculations. For example, to knock out CycD in the mammalian cell cycle network, we call
<<>>=
data(cellcycle)
knockedOut <- fixGenes(cellcycle, "CycD", 0)
@
or alternatively use the gene index
<<>>=
knockedOut <- fixGenes(cellcycle, 1, 0)
@
This sets the gene constantly to 0. To over-express the gene (i.e. to fix it to 1), the corresponding call is
<<>>=
overExpressed <- fixGenes(cellcycle, "CycD", 1)
@
The command
<<>>=
originalNet <- fixGenes(knockedOut, "CycD", -1)
@
reactivates the gene (for both knock and overexpression) and resets the network to its original state. 

The function also accepts multiple genes in a single call, such as
<<>>=
newNet <- fixGenes(cellcycle, c("CycD","CycE"), c(0,1))
@
which knocks out CycD and overexpresses CycE.

\section{Network analysis}\label{sec:networkanalysis}

\subsection{Simulation of state transitions}

To simulate a state transition and identify successor states of a given state, \texttt{BoolNet} includes the function \texttt{stateTransition()}. The function supports transitions for all four types of networks.

The following code performs a synchronous state transition for the state in which all genes are set to 1 on the mammalian cell cycle network:
<<>>=
data(cellcycle)
stateTransition(cellcycle, rep(1,10))
@

To calculate all state transitions in a synchronous network until an attractor is reached, you can call
<<>>=
path <- getPathToAttractor(cellcycle, rep(0,10))
path
@
The returned matrix consists of the subsequent states until an attractor is reached. Depending on the optional parameter \texttt{includeAttractorStates}, the sequence comprises all attractor states, only the first attractor state or none of the attractor states.

A sequence can be visualized by plotting a table of state changes:
<<echo=FALSE,results=hide>>=
pdf("sequence.pdf")
par(mar = c(1, 4, 2, 1))
plotSequence(sequence = path)
dev.off()
@
<<eval=FALSE>>=
plotSequence(sequence=path)
@

\begin{figure}[t]
\centering
\includegraphics[width=0.6\linewidth]{sequence}
\caption{Visualization of a sequence of states in the mammalian cell cycle network. The columns of the table represent consecutive states of the time series. The last state is the steady-state attractor of the network.}
\label{fig:sequence}
\end{figure}

The result is depicted in Figure~\ref{fig:sequence}. \texttt{plotSequence()} also includes a shortcut that calculates the sequence directly if a network and a start state are supplied. It also provides an alternative way of visualizing the sequence as a state transition by setting \texttt{mode="graph"}.

The function
<<eval=FALSE>>=
sequenceToLaTeX(sequence=path, file="sequence.tex")
@
creates a \LaTeX\ table similar to \texttt{plotSequence()} function document.

In many cases, start states are defined by a set of active genes. Instead of supplying a full state vector, one can also supply only these active genes using the \texttt{generateState()} function.
<<>>=
startState <- generateState(cellcycle, specs=c("CycD"=1,"CycA"=1))
stateTransition(cellcycle,startState)
@
\begin{sloppypar}
calculates a state transition starting from a state where only the genes \emph{CycD} and \emph{CycA} are active, while all other genes are inactive (which is controlled by the \texttt{default} parameter of \texttt{generateState()}). 
\end{sloppypar}

For temporal Boolean networks (objects of class \emph{SymbolicBooleanNetworks}), the above functions can be utilized mostly in the same way. We demonstrate this using a small temporal network example of the IGF (Insulin-like growth receptor) pathway that is included in the package and can be loaded via
<<>>=
data(igf)
@
The model illustrates the activation and feedback inhibition of the PI3K-Akt-mTOR signalling cascade through IGF and IRS.

A state transition from the initial state in which the trigger of the pathway -- IGF -- is active, can be performed using
<<>>=
startState <- generateState(igf, specs=c("IGF"=1))
stateTransition(igf, startState)
@
The IGF network incorporates time delays of up to 3. Therefore, the last three states have to be known to calculate a successor state. If only a single state is supplied -- as above -- the function assumes that the state was the same before. This can be seen when calculating the sequence to the attractor using
<<>>=
getPathToAttractor(network=igf,state=startState)
@

Here, the states at $t=-2, \ldots 0$ are the same as the generated start state.
If it is required to specify multiple predecessor states here, a matrix of states can be supplied instead of a vector. E.g.,  
<<>>=
startState <- generateState(igf, specs=list("IGF"=c(0,0,1)))

startState
@
specifies that the first two states ($t=-2, t=-1$) should have all genes inactive, while IGF is activated only at $t=0$. This time, we plot the sequence instead of printing it:
\begin{figure}[bt]
\centering
\includegraphics[width=0.6\linewidth]{sequence_igf}
\caption{Visualization of a sequence of states in the IGF network. The columns of the table represent consecutive states of the time series. After activation of IGF, the attractor consisting of 14 states is entered immediately. This attractor represents the activation and inactivation of the PI3K-Akt-mTOR signalling cascade through IGF and IRS.}
\label{fig:sequence_igf}
\end{figure}

<<echo=FALSE,results=hide>>=
pdf("sequence_igf.pdf")
par(mar = c(1, 9, 2, 1))
plotSequence(network=igf, startState=startState)
dev.off()
@
<<eval=FALSE>>=
plotSequence(network=igf, startState=startState)
@
The result is depicted in Figure~\ref{fig:sequence_igf}.


\texttt{stateTransition()} can also perform asynchronous updates. A random asynchronous transition is performed using
<<echo=FALSE>>=
set.seed(54321)
@
<<>>=
stateTransition(cellcycle, rep(1,10), type="asynchronous")
@
In this case, the fifth gene, CycA, was chosen at uniformly at random and updated. 

\begin{samepage}
We can also specify non-uniform probabilities for the genes, for example
<<echo=FALSE>>=
set.seed(4321)
@
<<>>=
stateTransition(cellcycle, rep(1,10), type="asynchronous", 
geneProbabilities=c(0.05,0.05,0.2,0.3,0.05,0.05,0.05,0.05,0.1,0.1))
@
This obviously increases probabilities for the genes 3 and 4 (E2F and CycE) to be chosen. In this case, CycE was chosen for the update.
\end{samepage}

\enlargethispage{0.5cm}
Sometimes you do not want a random update at all, but would like to specify which gene should be chosen for the update. This is possible via
<<>>=
stateTransition(cellcycle, rep(1,10), type="asynchronous", 
chosenGene="CycE")
@

In probabilistic Boolean networks, a state transition is performed by choosing one of the alternative functions for each gene and applying this set of functions to the current state. The following performs a state transition with a randomly chosen set of functions on the artificial probabilistic Boolean network taken from \cite{shmulevich02} with 3 genes, starting from state (0,1,1):
<<echo=FALSE>>=
set.seed(432)
@
<<>>=
data(examplePBN)
stateTransition(examplePBN, c(0,1,1), type="probabilistic")
@
You may get a different result, as the functions are chosen randomly according to the probabilities stored in the network. If you would like to execute a specific set of transition functions, you can supply this in an additional parameter:
<<>>=
stateTransition(examplePBN, c(0,1,1), type="probabilistic", 
chosenFunctions=c(2,1,2))
@
This call uses the second function for gene x1 and x3 and the first function for gene x2.  

\subsection{Identification of attractors}

Attractors are stable cycles of states in a Boolean network. As they comprise the states in which the network resides most of the time, attractors in models of gene-regulatory networks are expected to be linked to phenotypes \cite{kauffman93,li04}. Transitions from all states in a Boolean network eventually lead to an attractor, as the number of states in a network is finite. All states that lead to a certain attractor form its {\em basin of attraction}. \texttt{BoolNet} is able to identify attractors in synchronous and asynchronous Boolean networks. There are three types of attractors in these networks:
\begin{description}
\item[Simple attractors]{occur in synchronous and temporal Boolean networks and consist of a set of states whose synchronous transitions form a cycle.}
\item[Complex or loose attractors]{are the counterpart of simple attractors in asynchronous networks. As there is usually more than one possible transition for each state in an asynchronous network, a complex attractor is formed by two or more overlapping loops. Precisely, a complex attractor is a set of states in which all asynchronous state transitions lead to another state in the set, and a state in the set can be reached from all other states in the set.}
\item[Steady-state attractors]{are attractors that consist of only one state. All transitions from this state result in the state itself. These attractors are the same both for synchronous and asynchronous update of a network. Steady states are a special case of both simple attractors and complex attractors.}
\end{description}

The \texttt{getAttractors()} function incorporates several methods for the identification of attractors in synchronous and asynchronous networks. We present these methods using the included mammalian cell cycle network as an example. This network has one steady-state attractor, one simple synchronous attractor consisting of 7 states, and one complex asynchronous attractor with 112 states (see \cite{faure06}).

We first demonstrate the use of exhaustive synchronous state space search. This means that the software starts from all possible states of the network and performs synchronous state transitions until a simple or steady-state attractor is reached. 

\pagebreak[4]
<<eval=FALSE>>=
data(cellcycle)
attr <- getAttractors(cellcycle)
attr
@
\begin{lstlisting}[linewidth=.9\linewidth]
<<echo=FALSE,results=tex>>=
attr <- getAttractors(cellcycle)
attr
@
\end{lstlisting}


Typing \texttt{attr} calls a special print method that presents the attractor in a human-readable way. Here, a state in an attractor is represented by a binary vector, where each entry of the vector codes for one gene. An alternative is to print only the names of the active genes (i.e., the genes that are set to 1) instead of the full vector by calling the \texttt{print()} method explicitly with a changed parameter:
<<eval=FALSE>>=
print(attr, activeOnly=TRUE)
@
\begin{lstlisting}[linewidth=.9\linewidth]
<<echo=FALSE,results=tex>>=
print(attr, activeOnly=TRUE)
@
\end{lstlisting}

We can see that the search identified both synchronous attractors. 

\begin{sloppypar}
The \texttt{AttractorInfo} structure stores the attractors in an encoded form. The function \texttt{getAttractorSequence()} can be used to obtain the sequence of states that constitute a specific synchronous attractor as a table:
\end{sloppypar}
<<>>=
getAttractorSequence(attr, 2)
@
retrieves the states that make up the second (i.e., the 7-state attractor) as a data frame with the genes in the columns and the successive states in the rows.

\begin{figure}[t]
\centering
\includegraphics[width=0.65\linewidth]{attractor1}
\caption{Visualization of the state changes in an attractor. The columns of the table represent consecutive states of the attractor. On top, the percentage of states leading to the attractor is supplied.}
\label{fig:attractor1}
\end{figure}

Synchronous attractors can also be visualized by plotting a table of changes of gene values in the states of the attractor: 
<<eval=FALSE>>=
plotAttractors(attr, subset=2)
@
<<echo=FALSE,results=hide>>=
pdf("attractor1.pdf")
par(mar=c(1,5,1,1))
plotAttractors(attr, subset=2)
dev.off()
@
plots the state changes of the simple attractor with 7 states, as depicted in Figure~\ref{fig:attractor1}. Similarly,
<<eval=FALSE>>=
attractorsToLaTeX(attr, subset=2, file="attractors.tex")
@
exports the same state table to a \LaTeX\ document.

The advantage of the exhaustive search method is that apart from the attractors themselves, the complete transition table is calculated and stored in the return value. This table comprises information that is used by a number of analysis methods described below.

You can extract the transition table in a data frame and print it out using
<<eval=FALSE>>=
tt <- getTransitionTable(attr)
tt
@
\begin{verbatim}
     State      Next state  Attr. basin  # trans. to attr.
0000000000 =>   0110010111            1                  4
[...]
1111111111 =>   1000001110            2                  1

Genes are encoded in the following order: CycD Rb E2F CycE 
CycA p27 Cdc20 Cdh1 UbcH10 CycB
\end{verbatim}
In the printed table, the first column denotes the initial state, the second column contains the state after the transition, the first column contains the number of the attractor that is finally reached from this state, and the fourth column lists the number of state transitions required to attain this attractor.

A table of the same structure is returned by 
<<eval=FALSE>>=
getBasinOfAttraction(attr, 1)
@
which extracts all states from the transition table that belong to the basin of attraction of attractor one (i.e., whose attractor number in column 3 is 1).

If you are interested in information on a single state (here: the state with all genes set to 1), you can type
<<eval=FALSE>>=
getStateSummary(attr, c(1,1,1,1,1,1,1,1,1,1))
@
\begin{verbatim}
     State      Next state  Attr. basin  # trans. to attr.
1111111111 =>   1000001110            2                  1

Genes are encoded in the following order: CycD Rb E2F CycE 
CycA p27 Cdc20 Cdh1 UbcH10 CycB
\end{verbatim}

The visualization function \texttt{getStateGraph()} makes use of the transition table as well: It plots a transition graph in which the basins of attraction are drawn in different colors, and the attractors are highlighted.
The result of
<<echo=FALSE,results=hide>>=
pdf("stategraph1.pdf")
set.seed(43210)
par(mar=c(1,1,1,1))
plotStateGraph(attr)
dev.off()
@
<<eval=FALSE>>=
plotStateGraph(attr)
@
is depicted at the top of Figure~\ref{fig:stategraph}. The blue basin belongs to attractor 1, and the green basin belongs to attractor 2.

The above call does not ensure that the basins of attraction are clearly separated in the plot. If this is desired, one can choose to use a piecewise layout, which means that the layouting function is applied separately to each basin of attraction, and the basins are drawn side by side.
The result of
<<echo=FALSE,results=hide>>=
pdf("piecewisestategraph.pdf")
set.seed(43210)
par(mar=c(1,1,1,1))
plotStateGraph(attr, piecewise=TRUE)
dev.off()
@
<<eval=FALSE>>=
plotStateGraph(attr, piecewise=TRUE)
@
is depicted at the bottom of Figure~\ref{fig:stategraph}.

\begin{figure}[p]
\centering
\includegraphics[width=0.7\linewidth]{stategraph1}
\includegraphics[width=0.7\linewidth]{piecewisestategraph}
\caption{The state graph of the mammalian cell cycle network using the regular layout (top) and using a piecewise layout (bottom). Each node represents a state of the network, and each arrow is a state transition. The colors mark different basins of attraction. Attractors are highlighted using bold lines.}
\label{fig:stategraph}
\end{figure}

Exhaustive search consumes a high amount of time and memory with increasing size of the network, which makes it intractable for large networks (\texttt{BoolNet} currently supports networks with up to 29 genes for exhaustive state space search). Therefore, \texttt{BoolNet} also allows for other search techniques that can handle larger networks. Depending on the requirements, the following options are available:
\begin{description}
  \item[Heuristic state space search] basically uses the same technique as exhaustive attractor search, but only uses a subset of states as initial states instead of the full state space. It then identifies the attractors to which state transitions from these states lead. The start states can either be supplied, or they can be calculated randomly.
  \item[Exhaustive SAT-based search] exhaustively identifies all possible
  attractors of a network based on a formulation of attractor search as a
  satisfiability (SAT) problem and then applies the PicoSAT solver
  \cite{biere08} to the problem. The algorithm is an adaption of that of Dubrova and Teslenko \cite{dubrova11}. Unlike exhaustive state space search, this method only returns the attractors, but not the state transition table/graph. Depending on the network structure, it can be applied to networks comprising several hundreds of genes.
  \item[Restricted SAT-based search] uses another formulation as a satisfiability problem and only identifies attractors of a predefined maximum cycle length. It is exhaustive with regard to attractors of at most this size, but will never return any attractors that comprise more than the predefined number of states. Like the exhaustive SAT-based search, this method does not return a transition table.
\end{description}
The following shows the application of the above methods exemplarily.

A heuristic state space search from randomly chosen start states in the mammalian cell cycle network can be started via
<<>>=
attr <- getAttractors(cellcycle, method="random", startStates=100)
@
This chooses 100 random start states for the heuristic search and usually identifies both attractors.

<<>>=
attr <- getAttractors(cellcycle, 
                      method="chosen", 
                      startStates=list(rep(0,10),rep(1,10)))
@
starts from the states \texttt{(0,0,0,0,0,0,0,0,0,0)} and \texttt{(1,1,1,1,1,1,1,1,1,1)} and again identifies both synchronous attractors.
 
\begin{sloppypar}
For the above calls, only the subset of the transition table traversed by the heuristic search is returned. This means that there is no guarantee that, e.g. \texttt{getBasinOfAttraction()} returns the complete basin of attraction of an attractor in heuristic mode.
\end{sloppypar}

An exhaustive SAT-based search can be started using 
<<>>=
attr <- getAttractors(cellcycle, 
                      method="sat.exhaustive")
@
This will again return all attractors, but no transition table. 

If you are only interested in a subset of attractors that have a predefined length, the restricted SAT-based search can be applied. For example,
<<>>=
attr <- getAttractors(cellcycle, 
                      method="sat.restricted",
                      maxAttractorLength=1)
@
restricts the attractor search to steady states. Correspondingly, only the first attractor of the network is returned:
<<eval=FALSE>>=
attr
@
\begin{lstlisting}[linewidth=.9\linewidth]
<<echo=FALSE,results=tex>>=
attr
@
\end{lstlisting}

\texttt{BoolNet} can also identify attractors in asynchronous Boolean network.
For this purpose, another special heuristic algorithm is included. This algorithm starts from a small subset of states and makes a number of random transitions to reach an attractor with a high probability. After that, a validation step is performed to analyze whether a complex attractor has been identified.

The command
<<results=hide>>=
attr <- getAttractors(cellcycle, 
                      type="asynchronous",
                      method="random", 
                      startStates=500)
@
conducts an asynchronous search with 500 random start states on the mammalian cell cycle network.
In this case, the algorithm has identified both the steady-state attractor and the complex attractor:

\enlargethispage{0.5cm}
<<eval=FALSE>>=
attr
@
\begin{verbatim}
Attractor 1 is a simple attractor consisting of 1 state(s):

 |--<---------|
 V            |
 0100010100   |
 V            |
 |-->---------|

Genes are encoded in the following order: CycD Rb E2F CycE 
CycA p27 Cdc20 Cdh1 UbcH10 CycB

Attractor 2 is a complex/loose attractor consisting of 112 state(s) 
and 338 transition(s):

1011101111 => 1011101110
[...]
1000000000 => 1010000000

Genes are encoded in the following order: CycD Rb E2F CycE 
CycA p27 Cdc20 Cdh1 UbcH10 CycB
\end{verbatim}

For the complex attractor, the involved transitions are printed out. By default, the algorithm tries to avoid self-loops, i.e. transitions that lead to the same state again. This means that self-loop transitions are only allowed if there is no other transition that leads to a different state. If you would like to allow the algorithm to enter self-loops even if transitions to different states are possible, you can call
<<eval=FALSE>>=
attr <- getAttractors(cellcycle, 
                      type="asynchronous",
                      method="random", 
                      startStates=500, 
                      avoidSelfLoops=FALSE)
@
In the resulting complex attractor with 112 states, there are 450 transitions instead of 338 transitions, which is due to the additional self-loops. 

The asynchronous heuristic search does not return a transition table, such that the above analysis methods cannot be applied here.


\begin{figure}[tb]
\centering
\includegraphics[width=0.55\linewidth]{attractor2}
\caption{Graph representation of the complex attractor in the mammalian cell cycle network. Each node represents a state of the complex attractor, and each arrow represents a state transition.}
\label{fig:attractor2}
\end{figure}

As there are multiple possible transitions for each state, complex attractors cannot be visualized as in Figure~\ref{fig:attractor1}. For this reason, \texttt{plotAttractors()} supports a graph mode that visualizes the transitions among the states in the attractor:
<<echo=FALSE,results=hide>>=
pdf("attractor2.pdf")
par(mar=c(1,1,1,1))
plotAttractors(attr, subset=2, mode="graph", drawLabels=FALSE)
dev.off()
@
<<eval=FALSE>>=
plotAttractors(attr, subset=2, mode="graph", drawLabels=FALSE)
@
plots the 112-state attractor as depicted in Figure~\ref{fig:attractor2}. We omit the state labels (i.e. the gene values) due to the high number of states. This plot again requires the \texttt{igraph} package.

\begin{sloppypar}
Although \texttt{getAttractors()} can also be applied to temporal networks and other networks that are in a symbolic representation (i.e. \emph{SymbolicBooleanNetwork} objects), this function is only a shortcut to the  simulation function \texttt{simulateSymbolicModel()} in this case. It is advised to use \texttt{simulateSymbolicModel()} directly , as it provides more options. For the temporal model of the IGF pathway included in \texttt{BoolNet}, an exhaustive simulation can be performed as follows:
\end{sloppypar}

<<eval=FALSE>>=
sim <- simulateSymbolicModel(igf)
sim
@
\begin{lstlisting}[linewidth=.9\linewidth]
<<echo=FALSE,results=tex>>=
sim <- simulateSymbolicModel(igf)
sim
@
\end{lstlisting}

By default, the result object of class \emph{SymbolicSimulation} comprises several components:
\begin{itemize}
\item A list of sequences \texttt{sequences} from each start state to the corresponding attractor. If this component is not desired, the parameter \texttt{returnSequences} can be set to false.
\item A graph structure \texttt{graph} that comprises all traversed state transitions. If this component is not desired, the parameter \texttt{returnGraph} can be set to false.
\item The identified attractors \texttt{attractors}. If this component is not desired, the parameter \texttt{returnAttractors} can be set to false.
\end{itemize}
In this case, the network has two attractors: A steady state describes the inactive state of the pathway. The circular attractor describes the activation and inactivation of the PI3K-Akt-mTOR signalling cascade initiated by IGF. 

All visualization and analysis function described above can also be applied to the simulation results obtained by \texttt{simulateSymbolicModel}. For example, the cascade attractor can be visualized via

<<echo=FALSE,results=hide>>=
pdf("attractor3.pdf")
par(mar=c(1,5,1,1))
plotAttractors(sim, subset=2)
dev.off()
pdf("stategraph2.pdf")
par(mar=c(1,1,1,1))
plotStateGraph(sim)
dev.off()
@
<<eval=FALSE>>=
plotAttractors(sim, subset=2)
@
Similarly, 
<<eval=FALSE>>=
plotStateGraph(sim)
@
plots the state transition graph of the network. Unlike in classical synchronous networks, a state can have multiple successor states (outgoing edges) in temporal networks, as a state transition may also depend on the history of states before the current state.
The two plots are shown in Figure~\ref{fig:temporal_plots}.

\begin{figure}[p]
\centering
\includegraphics[width=0.6\linewidth]{attractor3}
\includegraphics[width=0.6\linewidth]{stategraph2}
\caption{Top: Visualization of an attractor that describes the activation and inactivation of the PI3K-Akt-mTOR signalling cascade through IGF and IRS. The columns of the table represent consecutive states of the attractor. On top, the percentage of states leading to the attractor is supplied.\newline Bottom: The state transition graph of the IGF pathway network. Each node represents a state of the network, and each arrow is a state transition. The colors mark different basins of attraction. }
\label{fig:temporal_plots}
\end{figure}

For temporal networks, it is often infeasible to perform an exhaustive state-space simulation, as the search space is not only exponential in the number of genes, but also in the time delays. There are two options to deal with this:

First, the search can be restricted to randomly generated or prespecified start states similarly to \texttt{getAttractors()}. If time delays of more than 1 are included in the network, not only single start states are generated, but the required history of states is generated as well. For example,
<<echo=FALSE>>=
set.seed(43851)
@
<<>>=
sim <- simulateSymbolicModel(igf, method="random", startStates=2)
@
generates two start state matrices, each comprising 3 states (the maximum delay of the IGF network), and uses them as the basis for a simulation. This can be seen when examining the sequences to the attractors:
<<>>=
sim$sequences
@
Both sequences comprise start states $t=-2$, $t=-1$ and $t=0$.

The second option is to use a SAT-based attractor search similar to that in \texttt{getAttractors()}. Here, only the attractors are identified, but no state sequences and no state transition graph are returned. 
<<>>=
sim <- simulateSymbolicModel(igf, method="sat.exhaustive")
@
identifies all attractors in the network in an exhaustive manner. Alternatively, one can only search for attractors of a specific maximum length:
<<>>=
sim <- simulateSymbolicModel(igf, method="sat.restricted", maxAttractorLength=1)
@
will only return the steady-state attractor representing pathway inactivity, but not the 14-state attractor.
Please note that this length-restricted search cannot be applied to temporal networks comprising the time-dependent predicates \texttt{timeis},\texttt{timelt} or \texttt{timegt}.

\begin{sloppypar}
Classical synchronous Boolean networks can also be simulated using the symbolic simulator \texttt{simulateSymbolicModel()} if they are in a symbolic form. However, in most cases, \texttt{getAttractors()} will be faster and will consume less memory for synchronous networks without temporal elements. Only if the number of inputs to genes is very high and exhaustive simulation is not required, it may be advisable to use the symbolic simulator.
\end{sloppypar}

\subsection{Markov chain simulations}

Another way of identifying relevant states in Boolean networks are Markov chain simulations. Instead of identifying cycles explicitly, these simulations calculate the probability that a certain state is reached after a predefined number of iterations. Of course, states in an attractor have a high probability of being reached if the number of iterations is chosen large enough. Markov chain simulations for probabilistic Boolean networks were introduced by Shmulevich et al. \cite{shmulevich02}. As a special case of probabilistic Boolean networks, these simulations are also suited for synchronous Boolean networks.

The following performs a Markov experiment with the predefined number of 1000 iterations on the example PBN described in \cite{shmulevich02}:

\enlargethispage{-0.5cm}
<<>>=
data(examplePBN)
sim <- markovSimulation(examplePBN)
sim
@

Only states with a non-zero probability are listed in the two tables. The first table shows the states that are reached after 1000 iterations. The second table is a transition table annotated with transition probabilities. This table can be suppressed by the parameter \texttt{returnTable=FALSE}. The results correspond exactly to those in \cite{shmulevich02}.

If the transition table is included in the simulation results, we can plot a graph of the network:
<<eval=FALSE>>=
plotPBNTransitions(sim)
@
<<echo=FALSE,results=hide>>=
pdf("pbntransitions.pdf")
set.seed(4961)
par(mar=c(1,1,1,1))
plotPBNTransitions(sim)
dev.off()
@
This graph is displayed in Figure~\ref{fig:pbntransitions}. The vertices are the states of the graph. The edges represent transitions and are annotated with the corresponding transition probabilities. For this plot, the \texttt{igraph} package must be installed.

\begin{figure}[h]
\centering
\includegraphics[width=0.65\linewidth]{pbntransitions}
\caption{State transition graph of the example probabilistic Boolean network included in \texttt{BoolNet}. Each node represents a state of the network, and each arrow is a possible state transition, annotated by the transition probability.}
\label{fig:pbntransitions}
\end{figure}

We can also use Markov chain simulations to identify the attractor states in the mammalian cell cycle network:
\begin{samepage}
<<>>=
data(cellcycle)
sim <- markovSimulation(cellcycle, 
                        numIterations=1024, 
                        returnTable=FALSE)
sim
@
\end{samepage}
We set the maximum number of iterations to 1024, which is the number of states in the network. In a deterministic network, this guarantees that all states are found.

The fourth state in the returned table is the steady-state attractor identified previously. It has a probability of 0.5, as the basin of attraction is exactly half of the states. The other 7 states belong to the simple synchronous attractor.

It is also possible to restrict the simulation to a certain set of input states instead of using all possible input states. In the following example, we only consider the state with all genes set to 1, and identify the state belonging to the steady-state attractor again:
\begin{samepage}
<<>>=
sim <- markovSimulation(cellcycle, 
                        numIterations=1024,
                        returnTable=FALSE, 
                        startStates=list(rep(1,10)))
sim
@
\end{samepage}

\subsection{Robustness assessment}

 A biological network is assumed to be robust to small amounts of noise. The plausibility of network models is therefore often assessed by testing its robustness to noise and mismeasurements. Typically, artificial noise is applied, and its influence on the behaviour of a network is measured.
 
There are two major ways of applying random noise: Either the current state of a network in a simulation can be perturbed, or the network structure itself can be perturbed. \texttt{BoolNet} includes functions for both types of robustness assessment.

The function \texttt{perturbTrajectories} measures the influence of noise that is applied to the current network state. It generates a set of initial states and creates perturbed copies of these states by randomly flipping bits. It then measures the influence of the flips on the further dynamic behaviour of the network. For example

<<echo=FALSE>>=
set.seed(3361)
@
<<>>=
data(cellcycle)
r <- perturbTrajectories(cellcycle, 
                         measure="hamming", 
                         numSamples=100, 
                         flipBits=1)
@
randomly generates 100 states and 100 copies with one bitflip and performs a single state transition for each state. It then measures the normalized Hamming distance (the fraction of different bits) between each state and the corresponding perturbed copy. A robust network is assumed to yield a low Hamming distance.
The average distance can be viewed by typing
<<>>=
r$value
@
A related measure is the average sensitivity. This measure assesses only a single transition function and counts the number of successor states that differ between the original states and the perturbed copies for the corresponding gene.
E.g.,
<<>>=
r <- perturbTrajectories(cellcycle, 
                         measure="sensitivity", 
                         numSamples=100, 
                         flipBits=1, 
                         gene="CycE")
@
<<>>=
r$value
@
measures the average sensitivity of the transition function for gene CycE.

The long-term behaviour can be evaluated by comparing the attractors that are reached from the initial states and their perturbed copies. 
<<>>=
r <- perturbTrajectories(cellcycle, 
                         measure="attractor", 
                         numSamples=100, 
                         flipBits=1)
r$value
@
measures the fraction of pairs of states and perturbed copies that yield the same attractors. It can be assumed that small changes in the state should not influence the long-term behaviour of the network, and hence the attractors should mostly be the same. 

The second class of perturbations adds random noise to the network itself. This is implemented in the \texttt{perturbNetwork()} function. Unlike \texttt{perturbTrajectories()}, this function does not perform any simulations, but returns a perturbed copy of the network that can be analyzed further.

\texttt{BoolNet} includes a set of different perturbation options that can be combined. For example,
<<>>=
perturbedNet <- perturbNetwork(cellcycle, 
                               perturb="functions", 
                               method="bitflip")
@
chooses a function of the network at random and flips a single bit in this function. By setting the parameter \texttt{maxNumBits}, you can also flip more than one bit at a time.

Instead of flipping bits,
<<>>=
perturbedNet <- perturbNetwork(cellcycle, 
                               perturb="functions", 
                               method="shuffle")
@
randomly permutes the output values of the chosen transition functions. This preserves the numbers of 0s and 1s, but may change the Boolean function completely.
These kinds of perturbations are supported for synchronous and asynchronous networks as well as for probabilistic networks.

For synchronous networks, a further perturbation mode is available:
<<>>=
perturbedNet <- perturbNetwork(cellcycle, 
                               perturb="transitions", 
                               method="bitflip", 
                               numStates=10)
@
Here, \texttt{BoolNet} calculates the complete transition table of the network and then flips a single bit in 10 states of the transition table. From this modified table, a network is reconstructed. Changes of this type only affect a few states (which might not be the case when perturbing the functions directly as above), but possibly several of the transition functions. As in the previous examples, it is also possible to modify the number of bits to be flipped or to choose \texttt{method="shuffle"}.

\begin{samepage}
A detailed listing of a perturbation experiment is shown below. In this experiment, 1000 perturbed copies of the cell cycle network are created, and the occurrences of the original synchronous attractors are counted in the perturbed copies. This is similar to the simulation in \texttt{perturbTrajectories()} with \texttt{measure="attractor"}. However, instead of comparing the outcomes of different initial states in the same network, it compares all attractors of different perturbed networks to all attractors of the original network by exhaustive search. 

\begin{footnotesize}\label{alg:perturbation}
<<eval=FALSE>>=
# Perform a robustness test on a network
# by counting the numbers of perturbed networks
# containing the attractors of the original net

library(BoolNet)

# load mammalian cell cycle network
data(cellcycle)

# get attractors in original network
attrs <- getAttractors(cellcycle, canonical=TRUE)

# create 1000 perturbed copies of the network and search for attractors
perturbationResults <- sapply(1:1000, function(i)
{
  # perturb network and identify attractors
  perturbedNet <- perturbNetwork(cellcycle, perturb="functions", method="bitflip")
  perturbedAttrs <- getAttractors(perturbedNet, canonical=TRUE)
  
  # check whether the attractors in the original network exist in the perturbed network
  attractorIndices <- sapply(attrs$attractors,function(attractor1)
        {
          index <- which(sapply(perturbedAttrs$attractors, function(attractor2)
            {
              identical(attractor1, attractor2)
            }))
          if (length(index) == 0)
            NA
          else
            index
        })
  return(attractorIndices)
})

# perturbationResults now contains a matrix
# with the first 2 columns specifying the indices or the 
# original attractors in the perturbed network 
# (or NA if the attractor was not found) and the next 2 
# columns counting the numbers of states
# in the basin of attraction (or NA if the attractor was not found)

# measure the total numbers of occurrences of the original attractors in the perturbed copies
numOccurrences <- apply(perturbationResults[seq_along(attrs$attractors),,drop=FALSE], 1,
                      function(row)sum(!is.na(row)))

# print original attractors
cat("Attractors in original network:\n")
print(attrs)

# print information
cat("Number of occurrences of the original attractors",
  "in 1000 perturbed copies of the network:\n")
for (i in 1:length(attrs$attractors))
{
  cat("Attractor ",i,": ",numOccurrences[i],"\n",sep="")
}
@
\end{footnotesize}
\end{samepage}

\pagebreak[4]
\begin{samepage}
The results of such an experiment could look like this:

\begin{verbatim}
Attractors in original network:
Attractor 1 is a simple attractor consisting of 1 state(s) 
and has a basin of 512 state(s):

 [...]
 
Attractor 2 is a simple attractor consisting of 7 state(s) 
and has a basin of 512 state(s):

[...]

Number of occurrences of the original attractors in 1000 
perturbed copies of the network:
Attractor 1: 622
Attractor 2: 589
\end{verbatim}
\end{samepage}
We see that the steady-state attractor is slightly more robust to perturbations than the simple attractor with 7 states, as it can be identified in a higher number of perturbed copies.

\subsection{Identifying specific properties of biological networks}

The described robustness measures could also be used to identify specific properties of real-world networks in comparison to arbitrary (random) networks. For example, one could assume that attractors in biological networks are more robust to perturbations than attractors in random networks with a similar structure, as they should be capable of compensating for small dysfunctions of their components. Similarly to the above code, one could execute a number of random perturbations on the biological network and measure the percentage of original attractors found in the perturbed copies.  Afterwards, one could repeat this process on a number of randomly generated networks -- i.e., generate perturbed copies from each of the random networks, and measure the percentage of attractors in the copies. If the percentage of the biological network is higher than most of the percentages of the random network, this suggests that the biological network exhibits a higher robustness. This is a kind of computer-intensive test.

\texttt{BoolNet} comprises a generic facility for such computer-intensive tests. This facility already includes several tests (mainly for synchronous Boolean networks) and can be extended by custom test functions. The outlined example of attractor robustness is one of the integrated functions:
<<eval=FALSE>>=
data(cellcycle)
res <- testNetworkProperties(cellcycle, 
                      numRandomNets=100, 
                      testFunction="testAttractorRobustness", 
                      testFunctionParams = list(copies=100, perturb="functions"))
@
creates a set of 100 random networks (each with the same number of input genes for the functions as the cell cycle network) and creates 100 perturbed copies for each of these networks and for the cell cycle network by applying \texttt{perturbNetwork()} with \texttt{perturb="functions"}. For each network, it then calculates the percentage of attractors that can still be found in the perturbed copies.

The function plots a histogram of this robustness measures of the random networks (see Figure~\ref{fig:robustness}, top panel). The corresponding value of the original cell cycle network is plotted as a red line, and the 95\% quantile is plotted as a blue line.

<<echo=FALSE,results=hide>>=
pdf("attractor_robustness.pdf")
par(mar=c(4,4,2,1))
data(cellcycle)
set.seed(3241)
res <- testNetworkProperties(cellcycle, 
                      numRandomNets=100, 
                      testFunction="testAttractorRobustness", 
                      testFunctionParams = list(copies=100, perturb="functions"))
dev.off()
@

\begin{figure}[p]
\centering
\includegraphics[width=0.6\linewidth]{attractor_robustness}
\includegraphics[width=0.6\linewidth]{transition_robustness}
\caption{Top: Attractor robustness of randomly generated networks (histogram) in comparison to the mammalian cell cycle network (red line).\newline
Bottom: Normalized Hamming distance of randomly generated networks (histogram) in comparison to the mammalian cell cycle network (red line).
}
\label{fig:robustness}
\end{figure}

We can see that the average percentage of found attractors is significantly higher in the biological network with a $p$-value of 0.01.

\begin{sloppypar}
It is also possible to perturb the states instead of the networks themselves by setting \texttt{perturb} to \texttt{"trajectories"}. In this case, the function applies \texttt{perturbTrajectories()} with \texttt{measure="attractor"} to the biological network and the randomly generated networks. It then tests whether the fraction of state pairs that yield the same attractor is higher in the biological network than in the randomly generated networks.

The second built-in test function also tests the robustness of the network behaviour by perturbing the network states: \texttt{testTransitionRobustness()} applies \texttt{perturbTrajectories()} with \texttt{measure="hamming"} to each network. It then checks whether random bit flips yield a higher Hamming distance of the successor states in randomly generated networks than in the biological model, i.e. whether noise in the states influences the randomly generated networks stronger than the biological model. In contrast to the previous measures for which a greater value was assumed in the biological model, the Hamming distance is assumed to be smaller. Hence, we must specify the test alternative as \texttt{alternative="less"}.
\end{sloppypar}

<<eval=FALSE>>=
testNetworkProperties(cellcycle, 
                      numRandomNets=100,
                      testFunction="testTransitionRobustness",
                      testFunctionParams=list(numSamples=100),
                      alternative="less")  
@

<<echo=FALSE,results=hide>>=
pdf("transition_robustness.pdf")
par(mar=c(4,4,2,1))
data(cellcycle)
set.seed(22652)
testNetworkProperties(cellcycle, 
                      numRandomNets=100,
                      testFunction="testTransitionRobustness",
                      testFunctionParams=list(numSamples=100),
                      alternative="less")  
dev.off()
@
The results are shown in the bottom panel of Figure~\ref{fig:robustness}. Again, the result is highly significant (indeed, the Hamming distances are \emph{always} lower in the biological network), which means that the biological network is considerably more robust to noise in the states than the randomly generated models.

Another network property can also be tested using a built-in function: When looking at the state graph of a biological network (which can be generated using \texttt{plotStateGraph()}), it can often be observed that many state transitions lead to the same successor states, which means that the dynamics of the network quickly concentrate on a few states after a number of state transitions. We call the number of states whose synchronous state transitions lead to a state $s$ the {\em in-degree} of state $s$. We expect the biological network to have a few states with a high in-degree and many states with a low in-degree. A characteristic to summarize the in-degrees is the Gini index, which is a measure of inhomogeneity. If all states have an in-degree of 1, the Gini index is 0; if all state transitions lead to only one state, the Gini index is 1.
<<eval=FALSE>>=
testNetworkProperties(cellcycle, 
                      numRandomNets=100, 
                      testFunction="testIndegree")
@
<<echo=FALSE,results=hide>>=
pdf("indegree.pdf")
par(mar=c(4,4,2,1))
set.seed(6314)
testNetworkProperties(cellcycle, 
                      numRandomNets=100, 
                      testFunction="testIndegree")
dev.off()
@
plots an histogram of Gini indices in 100 random networks and draws the Gini index of the cell cycle network as a red line, as depicted in the top panel of Figure~\ref{fig:indegree}.


\begin{figure}[p]
\centering
\includegraphics[width=0.6\linewidth]{indegree}
\includegraphics[width=0.6\linewidth]{indegree_kl}
\caption{Top: Gini indices of state in-degrees of randomly generated networks (histogram) in comparison to the mammalian cell cycle network (red line)\newline
Bottom: Kullback-Leibler distances of in-degrees of the mammalian cell cycle network and 100 random networks.}
\label{fig:indegree}
\end{figure}

The histogram shows that the Gini index of the in-degrees is always higher in the biological network. This is probably due to the special structure of functions in biological networks.

Instead of accumulating the in-degrees using the Gini index, it is also possible to compare the distributions of the in-degrees across the networks. For this purpose, the Kullback-Leibler distances of the in-degrees of the supplied network and each of the random networks are calculated and plotted in a histogram. The Kullback-Leibler distance (also called relative entropy) is an asymetric measure of similarity of two distributions \cite{cover91}. If the distributions are equal, the Kullback-Leibler distance is 0, otherwise it is greater than 0. 
<<eval=FALSE>>=
testNetworkProperties(cellcycle, 
                      numRandomNets=100, 
                      testFunction="testIndegree", 
                      accumulation="kullback_leibler")
@
<<echo=FALSE,results=hide>>=
pdf("indegree_kl.pdf")
par(mar=c(4,4,2,1))
set.seed(234256)
testNetworkProperties(cellcycle, 
                      numRandomNets=100, 
                      testFunction="testIndegree", 
                      accumulation="kullback_leibler")
dev.off()
@
results in the plot displayed in the bottom panel of Figure~\ref{fig:indegree}.

It is possible to switch between the histogram of an accumulated characteristic (e.g. the Gini index) and the histogram of the Kullback-Leibler distances for all tests. 

\begin{sloppypar}
You can also easily implement your own tests. To do this, the only thing you have to do is implement a custom testing function that replaces \texttt{testIndegree()} or \texttt{testAttractorRobustness()}. Testing functions have the following signature:
\end{sloppypar}
\begin{verbatim}
function(network, accumulate=TRUE, params)
\end{verbatim}
The first parameter is the network that should be tested. The parameter \texttt{accumulate} specifies whether a single characteristic value (e.g., the Gini index of the in-degrees) should be calculated, or whether a distribution of values (e.g., a vector of all in-degrees) should be returned. The third parameter is a list of further arguments needed by your function. 

If, for example, we would like to compare the sizes of the basins of attractions of synchronous attractors in biological and random networks, we would write a function like this:
\begin{samepage}
<<>>=
testBasinSizes <- function(network, accumulate=TRUE, params)
{
  attr <- getAttractors(network)
  basinSizes <- sapply(attr$attractors, function(a)
                      { 
                         a$basinSize
                      })
   if (accumulate)
     return(mean(basinSizes))
   else
     return(basinSizes)                  
}
@
\end{samepage}
This function calculates the mean basin size as a characteristic value if accumulation is required, or returns the sizes of all basins of attraction in a vector otherwise. It does not need any further parameters in \texttt{params}.

Now, we can start a test using
<<eval=FALSE>>=
testNetworkProperties(cellcycle, 
                      numRandomNets=100,
                      testFunction="testBasinSizes",
                      xlab="Average size of basins of attraction")
@
<<echo=FALSE,results=hide>>=
pdf("basinsize.pdf")
par(mar=c(4,4,2,1))
set.seed(6724)
testNetworkProperties(cellcycle, 
                      numRandomNets=100,
                      testFunction="testBasinSizes",
                      xlab="Average size of basins of attraction")
dev.off()
@
to produce the plot shown in Figure~\ref{fig:basinsize}. Apparently, the average basin sizes do not differ as much as the built-in test characteristics between the random networks and the cell cycle network.

\begin{figure}[h]
\centering
\includegraphics[width=0.6\linewidth]{basinsize}
\caption{A custom test statistic measuring the basin sizes on randomly generated networks (histogram) and the mammalian cell cycle network (red line).}
\label{fig:basinsize}
\end{figure}

\begin{sloppypar}
By writing custom test functions, you can extend the test facility to perform a wide variety computer-intensive test. Of course, it is also possible to plot the Kullback-Leibler distances with such new methods by using \verb+accumulation="kullback_leibler"+.

\texttt{testNetworkProperties()} accepts most of the parameters of \texttt{generateRandomNKNetwork()}. If necessary, you can generate more specialized kinds of random networks which resemble the original network in certain aspects, for example by specifying a generation function or by setting a proportion of 0 and 1 in the function outputs similar to the original network using \verb+functionGeneration="biased"+. 
\end{sloppypar}

\clearpage
\section{Import and export}\label{sec:importexport}

\subsection{Saving networks in the \texttt{BoolNet} file format}

Corresponding to the \texttt{loadNetwork()} command, a network can be saved using \texttt{saveNetwork()}. This stores the network in the network file format described in Section~\ref{sec:appendix} and can be applied to all types of networks supported by \texttt{BoolNet}. For example, the cell cycle network can be saved using
<<eval=FALSE>>=
saveNetwork(cellcycle, file="cellcycle.txt")
@
\begin{sloppypar}
The function stores the expressions that describe the transition functions. In some cases, there may not always be a valid symbolic description of the networks (e.g. for networks returned by \texttt{generateRandomNKNetwork()} when the \texttt{readableFunctions} parameter was not set). In this case, \texttt{saveNetwork()} can generate symbolic representations of the transition functions in Disjunctive Normal Form (DNF):
\end{sloppypar}
<<>>=
net <- generateRandomNKNetwork(n=10, k=3, readableFunctions=FALSE)
saveNetwork(net, file="randomnet.txt", generateDNF=TRUE)
@
The \texttt{generateDNF} parameter can also be used to detail which type of DNF formulae should be exported: \texttt{generateDNF="canonical"} exports canonical DNF formulae. \texttt{generateDNF="short"} minimizes the canonical functions by joining terms. By simply setting \texttt{generateDNF=TRUE}, formulae with up to 12 inputs are minimized, and formulae with more than 12 inputs are exported in a canonical form, as a minization is very time-consuming in this case.

\subsection{Import from and export to SBML}

\begin{sloppypar}
\texttt{BoolNet} provides an interface to the widely used Systems Biology Markup Language (SBML) via the import function \texttt{loadSBML()} and the export function \texttt{saveSBML()}. As the core SBML does not fully support Boolean models, import and export of SBML models is based on the \texttt{sbml-qual} package which extends SBML by several qualitative modeling approaches, such as general logical models and Petri nets. For a full description of \texttt{sbml-qual}, refer to \href{http://sbml.org/Documents/Specifications/SBML_Level_3/Packages/Qualitative_Models_(qual)}{http://sbml.org/Documents/Specifications/SBML\_Level\_3/Packages/Qualitative\_Models\_(qual)}.
\end{sloppypar}

\texttt{BoolNet} only supports a subset of \texttt{sbml-qual}. It can read and write logical models with two possible values for each state, which are equivalent to Boolean networks. Logical models with more than two values for a gene or Petri nets cannot currently be handled by \texttt{BoolNet}.

An export to SBML is usually not associated with any loss of information. For example, we can write the cell cycle network to a file and re-import it into \texttt{BoolNet}:

\begin{footnotesize}
<<>>=
toSBML(cellcycle, file="cellcycle.sbml")
sbml_cellcycle <- loadSBML("cellcycle.sbml")
sbml_cellcycle
@
\end{footnotesize}

Apart from some additional brackets, the re-imported network coincides with the original network. 

Similar to the \texttt{saveNetwork()} function, \texttt{toSBML()} exports a symbolic representation of the network transition functionss, which may not always be available. As for \texttt{saveNetwork()}, there is a parameter \texttt{generateDNF} that can be set to generate a symbolic representation in Disjunctive Normal Form from the truth tables.

\subsection{Importing networks from BioTapestry}

BioTapestry is a widely-used application for visual modeling of gene-regulatory networks \cite{longabaugh05}. It can be freely accessed at \url{http://www.biotapestry.org}. Although its primary purpose is visualization, the software supports specifying logical functions for the genes. \texttt{BoolNet} can read in the top-level (``Full genome'') plot of a BioTapestry file (*.btp) and convert it into a Boolean network.

As an example, we assume the following BioTapestry model with 5 genes (2~inputs and 3~dependent genes):

\begin{center}
\includegraphics[width=0.8\linewidth]{biotap_model}
\end{center}

The corresponding BioTapestry file is included in \texttt{BoolNet}. You can determine its path using
<<eval=FALSE>>=
system.file("doc/example.btp", package="BoolNet")
@
\label{cmd:example.btp}
to access it in BioTapestry or \texttt{BoolNet}.

For the import, \texttt{BoolNet} needs to know the type of influence a gene has on another gene. Therefore, imported networks should only use links that are either enhancers or repressors. Neutral links are ignored in the import.

We now set further simulation parameters for the model. These parameters are imported by \texttt{BoolNet} to construct the functions of the Boolean network. First, we want to change the function of Gene~2 to \texttt{OR}. Right-click on Gene~2 and choose \texttt{Simulation Properties...}. 

\begin{center}
\includegraphics[width=0.8\linewidth]{biotap_sim_properties}
\end{center}

In the properties dialog, choose the \texttt{Logic} tab, and select \texttt{OR} for the logical function.

\begin{center}
\includegraphics[width=0.8\linewidth]{biotap_logic}
\end{center}

Now set the function of Gene~1 to \texttt{XOR} (exclusive or) in the same way.

\begin{samepage}
You can also specify initial values for constant genes, i.e., genes with no input links. Choose the simulation properties of Input~1, and change to the \texttt{Parameters} tab. Choose \texttt{initVal} and set it to 1. 

\begin{center}
\includegraphics[width=0.8\linewidth]{biotap_initval}
\end{center}
\end{samepage}

Press Return to store the result, and exit the dialog with \texttt{OK}. This will create a fixed gene with value 1 (i.e., an over-expressed gene) in the \texttt{BoolNet} import. Note that values other than 0 and 1 are ignored by the import, as well as initialization values for non-constant genes.

We assume that you save the network to a file ``example.btp'' in your working directory. In \texttt{R}, type
<<echo=FALSE>>=
net <- loadBioTapestry(system.file("doc/example.btp", package="BoolNet"))
@
<<eval=FALSE>>=
net <- loadBioTapestry("example.btp")
@
to import the network. Alternatively, replace the file name by the command on page~\pageref{cmd:example.btp} to use the file in the package if you do not want to create the file yourself.

The imported network looks like this:
<<eval=FALSE>>=
net
@
\begin{verbatim}
Boolean network with 5 genes

Involved genes:
Input 1 Input 2 Gene 1 Gene 2 Gene 3
\end{verbatim}
\pagebreak[4]
\begin{verbatim}
Transition functions:
Input 1 = 1
Input 2 = Input 2
Gene 1 = (!Gene 1 & !Input 1 & Input 2) | (!Gene 1 & Input 1 & !Input 2) 
         | (Gene 1 & !Input 1 & !Input 2) | (Gene 1 & Input 1 & Input 2)
Gene 2 = Gene 1 & Gene 3 & !Input 2
Gene 3 = Gene 1 | Gene 2

Knocked-out and over-expressed genes:
Input 1 = 1
\end{verbatim}

We can see that Input~1 is specified as an over-expressed constant gene. Input~2 is modeled as depending only on itself, i.e. it keeps its initial value. Gene~1 is a representation of the XOR function in Disjunctive Normal Form (DNF), using only logical ANDs, logical ORs, and negations. Gene~2 and Gene~3 consist of conjunctions and disjunctions of their inputs respectively. In addition to this textual description, we can visually verify the network by plotting its wiring:
<<eval=FALSE>>=
plotNetworkWiring(net)
@
<<echo=FALSE,results=hide>>=
pdf("wiring_biotap.pdf")
par(mar=c(1,1,1,1))
set.seed(559652)
plotNetworkWiring(net)
dev.off()
@
The resulting plot is shown in Figure~\ref{fig:wiring_biotap}.
\begin{figure}[h]
\centering
\includegraphics[width=0.6\linewidth]{wiring_biotap}
\caption{The wiring graph of the imported network specified in BioTapestry.}
\label{fig:wiring_biotap}
\end{figure}

You can now use the imported network just like any other network in \texttt{BoolNet}.

\clearpage

\subsection{Exporting network simulations to Pajek}

For further analysis, network simulations can be exported to Pajek, a Windows application that provides visualization and analysis methods for graph structures \cite{batagelij98}. For more information on Pajek, please refer to \url{http://pajek.imfm.si/doku.php}.

The export function writes the state transition graph to a Pajek file (*.net). This requires a synchronous exhaustive attractor search in \texttt{BoolNet} to build the full transition table. 

To export the mammalian cell cycle network to Pajek, call
<<eval=FALSE>>=
data(cellcycle)
attr <- getAttractors(cellcycle)
toPajek(attr, file="cellcycle.net")
@

This will export the graph of the state transitions, which is usually sufficient for plotting. If you want to include the state information (i.e., the gene assignment vectors), call
<<eval=FALSE>>=
toPajek(attr, file="cellcycle.net", includeLabels=TRUE)
@

Now, start Pajek, load the network with \texttt{File | Network | Read}, and check out the tools provided by this application. For example, visualizations can be accessed using the menu item \texttt{Draw | Draw}. 

Figure~\ref{fig:pajek} shows a plot of the cell cycle network with the Kamada-Kawai layout (Menu entry \texttt{Layout | Energy | Kamada-Kawai | Separate Components}).

\begin{figure}[h]
\centering
\includegraphics[width=0.75\linewidth]{pajek}
\caption{A visualization of the mammalian cell cycle network in Pajek.}
\label{fig:pajek}
\end{figure}

\clearpage

\bibliographystyle{plain}
\bibliography{BoolNet_package_vignette}

\section{Appendix}\label{sec:appendix}

\subsection{Network file format}

This section provides a full language description for the network file format of \texttt{BoolNet}. The language is described in Extended Backus-Naur Form (EBNF).

For synchronous, asynchronous and probabilistic Boolean networks, the supported format is as follows:

\begin{verbatim}
Network           = Header Newline {Rule Newline | Comment Newline};
Header            = "targets" Separator "factors";
Rule              = GeneName Separator BooleanExpression [Separator Probability];
Comment           = "#" String;
BooleanExpression = GeneName 
                    | "!" BooleanExpression 
                    | "(" BooleanExpression ")" 
                    | BooleanExpression " & " BooleanExpression  
                    | BooleanExpression " | " BooleanExpression;
                    | "all(" BooleanExpression {"," BooleanExpression} ")"
                    | "any(" BooleanExpression {"," BooleanExpression} ")"
                    | "maj(" BooleanExpression {"," BooleanExpression} ")"
                    | "sumgt(" BooleanExpression {"," BooleanExpression} "," Integer ")"
                    | "sumlt(" BooleanExpression {"," BooleanExpression} "," Integer ")";
GeneName          = ? A gene name from the list of involved genes ?;
Separator         = ",";
Integer           = ? An integer value?;
Probability       = ? A floating-point number ?;
String            = ? Any sequence of characters (except a line break) ?;
Newline           = ? A line break character ?;
\end{verbatim}

\pagebreak[4]
The extended format for temporal networks includes additional time specifications and temporal predicates is defined as follows:

\begin{verbatim}
Network               = Header Newline
                        {Function Newline | Comment Newline};
Header                = "targets" Separator "factors";
Function              = GeneName Separator BooleanExpression;
Comment               = "#" String;
BooleanExpression     = GeneName 
                        | GeneName TemporalSpecification 
                        | BooleanOperator 
                        | TemporalOperator
BooleanOperator       =   BooleanExpression 
                        | "!" BooleanExpression 
                        | "(" BooleanExpression ")" 
                        | BooleanExpression " & " BooleanExpression  
                        | BooleanExpression " | " BooleanExpression;
TemporalOperator      =   "all" [TemporalIteratorDef] 
                                "(" BooleanExpression {"," BooleanExpression} ")"
                        | "any" [TemporalIteratorDef] 
                                "(" BooleanExpression {"," BooleanExpression} ")"
                        | "maj" [TemporalIteratorDef] 
                                "(" BooleanExpression {"," BooleanExpression} ")"
                        | "sumgt" [TemporalIteratorDef] 
                                  "(" BooleanExpression {"," BooleanExpression} "," Integer ")"
                        | "sumlt" [TemporalIteratorDef] 
                                  "(" BooleanExpression {"," BooleanExpression} "," Integer ")"
                        | "timeis" "(" Integer ")"
                        | "timegt" "(" Integer ")"
                        | "timelt" "(" Integer ")";
TemporalIteratorDef   = "[" TemporalIterator "=" Integer ".." Integer "]";
TemporalSpecification = "[" TemporalOperand {"+" TemporalOperand | "-" TemporalOperand} "]";
TemporalOperand       = TemporalIterator | Integer
TemporalIterator      = ? An alphanumeric string ?;
GeneName              = ? A gene name from the list of involved genes ?;
Separator             = ",";
Integer               = ? An integer value?;
String                = ? Any sequence of characters (except a line break) ?;
Newline               = ? A line break character ?;
\end{verbatim}

\end{document}

\name{print.MarkovSimulation}
\alias{print.MarkovSimulation}
\title{
Print the results of a Markov chain simulation
}
\description{
A specialized method to print an object of class \code{MarkovSimulation}. This prints all states that have a non-zero probability to be reached after the number of iterations in the Markov simulation. If the simulation was run with \code{returnTable=TRUE}, it also prints a table of state transitions and their probabilities to be chosen in a single step.
}
\usage{
\method{print}{MarkovSimulation}(x,
      activeOnly = FALSE,
      ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{

\item{x}{
An object of class \code{MarkovSimulation} to be printed
}

\item{activeOnly}{
If set to true, a state is represented by a list of active genes (i.e., genes which are set to 1). If set to false, a state is represented by a binary vector with one entry for each gene, specifying whether the gene is active or not. Defaults to \code{FALSE}. 
}

\item{\dots}{
Further parameters for the \code{\link{print}} method. Currently not used.
}
}

\value{
Invisibly returns the printed object
}

\seealso{
\code{\link{print}}, \code{\link{markovSimulation}}
}

\name{getBasinOfAttraction}
\Rdversion{1.1}
\alias{getBasinOfAttraction}
\title{
Get states in basin of attraction
}
\description{
Extracts information on all states in the basin of a supplied attractor
}
\usage{
getBasinOfAttraction(attractorInfo, attractorNo)
}
\arguments{
  \item{attractorInfo}{
	An object of class \code{AttractorInfo}, as returned by \code{\link{getAttractors}}, or of class \code{SymbolicSimulation}, as returned by \code{\link{simulateSymbolicModel}}. As the transition table information in this structure is required, \code{getAttractors} must be called in synchronous mode and with \code{returnTable} set to TRUE. Similarly, \code{simulateSymbolicModel} must be called with \code{returnGraph=TRUE}.
}
  \item{attractorNo}{
	The index of the attractor in \code{attractorInfo} whose basin should be identified
}
}
\details{
	The function outputs a transition table containing only the states that are contained in the basin of attraction, and displays additional information on these states. If \code{attractorInfo} is the result of an exhaustive synchronous attractor search, the complete basin of attraction is returned. If \code{attractorInfo} is the result of a heuristic synchronous search, there is no guarantee that the complete basin of attraction is returned, as only the calculated states are included. Asynchronous search results are not supported, as no transition table is calculated.
}
\value{
Returns a generic dataframe of the class \code{TransitionTable}. For n genes, the first n columns code for the original state, i.e. each column represents the value of one gene. The next n columns code for the successive state after a transition. The column \code{attractorAssignment} indicates the attractor to the state is assigned (in this case, \code{attractorNo}). If this information is available, the column \code{stepsToAttractor} indicates how many transitions are needed from the original state to the attractor.
The \code{TransitionTable} class supports pretty printing using the \code{\link{print}} method.
}
\seealso{
\code{\link{getStateSummary}}, \code{\link{getTransitionTable}}, \code{\link{getAttractors}}, \code{\link{simulateSymbolicModel}}
}
\examples{
\dontrun{
# load example data
data(cellcycle)

# get attractors
attractors <- getAttractors(cellcycle)

# print basin of first attractor
print(getBasinOfAttraction(attractors, 1))
}
}

\name{saveNetwork}
\alias{saveNetwork}
\title{
Save a network
}
\description{
Saves synchronous, asynchronous, probabilistic and temporal networks in the \pkg{BoolNet} network file format .
}
\usage{
saveNetwork(network, 
            file, 
            generateDNFs = FALSE,
            saveFixed = TRUE)
}
\arguments{
  \item{network}{
An object of class \code{BooleanNetwork} or \code{SymbolicBooleanNetwork} to be exported
}
  \item{file}{
The name of the network file to be created
}

  \item{generateDNFs}{
If \code{network} is a \code{BooleanNetwork} object, this parameter specifies whether formulae in Disjunctive Normal Form are exported instead of the expressions that describe the transition functions. If set to FALSE, the original expressions are exported. If set to "canonical", a canonical Disjunctive Normal Form is generated from each truth table. If set to "short", the canonical DNF is minimized by joining terms (which can be time-consuming for functions with many inputs). If set to TRUE, a short DNF is generated for functions with up to 12 inputs, and a canonical DNF is generated for functions with more than 12 inputs.
For objects of class \code{SymbolicBooleanNetwork}, this parameter is ignored.
}

  \item{saveFixed}{
If set to TRUE, knock-outs and overexpression of genes override their transition functions. That is, if a gene in the network is fixed to 0 or 1, this value is saved, regardless of the transition function. If set to FALSE, the transition function is saved. Defaults to TRUE.
}

}
\details{
The network is saved in the \pkg{BoolNet} file format (see \code{\link{loadNetwork}} for details).

If the expressions in the transition functions cannot be parsed or \code{generateDNFs} is true, a DNF representation of the transition functions is generated.
}

\seealso{
\code{\link{loadNetwork}}
}
\examples{
\dontrun{
# load the cell cycle network
data(cellcycle)

# save it to a file
saveNetwork(cellcycle, file="cellcycle.txt")

# reload the model
print(loadNetwork("cellcycle.txt"))
}
}

\name{perturbNetwork}
\Rdversion{1.1}
\alias{perturbNetwork}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Perturb a Boolean network randomly
}
\description{
Modifies a synchronous, asynchronous, or probabilistic Boolean network by randomly perturbing  either the functions for single genes or the state transitions. Random perturbations can be employed to assess the stability of the network.
}
\usage{
perturbNetwork(network, 
               perturb = c("functions","transitions"), 
               method = c("bitflip","shuffle"), 
               simplify = (perturb[1]!="functions"), 
               readableFunctions = FALSE, 
               excludeFixed = TRUE, 
               maxNumBits = 1, 
               numStates = max(1,2^length(network$genes)/100))
}
\arguments{
  \item{network}{
	A network structure of class \code{BooleanNetwork} or \code{ProbabilisticBooleanNetwork}. These networks can be read from files by \code{\link{loadNetwork}}, generated by \if{latex}{\cr}\code{\link{generateRandomNKNetwork}}, or reconstructed by \code{\link{reconstructNetwork}}.
}
  \item{perturb}{
	If set to "functions", a transition function of a single gene is chosen at random and perturbed directly. This is the default mode. If set to "transitions", the transition table is generated, one or several state transitions are perturbed randomly, and the gene transition functions are rebuilt from the modified transition table. \code{perturb="transitions"} is only allowed for non-probabilistic networks of class \code{BooleanNetworks}.
}
  \item{method}{
	The perturbation method to be applied to the functions or transitions. "bitflip" randomly inverts one or several bits (depending on the value of \code{maxNumBits}). "shuffle" generates a random permutation of the positions in the function or state and rearranges the bits according to this permutation.
}
  \item{simplify}{
	If this is true, \code{\link{simplifyNetwork}} is called to simplify the gene transition functions after the perturbation. This removes irrelevant input genes. Defaults to TRUE if \code{perturb} is "transitions", and to FALSE otherwise.
}
  \item{readableFunctions}{
	If this is true, readable DNF representations of the truth tables of the functions are generated. These DNF are displayed when the network is printed. The DNF representations are not minimized and can thus be very long. If set to FALSE, the truth table result column is displayed.	
}

\item{excludeFixed}{
	Determines whether fixed variables can also be perturbed (if set to FALSE) or if they are excluded from the perturbation (if set to TRUE). Default is TRUE.
}

  \item{maxNumBits}{
	The maximum number of bits to be perturbed in one function or state. Defaults to 1.
}
  \item{numStates}{
	The number of state transitions to be perturbed if \code{perturb} is "transitions". Defaults to 1% of all states.
}
}

\value{
Depending on the input, an object of class \code{BooleanNetwork} or \code{ProbabilisticBooleanNetwork} containing the perturbed copy of the original network is returned. The classes \code{BooleanNetwork} and \code{ProbabilisticBooleanNetwork} are described in more detail in \code{\link{loadNetwork}}.
}
\seealso{
\code{\link{loadNetwork}}, \code{\link{generateRandomNKNetwork}}, \code{\link{reconstructNetwork}}, \code{\link{simplifyNetwork}}}

\examples{
\dontrun{
# load example data
data(cellcycle)

# perturb the network
perturbedNet1 <- perturbNetwork(cellcycle, perturb="functions", method="shuffle")
perturbedNet2 <- perturbNetwork(cellcycle, perturb="transitions", method="bitflip")

# get attractors
print(getAttractors(perturbedNet1))
print(getAttractors(perturbedNet2))
}
}
\references{
Y. Xiao and E. R. Dougherty (2007), The impact of function perturbations in Boolean networks. Bioinformatics 23(10):1265--1273.

I. Shmulevich, E. R. Dougherty, W. Zhang (2002), Control of stationary behavior in probabilistic Boolean networks by means of structural intervention. Journal of Biological Systems 10(4):431--445.
}

\name{symbolicToTruthTable}
\alias{symbolicToTruthTable}

\title{
Convert a symbolic network into a truth table representation
}
\description{
Converts an object of class \code{SymbolicBooleanNetwork} into an object of class \code{BooleanNetwork} by generating truth tables from the symbolic expression trees.
}
\usage{
symbolicToTruthTable(network)
}

\arguments{
  \item{network}{
An object of class \code{SymbolicBooleanNetwork} to be converted.
}
}

\details{
The symbolic network \code{network} must not contain temporal operators, as these are not compatible with the truth table representation in \code{BooleanNetwork} objects.
}

\value{
Returns an object of class \code{BooleanNetwork}, as described in \code{\link{loadNetwork}}.
}

\seealso{
\code{\link{truthTableToSymbolic}}, \code{\link{loadNetwork}}
}
\examples{
\dontrun{
# Convert a truth table representation into a 
# symbolic representation and back
data(cellcycle)

symbolicNet <- truthTableToSymbolic(cellcycle)
print(symbolicNet)

ttNet <- symbolicToTruthTable(symbolicNet)
print(cellcycle)
}
}

\name{binarizeTimeSeries}
\alias{binarizeTimeSeries}
\title{
Binarize a set of real-valued time series
}
\description{
Binarizes a set of real-valued time series using k-means clustering, edge detection, or scan statistics.
}
\usage{
binarizeTimeSeries(measurements, 
                   method = c("kmeans","edgeDetector","scanStatistic"), 
                   nstart = 100, 
                   iter.max = 1000, 
                   edge = c("firstEdge","maxEdge"),                    
                   scaling = 1, 
                   windowSize = 0.25, 
                   sign.level = 0.1,
                   dropInsignificant = FALSE)
}
\arguments{
  \item{measurements}{
A list of matrices, each corresponding to one time series. Each row of these matrices contains real-valued measurements for one gene on a time line, i. e. column \code{i+1} contains the successor states of column \code{i+1}. The genes must be the same for all matrices in the list. 
}
  \item{method}{The employed binarization technique. "kmeans" uses k-means clustering for binarization. "edgeDetector" searches for a large gradient in the sorted measurements. "scanStatistic" searches for accumulations in the measurements. See Details for descriptions of the techniques.}

  \item{nstart}{
If \code{method="kmeans"}, this is the number of restarts for k-means. See \code{\link{kmeans}} for details.
}
  \item{iter.max}{
If \code{method="kmeans"}, the maximum number of iterations for k-means. See \code{\link{kmeans}} for details.
}

  \item{edge}{If \code{method="edgeDetector"}, this decides which of the edges is used as a threshold for binarization. If set to "firstEdge",the binarization threshold is the first combination of two successive sorted values whose difference exceeds a predefined value (average gradient * \code{scaling}). The parameter \code{scaling} can be used to adjust this value. 
  
  If set to "maxEdge", the binarization threshold is the position of the edge with the overall highest gradient.}
  
  \item{scaling}{If \code{method="edgeDetector"} and \code{edge="firstEdge"}, this holds the scaling factor used for adjustment of the average gradient.}
  
  \item{windowSize}{If \code{method="scanStatistic"}, this specifies the size of the scanning window (see Details). The size is given as a fraction of the whole range of input values for a gene. Default is 0.25.}
  
  \item{sign.level}{If \code{method="scanStatistic"}, the significance level used for the scan statistic (see Details).
}

  \item{dropInsignificant}{
If this is set to true, genes whose binarizations are insignificant in the scan statistic (see Details) are removed from the binarized time series. Otherwise, a warning is printed if such genes exist.
}

}
\details{

This method supports three binarization techniques:
\describe{
\item{k-means clustering}{For each gene, k-means clusterings are performed to determine a good separation of groups. The values belonging to the cluster with the smaller centroid are set to 0, and the values belonging to the greater centroid are set to 1.} 

\item{Edge detector}{This approach first sorts the measurements for each gene. In the sorted measurements, the algorithm searches for differences of two successive values that satisfy a predefined condition: If the "firstEdge" method was chosen, the pair of values whose difference exceeds the scaled average gradient of all values is chosen and used as maximum and minimum value of the two groups. If the "maxEdge" method was chosen, the largest difference between two successive values is taken. For details, see Shmulevich et al.}

\item{Scan statistic}{The scan statistic assumes that the measurements for each gene are uniformly and independently distributed independently over a certain range. The scan statistic shifts a scanning window across the data and decides for each window position whether there is an unusual accumulation of data points based on an approximated test statistic (see Glaz et al.). The window with the smallest p-value is remembered. The boundaries of this window form two thresholds, from which the value that results in more balanced groups is taken for binarization. Depending on the supplied significance level, gene binarizations are rated according to the p-value of the chosen window.}

}
}
\value{
Returns a list with the following elements:
\item{binarizedMeasurements}{A list of matrices with the same structure as \code{measurements} containing the binarized time series measurements}
\item{reject}{If \code{method="scanStatistic"}, a Boolean vector indicating for each gene whether the scan statistic algorithm was able to find a significant binarization window (FALSE) or not (TRUE). Rejected genes should probably be excluded from the data.}
\item{thresholds}{The thresholds used for binarization}
}
\references{
I. Shmulevich and W. Zhang (2002), Binary analysis and optimization-based normalization of gene expression data. Bioinformatics 18(4):555--565.

J. Glaz, J. Naus, S. Wallenstein (2001), Scan Statistics. New York: Springer.
}
\seealso{
\code{\link{reconstructNetwork}}
}
\examples{
# load test data
data(yeastTimeSeries)
			
# perform binarization with k-means
bin <- binarizeTimeSeries(yeastTimeSeries)
print(bin)

# perform binarization with scan statistic
# - will find and remove 2 insignificant genes!
bin <- binarizeTimeSeries(yeastTimeSeries, method="scanStatistic",
                          dropInsignificant=TRUE, sign.level=0.2)
print(bin)

# perform binarization with edge detector
bin <- binarizeTimeSeries(yeastTimeSeries, method="edgeDetector")
print(bin)

# reconstruct a network from the data
reconstructed <- reconstructNetwork(bin$binarizedMeasurements,
                                    method="bestfit", maxK=4)
print(reconstructed)
}

\name{attractorsToLaTeX}
\Rdversion{1.1}
\alias{attractorsToLaTeX}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Create LaTeX state table of attractors
}
\description{
Exports state tables of attractors (corresponding to the plot generated by \code{\link{plotAttractors}} with \code{mode="table"}) to a LaTeX document.
}
\usage{
attractorsToLaTeX(attractorInfo, 
                  subset, 
                  title = "", 
                  grouping = list(), 
                  plotFixed = TRUE,
                  onColor = "[gray]{0.9}",
                  offColor = "[gray]{0.6}",
                  reverse = FALSE, 
                  file = "attractors.tex")
}
\arguments{
  \item{attractorInfo}{
	An object of class \code{AttractorInfo}, as returned by \code{\link{getAttractors}}, or an object of class \code{SymbolicSimulation}, as returned by \code{\link{simulateSymbolicModel}}.
}

  \item{subset}{An subset of attractors to be exported. This is a vector of attractor indices in \code{attractorInfo}.}
  
  \item{grouping}{An optional structure to form groups of genes in the plot. This is a list with the following elements: 
 \describe{
  	\item{class}{A vector of names for the groups. These names will be printed in the region belonging to the group in the table.}
  	\item{index}{A list with the same length as \code{class}. Each element is a vector of gene indices belonging to the group.}}
}
 
  \item{title}{
	An optional title for the plot
}
  \item{plotFixed}{
	If this is true, genes with fixed values are included in the plot. Otherwise, these genes are not shown.
}
  \item{onColor}{
	An optional color value for the 1/ON values in the table. Defaults to dark grey.
}
  \item{offColor}{
	An optional color value for the 0/OFF values in the table. Defaults to light grey.	
}

  \item{reverse}{
Specifies the order of the genes in the plot. By default, the first gene is placed in the first row of the table. If \code{reverse=TRUE}, the first gene in the network is placed in the bottom row of the table. 
}

  \item{file}{
	The file to which the LaTeX document is written. Defaults to "attractors.tex".
}
}
\details{
This function creates LaTeX tables that visualize the states of synchronous attractors. Asynchronous attractors are ignored.
Attractors in \code{attractorInfo} are first grouped by length. Then, a LaTeX table environment is created for each attractor length (i.e. one plot with all attractors consisting of 1 state, one plot with all attractors consisting of 2 states, etc.).
The output file does not contain a document header and requires the inclusion of the packages \code{tabularx} and \code{colortbl}. The tables have the genes in the rows and the states of the attractors in the columns. If not specified otherwise, cells of the table are light grey for 0/OFF values and dark grey for 1/ON values. If \code{grouping} is set, the genes are rearranged according to the indices in the group, horizontal separation lines are plotted between the groups, and the group names are printed.
}
\value{
A list of matrices corresponding to the plots is returned. Each of these matrices has the genes in the rows and the states of the attractors in the columns.
}
\seealso{
\code{\link{getAttractors}}, \code{\link{plotAttractors}}, \code{\link{sequenceToLaTeX}}, \code{\link{plotSequence}}
}
\examples{
\dontrun{
# load example data
data(cellcycle)

# get attractors
attractors <- getAttractors(cellcycle)

# output LaTeX document
attractorsToLaTeX(attractors, file="attractors.tex")
}
}

\name{print.AttractorInfo}
\Rdversion{1.1}
\alias{print.AttractorInfo}
\title{
Print attractor cycles
}
\description{
Specialized print method to print the attractor cycles stored in an \code{AttractorInfo} object. For simple or steady-state attractors, the states of the attractors are printed in binary encoding in the order they are reached. For asynchronous complex/loose attractors, the possible transitions of the states in the attractor are printed. The method can print either the full states, or only the active genes of the states.
}
\usage{
\method{print}{AttractorInfo}(x,
      activeOnly = FALSE, 
      ...)
}
\arguments{

\item{x}{
An object of class \code{AttractorInfo} to be printed
}
  
\item{activeOnly}{
If set to true, a state is represented by a list of active genes (i.e., genes which are set to 1). If set to false, a state is represented by a binary vector with one entry for each gene, specifying whether the gene is active or not. Defaults to \code{FALSE}. 
}
  
\item{\dots}{
Further parameters for the \code{\link{print}} method. Currently not used.
}
}
\value{
Invisibly returns the printed object
}

\seealso{
\code{\link{print}}, \code{\link{getAttractors}}
}

\name{plotStateGraph}
\Rdversion{1.1}
\alias{plotStateGraph}
\title{
Visualize state transitions and attractor basins
}
\description{
Plots a graph containing all states visited in \code{stateGraph}, and optionally highlights attractors and basins of attraction. This requires the \pkg{igraph} package.
}
\usage{
plotStateGraph(stateGraph, highlightAttractors = TRUE, 
               colorBasins = TRUE, colorSet, 
               drawLegend = TRUE, drawLabels = FALSE, 
               layout = layout.kamada.kawai,
               piecewise = FALSE,  
               basin.lty = 2, attractor.lty = 1,
               plotIt = TRUE, 
               colorsAlpha = c(colorBasinsNodeAlpha    = .3,
                               colorBasinsEdgeAlpha    = .3,
                               colorAttractorNodeAlpha = 1,
                               colorAttractorEdgeAlpha = 1),
               ...)
}

\arguments{
  \item{stateGraph}{An object of class \code{AttractorInfo} or \code{SymbolicSimulation}, as returned by \code{\link{getAttractors}} and \code{\link{simulateSymbolicModel}} respectively. As the transition table information in this structure is required, \code{getAttractors} must be called in synchronous mode and with \code{returnTable} set to TRUE. Similarly, \code{simulateSymbolicModel} must be called with \code{returnGraph=TRUE}. Alternatively, \code{stateGraph} can be an object of class \code{TransitionTable}, which can be extracted using the functions \code{\link{getTransitionTable}}, \code{\link{getBasinOfAttraction}}, or \code{\link{getStateSummary}}
} 
  \item{highlightAttractors}{
	If this parameter is true, edges in attractors are drawn bold and with a different line type (which can be specified in \code{attractor.lty}). Defaults to TRUE.
}
  \item{colorBasins}{
	If set to true, each basin of attraction is drawn in a different color. Colors can be specified in \code{colorSet}. Defaults to TRUE.
}
  \item{colorSet}{
	An optional vector specifying the colors to be used for the different attractor basins. If not supplied, a default color set is used.
}
  \item{drawLegend}{
	If set to true and \code{colorBasins} is true, a legend for the colors of the basins of attraction is drawn. Defaults to TRUE.
}
  \item{drawLabels}{
	If set to true, the binary encodings of the states are drawn beside the vertices of the graph. As this can be confusing for large graphs, the default value is FALSE.
}
  \item{layout}{
	A layouting function that determines the placement of the nodes in the graph. Please refer to the \code{\link[igraph]{layout}} manual entry in the \pkg{igraph} package for further details. By default, the Fruchterman-Reingold algorithm is used.
}

  \item{piecewise}{
  If set to true, a piecewise layout is used, i.e. the subgraphs corresponding to different basins of attraction are separated and layouted separately.  
}

  \item{basin.lty}{
	The line type used for edges in a basin of attraction. Defaults to 2 (dashed).
}
  \item{attractor.lty}{
	If \code{highlightAttractors} is true, this specifies the line type for edges in an attractor. Defaults to 1 (straight).
}
  \item{plotIt}{
	If this is true, a plot is generated. Otherwise, only an object of class \code{igraph} is returned, but no plot is drawn.
}
  \item{colorsAlpha}{
	These parameters apply alpha correction to the colors of basins and attractors in the following order: basin node, basin edge, attractor node, attractor edge. Defaults to a vector of length 4 with settings \code{alpha = 0.3} for basins and \code{alpha = 1} for attractors.
}
  \item{\dots}{
	Further graphical parameters to be passed to \code{\link[igraph:plot.graph]{plot.igraph}}.
}
}
\details{
	This function uses the \code{\link[igraph:plot.graph]{plot.igraph}} function from the \pkg{igraph} package. The plots are customizeable using the \code{\dots} argument. For details on possible parameters, please refer to \code{\link[igraph:plot.common]{igraph.plotting}}.
}
\value{
	Returns an invisible object of class \code{igraph} containing the state graph, including color and line attributes.
}
\seealso{
\code{\link{getAttractors}}, \code{\link{simulateSymbolicModel}}, \code{\link{getTransitionTable}}, \code{\link{getBasinOfAttraction}}, \code{\link{getStateSummary}}, \code{\link{plotNetworkWiring}}, \code{\link[igraph:plot.common]{igraph.plotting}}
}
\examples{
# load example data
data(cellcycle)

# get attractors
attractors <- getAttractors(cellcycle)

# plot state graph
\dontrun{
plotStateGraph(attractors, main = "Cell cycle network", layout = layout.fruchterman.reingold)
}
}

\name{print.ProbabilisticBooleanNetwork}
\Rdversion{1.1}
\alias{print.ProbabilisticBooleanNetwork}
\alias{print.BooleanNetworkCollection}
\title{
Print a probabilistic Boolean network
}
\description{
A specialized method to print an object of class \code{ProbabilisticBooleanNetwork}. For backward compatibility, this method also prints objects of class \code{BooleanNetworkCollection}, which have been replaced by \code{ProbabilisticBooleanNetwork}.
This prints all alternative transition functions and their probabilities. If the network is the result of a reconstruction from time series measurements, it also outputs the error the functions make on the time series. If genes are knocked-out or over-expressed, these genes are listed below the functions.
}
\usage{
\method{print}{ProbabilisticBooleanNetwork}(x, ...)
                               
\method{print}{BooleanNetworkCollection}(x, ...)
}
\arguments{

\item{x}{
An object of class \code{ProbabilisticBooleanNetwork} or \code{BooleanNetworkCollection} to be printed
}

\item{\dots}{
Further parameters for the \code{\link{print}} method. Currently not used.
}
}
\value{
Invisibly returns the printed object
}

\seealso{
\code{\link{print}}, \code{\link{reconstructNetwork}}, \code{\link{loadNetwork}}
}

\name{print.BooleanNetwork}
\Rdversion{1.1}
\alias{print.BooleanNetwork}
\title{
Print a Boolean network
}
\description{
A specialized method to print an object of class \code{BooleanNetwork}. This prints the transition functions of all genes. If genes are knocked-out or over-expressed, these genes are listed below the functions.
}
\usage{
\method{print}{BooleanNetwork}(x, ...)
}
\arguments{

\item{x}{
An object of class \code{BooleanNetwork} to be printed
}

\item{\dots}{
Further parameters for the \code{\link{print}} method. Currently not used.
}
}
\value{
Invisibly returns the printed object
}

\seealso{
\code{\link{print}}, \code{\link{loadNetwork}}
}

\name{BoolNet-package}
\Rdversion{1.1}
\alias{BoolNet-package}
\alias{BoolNet}
\docType{package}
\title{Construction, Simulation and Analysis of Boolean Networks}
\description{Tools for reconstruction, analysis and visualization of synchronous, asynchronous and probabilistic Boolean networks, in particular for the identification of attractors in gene-regulatory networks}
\details{
\tabular{ll}{
Package: \tab BoolNet\cr
Type: \tab Package\cr
Version: \tab 2.1.4\cr
Date: \tab 2018-05-23\cr
License: \tab Artistic-2.0\cr
LazyLoad: \tab yes\cr
}
This package provides useful methods for the construction and generation of Boolean networks and for their analysis. In particular, it is designed for the analysis of gene-regulatory networks. The software supports four types of networks:
\describe{

\item{Synchronous Boolean networks}{These networks consist of a set of Boolean variables (genes) \var{X} and a set of transition functions, one for each variable. These transition functions map an input from the set \var{X} to a Boolean value. A state is a vector of values for each of the variables in \var{X}. Then, the next state of the network is calculated by applying \emph{all} transition functions to the state.}

\item{Asynchronous Boolean networks}{Asynchronous networks have the same structure as synchronous Boolean networks. Yet, the next state of the network is calculating by choosing only \emph{one} of the transition functions at random and updating the corresponding Boolean variable (gene). This corresponds to the assumption that in a genetic network, gene expression levels are likely to change at different points of time.}

\item{Synchronous Boolean networks with time delays}{These networks additionally include dependencies on genes at time steps other than the previous time step. That is, not only the immediate predecessor state is considered to determine the next state of the network, but earlier states can be considered as well. Furthermore, it is possible to use predicates that depend on the absolute time point, i.e. the number of transitions from an initial state. }

\item{Probabilistic Boolean networks}{Probabilistic networks allow for specifying more than one transition function per variable/gene. Each of these functions has a probability to be chosen, where the probabilities of all functions for one variable sum up to 1. Transitions are performed synchronously by choosing one transition function for each gene according to their probabilities and applying them to the current state.}
}

Networks can be assembled in several ways using \pkg{BoolNet}: The \code{\link{reconstructNetwork}} function infers Boolean networks from time series of measurements using several popular reconstruction algorithms. \code{\link{binarizeTimeSeries}} provides a means of binarizing real-valued time series for these reconstruction algorithms.
Boolean networks (synchronous, asynchronous, and probabilistic networks) can also be expressed in a description language and loaded from files using \code{\link{loadNetwork}} or stored to files using \code{\link{saveNetwork}}. Furthermore, networks can be imported from BioTapestry using \code{\link{loadBioTapestry}} and from SBML with the \code{sbml-qual} package using \code{\link{loadSBML}}. The package also includes an export to SBML (see \code{\link{toSBML}}).

Via \code{\link{generateRandomNKNetwork}} and \code{\link{perturbNetwork}}, the package supports various methods of generating random networks and perturbing existing networks for robustness analysis.

The \code{\link{getAttractors}} function identifies attractor cycles in a synchronous or asynchronous Boolean network. Attractors can be identified by exhaustive search or heuristic methods. For networks with time delays, the function \code{simulateSymbolicModel} simulates the model and identifies attractors.

The \code{\link{markovSimulation}} function identifies relevant states in probabilistic Boolean networks by performing a Markov chain simulation.

The package also provides methods to visualize state transitions and basins of attraction \if{latex}{\cr}
(\code{\link{plotPBNTransitions}}, \code{\link{plotStateGraph}}), to plot the wiring of a network \if{latex}{\cr}(\code{\link{plotNetworkWiring}}), to plot attractor state tables (\code{\link{plotAttractors}}) and sequences of states (\code{\link{plotSequence}}), and to export them to LaTeX (\code{\link{attractorsToLaTeX}} and \code{\link{sequenceToLaTeX}}) and Pajek (\code{\link{toPajek}}). 

Transition tables of the network can be analyzed using \code{\link{getTransitionTable}}. Paths from start states to their corresponding attractors are identified using \code{\link{getPathToAttractor}}.
}
\author{
Christoph Müssel, Martin Hopfensitz, Dao Zhou, Hans A. Kestler

Contributors: Armin Biere (contributed PicoSAT code), Troy D. Hanson (contributed uthash macros)

Maintainer: Hans A. Kestler <hans.kestler@uni-ulm.de>

}
\references{
S. A. Kauffman (1969), Metabolic stability and epigenesis in randomly constructed nets. J. Theor. Biol. 22:437--467.

S. A. Kauffman (1993), The Origins of Order. Oxford University Press.

Further references are listed in the corresponding help sections.
}
\examples{
##################################
# Example 1: identify attractors #
################################## 

# load example data
data(cellcycle)

# get all synchronous attractors by exhaustive search
attractors <- getAttractors(cellcycle)

# plot attractors side by side
par(mfrow=c(2,length(attractors$attractors)))
plotAttractors(attractors)

# identifies asynchronous attractors
attractors <- getAttractors(cellcycle, 
                            type="asynchronous", startStates=100)
                            
plotAttractors(attractors, mode="graph")

####################################
# Example 2: reconstruct a network #
####################################

# load example data
data(yeastTimeSeries)

# perform binarization with k-means
bin <- binarizeTimeSeries(yeastTimeSeries)

# reconstruct networks from transition table
net <- reconstructNetwork(bin$binarizedMeasurements, 
                          method="bestfit", maxK=3, returnPBN=TRUE)

# analyze the network using a Markov chain simulation
print(markovSimulation(net, returnTable=FALSE))
}

\name{markovSimulation}
\alias{markovSimulation}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Identify important states in probabilistic Boolean networks
}
\description{
Identifies important states in probabilistic Boolean networks (PBN) using a Markov chain simulation
}
\usage{
markovSimulation(network, 
                 numIterations = 1000, 
                 startStates = list(), 
                 cutoff = 0.001, 
                 returnTable = TRUE)
}

\arguments{
  \item{network}{
  An object of class \code{ProbabilisticBooleanNetwork} or \code{BooleanNetwork} whose transitions are simulated
}
  \item{numIterations}{
  The number of iterations for the matrix multiplication, which corresponds to the number of state transitions to simulate
}
  \item{startStates}{
  An optional list of start states. Each entry of the list must be a vector with a 0/1 value for each gene. If specified, the simulation is restricted to the states reachable from the supplied start states. Otherwise, all states are considered.
}
  \item{cutoff}{
  The cutoff value used to determine if a probability is 0. All output probabilities less than or equal to this value are set to 0.
}
  \item{returnTable}{
  If set to true, a transition table annotated with the probabilities for the transitions is included in the results. This is required by \code{\link{plotPBNTransitions}} and \code{\link{getTransitionProbabilities}}.
}
}
\details{
The algorithm identifies important states by performing the following steps: First, a Markov matrix is calculated from the set of transition functions, where each entry of the matrix specifies the probability of a state transition from the state belonging to the corresponding row to the state belonging to the corresponding column. A vector is initialized with uniform probability for all states (or -- if specified -- uniform probability for all start states) and repeatedly multiplied with the Markov matrix. The method returns all states with non-zero probability in this vector.
See the references for more details.
}
\value{
An object of class \code{MarkovSimulation} with the following components:
\item{reachedStates}{A data frame with one state in each row. The first columns specify the gene values of the state, and the last column holds the probability that the corresponding state is reached after \code{numIterations} transitions. Only states with a probability greater than \code{cutoff} are included in this table.}
\item{genes}{A vector of gene names of the input network}
\item{table}{If \code{returnTable=TRUE}, this structure holds a table of transitions with the corresponding probabilities that transitions are chosen. This is a list with the following components:
\describe{
    \item{initialStates}{A matrix of encoded start states of the transitions}
    \item{nextStates}{The encoded successor states of the transitions}
    \item{probabilities}{The probabilities that the transitions are chosen in a single step}
}}
}

\references{
I. Shmulevich, E. R. Dougherty, S. Kim, W. Zhang (2002), Probabilistic Boolean networks: a rule-based uncertainty model for gene regulatory networks. Bioinformatics 18(2):261--274.
}

\seealso{
\code{\link{reconstructNetwork}}, \code{\link{plotPBNTransitions}}, \code{\link{getTransitionProbabilities}}
}
\examples{
  \dontrun{
# load example network
data(examplePBN)

# perform a Markov chain simulation
sim <- markovSimulation(examplePBN)

# print the relevant states and transition probabilities
print(sim)

# plot the transitions and their probabilities
plotPBNTransitions(sim)
}
}

\name{plotNetworkWiring}
\alias{plotNetworkWiring}
\title{
 Plot the wiring of a Boolean network
}
\description{
Plots the wiring of genes (i.e. the gene dependencies) of a synchronous or probabilistic Boolean network. The nodes of the graph are the genes, and the directed edges show the dependencies of the genes. This requires the \pkg{igraph} package.
}
\usage{
plotNetworkWiring(network, 
                  layout = layout.fruchterman.reingold, 
                  plotIt = TRUE, ...)
}
\arguments{
  \item{network}{
	A network structure of class \code{BooleanNetwork}, \code{SymbolicBooleanNetwork} or \code{ProbabilisticBooleanNetwork}. These networks can be read from files by \code{\link{loadNetwork}}, generated by \if{latex}{\cr}\code{\link{generateRandomNKNetwork}}, or reconstructed by \code{\link{reconstructNetwork}}.
}
  \item{layout}{
A layouting function that determines the placement of the nodes in the graph. Please refer to the \code{\link[igraph]{layout}} manual entry in the \pkg{igraph} package for further details. By default, the Fruchterman-Reingold algorithm is used.
}
  \item{plotIt}{
If this is true, a plot is generated. Otherwise, only an object of class \code{igraph} is returned, but no plot is drawn.
}
  \item{\dots}{
Further graphical parameters to be passed to \code{\link[igraph:plot.graph]{plot.igraph}}.
}
}
\details{
	This function uses the \code{\link[igraph:plot.graph]{plot.igraph}} function from the \pkg{igraph} package. The plots are customizeable using the \code{\dots} argument. For details on possible parameters, please refer to \code{\link[igraph:plot.common]{igraph.plotting}}.
}
\value{
	Returns an invisible object of class \code{igraph} containing the wiring graph.
}

\seealso{
\code{\link{loadNetwork}}, \code{\link{generateRandomNKNetwork}}, \code{\link{reconstructNetwork}}, \code{\link{plotStateGraph}}, \code{\link[igraph:plot.graph]{igraph.plotting}}
}
\examples{
\dontrun{
# load example data
data(cellcycle)

# plot wiring graph
plotNetworkWiring(cellcycle)
}
}

\name{getTransitionProbabilities}
\alias{getTransitionProbabilities}
\title{
Get a matrix of transitions and their probabilities in probabilistic Boolean networks
}
\description{
Retrieves the state transitions and their probabilities in a probabilistic Boolean network. This takes the transition table information calculated by the \code{\link{markovSimulation}} method. 
}
\usage{
getTransitionProbabilities(markovSimulation)
}

\arguments{
  \item{markovSimulation}{
  An object of class \code{MarkovSimulation}, as returned by \code{\link{markovSimulation}}. As the transition table information in this structure is required, \code{\link{markovSimulation}} must be called with \code{returnTable} set to TRUE.
}
}

\value{
Returns a data frame with the first \code{n} columns describing the values of the genes before the transition, the next \code{n} columns describing the values of the genes after the transition, and the last column containing the probability of the transition. Here, \code{n} is the number of genes in the underlying network. Only transitions with non-zero probability are included.
}

\seealso{
\code{\link{markovSimulation}}
}
\examples{
\dontrun{
# load example network
data(examplePBN)

# perform a Markov chain simulation
sim <- markovSimulation(examplePBN)

# print out the probability table
print(getTransitionProbabilities(sim))
}
}

\name{getStateSummary}
\Rdversion{1.1}
\alias{getStateSummary}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Retrieve summary information on a state
}
\description{
Returns information on the supplied state, i.e. the successive state after a transition, the (synchronous) attractor to which the state leads, and the distance to this attractor.
}
\usage{
getStateSummary(attractorInfo, state)
}

\arguments{
  \item{attractorInfo}{
	An object of class \code{AttractorInfo}, as returned by \code{\link{getAttractors}}, or of class \code{SymbolicSimulation}, as returned by \code{\link{simulateSymbolicModel}}. As the transition table information in this structure is required, \code{getAttractors} must be called in synchronous mode and with \code{returnTable} set to TRUE. Similarly, \code{simulateSymbolicModel} must be called with \code{returnGraph=TRUE}.
}
  \item{state}{
	A 0-1 vector with n elements (where n is the number of genes in the underlying networks) describing the state.
}
}
\value{
Returns a generic dataframe of the class \code{TransitionTable}. For n genes, the first n columns code for the original state (in this case, the \code{state} parameter), i.e. each column represents the value of one gene. The next n columns code for the successive state after a transition. The column \code{attractorAssignment} indicates the attractor to the state is assigned. If this information is available,  the column \code{stepsToAttractor} indicates how many transitions are needed from the original state to the attractor. In this case, the table has only one row describing the supplied state.
The \code{TransitionTable} class supports pretty printing using the \code{\link{print}} method.
}
\seealso{
\code{\link{getBasinOfAttraction}}, \code{\link{getTransitionTable}}, \code{\link{getAttractors}}, \code{\link{simulateSymbolicModel}}
}
\examples{
\dontrun{
# load example data
data(cellcycle)

# get attractors
attractors <- getAttractors(cellcycle)

# print information for an arbitrary state
print(getStateSummary(attractors, c(1,1,1,1,1,1,1,1,1,1)))
}
}

\name{reconstructNetwork}
\alias{reconstructNetwork}
\title{
Reconstruct a Boolean network from time series of measurements
}
\description{
Reconstructs a Boolean network from a set of time series or from a transition table using the best-fit extension algorithm or the REVEAL algorithm.
}
\usage{
reconstructNetwork(measurements, 
                   method = c("bestfit", "reveal"), 
                   maxK = 5, 
                   requiredDependencies = NULL, 
                   excludedDependencies = NULL, 
                   perturbations=NULL,
                   readableFunctions=FALSE, 
                   allSolutions=FALSE, 
                   returnPBN=FALSE)
}
\arguments{
  \item{measurements}{
	This can either be an object of class \code{TransitionTable} as returned by \if{latex}{\cr}\code{\link{getTransitionTable}}, or a set of time series of measurements. In this case, \code{measurements} must be a list of matrices, each corresponding to one time series. Each row of these matrices contains measurements for one gene on a time line, i. e. column \code{i+1} contains the successor states of column \code{i}. The genes must be the same for all matrices in the list. Real-valued time series can be binarized using \code{\link{binarizeTimeSeries}}.
}
  \item{method}{
	This specifies the reconstruction algorithm to be used. If set to "bestfit", Laehdesmaeki's Best-Fit Extension algorithm is employed. This algorithm is an improvement of the algorithm by Akutsu et al. with a lower runtime complexity. It determines the functions with a minimal error for each gene. If set to "reveal", Liang's REVEAL algorithm is used. This algorithm searches for relevant input genes using the mutual information between the input genes and the output gene. 
}
  \item{maxK}{
	The maximum number of input genes for one gene to be tested. Defaults to 5.
}

  \item{requiredDependencies}{An optional specification of known dependencies that must be included in reconstructed networks. This is a named list containing a vector of gene names (regulators) for each target.}
  
  \item{excludedDependencies}{Analogous to \code{requiredDependencies}, this is an optional specification of dependencies that must not be included in reconstructed networks. This is a named list containing a vector of gene names (prohibited regulators) for each target.}
    
  \item{perturbations}{If \code{measurements} contains data obtained from perturbation experiments (i.e. different targeted knock-outs and overexpressions), this optional parameter is a matrix with one column for each entry in \code{measurements} and a row for each gene. A matrix entry is 0 for a knock-out of the corresponding gene in the corresponding time series, 1 for overexpression, and NA or -1 for no perturbation. If \code{measurements} has an element \code{perturbations} and this argument is not specified, the element of \code{measurements} is taken. 
}  
  
  \item{readableFunctions}{
	If this is true, readable DNF representations of the truth tables of the functions are generated. These DNF are displayed when the network is printed. The DNF representations are not minimized and can thus be very long. If set to FALSE, the truth table result column is displayed.
}

  \item{allSolutions}{
  If this is true, all solutions with the minimum error and up to \code{maxK} inputs are returned. By default, \code{allSolutions=FALSE}, which means that only the solutions with both minimum error and minimum \code{k} are returned.
  }
  
  \item{returnPBN}{Specifies the way unknown values in the truth tables of the transition functions ("don't care" values) are processed. If \code{returnPBN=TRUE}, all possible functions are enumerated recursively, and an object of class \code{ProbabilisticBooleanNetwork} is returned. This can consume a high amount of memory and computation time. If \code{returnPBN=FALSE}, the transition functions may contain "don't care" (*) values, and an object of class \code{BooleanNetworkCollection} is returned. \code{returnPBN=TRUE} corresponds to the behaviour prior to version 2.0. The default value is \code{returnPBN=FALSE}.
  }

}

\details{
Both algorithms iterate over all possible input combinations. While Best-Fit Extension is capable of returning functions that do not perfectly explain the measurements (for example, if there are inconsistent measurements or if \code{maxK} was specified too small), REVEAL only finds functions that explain all measurements. For more information, please refer to the cited publications.
}
\value{
If \code{returnPBN=TRUE}, the function returns an object of class \code{ProbabilisticBooleanNetwork}, with each alternative function of a gene having the same probability. The structure is described in detail in \code{\link{loadNetwork}}. In addition to the standard components, each alternative transition function has a component \code{error} which stores the error of the function on the input time series data.
If \code{returnPBN=FALSE}, the function returns an object of class \code{BooleanNetworkCollection} that has essentially the same structure as \code{ProbabilisticBooleanNetwork}, but does not store probabilities and keeps "don't care" values in the functions. Due to the "don't care" (*) values, this collection cannot be simulated directly. However, a specific Boolean network of class \code{BooleanNetwork} can be extracted from both \code{BooleanNetworkCollection} and \code{ProbabilisticBooleanNetwork} structures using \code{\link{chooseNetwork}}.

}
\references{
H. Laehdesmaeki, I. Shmulevich and O. Yli-Harja (2003), On Learning Gene-Regulatory Networks Under the Boolean Network Model. Machine Learning 52:147--167.

T. Akutsu, S. Miyano and S. Kuhara (2000). Inferring qualitative relations in genetic networks and metabolic pathways. Bioinformatics 16(8):727--734.

S. Liang, S. Fuhrman and R. Somogyi (1998), REVEAL, a general reverse engineering algorithm for inference of genetic network architectures. Pacific Symposium on Biocomputing 3:18--29.

}


\seealso{
\code{\link{generateTimeSeries}}, \code{\link{binarizeTimeSeries}}, \code{\link{chooseNetwork}}
}
\examples{
\dontrun{
# load example data
data(yeastTimeSeries)

# perform binarization with k-means
bin <- binarizeTimeSeries(yeastTimeSeries)

# reconstruct networks from binarized measurements
net <- reconstructNetwork(bin$binarizedMeasurements, method="bestfit", maxK=3, returnPBN=TRUE)

# print reconstructed net
print(net)

# plot reconstructed net
plotNetworkWiring(net)
}
}

\name{examplePBN}
\alias{examplePBN}
\docType{data}
\title{
An artificial probabilistic Boolean network
}
\description{
An artificial probabilistic Boolean network example introduced by Shmulevich et al.
}
\usage{data(examplePBN)}

\details{
This artificial network is introduced by Shmulevich et al. for a step-by-step description of their Markov chain algorithm. It is included as a general example for a probabilistic Boolean network. The network consists of 3 genes, where gene 1 and gene 3 have two alternative transition functions, and gene 1 has a unique transition function.
}
\source{
I. Shmulevich, E. R. Dougherty, S. Kim, W. Zhang (2002), Probabilistic Boolean networks: a rule-based uncertainty model for gene regulatory networks. Bioinformatics 18(2):261--274.
}
\examples{
data(examplePBN)

# the network is stored in a variable called 'examplePBN'
print(examplePBN)
}

\name{toPajek}
\Rdversion{1.1}
\alias{toPajek}
\title{
Export a network to the Pajek file format
}
\description{
Exports a network to the Pajek file format to visualize transition trajectories. For more information on Pajek, please refer to \url{http://pajek.imfm.si/doku.php}
}
\usage{
toPajek(stateGraph, file = "boolean.net", includeLabels=FALSE, ...)
}
\arguments{
  \item{stateGraph}{
	An object of class \code{AttractorInfo} or \code{SymbolicSimulation}, as returned by \code{\link{getAttractors}} and \code{\link{simulateSymbolicModel}} respectively. As the transition table information in this structure is required, \code{getAttractors} must be called in synchronous mode and with \code{returnTable} set to TRUE. Similarly, \code{simulateSymbolicModel} must be called with \code{returnGraph=TRUE}. Alternatively, \code{stateGraph} can be an object of class \code{TransitionTable}, which can be extracted using the functions \code{\link{getTransitionTable}}, \code{\link{getBasinOfAttraction}}, or \code{\link{getStateSummary}} .
}
  \item{file}{
	The name of the output file for Pajek. Defaults to "boolean.net".
}
 \item{includeLabels}{
	If set to true, the vertices of the graph in the output file are labeled with the binary encodings of the states. Defaults to FALSE.
}

 \item{\dots}{
This is only for compatibility with previous versions and should not be used. 
}
}
\value{
This function has no return value.
}
\seealso{
\code{\link{getAttractors}}, \code{\link{simulateSymbolicModel}}, \code{\link{getTransitionTable}}, \code{\link{getBasinOfAttraction}}, \code{\link{getStateSummary}}, \code{\link{toSBML}}
}
\examples{
\dontrun{
# load example data
data(cellcycle)

# get attractors
attractors <- getAttractors(cellcycle)

# export to Pajek
toPajek(attractors, file="pajek_export.net")
}
}

\name{toSBML}
\alias{toSBML}
\title{
Export a network to SBML
}
\description{
Exports a synchronous or asynchronous Boolean network to SBML with the \code{sbml-qual} extension package.
}
\usage{
toSBML(network, 
       file,
       generateDNFs = FALSE,
       saveFixed = TRUE)
}
\arguments{
  \item{network}{
An object of class \code{BooleanNetwork} or \code{SymbolicBooleanNetwork} to be exported
}
  \item{file}{
The name of the SBML file to be created
}

  \item{generateDNFs}{
If \code{network} is a \code{BooleanNetwork} object, this parameter specifies whether formulae in Disjunctive Normal Form are exported instead of the expressions that describe the transition functions. If set to FALSE, the original expressions are exported. If set to "canonical", a canonical Disjunctive Normal Form is generated from each truth table. If set to "short", the canonical DNF is minimized by joining terms (which can be time-consuming for functions with many inputs). If set to TRUE, a short DNF is generated for functions with up to 12 inputs, and a canonical DNF is generated for functions with more than 12 inputs.
For objects of class \code{SymbolicBooleanNetwork}, this parameter is ignored.
}

  \item{saveFixed}{
If set to TRUE, knock-outs and overexpression of genes override their transition functions. That is, if a gene in the network is fixed to 0 or 1, this value is exported, regardless of the transition function. If set to FALSE, the transition function is exported. Defaults to TRUE.
}
}
\details{
The export creates an SBML file describing a general logical model that corresponds to the Boolean network. Importing tools must support the \code{sbml-qual} extension package version 1.0.

The export translates the expressions that describe the network transition functions to a MathML description. If these expressions cannot be parsed or \code{generateDNFs} is true, a DNF representation of the transition functions is generated and exported.

For symbolic networks, temporal operators and delays of more than one time step are not allowed, as they are not compatible with SBML.
}

\references{
\url{http://sbml.org/Documents/Specifications/SBML_Level_3/Packages/Qualitative_Models_(qual)}
}

\seealso{
\code{\link{loadSBML}}, \code{\link{loadNetwork}}, \code{\link{saveNetwork}}, \code{\link{toPajek}}
}
\examples{
\dontrun{
# load the cell cycle network
data(cellcycle)

# export the network to SBML
toSBML(cellcycle, file="cellcycle.sbml")

# reimport the model
print(loadSBML("cellcycle.sbml"))
}
}

\name{plotPBNTransitions}
\alias{plotPBNTransitions}
\title{
Visualize the transitions in a probabilistic Boolean network
}
\description{
Visualizes the state transitions and their probabilities in a probabilistic Boolean network. This takes the transition table information calculated by the \code{\link{markovSimulation}} method. Only transitions with non-zero probability are included in the plot. The function requires the \pkg{igraph} package.
}
\usage{
plotPBNTransitions(markovSimulation, 
                   stateSubset, 
                   drawProbabilities = TRUE, 
                   drawStateLabels = TRUE, 
                   layout = layout.fruchterman.reingold, 
                   plotIt = TRUE, ...)
}
\arguments{
  \item{markovSimulation}{
  An object of class \code{MarkovSimulation}, as returned by \code{\link{markovSimulation}}. As the transition table information in this structure is required, \code{\link{markovSimulation}} must be called with \code{returnTable} set to TRUE.
}
  \item{stateSubset}{
An optional list of states, where each element of the list must be a vector with a 0/1 entry for each gene. If this argument is supplied, the graph only contains the specified states and transitions between these states.
}
  \item{drawProbabilities}{
If set to true, the edges of the graph are annotated with the probabilities of the corresponding transitions. Default is TRUE.
}
  \item{drawStateLabels}{
If set to true, the vertices of the graph are annotated with the gene values of the corresponding states. Defaults to TRUE.
}
  \item{layout}{
A layouting function that determines the placement of the nodes in the graph. Please refer to the \code{\link[igraph]{layout}} manual entry in the \pkg{igraph} package for further details. By default, the Fruchterman-Reingold algorithm is used.
}
  \item{plotIt}{
If this is true, a plot is generated. Otherwise, only an object of class \code{igraph} is returned, but no plot is drawn.
}
  \item{\dots}{
Further graphical parameters to be passed to \code{\link[igraph:plot.graph]{plot.igraph}}.
}
}

\value{
Returns an invisible object of class \code{igraph} containing the wiring graph.
}

\details{
This function uses the \code{\link[igraph:plot.graph]{plot.igraph}} function from the \pkg{igraph} package. The plots are customizeable using the \code{\dots} argument. For details on possible parameters, please refer to \code{\link[igraph:plot.common]{igraph.plotting}}.
}

\seealso{
\code{\link{markovSimulation}}
}
\examples{
\dontrun{
# load example network
data(examplePBN)

# perform a Markov chain simulation
sim <- markovSimulation(examplePBN)

# plot the transitions and their probabilities
plotPBNTransitions(sim)
}
}

\name{fixGenes}
\alias{fixGenes}

\title{
Simulate knocked-out or over-expressed genes
}
\description{
Simulates knocked-out or over-expressed genes by fixing the values of genes to 0 or 1, or turn off knock-out or over-expression of genes.
}
\usage{
fixGenes(network, fixIndices, values)
}
\arguments{
  \item{network}{
	The original network of class \code{BooleanNetwork}, \if{latex}{\cr} \code{SymbolicBooleanNetwork} or \code{ProbabilisticBooleanNetwork} containing the genes to be fixed
}
  \item{fixIndices}{
	A vector of names or indices of the genes to be fixed
}
  \item{values}{
	Either one single value, or a vector with the same length as \code{fixIndices}. For each gene, a value of 1 means that the gene is always turned on (over-expressed), a value of 0 means that the gene is always turned off (knocked-out), and a value of -1 means that the gene is not fixed.
}
}
\value{
Depending on the input, an object of class \code{BooleanNetwork}, \code{SymbolicBooleanNetwork} or \code{ProbabilisticBooleanNetwork} containing the fixed genes is returned. These classes are described in more detail in \code{\link{loadNetwork}}.
}
\seealso{
\code{\link{loadNetwork}}}

\examples{
\dontrun{
# load example data
data(cellcycle)

# knock out gene CycD (index 1)
net <- fixGenes(cellcycle, 1, 0)
# or
net <- fixGenes(cellcycle, "CycD", 0)

# get attractors by exhaustive search
attractors <- getAttractors(net)

print(attractors)
}
}

\name{truthTableToSymbolic}
\alias{truthTableToSymbolic}
\title{
Convert a network in truth table representation into a symbolic representation
}
\description{
Converts an object of class \code{BooleanNetwork} into an object of class \code{SymbolicBooleanNetwork} by generating symbolic expression trees.
}
\usage{
truthTableToSymbolic(network, generateDNFs = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{network}{
An object of class \code{BooleanNetwork} to be converted.
}

\item{generateDNFs}{
This parameter specifies whether formulae in Disjunctive Normal Form are generated instead of the parsing the string expressions that describe the transition functions. If set to FALSE, the original expressions are parsed. If set to "canonical", a canonical Disjunctive Normal Form is generated from each truth table. If set to "short", the canonical DNF is minimized by joining terms (which can be time-consuming for functions with many inputs). If set to TRUE, a short DNF is generated for functions with up to 12 inputs, and a canonical DNF is generated for functions with more than 12 inputs.
}
}

\value{
Returns an object of class \code{SymbolicBooleanNetwork}, as described in \code{\link{loadNetwork}}.
}

\seealso{
\code{\link{truthTableToSymbolic}}, \code{\link{loadNetwork}}
}
\examples{
\dontrun{
# Convert a truth table representation into a 
# symbolic representation and back
data(cellcycle)

symbolicNet <- truthTableToSymbolic(cellcycle)
print(symbolicNet)

ttNet <- symbolicToTruthTable(symbolicNet)
print(cellcycle)
}
}

\name{getPathToAttractor}
\alias{getPathToAttractor}
\title{
Get state transitions between a state and its attractor
}
\description{
Lists the states in the path from a specified state to the corresponding synchronous attractor.
}
\usage{
getPathToAttractor(network, 
                   state,
                   includeAttractorStates = c("all","first","none"))
}
\arguments{
  \item{network}{Either a network structure of class \code{BooleanNetwork} or \code{SymbolicBooleanNetwork} , or an attractor search result of class \code{AttractorInfo}. In the former case, a synchronous attractor search starting from \code{state} is conducted. In the latter case, \code{network} must be the result of a call to \code{\link{getAttractors}} with \code{returnTable=TRUE}, and its transition table must include \code{state}.    
}

  \item{state}{
  A binary vector with exactly one entry per gene in the network. If \code{network} is of class \code{SymbolicBooleanNetwork} and makes use of more than one predecessor state, this can also be a matrix with the genes in the columns and multiple predecessor states in the rows.
}

  \item{includeAttractorStates}{
  Specifies whether the actual attractor states are included in the resulting table or not. If \code{includeAttractorStates = "all"} (which is the default behaviour), the sequence ends when the attractor was traversed once. If \code{includeAttractorStates = "first"}, only the first state of attractor is added to the sequence. This corresponds to the behaviour prior to \pkg{BoolNet} version 1.5. If {includeAttractorStates = "none"}, the sequence ends with the last non-attractor state. In this case, the sequence can be empty if the start state is an attractor state.
}
}
\value{
Returns a data frame with the genes in the columns. The rows are the successive states from \code{state} to the  the corresponding attractor. Depending on \code{includeAttractorStates}, attractor states are included or not. The data frame has an attribute \code{attractor} specifying the indices of the states that belong to the attractor. If \code{includeAttractorStates} is \code{"first"} or \code{"none"}, these indices may correspond to states that are not included in the sequence itself. This attribute is used by \code{\link{plotSequence}} to highlight the attractor states.
}
\seealso{
\code{\link{getAttractors}}, \code{\link{simulateSymbolicModel}}, \code{\link{getTransitionTable}}, \code{\link{getBasinOfAttraction}}, \code{\link{plotSequence}}, \code{\link{attributes}}
}
\examples{
\dontrun{
# load example network
data(cellcycle)

# get path from a state to its attractor
# include all attractor states
path <- getPathToAttractor(cellcycle, rep(1,10), 
                           includeAttractorStates="all")
print(path)

# include only the first attractor state
path <- getPathToAttractor(cellcycle, rep(1,10), 
                           includeAttractorStates="first")
print(path)

# exclude attractor states
path <- getPathToAttractor(cellcycle, rep(1,10), 
                           includeAttractorStates="none")
print(path)
}
}

\name{perturbTrajectories}
\alias{perturbTrajectories}

\title{
Perturb the state trajectories and calculate robustness measures
}
\description{
Perturbs the state trajectories of a network and assesses the robustness by comparing the successor states or the attractors of a set of initial states and a set of perturbed copies of these initial states.
}
\usage{
perturbTrajectories(network, 
                    measure = c("hamming", "sensitivity", "attractor"), 
                    numSamples = 1000, 
                    flipBits = 1, 
                    updateType = c("synchronous", "asynchronous", "probabilistic"), 
                    gene, 
                    ...)
}
\arguments{
  \item{network}{
A network structure of class \code{BooleanNetwork}, \code{SymbolicBooleanNetwork} or \code{ProbabilisticBooleanNetwork} whose robustness is measured.
}
  \item{measure}{
Defines the way the robustness is measured (see Details).
}
  \item{numSamples}{
The number of randomly generated pairs of initial states and perturbed copies. Defaults to 1000.
}
  \item{flipBits}{
The number of bits that are flipped to generate a perturbed copy of an initial state. Defaults to 1.
}
  \item{updateType}{
If \code{measure="hamming"}, the type of update that is performed to calculate successor states.
}
  \item{gene}{
If \code{measure="sensitivity"}, the name or index of the gene for whose transition function the average sensitivity is calculated.
}
  \item{\dots}{
Further parameters to \code{\link{stateTransition}} and \code{\link{getAttractors}}.
}
}

\details{
The function generates a set of \code{numSamples} initial states and then applies \code{flipBits} random bit flips to each initial state to generate a perturbed copy of each initial state. For each pair of initial state and perturbed state, a robustness statistic is calculated depending \code{measure}:

If \code{measure="hamming"}, the normalized Hamming distances between the successor states of each initial state and the corresponding perturbed state are calculated. 

If \code{measure="sensitivity"}, the average sensitivity of a specific transition function (specified in the \code{gene} parameter) is approximated: The statistic is a logical vector that is \code{TRUE} if \code{gene} differs in the successor states of each initial state and the corresponding perturbed state.

If \code{measure="attractor"}, the attractors of all initial states and all perturbed states are identified. The statistic is a logical vector specifying whether the attractors are identical in each pair of initial state and perturbed initial state.
}
\value{
A list with the following items:
 \item{stat}{A vector of size \code{numSamples} containing the robustness statistic for each pair of initial state and perturbed copy.}
 \item{value}{The summarized statistic (i.e. the mean value) over all state pairs.}
}
\references{
I. Shmulevich and S. A. Kauffman (2004), Activities and Sensitivities in Boolean Network Models. Physical Review Letters 93(4):048701.
}

\seealso{
\code{\link{testNetworkProperties}}, \code{\link{perturbNetwork}}
}
\examples{
\dontrun{
data(cellcycle)

# calculate average normalized Hamming distance of successor states
hamming <- perturbTrajectories(cellcycle, measure="hamming", numSamples=100)
print(hamming$value)

# calculate average sensitivity of transition function for gene "Cdh1"
sensitivity <- perturbTrajectories(cellcycle, measure="sensitivity", numSamples=100, gene="Cdh1")
print(sensitivity$value)

# calculate percentage of equal attractors for state pairs
attrEqual <- perturbTrajectories(cellcycle, measure="attractor", numSamples=100)
print(attrEqual$value)
}
}

\name{simulateSymbolicModel}
\alias{simulateSymbolicModel}

\title{
Simulate a symbolic Boolean network
}
\description{
This function simulates Boolean networks in a symbolic representation, possibly with additional temporal qualifiers. The function can identify attractors, determine the state transition graph, and generate sequences of successive states.
}
\usage{
simulateSymbolicModel(network, 
              method = c("exhaustive", 
                         "random", 
                         "chosen", 
                         "sat.exhaustive", 
                         "sat.restricted"), 
              startStates = NULL,               
              returnSequences = 
                (!(match.arg(method) \%in\% 
              	  c("sat.exhaustive", "sat.restricted"))), 
              returnGraph = 
                  (!(match.arg(method) \%in\% 
              	    c("sat.exhaustive", "sat.restricted"))), 
              returnAttractors = TRUE, 
              maxTransitions = Inf,
              maxAttractorLength = Inf,
              canonical = TRUE)
}

\arguments{
  \item{network}{
A network structure of class \code{SymbolicBooleanNetwork}. These networks can be read from files by \code{\link{loadNetwork}}, \code{\link{loadBioTapestry}} or \code{\link{loadSBML}} with the \code{symbolic=TRUE} flag.
}
  \item{startStates}{
An optional parameter specifying the start states. If this is an integer value, it denotes the number of random start states to generate. Otherwise, it has to be a list of states. The list elements must either be vectors with one value for each gene in the network, or matrices with the genes in the columns and multiple predecessor states in the rows. These predecessor states may be evaluated if temporal predicates in the network have a time delay of more than one. If the number of supplied predecessor states is smaller than the maximum time delay in the network, genes are assumed to have had the same value as in the first supplied state prior to this state. In particular, if only a single state is supplied, it is assumed that the network always resided in this state prior to starting the simulation.
}
  \item{method}{
The simulation method to be used (see details). If \code{method} is not specified, the desired method is inferred from the type of \code{startStates}.
}
  \item{returnSequences}{
If set to true (and no SAT-based method is chosen), the return value has an element \code{sequences} specifying the sequences of states to the attractor.
}
  \item{returnGraph}{
If set to true (and no SAT-based method is chosen), the return value has an element \code{graph} specifying the state transition graph of the network.
}
  \item{returnAttractors}{
If set to true, the return value has an element \code{attractors} containing a list of identified attractors.
}
  \item{maxTransitions}{
The maximum number of state transitions to be performed for each start state (defaults to \code{Inf}).
}

\item{maxAttractorLength}{If \code{method="sat.restricted"}, this required parameter specifies the maximum size of attractors (i.e. the number of states in the loop) to be searched. For \code{method="sat.exhaustive"}, this parameter is optional and specifies the maximum attractor length for the initial length-restricted search phase that is performed to speed up the subsequent exhaustive search. In this case, changing this value might bring performance benefits, but does not change the results.}

 \item{canonical}{
	If set to true and \code{returnAttractors=TRUE}, the states in the attractors are rearranged such that the state whose binary encoding
makes up the smallest number is the first element of the vector. This ensures that attractors determined in runs with different start states are comparable, as the cycles may have been entered at different states.
}
}

\details{
Similarly to \code{\link{getAttractors}}, the symbolic simulator supports different simulation modes which can be specified in the \code{method} parameter:
\itemize{
\item{Exhaustive search} {If \code{method="exhaustive"}, all possible states in the network are used as start states. If the network has time delays greater than one (temporal network), this means that exhaustive search does not only cover all 2^n possible states for a network with n genes, but also all possible state histories of those genes for which longer delays are required.}
  	
\item{Heuristic search} {For \code{method="random"} or \code{method="chosen"}, a subset of states is used as start states for the simulation.

If \code{method="random"}, \code{startStates} is interpreted as an integer value specifying the number of states to be generated randomly. The algorithm is then initialized with these random start states.
  	
If \code{method="chosen"}, \code{startStates} is interpreted as a list of binary vectors, each specifying one start state (see also parameter description above for details). }

\item{SAT-based attractor search} { 	
If \code{method} is "sat.exhaustive" or "sat.restricted", the simulator transforms the network into a satisfiability problem and solves it using Armin Biere's PicoSAT solver (see also \code{\link{getAttractors}} for more details). If \code{method="sat.restricted"}, only attractors comprising up to \code{maxAttractorLength} states are identified. Otherwise, the algorithm by Dubrova and Teslenko is applied to identify all attractors. As the SAT-based approaches identify attractors directly, no state sequences and no transition graph are returned.}
}
}

\value{
Returns a list of class \code{SymbolicSimulation} containing the simulation results:

If \code{returnSequences} is true and no SAT-based method was chosen, the list contains an element \code{sequences} consisting of a list of data frames, each representing the state transitions performed from one start state (denoted as time step 0) to the attractor. Here, the columns correspond to the genes in the network, and the rows correspond to the states. Apart from the immediate start state, the sequences may also contain the supplied or assumed predecessor states of the start state (marked by a negative time step t) if the network contains time delays greater than one. 

If \code{returnGraph} is true and no SAT-based method was chosen, the list contains an element \code{graph} of class \code{TransitionTable}. Each row of the table corresponds to one state transition from an initial state to a successor state, i.e. an edge in the state transition graph.

If \code{returnAttractors} is true, the list contains an element \code{attractors}, which itself is a list of data frames. Each data frame represents one unique attractor, where each column corresponds to a gene, and each row corresponds to one state in the attractor.


If both \code{returnSequences} and \code{returnAttractors} are true, there is an additional element \code{attractorAssignment}. This integer vector specifies the indices of the attractors to which the sequences lead.

The structure supports pretty printing using the \code{\link{print}} method.}

\references{
E. Dubrova, M. Teslenko (2011), A SAT-based algorithm for finding attractors in synchronous Boolean networks. IEEE/ACM Transactions on Computational Biology and Bioinformatics 8(5):1393--1399.

A. Biere (2008), PicoSAT Essentials. Journal on Satisfiability, Boolean Modeling and Computation 4:75-97.
}

\seealso{
\code{\link{loadNetwork}}, \code{\link{loadBioTapestry}}, \code{\link{loadSBML}}, \code{\link{getAttractors}}, \code{\link{plotAttractors}}, \code{\link{attractorsToLaTeX}}, \code{\link{getTransitionTable}}, \code{\link{getBasinOfAttraction}}, \code{\link{getAttractorSequence}}, \code{\link{getStateSummary}}, \code{\link{getPathToAttractor}}, \code{\link{fixGenes}}
}

\examples{
\dontrun{
data(igf)

# exhaustive state space simulation
sim <- simulateSymbolicModel(igf)
plotAttractors(sim)

# exhaustive attractor search using SAT solver
sim <- simulateSymbolicModel(igf, method="sat.exhaustive")
plotAttractors(sim)
}
}

\name{stateTransition}
\Rdversion{1.1}
\alias{stateTransition}

\title{
Perform a transition to the next state
}
\description{
Calculates the next state in a supplied network for a given current state
}
\usage{
stateTransition(network,
                state,
                type = c("synchronous","asynchronous","probabilistic"),
                geneProbabilities, 
                chosenGene, 
                chosenFunctions,
                timeStep = 0)
}
\arguments{
  \item{network}{
	A network structure of class \code{BooleanNetwork}, \code{SymbolicBooleanNetwork} or \code{ProbabilisticBooleanNetwork}. These networks can be read from files by \code{\link{loadNetwork}}, generated by \if{latex}{\cr}\code{\link{generateRandomNKNetwork}}, or reconstructed by \code{\link{reconstructNetwork}}.
}
  \item{state}{
	The current state of the network, encoded as a vector with one 0-1 element for each gene. If \code{network} is of class \code{SymbolicBooleanNetwork} and makes use of more than one predecessor state, this can also be a matrix with the genes in the columns and multiple predecessor states in the rows.
}
  \item{type}{The type of transition to be performed. 
  
  If set to "synchronous", all genes are updated using the corresponding transition functions. 
  
  If set to "asynchronous", only one gene is updated. This gene is either chosen randomly or supplied in parameter \code{chosenGene}. 
  
  If set to "probabilistic", one transition function is chosen for each gene, and the genes are updated synchronously. The functions are either chosen randomly depending on their probabilities, or they are supplied in parameter \code{chosenFunctions}.
    
  Default is "synchronous" for objects of class \code{BooleanNetwork} and \code{SymbolicBooleanNetwork}, and "probabilistic" for objects of class \code{ProbabilisticBooleanNetwork}.}
  
  \item{geneProbabilities}{An optional vector of probabilities for the genes if \code{type="asynchronous"}. By default, each gene has the same probability to be chosen for the next state transition. These probabilities can be modified by supplying a vector of probabilities for the genes which sums up to one.}
  
  \item{chosenGene}{If \code{type="asynchronous"} and this parameter is supplied, no random update is performed. Instead, the gene with the supplied name or index is chosen for the next transition.}
  
  \item{chosenFunctions}{If \code{type="probabilistic"}, this parameter can contain a set of function indices for each gene. In this case, transition functions are not chosen randomly, but the provided functions are used in the state transition.}
  
  \item{timeStep}{An optional parameter that specifies the current time step associated with \code{state}. This is only relevant for networks of class \code{SymbolicBooleanNetwork} that make use of time-dependent predicates (\code{timelt, timeis, timegt}). Otherwise, this parameter is ignored.  
  }
}

\value{
	The subsequent state of the network, encoded as a vector with one 0-1 element for each gene.}
\seealso{
\code{\link{loadNetwork}}, \code{\link{generateRandomNKNetwork}}, \code{\link{generateState}}
}
\examples{
\dontrun{
# load example network
data(cellcycle)

# calculate a synchronous state transition
print(stateTransition(cellcycle, c(1,1,1,1,1,1,1,1,1,1)))

# calculate an asynchronous state transition of gene CycA
print(stateTransition(cellcycle, c(1,1,1,1,1,1,1,1,1,1),
                      type="asynchronous", chosenGene="CycA"))
                      
# load probabilistic network
data(examplePBN)

# perform a probabilistic state transition
print(stateTransition(examplePBN, c(0,1,1),
                      type="probabilistic"))       
}
}

\name{loadSBML}
\alias{loadSBML}
\title{
Load an SBML document
}
\description{
Loads an SBML document that specifies a qualitative model using the \code{sbml-qual} extension package.
}
\usage{
loadSBML(file, symbolic=FALSE)
}

\arguments{
  \item{file}{
The SBML document to be imported
}

  \item{symbolic}{
If set to \code{TRUE}, the function returns an object of class \code{SymbolicBooleanNetwork} with an expression tree representation. Otherwise, it returns an object of class \code{BooleanNetwork} with a truth table representation.
}
}
\details{
The import assumes an SBML level 3 version 1 document with the \code{sbml-qual} extension package version 1.0.
\pkg{BoolNet} only supports a subset of the \code{sbml-qual} standard. The function tries to import those documents that describe a logical model with two possible values per species. It does not support general logical models with more than two values per species or Petri nets. 

Further details on the import:
\itemize{
\item{The import supports multiple function terms with the same output for a transition and interprets them as a disjunction, as proposed in the specification.}
\item{Comparison operators are converted to the corresponding Boolean expressions.}
\item{Compartments are ignored.}
}

For the import, the \pkg{XML} package is required.
}
\value{
Returns a structure of class \code{BooleanNetwork} or \code{SymbolicBooleanNetwork}, as described in \code{\link{loadNetwork}}.
}
\references{
\url{http://sbml.org/Documents/Specifications/SBML_Level_3/Packages/Qualitative_Models_(qual)}
}

\seealso{
\code{\link{toSBML}}, \code{\link{loadNetwork}}
}
\examples{
\dontrun{
# load the cell cycle network
data(cellcycle)

fil <- tempfile()
# export the network to SBML
toSBML(cellcycle, fil)

# reimport the model
print(loadSBML(fil))
}
}

\name{loadBioTapestry}
\alias{loadBioTapestry}
\title{
Import a network from BioTapestry
}
\description{
Imports a Boolean network from a BioTapestry file (*.btp). BioTapestry is an interactive tool for building, visualizing, and simulating gene-regulatory networks, and can be accessed at \url{http://www.biotapestry.org}. 
}
\usage{
loadBioTapestry(file,
                symbolic = FALSE)
}
\arguments{
  \item{file}{
The name of the file to import. This must be a BioTapestry XML file (*.btp).
}
  
  \item{symbolic}{
If set to \code{TRUE}, the function returns an object of class \code{SymbolicBooleanNetwork} with an expression tree representation. Otherwise, it returns an object of class \code{BooleanNetwork} with a truth table representation.
}
}
\details{
The function builds up a Boolean network by importing the nodes, the links between these nodes, and the simulation parameters of the top-level plot of a BioTapestry file. The BioTapestry network should have the following properties:
\itemize{

\item All links should be either enhancers or repressors. Unspecified ("neutral") links are ignored.
\item In the simulation parameters, each node should specify the correct logical function (AND, OR, XOR) for its inputs.
\item Constant genes can be generated by modeling a gene without any input link and setting the simulation parameter \code{initVal} to 0 or 1.
}
}
\value{
A network of class \code{BooleanNetwork} or \code{SymbolicBooleanNetwork}, as described in \code{\link{loadNetwork}}.
}
\references{
W. J. R. Longabaugh, E. H. Davidson, H. Bolour (2005), Computational representation of developmental genetic regulatory networks. Developmental Biology 283(1):1--16.
}

\seealso{
\code{\link{loadNetwork}}, \code{\link{loadSBML}}
}
\examples{
# import the example BioTapestry file
# included in the package vignette
exampleFile <- system.file("doc/example.btp",
                           package="BoolNet")
net <- loadBioTapestry(exampleFile)

# print the imported network
print(net)
}

\name{getAttractorSequence}
\alias{getAttractorSequence}
\title{
Decode the state sequence of a synchronous attractor
}
\description{
Obtains the sequence of states belonging to a single synchronous attractor from the encoded data in an \code{AttractorInfo} structure or in a \code{SymbolicSimulation} structure.
}
\usage{
getAttractorSequence(attractorInfo, attractorNo)
}
\arguments{
  \item{attractorInfo}{
	An object of class \code{AttractorInfo}, as returned by \code{\link{getAttractors}}, or of class \code{SymbolicSimulation}, as returned by \code{\link{simulateSymbolicModel}}. As the transition table information in this structure is required, \code{getAttractors} must be called in synchronous mode and with \code{returnTable} set to TRUE. Similarly, \code{simulateSymbolicModel} must be called with \code{returnGraph=TRUE}.
}

  \item{attractorNo}{
The index of the attractor in \code{attractorInfo} whose state sequence should be obtained
}
}

\value{
Returns a data frame with the genes in the columns. The rows are the successive states of the attractor. The successor state of the last state (i.e. the last row) is the first state (i.e. the first row). 
}

\seealso{
\code{\link{getAttractors}}, \code{\link{simulateSymbolicModel}}, \code{\link{getPathToAttractor}}, \code{\link{plotSequence}}, \code{\link{sequenceToLaTeX}}
}
\examples{
\dontrun{
# load example data
data(cellcycle)

# get attractors
attractors <- getAttractors(cellcycle)

# print basin of 7-state attractor
print(getAttractorSequence(attractors, 2))
}
}

\name{getTransitionTable}
\Rdversion{1.1}
\alias{getTransitionTable}
\title{
Retrieve the transition table of a network
}
\description{
Retrieves the transition table and additional attractor information of a network. 
}
\usage{
getTransitionTable(attractorInfo)
}
\arguments{
  \item{attractorInfo}{
	An object of class \code{AttractorInfo}, as returned by \code{\link{getAttractors}}, or of class \code{SymbolicSimulation}, as returned by \code{\link{simulateSymbolicModel}}. As the transition table information in this structure is required, \code{getAttractors} must be called in synchronous mode and with \code{returnTable} set to TRUE. Similarly, \code{simulateSymbolicModel} must be called with \code{returnGraph=TRUE}.
}
}
\details{
Depending on the configuration of the call to \code{getAttractors} or \code{simulateSymbolicModel} that returned \code{attractorInfo}, this function either returns the complete transition table (for exhaustive synchronous search) or the part of the transition table calculated in a heuristic synchronous search. Asynchronous search is not supported, as no transition table is calculated.
}

\value{
Returns a generic dataframe of the class \code{TransitionTable}. For n genes, the first n columns code for the original state (in this case, the \code{state} parameter), i.e. each column represents the value of one gene. The next n columns code for the successive state after a transition. The column \code{attractorAssignment} indicates the attractor to the state is assigned. If this information is available,  the column \code{stepsToAttractor} indicates how many transitions are needed from the original state to the attractor. The table has a row for each possible input state.
The \code{TransitionTable} class supports pretty printing using the \code{\link{print}} method.
}
\seealso{
\code{\link{getStateSummary}}, \code{\link{getBasinOfAttraction}}, \code{\link{getAttractors}}, \code{\link{simulateSymbolicModel}}
}
\examples{
\dontrun{
# load example data
data(cellcycle)

# get attractors
attractors <- getAttractors(cellcycle)

# print the transition table
print(getTransitionTable(attractors))
}
}

\name{yeastTimeSeries}
\alias{yeastTimeSeries}
\docType{data}
\title{
Yeast cell cycle time series data
}
\description{
Preprocessed time series measurements of four genes from the yeast cell cycle data by Spellman et al.
}
\usage{data(yeastTimeSeries)}
\format{
  A matrix with 14 measurements for the genes Fhk2, Swi5, Sic1, and Clb1. Each gene is a row of the matrix, and each column is a measurement.
}
\details{
The data were obtained from the web site of the yeast cell cycle analysis project. The time series synchronized with the elutriation method were extracted for the genes Fhk2, Swi5, SIC1, and Clb1. In a preprocessing step, missing values were imputed by taking the means of the measurements of the same genes at neighbouring time points.
}
\source{
P. T. Spellman, G. Sherlock, M. Q. Zhang, V. R. Iyer, K. Anders, M. B. Eisen, P. O. Brown, D. Botstein, B. Futcher (1998), Comprehensive Identification of Cell Cycle-regulated Genes of the Yeast Saccharomyces cerevisiae by Microarray Hybridization. Molecular Biology of the Cell 9(12):3273--3297.
}
\examples{
data(yeastTimeSeries)

# the data set is stored in a variable called 'yeastTimeSeries'
print(yeastTimeSeries)
}

\name{chooseNetwork}
\alias{chooseNetwork}
\title{
Extract a single Boolean network from a probabilistic Boolean network
}
\description{
Creates a \code{BooleanNetwork} object with exactly one function per gene by extracting a specified set of transition functions from a \code{ProbabilisticBooleanNetwork} or \code{BooleanNetworkCollection} object.
}
\usage{
chooseNetwork(probabilisticNetwork, 
              functionIndices, 
              dontCareValues=NULL, 
              readableFunctions=FALSE)
}
\arguments{
  \item{probabilisticNetwork}{
A \code{ProbabilisticBooleanNetwork} or \code{BooleanNetworkCollection} object as returned by \code{\link{reconstructNetwork}} or \code{\link{loadNetwork}}
}
  \item{functionIndices}{
A vector of function indices with one entry for each gene
}
  
  \item{dontCareValues}{
If \code{probabilisticNetwork} is of class \code{BooleanNetworkCollection}, this specifies the values to fill in for "don't care" (*) values in the truth tables of the transition functions. This is a list containing one vector of Boolean values for each gene. The lengths of the vectors must coincide with the numbers of "don't care" values in the functions.
}
  \item{readableFunctions}{
If \code{probabilisticNetwork} is of class \code{BooleanNetworkCollection}, the string representations of the transition functions must be refreshed after filling in values for the "don't care" entries. 
This parameter specifies if readable DNF representations of the transition function truth tables are generated and displayed when the network is printed. If set to FALSE, the truth table result column is displayed. If set to "canonical", a canonical Disjunctive Normal Form is generated from each truth table. If set to "short", the canonical DNF is minimized by joining terms (which can be time-consuming for functions with many inputs). If set to TRUE, a short DNF is generated for functions with up to 12 inputs, and a canonical DNF is generated for functions with more than 12 inputs.
}  
}
\value{
Returns an object of class \code{BooleanNetwork} consisting of the transition functions whose indices were specified in \code{functionIndices}. The class \code{BooleanNetwork} is described in more detail in \code{\link{loadNetwork}}.

Constant genes are automatically fixed (e.g. knocked-out or over-expressed). This means that they are always set to the constant value, and states with the complementary value are not considered in transition tables etc. If you would like to change this behaviour, use \code{\link{fixGenes}} to reset the fixing.
}
\seealso{
\code{\link{reconstructNetwork}}, \code{\link{loadNetwork}}
}
\examples{
  \dontrun{
# load example data
data(examplePBN)

# extract a unique network
# - always use the first function
net <- chooseNetwork(examplePBN, rep(1, length(examplePBN$genes)))

# get attractors from this network
print(getAttractors(net))
}
}

\name{generateTimeSeries}
\alias{generateTimeSeries}

\title{
Generate time series from a network
}
\description{
Generates time series by simulating successive state transitions from random start states. In addition, the resulting matrices can be perturbed by Gaussian noise.
}
\usage{
generateTimeSeries(network, 
                   numSeries, 
                   numMeasurements, 
                   type = c("synchronous","asynchronous","probabilistic"),
                   geneProbabilities, 
                   perturbations = 0,
                   noiseLevel = 0)
}

\arguments{
  \item{network}{
An object of class \code{BooleanNetwork} or \code{SymbolicBooleanNetwork} that contains the network for which time series are generated
}

  \item{numSeries}{
The number of random start states used to generate successive series of states, that is, the number of time series matrices to generate
}

  \item{numMeasurements}{
The number of states in each of the time series matrices. The first state of each time series is the randomly generated start state. The remaining \code{numMeasurements - 1} states are obtained by successive state transitions.
}

  \item{type}{
The type of state transitions to be performed (see \code{\link{stateTransition}})
}

\item{geneProbabilities}{
An optional vector of probabilities for the genes if \code{type="asynchronous"}. By default, each gene has the same probability to be chosen for the next state transition. These probabilities can be modified by supplying a vector of probabilities for the genes which sums up to one.
}

  \item{perturbations}{
If this argument has a value greater than 0, artificial perturbation experiments are generated. That is, \code{perturbations} genes in each time series are knocked out or overexpressed artificially using the \code{\link{fixGenes}} function.   
}

  \item{noiseLevel}{
If this is non-zero, it specifies the standard deviation of the Gaussian noise which is added to all entries of the time series matrices. By default, no noise is added to the time series.
}
}
\value{
A list of matrices, each corresponding to one time series. Each row of these matrices contains measurements for one gene on a time line, i. e. column \code{i+1} contains the successor states of column \code{i+1}. If \code{noiseLevel} is non-zero, the matrices contain real values, otherwise they contain only 0 and 1. 

If \code{perturbations>0}, the result list contains an additional matrix \code{perturbations} specifying the artificial perturbations applied to the different time series. This matrix has \code{numSeries} columns and one row for each gene in the network. A matrix entry is 0 for a knock-out of the corresponding gene in the corresponding time series, 1 for overexpression, and NA for no perturbation.

The result format is compatible with the input parameters of \code{\link{binarizeTimeSeries}} \if{latex}{\cr} and \code{\link{reconstructNetwork}}.
}

\seealso{
\code{\link{stateTransition}}, \code{\link{binarizeTimeSeries}}, \code{\link{reconstructNetwork}}
}
\examples{
\dontrun{
# generate noisy time series from the cell cycle network
data(cellcycle)
ts <- generateTimeSeries(cellcycle, numSeries=50, numMeasurements=10, noiseLevel=0.1)

# binarize the noisy time series
bin <- binarizeTimeSeries(ts, method="kmeans")$binarizedMeasurements

# reconstruct the network
print(reconstructNetwork(bin, method="bestfit"))
}
}

\name{generationFunctions}
\alias{generateCanalyzing}
\alias{generateNestedCanalyzing}
\title{
Generation functions for biologically relevant function classes
}
\description{
These generation functions randomly generate canalyzing or nested canalyzing Boolean functions. These functions are usually not called directly, but are supplied to the \code{functionGeneration} parameter of \code{\link{generateRandomNKNetwork}}.
}
\usage{
generateCanalyzing(input)
generateNestedCanalyzing(input)
}

\arguments{
  \item{input}{
  A vector of input gene indices for the Boolean function
}
}
\value{
A binary vector corresponding to the result column of the truth table that represents the canalyzing/nested canalyzing function.
}
\references{
S. Kauffman and C. Peterson and B. Samuelsson and C. Troein (2004), Genetic networks with canalyzing Boolean rules are always stable. PNAS 101(49):7102--17107.
}


\seealso{
\code{\link{generateRandomNKNetwork}}
}
\examples{
\dontrun{
  # generate a random network with canalyzing functions
  net1 <- generateRandomNKNetwork(n=10, k=5, 
                                  functionGeneration="generateCanalyzing")
  print(net1)
                                  
  # generate a random network with nested canalyzing functions
  net2 <- generateRandomNKNetwork(n=10, k=5, 
                                  functionGeneration="generateNestedCanalyzing")
  print(net2)                                  
}  
}

\name{cellcycle}
\alias{cellcycle}
\docType{data}
\title{
Mammalian cell cycle network
}
\description{
The mammalian cell cycle network as described by Faure et al.
}
\usage{data(cellcycle)}
\details{
The data consists of a variable \code{cellcycle} of class \code{BooleanNetwork} with 10 genes describing the four phases of the mammalian cell cycle. The network has one steady-state attractor. Furthermore, it has one synchronous attractor with 7 states and one asynchronous complex/loose attractor with 112 states. The class \code{BooleanNetwork} is described in more detail in \code{\link{loadNetwork}}.
}
\source{
A. Faure, A. Naldi, C. Chaouiya and D. Thieffry (2006), Dynamical analysis of a generic Boolean model for the control of the mammalian cell cycle. Bioinformatics 22(14):e124--e131.
}
\examples{
data(cellcycle)

# the network is stored in a variable called 'cellcycle'
print(cellcycle)

}

\name{testNetworkProperties}
\alias{testNetworkProperties}
\alias{testIndegree}
\alias{testAttractorRobustness}
\alias{testTransitionRobustness}
\title{
Test properties of networks by comparing them to random networks
}
\description{
This is a general function designed to determine unique properties of biological networks by comparing them to a set of randomly generated networks with similar structure.
}
\usage{
testNetworkProperties(network, 
                      numRandomNets = 100, 
                      testFunction = "testIndegree", 
                      testFunctionParams = list(), 
                      accumulation = c("characteristic", "kullback_leibler"), 
                      alternative=c("greater","less"),
                      sign.level = 0.05, 
                      drawSignificanceLevel = TRUE, 
                      klBins, 
                      klMinVal = 1e-05, 
                      linkage = c("uniform", "lattice"), 
                      functionGeneration = c("uniform", "biased"), 
                      validationFunction, failureIterations=10000,
                      simplify = FALSE, 
                      noIrrelevantGenes = TRUE, 
                      d_lattice = 1, 
                      zeroBias = 0.5, 
                      title = "", 
                      xlab, 
                      xlim, 
                      breaks = 30, 
                      ...)
}
\arguments{
  \item{network}{
  A network structure of class \code{BooleanNetwork} or \code{SymbolicBooleanNetwork}
}
  \item{numRandomNets}{
   The number of random networks to generate for comparison
}
  \item{testFunction}{
  The name of a function that calculates characteristic values that describe properties of the network. There are two built-in functions: "testIndegree" calculates the in-degrees of states in the network, and "testAttractorRobustness" counts the occurrences of attractors in perturbed copies. It is possible to supply user-defined functions here. See Details.
}
  \item{testFunctionParams}{
A list of parameters to \code{testFunction}. The elements of the list depend on the chosen function.
}
  \item{accumulation}{
If "characteristic" is chosen, the test function is required to return a single value that describes the network. In this case, a histogram of these values in random networks is plotted, and the value of the original network is inserted as a vertical line.

If "kullback_leibler" is chosen, the test function can return a vector of values which is regarded as a sample from a probability distribution. In this case, the Kullback-Leibler distances of the distributions from the original network and each of the random networks are calculated and plotted in a histogram. The Kullback-Leibler distance measures the difference between two probability distributions. In this case, the resulting histogram shows the distribution of differences between the original network and randomly generated networks.
}
  
  \item{alternative}{
If \code{accumulation="characteristic"}, this specifies whether the characteristic value is expected to be greater or less than the random results under the alternative hypothesis.  
}  

  \item{sign.level}{
If \code{accumulation="characteristic"}, this specifies a significance level for a computer-intensive test.

If \code{alternative="greater"}, the test is significant if the characteristic value is greater than at least \code{(1-sign.level)*100}\% of the characteristic values of the random networks. 

If \code{alternative="less"}, the test is significant if the characteristic value is less than at most \code{sign.level*100}\% of the characteristic values of the random networks.
}
  \item{drawSignificanceLevel}{
If \code{accumulation="characteristic"} and this is true, a vertical line is plotted for the significance level in the histogram.
}
  \item{linkage, functionGeneration, validationFunction, failureIterations, simplify, noIrrelevantGenes, d_lattice, zeroBias}{
  The corresponding parameters of \code{\link{generateRandomNKNetwork}} used to generate the random networks. This allows for customization of the network generation process. The three remaining parameters of \code{\link{generateRandomNKNetwork}} are set to values that ensure structural similarity to the original network: The parameters \code{n} and \code{k} are set to the corresponding values of the original network, and \code{topology="fixed"}. 
}

  \item{klBins}{
If \code{accumulation="kullback_leibler"}, the number of bins used to discretize the samples for the Kullback-Leibler distance calculations. By default, each unique value in the samples has its own bin, i.e. no further discretization is performed. The influence of discretization on the resulting histogram may be high.
}
  \item{klMinVal}{
If \code{accumulation="kullback_leibler"}, this defines the minimum probability for the calculation of the Kullback-Leibler distance to ensure stability of the results.  
}

  \item{title}{
  The title of the plots. This is empty by default.
}
  \item{xlab}{
Customizes label of the x axis of the histogram. For the built-in test functions, the x axis label is set automatically.
}
  \item{xlim}{
Customizes the limits of the x axis of the histogram. For the built-in test functions, suitable values are chosen automatically. 
}
  \item{breaks}{
Customizes the number of breaks in the 
}
  \item{\dots}{
Further graphical parameters for \code{\link{hist}}
}
}
\details{
This function generically compares properties of biological networks to a set of random networks. It can be extended by supplying custom functions to the parameter \code{testFunction}. Such a function must have the signature

\code{function(network,
               accumulate=TRUE,
               params)}

\describe{
\item{network}{This is the network to test. In the process of the comparison, both the original network and the random networks are passed to the function}

\item{accumulate}{If \code{accumulate=TRUE}, the function must return a single value quantifying the examined property of the network. If \code{accumulate=FALSE}, the function can return a vector of values (e.g., one value for each gene/state etc.)}

\item{params}{A list of further parameters for the function supplied by the user in \code{testFunctionParams} (see above). This can contain any type of information your test function needs.}
}

Three built-in functions for synchronous Boolean networks already exist:
\describe{
\item{testIndegree}{This function is based on the observation that, often, in biological networks, many state transitions lead to the same states. In other words, there is a small number of "hub" states. In the state graph, this means that the in-degree of some states (i.e., the number of transitions leading to it) is high, while the in-degree of many other states is 0. We observed that random networks do not show this behaviour, thus it may be a distinct property of biological networks. For this function, the parameter \code{alternative} of \code{testNetworkProperties} should be set to "greater".

The function does not require any parameter entries in \code{params}. If \code{accumulate=FALSE}, it returns the in-degrees of all synchronous states in the network. If \code{accumulate=TRUE}, the Gini index of the in-degrees is returned as a characteristic value of the network. The Gini index is a measure of inequality. If all states have an in-degree of 1, the Gini index is 0. If all state transitions lead to one single state, the Gini index is 1.

This function requires the \pkg{igraph} package for the analysis of the in-degrees.
}

\item{testAttractorRobustness}{This function tests the robustness of attractors in a network to noise. We expect attractors in a real network to be less susceptible to noise than attractors in randomly generated networks, as biological processes can be assumed to be comparatively stable. There are modes of generating noise: Either the functions of the network can be perturbed, or the state trajectories can be perturbed in a simulation of the network. If \code{perturb="functions"} or \code{perturb="transitions"}, the function generates a number of perturbed copies of the network using \code{\link{perturbNetwork}} and checks whether the original attractors can still be found in the network.
If \code{perturb="trajectories"}, the network itself is not perturbed. Instead, a set of random initial states is generated, and a set of perturbed states is generated from these initial states by flipping one or more bits. Then, the function tests whether the attractors are the same for the initial states and the corresponding perturbed states. This corresponds to calling \code{\link{perturbTrajectories}} with \code{measure="attractor"}. 

\code{params} can hold a number of parameters:
\describe{
\item{numSamples}{If \code{perturb="trajectories"}, the number of randomly generated state pairs to generate. Otherwise the number of perturbed networks that are generated.}
\item{perturb}{Specifies the type of perturbation to be applied (possible values: \code{"functions"}, \code{"transitions"} and \code{"trajectories"} -- see above).}
\item{method, simplify, readableFunctions, excludeFixed, maxNumBits, numStates}{ If \code{perturb="functions"} or \code{perturb="transitions"}, these are the corresponding parameters of \code{\link{perturbNetwork}} that influence the way the network is perturbed.}
\item{flipBits}{ If \code{perturb="trajectories"}, the are the corresponding parameters of \code{\link{perturbTrajectories}} that defines how many bits are flipped.}
}

If {perturb="functions"} or \code{perturb="transitions"} and \code{accumulate=FALSE}, the function returns a vector of percentages of original attractors found in each of the perturbed copies of the original network. If \code{accumulate=TRUE}, the function returns the overall percentage of original attractors found in all perturbed copies.

If \code{perturb="trajectories"} and \code{accumulate=FALSE}, the function returns a logical vector of length \code{numSamples} specifying whether the attractor was the same for each initial state and the corresponding perturbed state. If \code{accumulate=TRUE}, the function returns the percentage of pairs of initial states and perturbed states for which the attractors were the same.

For this function, the parameter \code{alternative} of \code{testNetworkProperties} should be set to "greater".
}

\item{testTransitionRobustness}{This function calls \code{perturbTrajectories} with \code{measure="hamming"} to measure the average Hamming distance between successor states of randomly generated initial states and perturbed copies of these states.

code{params} can hold parameters \code{numSamples, flipBits} corresponding to the parameters of \code{\link{perturbTrajectories}} that define how many initial states are drawn and how many bits are flipped.

If \code{accumulate=FALSE}, the function returns a numeric vector of length \code{numSamples} with the normalized Hamming distances of all pairs of initial states and perturbed copies. If \code{accumulate=TRUE}, the mean normalized Hamming distance over all pairs is returned.

For this function, the parameter \code{alternative} of \code{testNetworkProperties} should be set to "less".
}

} 
               
}
\value{
The function returns a list with the following elements
 \item{hist}{The histogram that was plotted. The type of histogram depends on the parameter \code{accumulation}.}
 \item{pval}{If \code{accumulation="characteristic"}, a p-value for the alternative hypothesis that the test statistic value of the supplied network is greater than the value of a randomly generated network is supplied.}
 \item{significant}{If \code{accumulation="characteristic"}, this is true for \code{pval < sign.level}.} 
}

\seealso{
\code{\link{generateRandomNKNetwork}}, \code{\link{perturbNetwork}}, \code{\link{perturbTrajectories}}, \code{\link{plotStateGraph}}, \code{\link{getAttractors}}
}
\examples{
\dontrun{
# load mammalian cell cycle network
data(cellcycle)

if (interactive())
# do not run these examples in the package check, as they take some time
{
  # compare the in-degrees of the states in the
  # cell cycle network to random networks
  testNetworkProperties(cellcycle, testFunction="testIndegree", alternative="greater")

  # compare the in-degrees of the states in the
  # cell cycle network to random networks,
  # and plot the Kullback-Leibler distances of the 100 experiments
  testNetworkProperties(cellcycle, testFunction="testIndegree",
                        accumulation = "kullback_leibler")

  # compare the robustness of attractors in the cell cycle network
  # to random networks by perturbing the networks
  testNetworkProperties(cellcycle, testFunction="testAttractorRobustness",
                        testFunctionParams=list(perturb="functions", numSamples=10),
                        alternative="greater")

  # compare the robustness of attractors in the cell cycle network
  # to random networks by perturbing the state trajectories
  testNetworkProperties(cellcycle, testFunction="testAttractorRobustness",
                        testFunctionParams=list(perturb="trajectories", numSamples=10),
                        alternative="greater")
                      
  # compare the robustness of single state transitions in the cell cycle network
  testNetworkProperties(cellcycle, testFunction="testTransitionRobustness",
                        testFunctionParams=list(numSamples=10),
                        alternative="less")
}                     
}
}
